//---------------------------------------------------------------------------
// This sourcefile is created by Peter Jodda / peter@jodda.de
// It is under the GPL license.
// If you expand this file, please contact me, that I can integrate the
// enhancements in the original file.
//---------------------------------------------------------------------------
// Module: ApplicationFrame
//---------------------------------------------------------------------------
// Purpose: Hold several image frames in one application.
// Give also access to application global functions (Utilities, Icons..)
//---------------------------------------------------------------------------

#include <stdlib.h>
#include <unistd.h>
#include <sys/stat.h>
#include <sys/types.h>


#include <qtranslator.h>
#include <qtextcodec.h>
#include <qdir.h>

#include "IconLibrary.h"
#include "QtUtilities.h"
#include "ApplicationFrame.h"
#include "ApplicationStorage.h"
#include "MainFrame.h"
#include "BaseData.h"
#include "BaseDataStrings.h"
#include "SettingsBox.h"
#include "AutosaveBox.h"
#include "AutosaveInfoDialog.h"
#include "FileLoader.h"


extern unsigned char IconLibraryMemoryData[];

//===========================================================================
//
//===========================================================================
ApplicationStorage *g_Application;

ApplicationFrame::ApplicationFrame(QApplication *pc_Application)
{
   m_i_NumberOfImages = 0;


   m_pc_ApplicationStorage = new ApplicationStorage;
   m_pc_ApplicationStorage->m_pc_IconLibrary = new IconLibrary(IconLibraryMemoryData);
   m_pc_ApplicationStorage->m_pc_QtUtility = new QtUtility();
   m_pc_ApplicationStorage->m_pc_ApplicationFrame = this;
   m_pc_ApplicationStorage->m_pc_QApplication = pc_Application;

   g_Application = m_pc_ApplicationStorage;

   // load basic preferences from file
   LoadBaseData();

   // try to load a translator for the application
   LoadTranslator();

   CalculateTempDirectory();

   // create a settingsbox
   m_pc_SettingsBox = new SettingsBox(0);

   m_pc_AutosaveBox = new AutosaveBox(0);
   if(m_pc_AutosaveBox->GetNumberOfFiles() >0)
   {
      // something is autosaved -> display a message
      bool b_AutosaveTellAtStartup;
      b_AutosaveTellAtStartup = true;
      g_Application->m_pc_BaseData->GetValue(
                                  BASEDATA_AUTOSAVE_TELL_AT_STARTUP,
                                  b_AutosaveTellAtStartup);
      if(b_AutosaveTellAtStartup)
      {
         AutosaveInfoDialog *pc_AutosaveInfoDialog;
         pc_AutosaveInfoDialog = new AutosaveInfoDialog(0);
         pc_AutosaveInfoDialog->show();
      }
   }

} // ApplicationFrame() - constructor

//===========================================================================
//
//===========================================================================

ApplicationFrame::~ApplicationFrame()
{
   delete m_pc_ApplicationStorage->m_pc_IconLibrary;
   delete m_pc_ApplicationStorage->m_pc_QtUtility;
   delete m_pc_ApplicationStorage;
} // ~ApplicationFrame()

//===========================================================================
//
//===========================================================================

void ApplicationFrame::AddMainFrame(MainFrame *pc_NewFrame)
{
   m_Images.append(pc_NewFrame);
   m_i_NumberOfImages +=1;
} // AddMainFrame()

//===========================================================================
//
//===========================================================================

void ApplicationFrame::RemoveMainFrame(MainFrame *pc_OldFrame)
{
   int i;
   for(i=0; i< m_i_NumberOfImages; i++)
   {
      if(m_Images.at(i) == pc_OldFrame)
      {
         m_Images.remove(i);
         m_i_NumberOfImages -= 1;
         break;
      }
   }
   if(m_i_NumberOfImages ==0)
   {
      Terminate();
   }
} // RemoveMainFrame()

//===========================================================================
//
//===========================================================================

void ApplicationFrame::Terminate()
{
   // later: save some settings

   if(   m_pc_ApplicationStorage != NULL
      && m_pc_ApplicationStorage->m_pc_BaseData != NULL)
   {
      m_pc_ApplicationStorage->m_pc_BaseData->SaveFile();
   }
   

   exit(0);
} // Terminate()

//===========================================================================
//
//===========================================================================

void ApplicationFrame::ExitCall(MainFrame *pc_CallingFrame)
{
   while (m_i_NumberOfImages >0)
   {
      MainFrame *pc_CurrentFrame;

      pc_CurrentFrame = m_Images.at(m_i_NumberOfImages -1);
      // delete the image
      
      delete pc_CurrentFrame;
      // remove it from the list
      m_Images.remove(m_i_NumberOfImages -1);

      m_i_NumberOfImages -= 1;

   } // while images there

   // terminate program
   Terminate();
} // ExitCall()

//===========================================================================
//
//===========================================================================

void ApplicationFrame::LoadBaseData()
{
   QString qs_BaseDataName;

   qs_BaseDataName = BaseData::CalculateFilename("Perico");

   m_pc_ApplicationStorage->m_pc_BaseData
               = new BaseData(qs_BaseDataName);

} // LoadBaseData;

//===========================================================================
//
//===========================================================================

void ApplicationFrame::LoadTranslator()
{
   QString   qs_Locale;
   QString   qs_TranslationName;
   bool      b_Loaded;

   m_pc_ApplicationStorage->m_pc_QTranslator     = new QTranslator(0);

   qs_TranslationName = "";
   qs_Locale          = QTextCodec::locale();

   if(qs_Locale.length()>1)
   {
      // if we have a locale
      qs_TranslationName ="Perico_"+qs_Locale+".qm";
   }
   else
   {
      char * cp_GetEnvResult;

      // look, whether rc_Lang is set
      cp_GetEnvResult = getenv("RC_LANG");
      if(cp_GetEnvResult!= NULL)
      {

         char ca_StartLetter[3];
         strncpy(ca_StartLetter,cp_GetEnvResult,2);
         ca_StartLetter[2]='\0';
         qs_Locale= ca_StartLetter;
         qs_TranslationName ="Perico_"+qs_Locale+".qm";
      }
   }

   // try to load the translator
   b_Loaded = m_pc_ApplicationStorage->
                            m_pc_QTranslator->load(qs_TranslationName,".");

   // if loaded, then install it
   if(!b_Loaded)
   {
      QString qs_LanguageFile;
      bool    b_Success;
      if(g_Application->m_pc_BaseData== NULL)
      {
         b_Success = false;
      }
      else
      {
         b_Success = g_Application->m_pc_BaseData->GetValue(
                                  BASEDATA_LANGUAGE_FILENAME,
                                  qs_LanguageFile);
      }
      if(b_Success)
      {
         b_Loaded = m_pc_ApplicationStorage->
                            m_pc_QTranslator->load(qs_LanguageFile,"/");
      }
   }

   if(b_Loaded)
   {
      m_pc_ApplicationStorage->m_pc_QApplication
            ->installTranslator(m_pc_ApplicationStorage->m_pc_QTranslator);
   
   }
        
} // LoadTranslator

//===========================================================================
//
//===========================================================================

void ApplicationFrame::OpenSettingsBox()
{
   // show the box
   m_pc_SettingsBox->show();
   
} // OpenSettingsBox()

//===========================================================================
//
//===========================================================================

void ApplicationFrame::CalculateTempDirectory()
{
   QString qs_TempDirectory =".";
   bool    b_Success;
   // try to load the directory from the basedata

   b_Success = g_Application->m_pc_BaseData->GetValue(
                                  BASEDATA_TEMP_DIRECTORY,
                                  qs_TempDirectory);
								  
   // need at least a not empty directory
   if(qs_TempDirectory.isEmpty())
   {
      b_Success = false;
   }

   if(b_Success)
   {
      m_pc_ApplicationStorage->m_qs_TempDirectory = qs_TempDirectory;
   }
   else
   {
      // not set, then create $HOME/.perico
      char * cp_GetEnvResult;

      // look, whether rc_Lang is set
      cp_GetEnvResult = getenv("HOME");
      if(cp_GetEnvResult!= NULL)
      {
         m_pc_ApplicationStorage->m_qs_TempDirectory = cp_GetEnvResult;
         
      }
      else
      {
         //take the current working directory
         char c_CwdBuffer[4096];
         char *cp_CwdResult;
         
         cp_CwdResult = getcwd(c_CwdBuffer,sizeof(c_CwdBuffer));
         if( cp_CwdResult != NULL)
         {
            m_pc_ApplicationStorage->m_qs_TempDirectory = c_CwdBuffer;
         }
         else
         {
            // take the current directory
            m_pc_ApplicationStorage->m_qs_TempDirectory = "." ;
         }
      } //else no home directory

      // add directory .perico as basis
      m_pc_ApplicationStorage->m_qs_TempDirectory += "/.perico";
      
   } // else undefined

   // now we have a directory

   // create it, in case of non existent
   FileLoader::MakeDirectory((const char *) m_pc_ApplicationStorage->m_qs_TempDirectory);
   
} // CalculateTempDirectory()

//===========================================================================
//
//===========================================================================


