//---------------------------------------------------------------------------
// This sourcefile is created by Peter Jodda / peter@jodda.de
// It is under the GPL license.
// If you expand this file, please contact me, that I can integrate the
// enhancements in the original file.
//---------------------------------------------------------------------------
// Module: BSpline.c
//---------------------------------------------------------------------------
// Purpose: 2-Dimensional BSpline area
//---------------------------------------------------------------------------

#include <stdio.h>
#include "D3Utilities.h"      
#include "BSpline.h"     

//===========================================================================
//
//===========================================================================

BSpline::BSpline()
{
   // set default order to 3
   m_i_OrderOfBSpline = 3;
}
   
//===========================================================================
//
//===========================================================================

BSpline::~BSpline()
{
}

//===========================================================================
//
//===========================================================================

int BSpline::Minimum(int i,int j)
{
   if( i<j)
   {
      return i;
   }
   return j;
} // Minimum()

//===========================================================================
//
//===========================================================================

int BSpline::TruncateIndex(int i_Index, int i_OrderOfBSpline, int i_NumberOfPoints)
{
   if(i_Index<i_OrderOfBSpline)
   {
      return i_OrderOfBSpline-1;
   }
   if(i_Index < i_NumberOfPoints)
   {
      return i_Index;
   }
   return i_NumberOfPoints;
} // TruncateIndex()

//===========================================================================
//
//===========================================================================


void BSpline::DeBoorPoints(
                   D3Vector     *pc_GivenDeBoorPoints,  
                   int          i_NumberOfPoints,      
                   double       d_ParameterValue,    
                   int          i_KnotIndex,      
                   D3Vector     *pc_DeBoorHelp, 
                   D3Coordinate &c_WishedCurvePoint    
                  )
{
   double d_RowFactor;  
   int    i_LineIndex;       
   int    i_RowIndex;       
   int    i_KnotIndexMinusOrder;    

   i_KnotIndexMinusOrder = i_KnotIndex - m_i_OrderOfBSpline + 1;                 
                                      

   // copy points into a help vector
   for (i_LineIndex = Minimum(i_KnotIndex, i_NumberOfPoints - 1);
        i_LineIndex >= i_KnotIndexMinusOrder;  
        i_LineIndex--)                           
   {
      pc_DeBoorHelp->Set(i_LineIndex - i_KnotIndexMinusOrder,
                         pc_GivenDeBoorPoints->Get(i_LineIndex));      
                                      
   }


   // additional rows 
   for (i_LineIndex = i_NumberOfPoints;
        i_LineIndex <= i_KnotIndex;
        i_LineIndex++)            
   {
      pc_DeBoorHelp->Set(i_LineIndex - i_KnotIndexMinusOrder,
                         pc_GivenDeBoorPoints->Get(i_LineIndex - i_NumberOfPoints));   
   }
  

   for (i_RowIndex = 0;
        i_RowIndex < m_i_OrderOfBSpline - 1;
        i_RowIndex++)         
   {
      // calculate deboor scheme from index-1
      for (i_LineIndex = m_i_OrderOfBSpline - 1;
           i_LineIndex > i_RowIndex;
           i_LineIndex--)      
      {                                
         D3Coordinate d_Result;
         d_RowFactor =   (d_ParameterValue  - (double)TruncateIndex(i_LineIndex + i_KnotIndexMinusOrder   ,m_i_OrderOfBSpline,i_NumberOfPoints))
                         /
                            (double)(   TruncateIndex(i_LineIndex + i_KnotIndex - i_RowIndex,m_i_OrderOfBSpline,i_NumberOfPoints)
                                     -  TruncateIndex(i_LineIndex + i_KnotIndexMinusOrder   ,m_i_OrderOfBSpline,i_NumberOfPoints)
                                    );


         d_Result =   pc_DeBoorHelp->Get (i_LineIndex - 1)
                    + 
                      (   pc_DeBoorHelp->Get (i_LineIndex)
                        - pc_DeBoorHelp->Get (i_LineIndex - 1))
                      * d_RowFactor;

         // put result back into the vector             
         pc_DeBoorHelp->Set (i_LineIndex,d_Result);
         
      }  // for i_LineIndex
   }  // for i_RowIndex

  
   // final element     
   c_WishedCurvePoint = pc_DeBoorHelp->Get(m_i_OrderOfBSpline - 1);

}  // DeBoorPoints()

//===========================================================================
//
//===========================================================================


int BSpline::ColumnCurve(
           double d_ParameterValue,       
           int  i_NodeIntervalRow,      
           D3Vector *pc_RowVector,     
           int  &i_RealPointCount,      
           D3Vector *pc_Help, 
           D3Vector *pc_CyclicHelpVektor,   
           D3Vector *pc_DeBoorHelp 
          )               

{
   D3Coordinate d_Result;
   int          i_RunRow;
   int          i_RunColumn;
   int          i_Error;

   // calculate temporary points
      
   for (i_RunColumn = 0;
        i_RunColumn < m_i_NumberPointsColumn;
        i_RunColumn++)             
   {                                    
      for (i_RunRow = i_NodeIntervalRow - m_i_OrderOfBSpline + 1;                  
           i_RunRow <= Minimum(i_NodeIntervalRow, m_i_NumberPointsRow - 1);             
           i_RunRow++)                           
      {
        pc_CyclicHelpVektor->Set(i_RunRow, m_pc_DeBoorPoints->Get(i_RunRow,i_RunColumn)); 
      }

    
      for (i_RunRow = 0;
           i_RunRow <= i_NodeIntervalRow - m_i_NumberPointsRow;
           i_RunRow++)        
      {
         pc_CyclicHelpVektor->Set(i_RunRow, m_pc_DeBoorPoints->Get(i_RunRow,i_RunColumn));                  
      }


      DeBoorPoints(pc_CyclicHelpVektor,
                   m_i_NumberPointsRow,
                   d_ParameterValue,
                   i_NodeIntervalRow,
                   pc_DeBoorHelp,
                   d_Result);         

      pc_Help->Set(i_RunColumn, d_Result);
   }  // for i_RunColumn
  

   i_Error = CalculateBSpline1Dimension(pc_Help,
                   m_i_NumberPointsColumn,
                   pc_RowVector,
                   i_RealPointCount);      

   return i_Error;

} // ColumnCurve()

//===========================================================================
//
//===========================================================================

int BSpline::CalculateBSpline1Dimension(
            D3Vector *pc_GivenDeBoorPoints,
            int  i_NumberOfPoints,
            D3Vector *pc_CalculatedCurvePoints,
            int  &i_RealPointCount
           )

{
   D3Coordinate  d3_Result;
   D3Vector     *pc_DeBoorHelp;
   double        d_StepWidth;
   double        d_ParameterValue;
   int           i_NumberOfRealIntervals;
   int           i_NumberPointsBetweenNodes;
   int           i_RunInNodeInterval;
   int           i;


   i_NumberOfRealIntervals = i_NumberOfPoints;
   i_NumberOfRealIntervals -= m_i_OrderOfBSpline - 1;

   if (   i_NumberOfPoints < 3
       || m_i_OrderOfBSpline > i_NumberOfPoints
       || i_RealPointCount - 1 < 2 * i_NumberOfRealIntervals)
   {
      return 1;
   }

   pc_DeBoorHelp  = new D3Vector( m_i_OrderOfBSpline);

   i_NumberPointsBetweenNodes = (i_RealPointCount - 1) / i_NumberOfRealIntervals;
   d_StepWidth   = 1.0 / (double)i_NumberPointsBetweenNodes;

   DeBoorPoints(pc_GivenDeBoorPoints,
               i_NumberOfPoints,
               (double)(m_i_OrderOfBSpline - 1),
               m_i_OrderOfBSpline - 1,
               pc_DeBoorHelp,
               d3_Result
               );

   pc_CalculatedCurvePoints->Set(0,d3_Result);
   i_RealPointCount = 1;

   i_NumberOfRealIntervals += m_i_OrderOfBSpline - 1 ;

   for (i_RunInNodeInterval = m_i_OrderOfBSpline - 1;
        i_RunInNodeInterval < i_NumberOfRealIntervals;
        i_RunInNodeInterval++)
   {
      d_ParameterValue = (double)i_RunInNodeInterval + d_StepWidth;
      
      for (i = 0;
           i < i_NumberPointsBetweenNodes;
           i++)
      {
         DeBoorPoints(pc_GivenDeBoorPoints,
                i_NumberOfPoints,
                d_ParameterValue,
                i_RunInNodeInterval,
                pc_DeBoorHelp,
                d3_Result);
         pc_CalculatedCurvePoints->Set(i_RealPointCount, d3_Result);

         d_ParameterValue += d_StepWidth;
         i_RealPointCount++;
      }
   }


   delete pc_DeBoorHelp;

   return 0;

} // CalculateBSpline1Dimension()

//===========================================================================
//
//===========================================================================

int BSpline::CalculateBSpline2Dimensions(
                         D3Matrix *pc_DeBoorPoints,
                         int  i_PointCountRow,
                         int  i_PointCountColumnx,
                         D3Matrix *pc_ResultPoints,
                         int  &i_RealPointCountRow,
                         int  &i_RealPointCountColumn
                        )
{
   D3Vector *pc_Help;
   D3Vector *pc_WCurveHelpvector;
   D3Vector *pc_DeBoorHelp;

   D3Vector *pc_RowVektor;
   double    d_StepWidth;
   double    d_CurrentParameterRow;
   int       i_RunRowInterval;
   int       i_RunIntoOneInterval;
   int       i_NumberIntervalsInRow;         
   int       i_NumberBetweenTwoRowPoints;         
   int       i_Error;   
   int       i_Y;

   m_pc_DeBoorPoints      = pc_DeBoorPoints ;
   m_i_NumberPointsRow    = i_PointCountRow;
   m_i_NumberPointsColumn = i_PointCountColumnx;
  
   i_NumberIntervalsInRow  = m_i_NumberPointsRow;
   i_NumberIntervalsInRow -= m_i_OrderOfBSpline - 1;

   if (   m_i_NumberPointsRow < 3
       || m_i_NumberPointsColumn < 3
       || m_i_OrderOfBSpline > m_i_NumberPointsRow 
       || i_RealPointCountRow - 1 < 2 * i_NumberIntervalsInRow)         
   {
      return 1;                              
   }

   i_NumberBetweenTwoRowPoints = (i_RealPointCountRow - 1) / i_NumberIntervalsInRow;
   d_StepWidth   = 1.0 / (double)i_NumberBetweenTwoRowPoints;


   pc_Help             = new D3Vector( m_i_NumberPointsColumn);
   pc_WCurveHelpvector = new D3Vector( m_i_NumberPointsRow);
   pc_DeBoorHelp       = new D3Vector( m_i_NumberPointsRow);
   pc_RowVektor        = new D3Vector( pc_ResultPoints->GetHeight() );

   for(i_Y=0;i_Y<pc_ResultPoints->GetHeight();i_Y++)
   {
      pc_RowVektor->Set(i_Y,pc_ResultPoints->Get(0,i_Y));
   }
   i_Error = ColumnCurve((double)(m_i_OrderOfBSpline - 1),
                  m_i_OrderOfBSpline - 1,  
                  pc_RowVektor,
                  i_RealPointCountColumn,     
                  pc_Help,
                  pc_WCurveHelpvector,
                  pc_DeBoorHelp);
   for(i_Y=0;i_Y<pc_ResultPoints->GetHeight();i_Y++)
   {
      pc_ResultPoints->Set(0,i_Y,pc_RowVektor->Get(i_Y));
   }
   if (i_Error)
   {
      delete pc_Help;
      delete pc_WCurveHelpvector;
      delete pc_DeBoorHelp;
      delete pc_RowVektor;
      return i_Error;     
   }

   i_RealPointCountRow = 1;

   i_NumberIntervalsInRow += m_i_OrderOfBSpline - 1;

   for (i_RunRowInterval = m_i_OrderOfBSpline - 1;
        i_RunRowInterval < i_NumberIntervalsInRow;                 
        i_RunRowInterval++)
   {
      d_CurrentParameterRow = (double)i_RunRowInterval + d_StepWidth;
       
      for (i_RunIntoOneInterval = 1;         
           i_RunIntoOneInterval <= i_NumberBetweenTwoRowPoints;                       
           i_RunIntoOneInterval++)
      {
         for(i_Y=0;i_Y<pc_ResultPoints->GetHeight();i_Y++)
         {
            pc_RowVektor->Set(i_Y,pc_ResultPoints->Get(i_RealPointCountRow,i_Y));
         }
         i_Error = ColumnCurve(d_CurrentParameterRow,
                     i_RunRowInterval,
                      pc_RowVektor,
                      i_RealPointCountColumn,        
                      pc_Help,
                      pc_WCurveHelpvector,
                      pc_DeBoorHelp);

         for(i_Y=0;i_Y<pc_ResultPoints->GetHeight();i_Y++)
         {
            pc_ResultPoints->Set(i_RealPointCountRow,i_Y,pc_RowVektor->Get(i_Y));
         }

         if (i_Error)
         {
            delete pc_Help;
            delete pc_WCurveHelpvector;
            delete pc_DeBoorHelp;
            delete pc_RowVektor;
            return i_Error;         
         }

         d_CurrentParameterRow += d_StepWidth;
         i_RealPointCountRow++;

      } // for  i_RunIntoOneInterval

   }  // for i_RunRowInterval

   delete pc_Help;
   delete pc_WCurveHelpvector;
   delete pc_DeBoorHelp;
   delete pc_RowVektor;

   return 0;                        
}  // CalculateBSpline2Dimensions()

//===========================================================================
//
//===========================================================================

void BSpline::SetOrder(int i_OrderOfBSpline)
{
   if(i_OrderOfBSpline >=3)
   {
      m_i_OrderOfBSpline= i_OrderOfBSpline;
   }
} // SetOrder()

//===========================================================================
//
//===========================================================================

