//---------------------------------------------------------------------------
// This sourcefile is created by Peter Jodda / peter@jodda.de
// It is under the GPL license.
// If you expand this file, please contact me, that I can integrate the
// enhancements in the original file.
//---------------------------------------------------------------------------
// Module: Blend.c
//---------------------------------------------------------------------------
// Purpose: Blend of Colors
//---------------------------------------------------------------------------

#include <math.h>
#include <stdlib.h>

#include "Utilities.h"
#include "Blend.h"
#include "Geometry.h"
#include "ColorCalculation.h"
#include "PixelBuffer.h"
#include "BufferPainterEdges.h"
#include "BufferPainter.h"
#include "BSpline.h"
#include "MathHermite.h"
#include "PList.h"

//===========================================================================
//
//===========================================================================

BlendPoint::BlendPoint()
{
   m_i_X             =0;
   m_i_Y             =0;
   m_p_Color         =0;
   m_b_IsBorderPoint =0;
}

//===========================================================================
//
//===========================================================================

BlendLine::BlendLine(BlendPoint *pc_Point1,BlendPoint *pc_Point2)
{
   double d_DiffX;
   double d_DiffY;
   
   m_pc_Point1 = pc_Point1;
   m_pc_Point2 = pc_Point2;

   d_DiffX = m_pc_Point1->m_i_X - m_pc_Point2->m_i_X;   
   d_DiffY = m_pc_Point1->m_i_Y - m_pc_Point2->m_i_Y;

   d_Length =  sqrt(d_DiffX*d_DiffX + d_DiffY*d_DiffY); 
} // Constructor()

//===========================================================================
//
//===========================================================================

bool BlendLine::CutsLine(BlendLine *pc_SecondLine)
{
   GeoCoordinate c_CutPoint;
   GeoLine       c_Line1;
   GeoLine       c_Line2;
   bool          b_Result;

   // is there a common endpoint, then lines don't cut
   if(    m_pc_Point1 ==  pc_SecondLine->m_pc_Point1
       || m_pc_Point1 ==  pc_SecondLine->m_pc_Point2
       || m_pc_Point2 ==  pc_SecondLine->m_pc_Point1
       || m_pc_Point2 ==  pc_SecondLine->m_pc_Point2
      ) 
   {
      return false;
   
   }

   // convert to geolines
   c_Line1.Set( m_pc_Point1->m_i_X,m_pc_Point1->m_i_Y,
                m_pc_Point2->m_i_X,m_pc_Point2->m_i_Y);
   c_Line2.Set( pc_SecondLine->m_pc_Point1->m_i_X,pc_SecondLine->m_pc_Point1->m_i_Y,
                pc_SecondLine->m_pc_Point2->m_i_X,pc_SecondLine->m_pc_Point2->m_i_Y);

   
   b_Result = c_Line1.CutsLine(c_Line2,c_CutPoint);

   return b_Result;
} // CutsLine()

//===========================================================================
//
//===========================================================================

BlendTriangle::BlendTriangle( BlendLine *pc_Line1,
                  BlendLine *pc_Line2,
                  BlendLine *pc_Line3,
                  BlendPoint *pc_Point1,
                  BlendPoint *pc_Point2,
                  BlendPoint *pc_Point3)                 
{
   m_pc_Line[0]  = pc_Line1;
   m_pc_Line[1]  = pc_Line2;
   m_pc_Line[2]  = pc_Line3;
   m_pc_Point[0] = pc_Point1;
   m_pc_Point[1] = pc_Point2;
   m_pc_Point[2] = pc_Point3;

   
} // BlendTriangle() constructor

//===========================================================================
//
//===========================================================================

void BlendTriangle::printf()
{
   fprintf(stderr,"points (%d %d) (%d %d) (%d %d)\n",
               m_pc_Point[0]->m_i_X,
               m_pc_Point[0]->m_i_Y,
               m_pc_Point[1]->m_i_X,
               m_pc_Point[1]->m_i_Y,
               m_pc_Point[2]->m_i_X,
               m_pc_Point[2]->m_i_Y
               );

   fprintf(stderr,"lines (%d %d - %d %d) (%d %d - %d %d) (%d %d - %d %d)\n",
               m_pc_Line[0]->m_pc_Point1->m_i_X,
               m_pc_Line[0]->m_pc_Point1->m_i_Y,
               m_pc_Line[0]->m_pc_Point2->m_i_X,
               m_pc_Line[0]->m_pc_Point2->m_i_Y,
               m_pc_Line[1]->m_pc_Point1->m_i_X,
               m_pc_Line[1]->m_pc_Point1->m_i_Y,
               m_pc_Line[1]->m_pc_Point2->m_i_X,
               m_pc_Line[1]->m_pc_Point2->m_i_Y,
               m_pc_Line[2]->m_pc_Point1->m_i_X,
               m_pc_Line[2]->m_pc_Point1->m_i_Y,
               m_pc_Line[2]->m_pc_Point2->m_i_X,
               m_pc_Line[2]->m_pc_Point2->m_i_Y
               
               );
               
} // printf

//===========================================================================
//
//===========================================================================

bool BlendTriangle::CoordinateInTriangle(float f_X, float f_Y)
{
   GeoCoordinate c_Point;
   GeoLine       c_Line;
  
   // set points to given coordinate
   c_Point.SetXY(f_X,f_Y);

   // set line object and check 3rd point and given point
   c_Line.Set(m_pc_Point[0]->m_i_X,m_pc_Point[0]->m_i_Y,
              m_pc_Point[1]->m_i_X,m_pc_Point[1]->m_i_Y
              );
   if( ! c_Line.IsOnSameSideOfLine(
                        GeoCoordinate( m_pc_Point[2]->m_i_X,m_pc_Point[2]->m_i_Y),
                        c_Point)
     )
   {
      return false;
   }

   // again for the next side
   c_Line.Set(m_pc_Point[1]->m_i_X,m_pc_Point[1]->m_i_Y,
              m_pc_Point[2]->m_i_X,m_pc_Point[2]->m_i_Y
              );
   if( ! c_Line.IsOnSameSideOfLine(
                        GeoCoordinate( m_pc_Point[0]->m_i_X,m_pc_Point[0]->m_i_Y),
                        c_Point)
     )
   {
      return false;
   }

   // final for the last side
   c_Line.Set(m_pc_Point[2]->m_i_X,m_pc_Point[2]->m_i_Y,
              m_pc_Point[0]->m_i_X,m_pc_Point[0]->m_i_Y
              );
   if( ! c_Line.IsOnSameSideOfLine(
                        GeoCoordinate( m_pc_Point[1]->m_i_X,m_pc_Point[1]->m_i_Y),
                        c_Point)
     )
   {
      return false;
   }

   
   return true;
} // CoordinateInTriangle()

//===========================================================================
//
//===========================================================================

Blend::Blend() : Tracer("Blend")
{
   // inititalize empty
   m_b_BlendOnlyInSelectionMask = false;
   m_pc_Destination             = NULL;
   m_pc_BlendPointList          = NULL;
   m_pc_InnerMask               = NULL;
   m_i_BlendPointListCount      = 0;
   m_b_TransparencyOn           = false;
   m_i_TransparencyFactor       = 50;
   m_i_SampleDistance           = 18;
   m_b_Smooth                   = true;

   m_c_ChangedRegion.SetEmpty();

} // Blend()

//===========================================================================
//
//===========================================================================

Blend::~Blend()
{
   if(m_pc_InnerMask != NULL)
   {
      delete m_pc_InnerMask;
      m_pc_InnerMask = NULL;
   }
   
} // ~Blend()

//===========================================================================
//
//===========================================================================

void Blend::SetPixelBuffer(PixelBuffer *pc_Buffer)
{
   m_pc_Destination = pc_Buffer;
} // SetPixelBuffer()

//===========================================================================
//
//===========================================================================

void Blend::DoBlend()
{
   D3Matrix *pc_InputMatrix;
   D3Matrix *pc_DestinationMatrix;
   BSpline   c_Spline;
   int       i_X;
   int       i_Y;
   int       i_Width;
   int       i_Height;
   int       i_InputWidth;
   int       i_InputHeight;

   // set changed region initital to empty
   m_c_ChangedRegion.SetEmpty();

      
   if(   m_pc_Destination ==NULL
      || m_i_BlendPointListCount <=1 )
   {
      return;
   }

   if(m_i_BlendPointListCount==2)
   {
      // perform a simple 2 point blend
      DoBlend2Points();
      return;
   }

   i_Width  = m_pc_Destination->GetWidth();
   i_Height = m_pc_Destination->GetHeight();

   i_InputWidth  = (i_Width  / m_i_SampleDistance)+3;
   i_InputHeight = (i_Height / m_i_SampleDistance)+3;

   // calculate a mask with inner and outer points
   CalculateBorderPoints();

   // fprintf(stderr,"CreateLineList\n");
   CreateLineList();

   // fprintf(stderr,"CreateTriangleList\n");
   CreateTriangleList();
   
   // fprintf(stderr,"after CreateTriangleList\n");

   if(m_b_Smooth)
   {
      int i_Error;
      int i_RealWidth;
      int i_RealHeight;

      pc_InputMatrix = new D3Matrix(i_InputWidth,i_InputHeight);

      pc_DestinationMatrix = new D3Matrix(i_Width,i_Height);

      // fill the input matrix
      FillD3Matrix(pc_InputMatrix);

      i_RealWidth = i_Width + m_i_SampleDistance;
      i_RealHeight = i_Height+ m_i_SampleDistance;
            
      // calculate the spline value
      i_Error = c_Spline.CalculateBSpline2Dimensions(
                    pc_InputMatrix,
                    i_InputWidth,
                    i_InputHeight,
                    pc_DestinationMatrix,
                    i_RealWidth,
                    i_RealHeight);
         /*
         fprintf(stderr,"i_Error = %d/, i_Width =%d/%d, i_Height =%d/%d\n",
               i_Error,
               i_Width,i_RealWidth,
               i_Height,i_RealHeight
               );
         */
      ApplySplineResults(pc_DestinationMatrix);      
      

      // delete the arrays
      delete pc_InputMatrix;

      delete pc_DestinationMatrix;

   }
   else // not smooth
   {

      for(i_Y=0;i_Y<i_Height;i_Y++)
      {
         for(i_X=0;i_X<i_Width;i_X++)
         {
            // now calculate the color for every point
            Pixel  p_New;

            p_New = 0;
            if(m_pc_InnerMask != NULL
               && m_pc_InnerMask->Get(i_X,i_Y)!= 0)
            {
               // look for the triangle
               BlendTriangle *pc_Triangle;
               unsigned int ui_TriangleIndex;
               bool b_Found;

               b_Found = false;            
               for(ui_TriangleIndex=0;
                   ui_TriangleIndex < m_TriangleList.count();
                   ui_TriangleIndex++)
               {
                  pc_Triangle = m_TriangleList.at( ui_TriangleIndex);
                  if( pc_Triangle->CoordinateInTriangle((float)i_X,(float)i_Y))
                  {
                     p_New = CalculateTrianglePoint(pc_Triangle,i_X,i_Y);
                     b_Found = true;
                     break;
                  }
               }

               if(b_Found)
               {
                  SetInResult(i_X,i_Y,p_New);
               }
               else
               {
                  p_New = CalculateOuterPoint(i_X,i_Y);
                  SetInResult(i_X,i_Y,p_New);
               }
            } // if in inner mask
            else
            {
               // not inner
               p_New = CalculateOuterPoint(i_X,i_Y);

               SetInResult(i_X,i_Y,p_New);

            }
         } // for i_X
      } // for i_Y
   } // else not smooth

   DeleteTriangleList();
   DeleteLineList();


} // DoBlend()

//===========================================================================
//
//===========================================================================

void Blend::ApplySplineResults(D3Matrix *pc_Result)
{
   D3Coordinate c_Result;
   int          i_X;
   int          i_Y;
   int          i_Width;
   int          i_Height;

   i_Width  = m_pc_Destination->GetWidth();
   i_Height = m_pc_Destination->GetHeight();

   for(i_Y=0; i_Y < i_Height;i_Y++)
   {
      for(i_X=0; i_X < i_Width;i_X++)
      {
         Pixel p_Value;
         int r;
         int g;
         int b;
         int i_Result;

         // get old
         p_Value = m_pc_Destination->Get(i_X,i_Y);


         c_Result = pc_Result->Get(i_X,i_Y);

         // get r
         i_Result = (int)(c_Result.z1*255.0);
         if(i_Result >255)
         {
            i_Result =255;
         }
         else if(i_Result <0)
         {
            i_Result =0;
         }
         r = i_Result;

         i_Result = (int)(c_Result.z2*255.0);
         if(i_Result >255)
         {
            i_Result =255;
         }
         else if(i_Result <0)
         {
            i_Result =0;
         }
         g = i_Result;

         
         i_Result = (int)(c_Result.z3*255.0);
         if(i_Result >255)
         {
            i_Result =255;
         }
         else if(i_Result <0)
         {
            i_Result =0;
         }
         b = i_Result;

         // combine the 3 colors
         p_Value = PIXEL_MAKE(r,g,b);

         SetInResult(i_X,i_Y,p_Value);

      } // for x
      
   }  // for y

} // ApplySplineResults()

//===========================================================================
//
//===========================================================================

void Blend::FillD3Matrix(D3Matrix *pc_Matrix)
{
   D3Coordinate c_Coordinate;
   Pixel        p_Value;
   int          i_ImageX;
   int          i_ImageY;
   int          i_ImageXClip;
   int          i_ImageYClip;
   int          i_InputWidth;
   int          i_InputHeight;
   int          i_X;
   int          i_Y;
   
   i_InputWidth  = pc_Matrix->GetWidth();
   i_InputHeight = pc_Matrix->GetHeight();

   for(i_Y=0;i_Y < i_InputHeight; i_Y++)
   {
      for(i_X=0;i_X < i_InputWidth; i_X++)
      {

         i_ImageX = i_X * m_i_SampleDistance;
         i_ImageY = i_Y * m_i_SampleDistance;
         i_ImageXClip = i_ImageX;
         i_ImageYClip = i_ImageY;

         if(i_ImageXClip >= m_pc_Destination->GetWidth())
         {
            i_ImageXClip =  m_pc_Destination->GetWidth() -1;
         }
         if(i_ImageYClip >= m_pc_Destination->GetHeight())
         {
            i_ImageYClip =  m_pc_Destination->GetHeight() -1;
         }
         p_Value = GetSamplePoint(i_ImageXClip,i_ImageYClip);

            
         c_Coordinate.x = (double) i_ImageX;
         c_Coordinate.y = (double) i_ImageX;
         c_Coordinate.z1 = PIXEL_RED  (p_Value) / 255.0;
         c_Coordinate.z2 = PIXEL_GREEN(p_Value) / 255.0;
         c_Coordinate.z3 = PIXEL_BLUE (p_Value) / 255.0;
                  
         pc_Matrix->Set(i_X,i_Y,c_Coordinate);
      } // for i_Y

   } // for i_Y
   

} // FillD3Matrix()

//===========================================================================
//
//===========================================================================

Pixel Blend::GetSamplePoint(int i_X, int i_Y)
{
   // now calculate the color for every point
   Pixel  p_Result;

   if(   m_pc_InnerMask != NULL
      && m_pc_InnerMask->Get(i_X,i_Y)!= 0)
   {
      // look for the triangle
      BlendTriangle *pc_Triangle;
      unsigned int   ui_TriangleIndex;
      bool           b_Found;

      b_Found = false;
      for(ui_TriangleIndex=0;
          ui_TriangleIndex < m_TriangleList.count();
          ui_TriangleIndex++)
      {
         pc_Triangle = m_TriangleList.at( ui_TriangleIndex);
         if( pc_Triangle->CoordinateInTriangle((float)i_X,(float)i_Y))
         {
            p_Result = CalculateTrianglePoint(pc_Triangle,i_X,i_Y);
            return p_Result;
         }
      }

      p_Result = CalculateOuterPoint(i_X,i_Y);
   } // if in inner mask
   else
   {
      // not inner
      p_Result = CalculateOuterPoint(i_X,i_Y);
   }

   return p_Result;
} // GetSamplePoint()

//===========================================================================
//
//===========================================================================

void Blend::SetBlendPointList(   BlendPoint *pc_BlendPointList, int i_Count)
{
   m_pc_BlendPointList       = pc_BlendPointList;
   m_i_BlendPointListCount = i_Count;
   
}  // SetBlendPointList()

//===========================================================================
//
//===========================================================================

void Blend::CalculateBorderPoints()
{
   PMemVector<PixelCoordinate>   c_List;
   PixelCoordinate               c_Coordinate;
   BufferPainter                 c_BufferPainter;
   double                        d_MinimumAngle;
   double                        d_MinimumDifferenceAngle;
   double                        d_LastAngle;
   bool                          b_FirstPoint;

   int                           i_CurrentMinX;
   int                           i_CurrentMinY;
   int                           i_IndexStartpoint;
   int                           i;
   int                           i_LastIndex;
   int                           i_CurrentIndex;

   m_BorderList.clear();
   d_MinimumAngle           =0.0;
   d_MinimumDifferenceAngle =0.0;
   i_CurrentIndex           =0;

   if(m_pc_InnerMask != NULL)
   {
      delete m_pc_InnerMask;
      m_pc_InnerMask = NULL;
   }

   // no border with only 2 points
   if(m_i_BlendPointListCount<2)
   {
      return;
   }

   
   // first look for the  upper leftstartpoint
   i_CurrentMinX = m_pc_BlendPointList[0].m_i_X;
   i_CurrentMinY = m_pc_BlendPointList[0].m_i_Y;
   i_IndexStartpoint =0;

   for(i=1;i<m_i_BlendPointListCount;i++)
   {
      if(    (m_pc_BlendPointList[i].m_i_Y< i_CurrentMinY)
          || (    m_pc_BlendPointList[i].m_i_Y== i_CurrentMinY
               && m_pc_BlendPointList[i].m_i_X< i_CurrentMinX
             ) )
      {
         // found point above, or same height, but left
         i_CurrentMinX = m_pc_BlendPointList[i].m_i_X;
         i_CurrentMinY = m_pc_BlendPointList[i].m_i_Y;
         i_IndexStartpoint =i;
         
      } 

   } // for i

   // put startpoint into the borderlist
   m_BorderList.append (& m_pc_BlendPointList[i_IndexStartpoint]);
   m_pc_BlendPointList[i_IndexStartpoint].m_b_IsBorderPoint=true;

   TRACE << "Start Punkt "
         << m_pc_BlendPointList[i_IndexStartpoint].m_i_X
         << " , "
         << m_pc_BlendPointList[i_IndexStartpoint].m_i_Y
         << "\n";
   

   // now we have the startpoint
   d_LastAngle = -M_PI;
   i_LastIndex = i_IndexStartpoint;


   d_LastAngle = Geometry::AngleNormalize(d_LastAngle);
   // loop until all borderpoints are fount
   do 
   {
      double d_DifferenceAngle;
      b_FirstPoint = true;
      
      // make lines from current to all other available points
      for(i=0;i < m_i_BlendPointListCount;i++)
      {
         double d_DiffX;
         double d_DiffY;
         double d_CurrentAngle;
       
         // the point itself ??
         if(i == i_LastIndex)
         {
            continue;
         }

         // a already found borderpoint, but not the startpoint
         if(    ( i != i_IndexStartpoint)
            &&  ( m_pc_BlendPointList[i].m_b_IsBorderPoint) )
         {
            // point already on the list
            continue;
         }

         // now make a line to the point
         d_DiffX =    (double)m_pc_BlendPointList[i_LastIndex].m_i_X
                   -  m_pc_BlendPointList[i].m_i_X;
         d_DiffY =    (double)m_pc_BlendPointList[i_LastIndex].m_i_Y
                   -  m_pc_BlendPointList[i].m_i_Y;

         // calculate angle
         d_CurrentAngle = atan2(d_DiffY,d_DiffX);
         d_CurrentAngle = Geometry::AngleNormalize(d_CurrentAngle);
         d_DifferenceAngle = d_CurrentAngle - d_LastAngle;
         d_DifferenceAngle = Geometry::AngleNormalize(d_DifferenceAngle);


         TRACE << "Angle to ("
                <<  m_pc_BlendPointList[i].m_i_X << ","
                <<  m_pc_BlendPointList[i].m_i_Y << ") ("
                <<  d_DiffX << "/"
                <<  d_DiffY << ") ="
                <<  d_CurrentAngle
                <<  " Last = "
                <<  d_LastAngle
                <<  " Difference = "
                <<  d_DifferenceAngle
                <<  "\n";
         
         if(b_FirstPoint)
         {
            d_MinimumAngle           = d_CurrentAngle;
            d_MinimumDifferenceAngle = d_DifferenceAngle;
            i_CurrentIndex           = i;
            b_FirstPoint             = false;
         }
         else if(d_DifferenceAngle < d_MinimumDifferenceAngle)
         {
            d_MinimumAngle           = d_CurrentAngle;
            d_MinimumDifferenceAngle = d_DifferenceAngle;
            i_CurrentIndex           = i;
         
         }

      } // for all points

      if(i_CurrentIndex == i_IndexStartpoint)
      {
         // reached the end
         break;
      }

      // remember point as new lastpoint
      d_LastAngle = d_MinimumAngle;
      i_LastIndex = i_CurrentIndex;

      // remeber that it is borderpoint
      m_pc_BlendPointList[i_LastIndex].m_b_IsBorderPoint=true;

      // add to the list
      m_BorderList.append (& m_pc_BlendPointList[i_LastIndex]);
      TRACE << "Borderpoint "
            << m_pc_BlendPointList[i_LastIndex].m_i_X
            << ","
            << m_pc_BlendPointList[i_LastIndex].m_i_Y
            << "taken \n";
      
   } while(i_CurrentIndex != i_IndexStartpoint) ;
      
   // create a mask and fill in the polygon defined by the
   // borderpoint list
   
   m_pc_InnerMask = new PixelBuffer(m_pc_Destination,1,false,false,false);
   if(m_pc_InnerMask == NULL)
   {
      return;
   }

   // create a vector list of pixel coordinates
   for(i=0;i< (int) m_BorderList.count();i++)
   {
      BlendPoint *pc_Point;
      pc_Point =  (BlendPoint *) m_BorderList.at(i);

      c_Coordinate.m_i_X = pc_Point->m_i_X;
      c_Coordinate.m_i_Y = pc_Point->m_i_Y;
      

      c_List.AddTail(c_Coordinate);
      
   } // for i
   
   c_BufferPainter.SetBorderColor(1);
   c_BufferPainter.SetInnerColor(1);
   c_BufferPainter.SetDrawInnerEnable(true);

   // draw in the mask
   c_BufferPainter.DrawFreeFigure(m_pc_InnerMask,
                       &c_List);
   
   

} // CalculateBorderPoints()

//===========================================================================
//
//===========================================================================

void Blend::DoBlend2Points()
{
   GeoCoordinate c_CutPoint;
   MathHermite   c_HermiteSpline;
   GeoLine       c_Line;
   int           i_X;
   int           i_Y;
   int           i_Width;
   int           i_Height;
   
   // make a line between the points
   c_Line.Set(m_pc_BlendPointList[0].m_i_X,
              m_pc_BlendPointList[0].m_i_Y,
              m_pc_BlendPointList[1].m_i_X,
              m_pc_BlendPointList[1].m_i_Y
              );

   i_Width  = m_pc_Destination->GetWidth();
   i_Height = m_pc_Destination->GetHeight();

   c_HermiteSpline.Set(0,0,1,0);
   
   // loop over all pixel
   for(i_Y=0;i_Y<i_Height;i_Y++)
   {
      for(i_X=0;i_X<i_Width;i_X++)
      {
         GeoCoordinate c_Actual;
         double        d_TFactor;
         Pixel         p_NewColor;

         if(m_b_BlendOnlyInSelectionMask)
         {
            if( ! m_pc_Destination->IsRawSelected(i_X,i_Y) )
            {
               continue;
            }
         }

         // make a coordinate
         c_Actual.SetXY(i_X,i_Y);

         d_TFactor = c_Line.PerpendicularOfPointCutsAt(c_Actual,c_CutPoint);

         if(d_TFactor <=0.0)
         {
            // left of the first point
            p_NewColor = m_pc_BlendPointList[0].m_p_Color;
         }
         else if(d_TFactor>=1.0)
         {
            // right of the second point
            p_NewColor = m_pc_BlendPointList[1].m_p_Color;
         }
         else
         {
            // in between, calculate a color
            int   i_BlendFactor;
            if(m_b_Smooth)
            {
               d_TFactor = c_HermiteSpline.Calc( d_TFactor);
            }
            i_BlendFactor = (int)(d_TFactor*100.0);

            p_NewColor = ColorCalculation::TransparentColor(
                              m_pc_BlendPointList[1].m_p_Color,
                              m_pc_BlendPointList[0].m_p_Color,
                              i_BlendFactor);
        
         } // else between the two pixels

         SetInResult(i_X,i_Y,p_NewColor);

      }  // for i_X

   } // for i_Y
   m_c_ChangedRegion.Set(m_pc_Destination);   

} //  DoBlend2Points()

//===========================================================================
//
// special blend for 3 point triangle
//===========================================================================

void Blend::DoBlend3Points()
{
   GeoCoordinate c_CutPoint;
   GeoLine       c_Line;
   int           i_X;
   int           i_Y;
   int           i_Width;
   int           i_Height;

   CalculateBorderPoints();

   // make a line between the points
   c_Line.Set(m_pc_BlendPointList[0].m_i_X,
              m_pc_BlendPointList[0].m_i_Y,
              m_pc_BlendPointList[1].m_i_X,
              m_pc_BlendPointList[1].m_i_Y
              );

   i_Width  = m_pc_Destination->GetWidth();
   i_Height = m_pc_Destination->GetHeight();

   // loop over all pixel
   for(i_Y=0;i_Y<i_Height;i_Y++)
   {
      for(i_X=0;i_X<i_Width;i_X++)
      {
         GeoCoordinate c_Actual;
         Pixel         p_NewColor;

         if(m_b_BlendOnlyInSelectionMask)
         {
            if( ! m_pc_Destination->IsRawSelected(i_X,i_Y) )
            {
               continue;
            }
         }

         if(m_pc_InnerMask != NULL
            && m_pc_InnerMask->Get(i_X,i_Y)!= 0)
         {
            // within the triangle
            p_NewColor = CalculateTrianglePoint(
                     m_pc_BlendPointList[0],
                     m_pc_BlendPointList[1],
                     m_pc_BlendPointList[2],
                       i_X,i_Y);

            if(m_b_TransparencyOn)
            {
               Pixel p_Old;
               p_Old =  m_pc_Destination->Get(i_X,i_Y);

               p_NewColor = ColorCalculation::TransparentColor(p_Old,
                                                       p_NewColor,
                                                       m_i_TransparencyFactor
                                                       );
             }

            m_pc_Destination->Set(i_X,i_Y,p_NewColor);

            m_c_ChangedRegion.AddPoint(i_X,i_Y);
         } // if in inner mask
         else
         {
            // not inner
            p_NewColor = CalculateOuterPoint(i_X,i_Y);

            if(m_b_TransparencyOn)
            {
               Pixel p_Old;
               p_Old =  m_pc_Destination->Get(i_X,i_Y);

               p_NewColor = ColorCalculation::TransparentColor(p_Old,
                                                       p_NewColor,
                                                       m_i_TransparencyFactor
                                                       );
             }

             m_pc_Destination->Set(i_X,i_Y,p_NewColor);

             m_c_ChangedRegion.AddPoint(i_X,i_Y);
         }


      }  // for i_X

   } // for i_Y
   m_c_ChangedRegion.Set(m_pc_Destination);

} //  DoBlend3Points()

//===========================================================================
//
//===========================================================================

RectRegion Blend::GetChangedRegion()
{
   return m_c_ChangedRegion;
} // GetChangedRegion()

//===========================================================================
//
//===========================================================================

void Blend::SetOnlyInSelectionMask(bool b_OnlyInMask)
{
   m_b_BlendOnlyInSelectionMask = b_OnlyInMask;
} // SetOnlyInSelectionMask

//===========================================================================
//
//===========================================================================

void Blend::SetTransparency( bool b_On, int i_Factor)
{

   m_b_TransparencyOn    = b_On;
   m_i_TransparencyFactor= i_Factor;

} // SetTransparency()

//===========================================================================
//
//===========================================================================

Pixel Blend::CalculateAveragePoint(int i_X, int i_Y)
{
   double d_Sum;
   double d_Red;
   double d_Green;
   double d_Blue;
   Pixel  p_Result;
   int    i_CurrentPoint;


   d_Sum =0;
   for(i_CurrentPoint =0;
       i_CurrentPoint <m_i_BlendPointListCount;
       i_CurrentPoint ++)
   {
      float f_Distance;

      // calculate the distance between current point and
      // all blendpoints
      f_Distance = Utilities::Hypothenusis(
                          (i_X - m_pc_BlendPointList[i_CurrentPoint].m_i_X),
                          (i_Y - m_pc_BlendPointList[i_CurrentPoint].m_i_Y));
      f_Distance *= f_Distance;
      f_Distance *= f_Distance;
      if(f_Distance<1.0)
      {
         f_Distance = 1.0;
      }
      
      m_d_FaktorList[i_CurrentPoint] = 1.0 / (double) f_Distance;
      d_Sum += m_d_FaktorList[i_CurrentPoint];

   } // for i_CurrentPoint

   // calculate inverse sum
   d_Sum = 1 / d_Sum;

   // normate the factors, that the sum of them will give 1
   for(i_CurrentPoint =0;
       i_CurrentPoint <m_i_BlendPointListCount;
       i_CurrentPoint ++)
   {
      m_d_FaktorList[i_CurrentPoint] *= d_Sum;
   }

   // now calculate the new color

   d_Red   =0;
   d_Green =0;
   d_Blue  =0;
   for(i_CurrentPoint =0;
       i_CurrentPoint <m_i_BlendPointListCount;
       i_CurrentPoint ++)
   {
      d_Red   += m_d_FaktorList[i_CurrentPoint] * PIXEL_RED  (m_pc_BlendPointList[i_CurrentPoint].m_p_Color);
      d_Green += m_d_FaktorList[i_CurrentPoint] * PIXEL_GREEN(m_pc_BlendPointList[i_CurrentPoint].m_p_Color);
      d_Blue  += m_d_FaktorList[i_CurrentPoint] * PIXEL_BLUE (m_pc_BlendPointList[i_CurrentPoint].m_p_Color);

   }
   p_Result = PIXEL_MAKE((int)d_Red,(int) d_Green,(int)d_Blue);

   return p_Result;
   
} // CalculateAveragePoint()

//===========================================================================
//
//===========================================================================

Pixel Blend::CalculateOuterPoint(int i_X, int i_Y)
{
   GeoCoordinate  c_CutPoint;
   BlendPoint    *pc_CurrentPoint;
   GeoLine        c_Line;
   
   Pixel   p_Result;
   int     i_Line;
   int     i_NumberOfPoints;

   double  d_CutValue;
   double  d_NearestDistance;
   int     i_NearestPointIndex;
   
   // search the nearest outer line

   d_NearestDistance =0.0;
   i_NearestPointIndex = -1;
   i_NumberOfPoints = m_BorderList.count();

   for(i_Line=0; i_Line < i_NumberOfPoints ;i_Line++)
   {
      BlendPoint *pc_NextPoint;
      double     d_CurrentDistance;
      int        i_NextPoint;

      i_NextPoint = i_Line+1;
      if(i_NextPoint >= i_NumberOfPoints)
      {
         i_NextPoint =0;
      }

      pc_CurrentPoint = m_BorderList.at(i_Line);
      pc_NextPoint = m_BorderList.at(i_NextPoint);
      // make a line from current to next point
      c_Line.Set(pc_CurrentPoint->m_i_X,
                 pc_CurrentPoint->m_i_Y,
                 pc_NextPoint->m_i_X,
                 pc_NextPoint->m_i_Y
                 );

      d_CutValue = c_Line.PerpendicularOfPointCutsAt(
                                 GeoCoordinate(i_X,i_Y),
                                 c_CutPoint);
      if(   d_CutValue > 0.0
         && d_CutValue < 1.0)
      {
         //found a line, now look for the correct side

         double d_AngleLine;
         double d_AnglePoint;

         d_AngleLine = atan2(pc_NextPoint->m_i_Y -  pc_CurrentPoint->m_i_Y,
                             pc_NextPoint->m_i_X -  pc_CurrentPoint->m_i_X);
         d_AnglePoint = atan2(i_Y -  pc_CurrentPoint->m_i_Y,
                              i_X -  pc_CurrentPoint->m_i_X);

         if(d_AngleLine< d_AnglePoint)
         {
            d_AngleLine += 2*M_PI;
         }
         if( (d_AngleLine-d_AnglePoint) < (M_PI_2))
         {
            // point is on the correct side
            // now calculate the point at the cut-value

             p_Result =  ColorCalculation::TransparentColor(
                                 pc_NextPoint->m_p_Color,
                                 pc_CurrentPoint->m_p_Color,
                                 (int)(d_CutValue*100)
                                 );
            return p_Result;
         }
      }  // if cut-value between 0..1

      d_CurrentDistance = Utilities::Hypothenusis(
                     pc_CurrentPoint->m_i_X - i_X,
                     pc_CurrentPoint->m_i_Y - i_Y);
      if(i_NearestPointIndex <0)
      {
         i_NearestPointIndex = i_Line;
         d_NearestDistance = d_CurrentDistance;
      }
      else if( d_CurrentDistance < d_NearestDistance)
      {
         i_NearestPointIndex = i_Line;
         d_NearestDistance = d_CurrentDistance;
      }
                     
      
      
   } // for all lines

   // not a perpendicular found, take color of nearst
   // border point;
   pc_CurrentPoint = m_BorderList.at(i_NearestPointIndex);
   p_Result = pc_CurrentPoint->m_p_Color;

   /*   
   p_Result = CalculateAveragePoint(pc_CurrentPoint->m_i_X,
                                    pc_CurrentPoint->m_i_Y);
   
   */
   
   return p_Result;
} // CalculateOuterPoint()

//===========================================================================
//
//===========================================================================

Pixel Blend::CalculateTrianglePoint(BlendPoint c_PointA,
            BlendPoint c_PointB,
            BlendPoint c_PointC,
            int        i_X,
            int        i_Y)
{
   GeoCoordinate c_CoordinateDummy;
   GeoLine c_Line0;
   GeoLine c_Line1;
   Pixel   p_Return;
   float   f_T;
   float   f_S;
   float   f_PercentRed;
   float   f_PercentGreen;
   float   f_PercentBlue;
   float   f_Red;
   float   f_Green;
   float   f_Blue;

   if(   i_X == c_PointA.m_i_X
      && i_Y == c_PointA.m_i_Y )
   {
      // point is on the a-point
      // just return its color
      return c_PointA.m_p_Color;
   }

   c_Line0.Set(GeoCoordinate(c_PointA.m_i_X,c_PointA.m_i_Y),
               GeoCoordinate(i_X,i_Y));
   c_Line1.Set(GeoCoordinate(c_PointB.m_i_X,c_PointB.m_i_Y),
               GeoCoordinate(c_PointC.m_i_X,c_PointC.m_i_Y));
   c_Line0.CutsLineExtended(c_Line1,c_CoordinateDummy,&f_S,&f_T);

   
   f_PercentRed   =   (1.0-f_T) * (float) PIXEL_RED(c_PointB.m_p_Color)
                    +   f_T     * (float) PIXEL_RED(c_PointC.m_p_Color);

   f_PercentGreen =   (1.0-f_T) * (float) PIXEL_GREEN(c_PointB.m_p_Color)
                    +  f_T      * (float) PIXEL_GREEN(c_PointC.m_p_Color);

   f_PercentBlue  =   (1.0-f_T) * (float) PIXEL_BLUE(c_PointB.m_p_Color)
                    +  f_T      * (float) PIXEL_BLUE(c_PointC.m_p_Color);

   if(f_S > 0.0)
   {
      f_Red   = PIXEL_RED(c_PointA.m_p_Color) * ((f_S-1.0)/f_S)
                + f_PercentRed * (1/f_S);
      f_Green = PIXEL_GREEN(c_PointA.m_p_Color) * ((f_S-1.0)/f_S)
                + f_PercentGreen * (1/f_S);
      f_Blue  = PIXEL_BLUE(c_PointA.m_p_Color) * ((f_S-1.0)/f_S)
                + f_PercentBlue * (1/f_S);
   }
   else
   {
      f_Red   = PIXEL_RED  (c_PointA.m_p_Color);
      f_Green = PIXEL_GREEN(c_PointA.m_p_Color);
      f_Blue  = PIXEL_BLUE (c_PointA.m_p_Color);
   }

   if(f_Red < 0)
   {
      f_Red =0;
   }
   else if (f_Red>255.0)
   {
      f_Red = 255.0;
   }

   if(f_Green < 0)
   {
      f_Green =0;
   }
   else if (f_Green>255.0)
   {
      f_Green = 255.0;
   }
   
   if(f_Blue < 0)
   {
      f_Blue =0;
   }
   else if (f_Blue>255.0)
   {
      f_Blue = 255.0;
   }

   p_Return = PIXEL_MAKE(f_Red,f_Green,f_Blue);

   return p_Return;
   
} // CalculateTrianglePoint()

//===========================================================================
//
//===========================================================================

Pixel Blend::CalculateTrianglePoint(BlendTriangle *c_Triangle,
            int        i_X,
            int        i_Y)
{
  return  CalculateTrianglePoint(
            *(c_Triangle->m_pc_Point[0]),
            *(c_Triangle->m_pc_Point[1]),
            *(c_Triangle->m_pc_Point[2]),
            i_X,
            i_Y);
            
} // CalculateTrianglePoint()

//===========================================================================
//
//===========================================================================

static int CompareLineLength(const void *pc_1,
                             const void *pc_2
                            )
{
   BlendLine *pc_Line1;
   BlendLine *pc_Line2;

   pc_Line1 = * (BlendLine **) pc_1;
   pc_Line2 = * (BlendLine **) pc_2;
   
   if(pc_Line1->d_Length < pc_Line2->d_Length)
   {
      return -1;
   }
   else if(pc_Line1->d_Length > pc_Line2->d_Length)
   {
      return 1;
   }

   // same size
   return 0;
} // CompareLineLength()

//===========================================================================
//
//===========================================================================

void Blend::CreateLineList()
{
   BlendLine ** pc_BlendLineList;
   BlendLine *  pc_NewLine;
   BlendLine *  pc_CurrentLine;
   BlendLine *  pc_CompareLine;
   int          i_MaximumLines;
   int          i_LineAddIndex;
   int          i_ActualPoint;
   int          i_ConnectPoint;
   int          i_CurrentLine;
   int          i_CompareLine;

   i_MaximumLines = (m_i_BlendPointListCount*m_i_BlendPointListCount + m_i_BlendPointListCount)
                     / 2;

   pc_BlendLineList = new BlendLine *[i_MaximumLines];
   i_LineAddIndex =0;

   // ---- first create all possible lines
   // for all points
   for(i_ActualPoint =0;
       i_ActualPoint < m_i_BlendPointListCount-1;
       i_ActualPoint ++)
   {
      // connect to the remaining points
      for(i_ConnectPoint = i_ActualPoint+1;
          i_ConnectPoint < m_i_BlendPointListCount;
          i_ConnectPoint ++)
      {
         // make a new line
         pc_NewLine = new BlendLine(
                  &m_pc_BlendPointList[i_ActualPoint],
                  &m_pc_BlendPointList[i_ConnectPoint]);

         pc_BlendLineList[i_LineAddIndex] = pc_NewLine;
         i_LineAddIndex ++;
      }
   }

   //--- sort all lines depending on the size
   qsort(pc_BlendLineList,
         i_LineAddIndex,
         sizeof(BlendLine *),
         CompareLineLength);

   if(TraceActive())
   {
      int i;
      TRACE <<  (int) i_LineAddIndex
            <<  " raw lines created \n";
      for(i=0;i<(int) i_LineAddIndex;i++)
      {
         BlendLine *pc_Line;
         pc_Line =  pc_BlendLineList[i];
         TRACE << "Line "
               << i
               << " is from "
               << pc_Line->m_pc_Point1->m_i_X << " "
               << pc_Line->m_pc_Point1->m_i_Y << " to "
               << pc_Line->m_pc_Point2->m_i_X << " "
               << pc_Line->m_pc_Point2->m_i_Y << "\n";

      }
   }

                  

   // -----    now delete cutting lines
   for(i_CurrentLine=0;
       i_CurrentLine< i_LineAddIndex -1;
       i_CurrentLine ++)
   {
      pc_CurrentLine = pc_BlendLineList[i_CurrentLine];
      if(pc_CurrentLine == NULL)
      {
         continue;
      }
      for(i_CompareLine = i_CurrentLine +1;
          i_CompareLine < i_LineAddIndex;
          i_CompareLine ++)
      {
         pc_CompareLine = pc_BlendLineList[i_CompareLine];
         if(pc_CompareLine == NULL)
         {
            continue;
         }
         if(pc_CurrentLine->CutsLine(pc_CompareLine))
         {
            // line cuts. delete the compare line
            delete pc_CompareLine;
            pc_BlendLineList[i_CompareLine] = NULL;
            TRACE << "Line Index "
                  << i_CompareLine
                  << " deleted by Line "
                  << i_CurrentLine
                  << "\n";
         }
      } // for compareline
   } // for currentline
       
   // ---- add all remaining lines to the list

   // init the linelist
   DeleteLineList();

   for(i_CurrentLine=0;
       i_CurrentLine< i_LineAddIndex ;
       i_CurrentLine ++)
   {
      pc_CurrentLine = pc_BlendLineList[i_CurrentLine];
      if(pc_CurrentLine == NULL)
      {
         continue;
      }
      // append the new line
      m_LineList.append(pc_CurrentLine);
   }

   // delete the helping array
   delete pc_BlendLineList;

   if(TraceActive())   
   {
      unsigned int i;
      TRACE <<  (int) m_LineList.count()
            <<  "lines created \n";
      for(i=0;i<m_LineList.count();i++)
      {
         BlendLine *pc_Line;
         pc_Line = m_LineList.at(i);
         TRACE << "Line "
               << i
               << " is from "
               << pc_Line->m_pc_Point1->m_i_X << " "
               << pc_Line->m_pc_Point1->m_i_Y << " to "
               << pc_Line->m_pc_Point2->m_i_X << " "
               << pc_Line->m_pc_Point2->m_i_Y << "\n";
      
      }
   }
} // CreateLineList()

//===========================================================================
//
//===========================================================================

void Blend::DeleteLineList()
{
   if(!m_LineList.isEmpty())
   {
      // remove older stored points
      m_LineList.setAutoDelete(true);
      m_LineList.clear();
      m_LineList.setAutoDelete(false);
   }

} // DeleteLineList()

//===========================================================================
//
//===========================================================================

void Blend::CreateTriangleList()
{
   BlendPoint * pc_Point1;
   BlendPoint * pc_Point2;
   BlendPoint * pc_Point3;
   BlendLine  * pc_Line1;
   BlendLine  * pc_Line2;
   BlendLine  * pc_Line3;
   bool         b_TriangleFound ;
   int          i_FirstLineIndex;
   int          i_SecondLineIndex;
   int          i_ThirdLineIndex;
   int          i_NumberOfLines;

   i_NumberOfLines = m_LineList.count();

   pc_Point1=NULL;
   pc_Point2=NULL;
   pc_Point3=NULL;

         
   // init the Trianglelist
   DeleteTriangleList();

   for (i_FirstLineIndex = 0;
        i_FirstLineIndex < i_NumberOfLines-2;
        i_FirstLineIndex++)
   {
      pc_Line1 = m_LineList.at(i_FirstLineIndex);

      b_TriangleFound = false;

      for (i_SecondLineIndex = i_FirstLineIndex+1;
           i_SecondLineIndex < i_NumberOfLines-1 /* && !b_TriangleFound */;
           i_SecondLineIndex++)
      {
         bool b_PointsTogether;
         
         pc_Line2 = m_LineList.at(i_SecondLineIndex);

         // check whether the second line has one of the points together with the first line
         b_PointsTogether = false;
         if     (pc_Line1->m_pc_Point1 == pc_Line2->m_pc_Point1)
         {
            b_PointsTogether = true;
            pc_Point1 =pc_Line1->m_pc_Point1;
            pc_Point2 =pc_Line1->m_pc_Point2;
            pc_Point3 =pc_Line2->m_pc_Point2;
            
         }
         else if(pc_Line1->m_pc_Point1 == pc_Line2->m_pc_Point2)
         {
            b_PointsTogether = true;
            pc_Point1 =pc_Line1->m_pc_Point1;
            pc_Point2 =pc_Line1->m_pc_Point2;
            pc_Point3 =pc_Line2->m_pc_Point1;
         }
         else if(pc_Line1->m_pc_Point2 == pc_Line2->m_pc_Point1)
         {
            b_PointsTogether = true;
            pc_Point1 =pc_Line1->m_pc_Point2;
            pc_Point2 =pc_Line1->m_pc_Point1;
            pc_Point3 =pc_Line2->m_pc_Point2;
         }
         else if(pc_Line1->m_pc_Point2 == pc_Line2->m_pc_Point2)
         {
            b_PointsTogether = true;
            pc_Point1 =pc_Line1->m_pc_Point2;
            pc_Point2 =pc_Line1->m_pc_Point1;
            pc_Point3 =pc_Line2->m_pc_Point1;
         }

         // no common point, then go directly to the next line
         if( ! b_PointsTogether)
         {
            continue;
         }
         
         for (i_ThirdLineIndex = i_SecondLineIndex+1;
              i_ThirdLineIndex < i_NumberOfLines /*&&  !b_TriangleFound */;
              i_ThirdLineIndex++)
         {
            pc_Line3 = m_LineList.at(i_ThirdLineIndex);
            if(   (   pc_Line3->m_pc_Point1 == pc_Point2 
                   && pc_Line3->m_pc_Point2 == pc_Point3)             
               || (   pc_Line3->m_pc_Point1 == pc_Point3
                   && pc_Line3->m_pc_Point2 == pc_Point2)
               )
            {
               // found also the third line
               // now create a triangle
               BlendTriangle *pc_NewTriangle;

               pc_NewTriangle = new BlendTriangle(pc_Line1,pc_Line2,pc_Line3,
                                                  pc_Point1,pc_Point2,pc_Point3);

               // add into the list
               m_TriangleList.append(pc_NewTriangle);

               // stop loop for line 2 and 3, back with next line 1
               b_TriangleFound = true;
               
            }  // if found


         } // for Third line

      } // for second line


      
   } // for first line

   /*
   fprintf(stderr,"%d triangle created \n",m_TriangleList.count());
   */
   
   // after all trinagles are created, check whether there is a surrounding
   // triangle, which includes others
   RemoveSuperTriangles();

   
} // CreateTriangleList

//===========================================================================
//
//===========================================================================

void Blend::DeleteTriangleList()
{
   if(!m_TriangleList.isEmpty())
   {
      // remove older stored points
      m_TriangleList.setAutoDelete(true);
      m_TriangleList.clear();
      m_TriangleList.setAutoDelete(false);
   }

} // DeleteTriangleList()

//===========================================================================
//
//===========================================================================

void Blend::RemoveSuperTriangles()
{
   BlendTriangle *pc_ActualTriangle;
   BlendPoint    *pc_ActualPoint;
   int            i_Triangle;
   int            i_Point;
   

   i_Triangle = m_TriangleList.count();

   while(i_Triangle >0)
   {
      // reduce Number first, then I can use it as index
      i_Triangle --;

      pc_ActualTriangle = m_TriangleList.at(i_Triangle);

      TRACE << "Traingle "
            << i_Triangle
            << " is:\n" ;
      if(TraceActive())
      {
         pc_ActualTriangle->printf();
      }

      // is the point one of the border points ??
      for(i_Point =0; i_Point < m_i_BlendPointListCount;i_Point++)
      {
         pc_ActualPoint = &m_pc_BlendPointList[i_Point];

         if( pc_ActualPoint ==  pc_ActualTriangle->m_pc_Point[0])
         {
            continue;
         }
         if( pc_ActualPoint ==  pc_ActualTriangle->m_pc_Point[1])
         {
            continue;
         }
         if( pc_ActualPoint ==  pc_ActualTriangle->m_pc_Point[2])
         {
            continue;
         }

         // point in triangle?
         if(pc_ActualTriangle->CoordinateInTriangle(
                                pc_ActualPoint->m_i_X,
                                pc_ActualPoint->m_i_Y)
           )
         {
            
            TRACE << "SuperTriangle found, actual = ("
                  << pc_ActualPoint->m_i_X
                  << " , "
                  << pc_ActualPoint->m_i_Y
                  << ")\n";

            if(TraceActive())
            {      
               pc_ActualTriangle->printf();
            }
            
            TRACE << "Actual ="               
                 << pc_ActualPoint
                 << " tr1="
                 << pc_ActualTriangle->m_pc_Point[0]
                 << " tr2="
                 << pc_ActualTriangle->m_pc_Point[1]
                 << " tr2="
                 << pc_ActualTriangle->m_pc_Point[2]
                 << "\n";
           
                 
         
            // delete this triangle
            m_TriangleList.remove(i_Triangle);
            delete pc_ActualTriangle;
            pc_ActualTriangle = NULL;

            // break the loop over the points
            break;
            
         } // if found
      }  // for i
      
   }  // while Number >0

} // RemoveSuperTriangles()

//===========================================================================
//
//===========================================================================

void Blend::SetSmooth(bool b_On)
{
  m_b_Smooth = b_On;
} // SetSmooth()

//===========================================================================
//
//===========================================================================

void Blend::SetInResult(int i_X, int i_Y, Pixel p_NewValue)
{
   if(m_b_BlendOnlyInSelectionMask)
   {
      if( ! m_pc_Destination->IsRawSelected(i_X,i_Y) )
      {
         return;
      }
   }
   if(m_b_TransparencyOn)
   {
      Pixel p_Old;
      p_Old =  m_pc_Destination->Get(i_X,i_Y);

      p_NewValue = ColorCalculation::TransparentColor(
                                            p_Old,
                                            p_NewValue,
                                            m_i_TransparencyFactor
                                            );
   }
   m_pc_Destination->Set(i_X,i_Y,p_NewValue);
   m_c_ChangedRegion.AddPoint(i_X,i_Y);

} // SetInResult()

//===========================================================================
//
//===========================================================================





