//---------------------------------------------------------------------------
// This sourcefile is created by Peter Jodda / peter@jodda.de
// It is under the GPL license.
// If you expand this file, please contact me, that I can integrate the
// enhancements in the original file.
//---------------------------------------------------------------------------
// Module: BlendBox
//---------------------------------------------------------------------------
// Purpose: dialog box for the color blending
//---------------------------------------------------------------------------

#include <stdio.h>
#include <stdlib.h>
#include <math.h>

#include <qlayout.h>
#include <qgrid.h>
#include <qlabel.h>
#include <qgroupbox.h>
#include <qvgroupbox.h>
#include <qhgroupbox.h>
#include <qfont.h>
#include <qfontmetrics.h>
#include <qcombobox.h>

#include "PixelBuffer.h"
#include "QtUtilities.h"
#include "ImageFrame.h"
#include "MessageServer.h"
#include "Utilities.h"

#include "QULabelSlider.h"
#include "Selection.h"
#include "Callback.h"

#include "Blend.h"
#include "BlendBox.h"
#include "MainFrame.h"

#define BLENDBOXLAYER "BlendLayer"

//===========================================================================
//
//===========================================================================

BlendBox::BlendBox(QWidget *parent,
                        GlobalStorage *pc_Global,
                        MainFrame *pc_MainFrame) :
                StandardDialog(parent,pc_Global),
                Tracer("BlendBox")
{
   QBoxLayout    *l_Layout;

   LoadPosition("BlendBox");

   setCaption(tr("Color Blend"));
   TRACE << "Blend Box Started\n" ;

   l_Layout = new  QVBoxLayout( m_w_TopWidget,1);


   m_b_ObjectModeActive    = false;
   m_b_InRedraw            = false;
   m_b_TransparencyEnabled = false;
   m_i_TransparencyFactor  = 50;
   m_b_DoSmoothing         = true;
   
   // create a new layer
   m_pc_ObjectLayer= m_pc_GlobalStorage->m_pc_ObjectMain->CreateLayer(BLENDBOXLAYER);
   m_pc_ObjectLayer->SetVisible(false);


   // create tha active checkbox
   m_ActivateCheck = new QCheckBox(tr("Active"),m_w_TopWidget);
   l_Layout->addWidget(m_ActivateCheck);


   // create the region box with 3 buttons
   m_RegionBox = new QHGroupBox(tr("Point "),m_w_TopWidget);

   l_Layout->addWidget(m_RegionBox);

   m_AddButton       = new QPushButton(tr("Add"),m_RegionBox);
   m_DeleteButton    = new QPushButton(tr("Delete"),m_RegionBox);
   m_DeleteAllButton = new QPushButton(tr("Delete All"),m_RegionBox);

   // create the color selection box
   if(pc_MainFrame == NULL)
   {
      m_w_ColorSelector = new QUColorDragLabeledWidget(m_w_TopWidget,
                                  tr("Color"),
                                  QUColorDragWidget::ModePipette,
                                   m_pc_GlobalStorage,
                                   NULL
                                  );
   }
   else
   {
      m_w_ColorSelector = new QUColorDragLabeledWidget(m_w_TopWidget,
                                  tr("Color"),
                                  QUColorDragWidget::ModePipette
                                  | QUColorDragWidget::ModeEnableColorselectBox,
                                   m_pc_GlobalStorage,
                                   pc_MainFrame->GetColorSelectBox()

                                  );

   }
   l_Layout->addWidget(m_w_ColorSelector); 

   m_OnlyInSelectionMaskCheck = new QCheckBox(tr("Only in Selection Mask"),
                                             m_w_TopWidget);
   l_Layout->addWidget(m_OnlyInSelectionMaskCheck);

   // -- box for transparency
   m_TransparencyBox = new QVGroupBox(tr("Transparency"),m_w_TopWidget);
   l_Layout->addWidget(m_TransparencyBox);

   {
      // inner is off
      m_TransparencyEnable = new QCheckBox(tr("Enable"),m_TransparencyBox);
      m_TransparencyEnable->setChecked(m_b_TransparencyEnabled);

      m_TransparencySlider = new QULabelSlider(m_TransparencyBox,
                                       QULabelSlider::LabelStyleAutomatic |
                                          QULabelSlider::LabelStyleEdit
                                       ,
                                       0,100,
                                       m_i_TransparencyFactor,
                                       tr("Factor"),
                                       6);
      connect( m_TransparencyEnable,
               SIGNAL(toggled(bool)) ,
               SLOT( TransparencyEnableChanged(bool) )
             );
      connect( m_TransparencySlider,
               SIGNAL(valueChanged(int)),
               SLOT( TransparencyFactorChanged(int )) );

   } // block for tranparency box

   // create tha smoothing checkbox
   m_DoSmoothingCheck = new QCheckBox(tr("Smooth"),m_w_TopWidget);
   l_Layout->addWidget(m_DoSmoothingCheck);
   if(m_b_DoSmoothing)
   {
      m_DoSmoothingCheck->setChecked(1);
   }
   
   // create the blend button
   m_BlendButton    = new QPushButton(tr("Blend"),m_w_TopWidget);
   l_Layout->addWidget(m_BlendButton);

   // connect all elements with slots
   connect( m_ActivateCheck,
            SIGNAL(toggled(bool)),
            SLOT( ActiveModeToggled(bool )));

   connect( m_AddButton,
            SIGNAL( clicked()),
            SLOT( AddButtonPressed( )));

   connect( m_DeleteButton,
            SIGNAL( clicked()),
            SLOT( DeleteButtonPressed( )));

   connect( m_DeleteAllButton,
            SIGNAL( clicked()),
            SLOT( DeleteAllButtonPressed( )));

   connect( m_w_ColorSelector,
            SIGNAL( ColorChanged(Pixel)),
            SLOT( ColorChanged(Pixel )));

   connect( m_DoSmoothingCheck,
            SIGNAL(toggled(bool)),
            SLOT( DoSmoothingToggled(bool )));

   connect( m_BlendButton,
            SIGNAL( clicked()),
            SLOT( BlendButtonPressed( )) );


   // set a default color into the color select box
   m_w_ColorSelector->SetColor(0xaaffaa);            
   SetSensitivity();

} // BlendBox()

//===========================================================================
//
//===========================================================================

BlendBox::~BlendBox()
{

} // ~BlendBox()

//===========================================================================
//
//===========================================================================


void BlendBox::OnMessageReceive(QString qs_MessageType,
                                 void *p_Data)
{
   TRACE << "Message Received member this = " << (int) this << "\n";

   if(qs_MessageType.compare("ImageResize") ==0)
   {
   }
   if(qs_MessageType.compare("ImageNewBuffer") ==0)
   {
   }
} // OnMessageReceive()

//===========================================================================
//
//===========================================================================

void BlendBox::GraphicObjectCallback(GraphicObjectMainEvent *pc_CallbackData)
{
   // now redisplay the size of the graphic object

   if(pc_CallbackData->i_CallbackType == GraphicObjectMainEvent::SelectionDisabled)
   {
      // disable the frame mode
      if( m_b_ObjectModeActive )
      {
         if(   m_pc_GlobalStorage->m_pc_ObjectMain->GetActiveInputLayer()
            == m_pc_ObjectLayer )
         {
            // reset the callback for graphic object changes
            m_pc_GlobalStorage->m_pc_ObjectMain->SetEventCallback(
                              Callback(NULL,NULL));
            // disnable active action
            m_pc_GlobalStorage->m_pc_ObjectMain->ActivateInput(false);
            m_pc_GlobalStorage->m_pc_ImageFrame->RedrawAll(true);
         }
         m_pc_ObjectLayer->SetVisible(false);

         m_b_InRedraw = true;
         m_ActivateCheck->setChecked(false);
         m_b_InRedraw = false;
      }
   }
   if(pc_CallbackData->i_CallbackType == GraphicObjectMainEvent::Selected)
   {
      // a point is selected
      GraphicObject *pc_Object;
      Pixel          p_Color;
      int            i_Number;
      i_Number = m_pc_ObjectLayer->GetNumberOfSelectedObjects(pc_Object);

      // nothing selected?
      if(i_Number != 1)
      {
         // we are ready
         return;
      }

      // get the color from the point
      p_Color = pc_Object->GetColor();
      // and set it into the color selection widget
      m_w_ColorSelector->SetColor(p_Color);
         
   }
   else  // regular callback event
   {
   }
} // GraphicObjectCallback()

//===========================================================================
//
//===========================================================================

void *BlendBox::GraphicObjectCallbackCB(void *thisData,void *GraphicObjectCallbackData)
{
   BlendBox *pc_This;
   pc_This = (BlendBox *) thisData;

   pc_This-> GraphicObjectCallback( (GraphicObjectMainEvent *)GraphicObjectCallbackData );

   return NULL;
} // GraphicObjectCallbackCB()

//===========================================================================
//
//===========================================================================

void BlendBox::DoActivate()
{
    if(m_b_ObjectModeActive )
    {
       return;
    }

    m_ActivateCheck->setChecked(true);

} // DoActivate()

//===========================================================================
//
//===========================================================================

void BlendBox::DoSmoothingToggled(bool b_Mode)
{
   m_b_DoSmoothing = b_Mode;
} // DoSmoothingToggled

//===========================================================================
//
//===========================================================================

void BlendBox::ActiveModeToggled(bool b_Mode)
{
   if(m_b_InRedraw)
   {
   }

   if(b_Mode == m_b_ObjectModeActive)
   {
      // no change
      return;
   }

   m_b_ObjectModeActive = b_Mode;

   if( m_b_ObjectModeActive)
   {
      // make the layer visible
      m_pc_ObjectLayer->SetVisible(true);
      m_pc_GlobalStorage->m_pc_ObjectMain->SetVisible(true);
      m_pc_GlobalStorage->m_pc_ImageFrame->RedrawAll(true);

      // set this layer as input layer
      m_pc_GlobalStorage->m_pc_ObjectMain->SetActiveLayer(BLENDBOXLAYER);

      // enable active action
      m_pc_GlobalStorage->m_pc_ObjectMain->ActivateInput();

      // set the callback for graphic object changes
      m_pc_GlobalStorage->m_pc_ObjectMain->SetEventCallback(
                              Callback(GraphicObjectCallbackCB,this));
   }
   else // not active
   {
      // make invisible
      m_pc_ObjectLayer->SetVisible(false);

      if(   m_pc_GlobalStorage->m_pc_ObjectMain->GetActiveInputLayer()
         == m_pc_ObjectLayer )
      {
         // remove the event callback
         m_pc_GlobalStorage->m_pc_ObjectMain->SetEventCallback(
                              Callback(NULL,NULL));

         // disable input
         m_pc_GlobalStorage->m_pc_ObjectMain->ActivateInput(false);

         // redraw the image
         m_pc_GlobalStorage->m_pc_ImageFrame->RedrawAll(true);
      }
   } // else not active
   
   SetSensitivity();
   
} // ActiveModeToggled()

//===========================================================================
//
//===========================================================================

void *BlendBox::SelectionEventArrived(void *pc_Data)
{
   SelectionCallbackData *cb;
   
   cb = (SelectionCallbackData *) pc_Data;
   if(cb->i_CallbackReason == SelectionCallbackData:: ReasonOperationDisabled)
   {
      // clean up unused structures

   }
   else if(cb->i_CallbackReason == SelectionCallbackData:: ReasonSelection)
   {
      GraphicObjectPoint *pc_NewPoint;
      Pixel               p_Color;
      int                 i_CenterX;
      int                 i_CenterY;
      
      if(m_pc_GlobalStorage == NULL)
      {
         return NULL;
      }

      // get coordianted from the selection
      m_pc_GlobalStorage->m_pc_Selection->GetSelectionPoint(i_CenterX,
                                                              i_CenterY);

      // create a new object                                                              
      pc_NewPoint = new GraphicObjectPoint(i_CenterX,i_CenterY);

      // add the ellipse into the layer
      m_pc_ObjectLayer->AddObject(pc_NewPoint);

      p_Color = m_w_ColorSelector->GetColor();
      pc_NewPoint->SetColor(p_Color);
      // force a redraw
      m_pc_GlobalStorage->m_pc_ImageFrame->RedrawAll();

      // enable the reduce button
      SetSensitivity();
   }
   return NULL;

}  // SelectionEventArrived()


void *BlendBox::SelectionEventArrivedCB(void *Thisdata,void *pc_Data)
{
   BlendBox *pc_This = (BlendBox *) Thisdata;

   return pc_This->SelectionEventArrived(pc_Data);

} // SelectionEventArrivedCB()

//===========================================================================
//
//===========================================================================

void BlendBox::AddButtonPressed()
{
   if(  m_pc_GlobalStorage != NULL)
   {
      m_pc_GlobalStorage->m_pc_Selection->ActivateShortTimeSelection(
                                 Selection::SelectionPoint,
                                 tr("Add Blend Point"),
                                 Callback(SelectionEventArrivedCB,this)
                                 );

   }


} // AddButtonPressed()

//===========================================================================
//
//===========================================================================

void BlendBox::DeleteButtonPressed()
{
   GraphicObject *pc_Object;
   int i_Number;

   // get number of selected objects
   // and the first selected one
   i_Number = m_pc_ObjectLayer->GetNumberOfSelectedObjects(pc_Object);
   
   while(i_Number >0)
   {
      m_pc_ObjectLayer->DeleteObject(pc_Object);
      
      // get new number;
      i_Number = m_pc_ObjectLayer->GetNumberOfSelectedObjects(pc_Object);
   }
   
   
   // force a redraw
   m_pc_GlobalStorage->m_pc_ImageFrame->RedrawAll();

   SetSensitivity();

} // DeleteButtonPressed()

//===========================================================================
//
//===========================================================================

void BlendBox::DeleteAllButtonPressed()
{
   // delete all objects from the layer
   m_pc_ObjectLayer->DeleteAllObjects();

   // force a redraw
   m_pc_GlobalStorage->m_pc_ImageFrame->RedrawAll();

   SetSensitivity();

} // DeleteAllButtonPressed()

//===========================================================================
//
//===========================================================================

void BlendBox::SetSensitivity()
{
   if(m_b_ObjectModeActive)
   {
      m_RegionBox->setEnabled(true);

      if(m_pc_ObjectLayer->GetObjectCount()>1)
      {
         m_BlendButton->setEnabled(true);
      }
      else
      {
         m_BlendButton->setEnabled(false);
      }
   }
   else
   {
      m_RegionBox->setEnabled(false);
      m_BlendButton->setEnabled(false);
   }

   // no more maximum points
   m_AddButton->setEnabled(true);

} // SetSensitivity()

//===========================================================================
//
//===========================================================================

void BlendBox::BlendButtonPressed()
{
   GraphicObject * pc_Object;
   BlendPoint    * pc_BlendPointList;
   RectRegion      c_ChangedRegion;
   Blend           c_Blend;
   bool            b_SelectionMask;
   int             i_NumberOfPoints;
   int             i_NumberOfObjects;
   int             i;

   // now put the points into the blend-object

   i_NumberOfObjects = m_pc_ObjectLayer->GetObjectCount();
   i_NumberOfPoints  =0;
   for(i=0;i<i_NumberOfObjects;i++)
   {
      pc_Object = m_pc_ObjectLayer->GetObject(i);
      if( pc_Object->GetObjectType() == GraphicObject::TypePoint)
      {
         i_NumberOfPoints += 1;
      }
   }
   
   if(i_NumberOfPoints <2)
   {
      // too less points, do nothing
      return;
   }

   // allocate an array
   pc_BlendPointList = new BlendPoint[i_NumberOfPoints];
   if(pc_BlendPointList == NULL)
   {
      return;
   }

   // second loop -- copy the points and attributes into the list
   i_NumberOfPoints  =0;
   for(i=0;i<i_NumberOfObjects;i++)
   {
      pc_Object = m_pc_ObjectLayer->GetObject(i);
      if( pc_Object->GetObjectType() == GraphicObject::TypePoint)
      {
         GraphicObjectPoint *pc_Point;
         float               f_X;
         float               f_Y;
         
         pc_Point = (GraphicObjectPoint *) pc_Object;
         pc_Point->GetPosition(f_X,f_Y);
         
         pc_BlendPointList[i_NumberOfPoints].m_i_X = (int)f_X;
         pc_BlendPointList[i_NumberOfPoints].m_i_Y = (int) f_Y;
         pc_BlendPointList[i_NumberOfPoints].m_p_Color = pc_Point->GetColor();
         
         i_NumberOfPoints += 1;
      }
   }

   // copy all attributes into the blend object
   c_Blend.SetBlendPointList(pc_BlendPointList,i_NumberOfPoints);
   c_Blend.SetPixelBuffer( m_pc_GlobalStorage->m_pc_MainImage);
   b_SelectionMask = m_OnlyInSelectionMaskCheck->isChecked();
   c_Blend.SetOnlyInSelectionMask(b_SelectionMask);

   
   c_Blend.SetTransparency(m_b_TransparencyEnabled,
                           m_i_TransparencyFactor);

   c_Blend.SetSmooth(m_b_DoSmoothing);                           
   // perform a blend

   // may take longer
   QtUtility::StartWaitingCursor();

   // do the blend
   c_Blend.DoBlend();

   // cursor to normal
   QtUtility::EndWaitingCursor();

   c_ChangedRegion = c_Blend.GetChangedRegion();   

   // delete the blendpoint list
   delete[] pc_BlendPointList;

   // update the image
   if( ! c_ChangedRegion.IsEmpty() )
   {
      m_pc_GlobalStorage->m_pc_ImageFrame->RedrawBufferRegion(c_ChangedRegion);

      m_pc_GlobalStorage->m_pc_MessageServer->SendMessage(0,
                                                       this,
                                                       "ImageChanged");
   }
} // ReduceButtonPressed()

//===========================================================================
//
//===========================================================================


int BlendBox::WriteFile(PericoFileWriter *pc_FileWriter,
                         QString qs_Name)
{
   pc_FileWriter->StartStructure(qs_Name,"BlendBox");

   m_pc_ObjectLayer->WriteFile(pc_FileWriter,"ObjectLayer");

   pc_FileWriter->FinishStructure();

   // everything is OK
   return 0;
} // WriteFile

//===========================================================================
//
//===========================================================================

int BlendBox::LoadFile(PericoFileWriter *pc_FileWriter)
{

   PERICO_READ_STRUCTURE_START

   if(    qs_Name.compare("ObjectLayer") == 0
       && qs_Value.compare("GraphicObjectLayer") == 0
       && b_IsStartStructure)
   {
      // delete old objects
      m_pc_ObjectLayer->DeleteAllObjects();

      // load objects from a file
      m_pc_ObjectLayer->LoadFile(pc_FileWriter);

   }

   PERICO_READ_STRUCTURE_END

   // change sensitivity according the new settings
   SetSensitivity();

   // everything is OK
   return 0;
} // LoadFile

//===========================================================================
//
//===========================================================================

void BlendBox::showEvent(QShowEvent *)
{
   m_ActivateCheck->setChecked(true);
   SetSensitivity();
}  // showEvent()

//===========================================================================
//
//===========================================================================

void BlendBox::hideEvent(QHideEvent *)
{
   // make the layer invisible
   m_ActivateCheck->setChecked(false);

}  // hideEvent()

//===========================================================================
//
//===========================================================================

void BlendBox::ColorChanged(Pixel p_NewColor)
{
   GraphicObject *pc_Object;
   int            i_Number;
   int            i;

   // get number of selected objects
   // and the first selected one
   i_Number = m_pc_ObjectLayer->GetNumberOfSelectedObjects(pc_Object);

   // nothing selected?
   if(i_Number <= 0)
   {
      // we are ready
      return;
   }
   
   for(i=0;i<i_Number;i++)
   {
      pc_Object->SetColor(p_NewColor);

      // get next object
      m_pc_ObjectLayer->GetNextSelectedObject(pc_Object);
      if(pc_Object == NULL)
      {
         break;
      }
   } // for all objects


   // force a redraw
   m_pc_GlobalStorage->m_pc_ImageFrame->RedrawAll();

   SetSensitivity();
} // ColorChanged()

//===========================================================================
//
//===========================================================================

void BlendBox::TransparencyEnableChanged(bool b_on)
{
   m_b_TransparencyEnabled = b_on;

   SetSensitivity();

} // ransparencyEnableChanged

//===========================================================================
//
//===========================================================================

void BlendBox::TransparencyFactorChanged(int i_Factor)
{
   m_i_TransparencyFactor = i_Factor;

} // TransparencyFactorChanged()

//===========================================================================
//
//===========================================================================


