//---------------------------------------------------------------------------
// This sourcefile is created by Peter Jodda / peter@jodda.de
// It is under the GPL license.
// If you expand this file, please contact me, that I can integrate the
// enhancements in the original file.
//---------------------------------------------------------------------------
// Module: BufferPainter
//---------------------------------------------------------------------------
// Purpose: simple drawing functions to paint into a 2dimensional
// pixelbuffer.
//---------------------------------------------------------------------------

#include <stdio.h>
#include <stdlib.h>
#include <math.h>

#include "BufferPainter.h"

#include "Utilities.h"
#include "PixelBuffer.h"
#include "ColorCalculation.h"
#include "CubicSpline.h"


//===========================================================================
//
//===========================================================================



BufferPainter::BufferPainter()
{
   m_BorderBrush.m_Color =(0xffaa88);
   m_InnerBrush.m_Color  =(0x55ff55);

   m_BlackBrush.m_Color  =(0x000000);
   m_WhiteBrush.m_Color  =(0xFFFFFF);

   m_i_LineWidth          = 1;
   m_b_DrawInnerPart      = true;
   m_b_DrawTransparent    = false;
   m_i_TransparencyFactor = 0;
   m_i_SpeckleSize        = 5;
   m_i_SpeckleDensity     = 1000;
   m_i_SmearStrength      = 10;
   m_pc_SmearCopy         = NULL;


   m_i_FilterWidth              = 1;
   m_b_FilterNearDistanceActive = false;
   m_i_FilterNearDistance       = 30;
   m_i_FilterAlgorithm          = FilterAlgorithmMiddle;
   // disable the onetimedraw buffer
   m_b_UseOneTimeDrawBuffer = false;
//   m_pc_OneTimeDrawBuffer   = NULL;

   SetMaskMode(false,false);
   SetRecolorParameter(0x0000ff,0xffff00,1);

} // BufferPainter

//===========================================================================
//
//===========================================================================


BufferPainter::~BufferPainter()
{
   if(m_pc_SmearCopy != NULL)
   {
      delete m_pc_SmearCopy;
      m_pc_SmearCopy = NULL;
   }
} // ~BufferPainter()

//===========================================================================
//
//===========================================================================

void BufferPainter::FilterPixelMiddle(PixelBuffer *pc_Buffer,
                                int i_X,
                                int i_Y)
{
   PixelBuffer *pc_Original;
   Pixel        p_Value;
   Pixel        p_Original;
   int          i_Width;
   int          i_Height;

   int          i_RangeFromY;
   int          i_RangeFromX;
   int          i_RangeToX;
   int          i_RangeToY;
   int          i_RangeTotalPoints;

   int          i_InnerX;
   int          i_InnerY;

   int          i_RedSum;
   int          i_GreenSum;
   int          i_BlueSum;

   // get the original picture, which must not get modified
   pc_Original = pc_Buffer->GetUsedPixelMask();

   // get the image dimension
   i_Width  = pc_Original->GetWidth();
   i_Height = pc_Original->GetHeight();

   // calculate the filter region
   i_RangeFromY = i_Y - m_i_FilterWidth;
   if(i_RangeFromY <0)
   {
      i_RangeFromY =0;
   }
   i_RangeToY = i_Y + m_i_FilterWidth;
   if(i_RangeToY >=i_Height)
   {
      i_RangeToY = i_Height-1;
   }

   i_RangeFromX = i_X - m_i_FilterWidth;
   if(i_RangeFromX <0)
   {
      i_RangeFromX =0;
   }
   i_RangeToX = i_X + m_i_FilterWidth;
   if(i_RangeToX >= i_Width)
   {
      i_RangeToX = i_Width-1;
   }

   // calculate the total points in the range
   i_RangeTotalPoints=   (i_RangeToX - i_RangeFromX +1)
                        *(i_RangeToY - i_RangeFromY +1);

   // set sums to zero
   i_RedSum  = 0;
   i_GreenSum= 0;
   i_BlueSum = 0;

   p_Original= 0;
   
   // in some modes the original is needed
   if(   m_b_FilterNearDistanceActive)
   {
      p_Original = pc_Original->Get(i_X,i_Y);
   }
         
   // inner loop - calculate the pixelsumms
   for(i_InnerY =  i_RangeFromY;
       i_InnerY <= i_RangeToY;
       i_InnerY ++)
   {
      for(i_InnerX =  i_RangeFromX;
          i_InnerX <= i_RangeToX;
          i_InnerX ++)
      {
         p_Value = pc_Original->Get(i_InnerX,i_InnerY);

         if(m_b_FilterNearDistanceActive)
         {
            int i_ActualDistance;
            i_ActualDistance = Utilities::PixelDistance(p_Value,p_Original);
            if( i_ActualDistance > m_i_FilterNearDistance)
            {
               // forget this point
               i_RangeTotalPoints -= 1;
               continue;
            }
         }
         
         i_RedSum   += PIXEL_RED  (p_Value);
         i_GreenSum += PIXEL_GREEN(p_Value);
         i_BlueSum  += PIXEL_BLUE (p_Value);

      } // for i_InnerX
   } // for i_InnerY

   if(i_RangeTotalPoints >= 1)
   {
      // divide summ by number of pixels
      i_RedSum   /= i_RangeTotalPoints;
      i_GreenSum /= i_RangeTotalPoints;
      i_BlueSum  /= i_RangeTotalPoints;
      // clipp maximum to 255
      if(i_RedSum>255)
      {
         i_RedSum = 255;
      }
      if(i_GreenSum>255)
      {
         i_GreenSum = 255;
      }
      if(i_BlueSum>255)
      {
         i_BlueSum = 255;
      }

      // compute the new pixel
      p_Value = PIXEL_MAKE(i_RedSum,i_GreenSum,i_BlueSum);

      // set the new pixel into the image
      pc_Buffer->Set(i_X,i_Y,p_Value);
   }


} // FilterPixelMiddle()

//===========================================================================
//
//===========================================================================

void BufferPainter::FilterPixelSharp(PixelBuffer *pc_Buffer,
                                int i_X,
                                int i_Y)
{
   PixelBuffer *pc_Original;
   Pixel        p_Value;
   Pixel        p_Original;
   int          i_Width;
   int          i_Height;

   int          i_RangeFromY;
   int          i_RangeFromX;
   int          i_RangeToX;
   int          i_RangeToY;
   int          i_RangeTotalPoints;

   int          i_InnerX;
   int          i_InnerY;

   int          i_RedSum;
   int          i_GreenSum;
   int          i_BlueSum;

   // get the original picture, which must not get modified
   pc_Original = pc_Buffer->GetUsedPixelMask();

   // get the image dimension
   i_Width  = pc_Original->GetWidth();
   i_Height = pc_Original->GetHeight();

   // calculate the filter region
   i_RangeFromY = i_Y - m_i_FilterWidth;
   if(i_RangeFromY <0)
   {
      i_RangeFromY =0;
   }
   i_RangeToY = i_Y + m_i_FilterWidth;
   if(i_RangeToY >=i_Height)
   {
      i_RangeToY = i_Height-1;
   }

   i_RangeFromX = i_X - m_i_FilterWidth;
   if(i_RangeFromX <0)
   {
      i_RangeFromX =0;
   }
   i_RangeToX = i_X + m_i_FilterWidth;
   if(i_RangeToX >= i_Width)
   {
      i_RangeToX = i_Width-1;
   }

   // calculate the total points in the range
   i_RangeTotalPoints=   (i_RangeToX - i_RangeFromX +1)
                        *(i_RangeToY - i_RangeFromY +1);

   // set sums to zero
   i_RedSum  = 0;
   i_GreenSum= 0;
   i_BlueSum = 0;

   p_Original = pc_Original->Get(i_X,i_Y);

   // inner loop - calculate the pixelsumms
   for(i_InnerY =  i_RangeFromY;
       i_InnerY <= i_RangeToY;
       i_InnerY ++)
   {
      for(i_InnerX =  i_RangeFromX;
          i_InnerX <= i_RangeToX;
          i_InnerX ++)
      {
         p_Value = pc_Original->Get(i_InnerX,i_InnerY);

         if(m_b_FilterNearDistanceActive)
         {
            int i_ActualDistance;
            i_ActualDistance = Utilities::PixelDistance(p_Value,p_Original);
            if( i_ActualDistance > m_i_FilterNearDistance)
            {
               // forget this point
               i_RangeTotalPoints -= 1;
               continue;
            }
         }

         i_RedSum   += PIXEL_RED  (p_Value);
         i_GreenSum += PIXEL_GREEN(p_Value);
         i_BlueSum  += PIXEL_BLUE (p_Value);

      } // for i_InnerX
   } // for i_InnerY

   if(i_RangeTotalPoints >= 1)
   {
      // do the sharpening
      i_RedSum   = PIXEL_RED(p_Original)*2*i_RangeTotalPoints - i_RedSum;
      i_GreenSum = PIXEL_GREEN(p_Original)*2*i_RangeTotalPoints - i_GreenSum;
      i_BlueSum  = PIXEL_BLUE(p_Original)*2*i_RangeTotalPoints - i_BlueSum;

      // divide summ by number of pixels
      i_RedSum   /= i_RangeTotalPoints;
      i_GreenSum /= i_RangeTotalPoints;
      i_BlueSum  /= i_RangeTotalPoints;
      // clipp maximum to 255
      if(i_RedSum>255)
      {
         i_RedSum = 255;
      }
      else if(i_RedSum<0)
      {
         i_RedSum = 0;
      }

      if(i_GreenSum>255)
      {
         i_GreenSum = 255;
      }
      else if(i_GreenSum<0)
      {
         i_GreenSum = 0;
      }

      if(i_BlueSum>255)
      {
         i_BlueSum = 255;
      }
      else if(i_BlueSum<0)
      {
         i_BlueSum = 0;
      }

      // compute the new pixel
      p_Value = PIXEL_MAKE(i_RedSum,i_GreenSum,i_BlueSum);

      // set the new pixel into the image
      pc_Buffer->Set(i_X,i_Y,p_Value);
   }


} // FilterPixelSharp()

//===========================================================================
//
//===========================================================================

void BufferPainter::FilterPixelMedian(PixelBuffer *pc_Buffer,
                                int i_X,
                                int i_Y)
{
   unsigned char *p_ArrayRed;
   unsigned char *p_ArrayGreen;
   unsigned char *p_ArrayBlue;
   PixelBuffer   *pc_Original;
   Pixel          p_Value;
   Pixel          p_Original;
   int            i_Width;
   int            i_Height;

   int            i_RangeFromY;
   int            i_RangeFromX;
   int            i_RangeToX;
   int            i_RangeToY;
   int            i_RangeTotalPoints;

   int            i_InnerX;
   int            i_InnerY;
   int            i_ActualPoint;


   // get the original picture, which must not get modified
   pc_Original = pc_Buffer->GetUsedPixelMask();

   // get the image dimension
   i_Width  = pc_Original->GetWidth();
   i_Height = pc_Original->GetHeight();

   // calculate the filter region
   i_RangeFromY = i_Y - m_i_FilterWidth;
   if(i_RangeFromY <0)
   {
      i_RangeFromY =0;
   }
   i_RangeToY = i_Y + m_i_FilterWidth;
   if(i_RangeToY >=i_Height)
   {
      i_RangeToY = i_Height-1;
   }

   i_RangeFromX = i_X - m_i_FilterWidth;
   if(i_RangeFromX <0)
   {
      i_RangeFromX =0;
   }
   i_RangeToX = i_X + m_i_FilterWidth;
   if(i_RangeToX >= i_Width)
   {
      i_RangeToX = i_Width-1;
   }

   // calculate the total points in the range
   i_RangeTotalPoints=   (i_RangeToX - i_RangeFromX +1)
                        *(i_RangeToY - i_RangeFromY +1);

   if(i_RangeTotalPoints <= 0)
   {
      return;
   }
   // allocate 3 arrays for the sorting vector
   p_ArrayRed    = new unsigned char[i_RangeTotalPoints];
   if(p_ArrayRed == NULL)
   {
      return ;
   }
   p_ArrayGreen  = new unsigned char[i_RangeTotalPoints];
   if(p_ArrayGreen == NULL)
   {
      delete p_ArrayRed;
      return ;
   }
   p_ArrayBlue   = new unsigned char[i_RangeTotalPoints];
   if(p_ArrayBlue == NULL)
   {
      delete p_ArrayRed;
      delete p_ArrayGreen;
      return ;
   }

   p_Original= 0;

   // in some modes the original is needed
   if(   m_b_FilterNearDistanceActive)
   {
      p_Original = pc_Original->Get(i_X,i_Y);
   }

   i_ActualPoint =0;

   // inner loop - calculate the pixelsumms
   for(i_InnerY =  i_RangeFromY;
       i_InnerY <= i_RangeToY;
       i_InnerY ++)
   {
      for(i_InnerX =  i_RangeFromX;
          i_InnerX <= i_RangeToX;
          i_InnerX ++)
      {
         p_Value = pc_Original->Get(i_InnerX,i_InnerY);

         if(m_b_FilterNearDistanceActive)
         {
            int i_ActualDistance;
            i_ActualDistance = Utilities::PixelDistance(p_Value,p_Original);
            if( i_ActualDistance > m_i_FilterNearDistance)
            {
               // forget this point
               continue;
            }
         }

         p_ArrayRed  [i_ActualPoint] = (unsigned char) PIXEL_RED  (p_Value);
         p_ArrayGreen[i_ActualPoint] = (unsigned char) PIXEL_GREEN(p_Value);
         p_ArrayBlue [i_ActualPoint] = (unsigned char) PIXEL_BLUE (p_Value);
         i_ActualPoint ++;

      } // for i_InnerX
   } // for i_InnerY

   // middle pixel makes only sense, if we have >=3 pixel
   if(i_ActualPoint > 2)
   {
      int i_Red;
      int i_Green;
      int i_Blue;

      // now sort the array
      Utilities::QSortBytes(p_ArrayRed  ,0,i_ActualPoint -1);
      Utilities::QSortBytes(p_ArrayGreen,0,i_ActualPoint -1);
      Utilities::QSortBytes(p_ArrayBlue ,0,i_ActualPoint -1);

      // take the point in the middle
      i_ActualPoint /=2;
      i_Red   = p_ArrayRed  [i_ActualPoint];
      i_Green = p_ArrayGreen[i_ActualPoint];
      i_Blue  = p_ArrayBlue [i_ActualPoint];

      // compute the new pixel
      p_Value = PIXEL_MAKE(i_Red,i_Green,i_Blue);

      // set the new pixel into the image
      pc_Buffer->Set(i_X,i_Y,p_Value);
   }

   // free the allocated arrays
   delete p_ArrayRed;
   delete p_ArrayGreen;
   delete p_ArrayBlue;

} // FilterPixelMedian()

//===========================================================================
//
//===========================================================================


void BufferPainter::DrawPixel(PixelBuffer *pc_Buffer,
                              PainterBrush *pc_Brush,
                              int i_X,
                              int i_Y)
{
   Pixel p_NewPixel;

   if(pc_Brush->m_b_Recolor)
   {
      // for recolor do only real change colors in the
      // usedpixelmask. So a slightly different algorithm
      // here.
      Pixel p_Old;
      Pixel p_New;
      if(   m_b_UseOneTimeDrawBuffer
          &&  pc_Buffer->UsedPixelMaskAvailable())
      {
         if(pc_Buffer->UsedPixelMaskGet(i_X,i_Y))
         {
            // already set
            return;
         }
      }

      // get old value
      p_Old = pc_Buffer->Get(i_X, i_Y);

      // calculate the new value
      p_New = CalculateRecolor(p_Old);
      if(p_New == p_Old)
      {
         return;
      }

      // store new value
      pc_Buffer->Set(i_X,i_Y,p_New);

      // set used buffer mask
      if(   m_b_UseOneTimeDrawBuffer
          &&  pc_Buffer->UsedPixelMaskAvailable())
      {
         pc_Buffer->UsedPixelMaskSet(i_X,i_Y,1);
      }

      // leave reolor at this piunt
      return;
   }
   
   // is the only overwrite once buffer used?
   if(    m_b_UseOneTimeDrawBuffer
      &&  pc_Buffer->UsedPixelMaskAvailable())
   {
      if(pc_Buffer->UsedPixelMaskGet(i_X,i_Y))
      {
         // already set
         return;
      }
      else
      {
         // set it for this pixel, to prevent the next time
         pc_Buffer->UsedPixelMaskSet(i_X,i_Y,1);
      }
   }

   if(pc_Brush->m_b_Lightness)
   {
      Pixel p_Old;
      Pixel p_New;
      int   i_Red;
      int   i_Green;
      int   i_Blue;
      int   i_Hue;
      int   i_Lightness;
      int   i_Saturation;

      p_Old = pc_Buffer->Get(i_X, i_Y);
      i_Red   = PIXEL_RED  (p_Old);
      i_Green = PIXEL_GREEN(p_Old);
      i_Blue  = PIXEL_BLUE (p_Old);

      // convert into HLS
      ColorCalculation::RGBToHLS(i_Red,i_Green,i_Blue,i_Hue,i_Lightness,i_Saturation);

      // modify the lightness
      i_Lightness += pc_Brush->m_i_LightnessFactor;
      if(i_Lightness <0)
      {
         i_Lightness =0;
      }
      else if(i_Lightness >100)
      {
         i_Lightness = 100;
      }

      // convert back into rgb
      ColorCalculation::HLSToRGB(i_Hue,i_Lightness,i_Saturation,i_Red,i_Green,i_Blue);
      p_New = PIXEL_MAKE(i_Red,i_Green,i_Blue);

      pc_Buffer->Set(i_X,i_Y,p_New);

   }
   else if(pc_Brush->m_b_Saturation)
   {
      Pixel p_Old;
      Pixel p_New;
      int   i_Red;
      int   i_Green;
      int   i_Blue;
      int   i_Hue;
      int   i_Lightness;
      int   i_Saturation;

      p_Old = pc_Buffer->Get(i_X, i_Y);
      i_Red   = PIXEL_RED  (p_Old);
      i_Green = PIXEL_GREEN(p_Old);
      i_Blue  = PIXEL_BLUE (p_Old);

      //convert into HLS
      ColorCalculation::RGBToHLS(i_Red,i_Green,i_Blue,i_Hue,i_Lightness,i_Saturation);

      // modify the saturation
      i_Saturation += pc_Brush->m_i_LightnessFactor;
      if(i_Saturation <0)
      {
         i_Saturation =0;
      }
      else if(i_Saturation >100)
      {
         i_Saturation = 100;
      }

      // convert back into rgb
      ColorCalculation::HLSToRGB(i_Hue,i_Lightness,i_Saturation,i_Red,i_Green,i_Blue);
      p_New = PIXEL_MAKE(i_Red,i_Green,i_Blue);
      pc_Buffer->Set(i_X,i_Y,p_New);

   }
   else if(pc_Brush->m_b_Filter)
   {
      if(m_i_FilterAlgorithm == FilterAlgorithmMiddle)
      {
         FilterPixelMiddle(pc_Buffer,i_X,i_Y);
      }
      else if(m_i_FilterAlgorithm == FilterAlgorithmMedian)
      {
         FilterPixelMedian(pc_Buffer,i_X,i_Y);
      }
      else if(m_i_FilterAlgorithm == FilterAlgorithmSharp)
      {
         FilterPixelSharp(pc_Buffer,i_X,i_Y);
      }
   }
   else
   {
      bool b_Valid;
      // regular pixel paint
      p_NewPixel = pc_Brush->GetBrushPixel(i_X,i_Y,b_Valid);

      // if pixel is not valid
      if(!b_Valid)
      {
         // leave it untouched
         return;
      }
      
      if(m_b_DrawTransparent
         && m_i_TransparencyFactor>0)
      {
         // make a transparent color
         Pixel p_Old;

         p_Old = pc_Buffer->Get(i_X, i_Y);

         p_NewPixel = ColorCalculation::TransparentColor(p_Old,
                                                         p_NewPixel,
                                                         m_i_TransparencyFactor);
         pc_Buffer->Set(i_X,i_Y,p_NewPixel);

      }
      else // not transparent
      {
         pc_Buffer->Set(i_X,i_Y,p_NewPixel);
      }
   }


} // DrawPixel() -- brush version

//===========================================================================
//
//===========================================================================


void BufferPainter::DrawPixel(PixelBuffer *pc_Buffer,
                              Pixel p_Pixel,
                              int i_X,
                              int i_Y)
{
   Pixel p_NewPixel;

   // is the only overwrite once buffer used?
   if(    m_b_UseOneTimeDrawBuffer
      &&  pc_Buffer->UsedPixelMaskAvailable())
   {
      if(pc_Buffer->UsedPixelMaskGet(i_X,i_Y))
      {
         // already set
         return;
      }
      else
      {
         // set it for this pixel, to prevent the next time
         pc_Buffer->UsedPixelMaskSet(i_X,i_Y,1);
      }
   }

//   fprintf(stderr,"m_b_DrawTransparent %d  m_i_TransparencyFactor %d\n",m_b_DrawTransparent,m_i_TransparencyFactor);
   {
      // regular pixel paint
      p_NewPixel = p_Pixel;

      if(m_b_DrawTransparent
         && m_i_TransparencyFactor>0)
      {
         // make a transparent color
         Pixel p_Old;

         p_Old = pc_Buffer->Get(i_X, i_Y);

         p_NewPixel = ColorCalculation::TransparentColor(p_Old,
                                                         p_NewPixel,
                                                         m_i_TransparencyFactor);
         pc_Buffer->Set(i_X,i_Y,p_NewPixel);

      }
      else // not transparent
      {
         pc_Buffer->Set(i_X,i_Y,p_NewPixel);
      }
   }


} // DrawPixel()

//===========================================================================
//
//===========================================================================

void BufferPainter::DrawHorizontalLine(PixelBuffer *pc_Buffer,
                                       PainterBrush *pc_Brush,
                                       int i_StartX,
                                       int i_EndX,
                                       int i_Y)
{
   int i;
   // fprintf(stderr,"Horizontal %d %d\n",i_StartX,i_EndX);
   if(i_StartX > i_EndX)
   {
      Utilities::swap(i_StartX,i_EndX);
   }

   for(i=i_StartX; i<=i_EndX;i++)
   {
      DrawPixel(pc_Buffer,pc_Brush,i,i_Y);
   }

} // DrawHorizontalLine()


//===========================================================================
//
//===========================================================================

void BufferPainter::DrawPoint(PixelBuffer *pc_Buffer,int i_X, int i_Y)
{
   DrawPixel(pc_Buffer,m_pc_CurrentBorderBrush,i_X,i_Y);

   m_ChangedRegion.SetXYWH(i_X,i_Y,1,1);

} // DrawPoint()

//===========================================================================
//
//===========================================================================

void BufferPainter::DrawLine(PixelBuffer *pc_Buffer,
                             int i_X1, int i_Y1,
                             int i_X2, int i_Y2)
{
   BaseDrawLine(  pc_Buffer,
                  m_pc_CurrentBorderBrush,
                  i_X1,i_Y1,
                  i_X2,i_Y2,
                  m_i_LineWidth,
                  m_ChangedRegion);


} // DrawLine()


//===========================================================================
//
//===========================================================================

void BufferPainter::DrawLineWithWidth1(PixelBuffer *pc_Buffer,
                                       PainterBrush *pc_Brush,
                                       int i_X1, int i_Y1,
                                       int i_X2, int i_Y2)
{

   int i_DiffX;
   int i_DiffY;
   
   // calculate differences
   i_DiffX = i_X2 - i_X1;
   if(i_DiffX<0)
   {
      i_DiffX = - i_DiffX;
   }

   i_DiffY = i_Y2 - i_Y1;
   if(i_DiffY<0)
   {
      i_DiffY = - i_DiffY;
   }

   if(i_DiffX >= i_DiffY)
   {
      int i_RunX;
      int i_RunY;
      int i_YOffset;
      int i_Divider;

      // make loop for X
      if(i_X1> i_X2)
      {
         Utilities::swap(i_X1,i_X2);   
         Utilities::swap(i_Y1,i_Y2);   
      }
      if(i_Y1>i_Y2) 
      {
         i_YOffset = -1;
      }
      else if(i_Y1<i_Y2)
      {
         i_YOffset = 1;
      }
      else
      {
         i_YOffset = 0;
      }

      i_Divider = i_DiffX /2;
      i_RunY = i_Y1;
      for(i_RunX= i_X1; i_RunX <= i_X2;i_RunX++)
      {
         DrawPixel(pc_Buffer,pc_Brush,i_RunX,i_RunY);
         i_Divider -= i_DiffY;
         if(i_Divider <0)
         {
            i_Divider += i_DiffX;
            i_RunY    += i_YOffset;
         }
      }

   }
   else  // DiffY ist larger
   {
      // make loop for Y
      int i_RunX;
      int i_RunY;
      int i_XOffset;
      int i_Divider;

      if(i_Y1> i_Y2)
      {
         Utilities::swap(i_X1,i_X2);   
         Utilities::swap(i_Y1,i_Y2);   
      }
      if(i_X1>i_X2) 
      {
         i_XOffset = -1;
      }
      else if(i_X1<i_X2)
      {
         i_XOffset = 1;
      }
      else
      {
         i_XOffset = 0;
      }

      i_Divider = i_DiffY /2;
      i_RunX = i_X1;
      for(i_RunY= i_Y1; i_RunY <= i_Y2;i_RunY++)
      {
         DrawPixel(pc_Buffer,pc_Brush,i_RunX,i_RunY);
         i_Divider -= i_DiffX;
         if(i_Divider <0)
         {
            i_Divider += i_DiffY;
            i_RunX    += i_XOffset;
         }
      }

   }


} // DrawLineWithWidth1()

//===========================================================================
//
//===========================================================================

RectRegion BufferPainter::GetChangedRegion()
{
   return m_ChangedRegion;
} // GetChangedRegion()

//===========================================================================
//
//===========================================================================


void BufferPainter::DrawRectangle(PixelBuffer *pc_Buffer,
                             int i_X1, int i_Y1,
                             int i_X2, int i_Y2)
{
   int i_RunX;
   int i_RunY;

   int i_Height;
   int i_Width;

   if(i_X1 > i_X2)
   {
      Utilities::swap(i_X1,i_X2);
   }

   if(i_Y1 > i_Y2)
   {
      Utilities::swap(i_Y1,i_Y2);
   }

   i_Width  = (i_X2 - i_X1)+1;
   i_Height = (i_Y2 - i_Y1)+1;


   // now x1 <= x2 and y1 <= y2

   if(    ( m_i_LineWidth * 2) >= i_Width
       || ( m_i_LineWidth * 2) >= i_Height )
   {
      // Draw full filled rectangle
      for(i_RunY = i_Y1; i_RunY <= i_Y2; i_RunY++)
      {
         for(i_RunX = i_X1; i_RunX <= i_X2; i_RunX++)
         {
            DrawPixel(pc_Buffer,m_pc_CurrentBorderBrush,i_RunX,i_RunY);
         }
      }
   }
   else
   {
      // draw inner part in another color

      // start and end of the inner part - including
      int i_LeftInner;
      int i_RightInner;


      i_LeftInner  = i_X1 + m_i_LineWidth;
      i_RightInner = i_X2 - m_i_LineWidth;


      // draw the upper line part
      for(i_RunY = i_Y1; i_RunY < i_Y1+m_i_LineWidth; i_RunY++)
      {
         for(i_RunX = i_X1; i_RunX <= i_X2; i_RunX++)
         {
            DrawPixel(pc_Buffer,m_pc_CurrentBorderBrush,i_RunX,i_RunY);
         }
      }
      // draw the middle Part
      for(i_RunY = i_Y1+m_i_LineWidth; 
          i_RunY <= i_Y2-(m_i_LineWidth);
          i_RunY++)
      {
         // draw left line
         for(i_RunX = i_X1; i_RunX < i_LeftInner; i_RunX++)
         {
            DrawPixel(pc_Buffer,m_pc_CurrentBorderBrush,i_RunX,i_RunY);
         }
         // draw middle part
         if(m_b_DrawInnerPart)
         {
            for(i_RunX = i_LeftInner; i_RunX <= i_RightInner; i_RunX++)
            {
               DrawPixel(pc_Buffer,m_pc_CurrentInnerBrush,i_RunX,i_RunY);
            }
         }
         // draw right line
         for(i_RunX = i_RightInner+1; i_RunX <= i_X2; i_RunX++)
         {
            DrawPixel(pc_Buffer,m_pc_CurrentBorderBrush,i_RunX,i_RunY);
         }
      }

      // draw the lower line part
      for(i_RunY = i_Y2-(m_i_LineWidth-1); i_RunY <= i_Y2; i_RunY++)
      {
         for(i_RunX = i_X1; i_RunX <= i_X2; i_RunX++)
         {
            DrawPixel(pc_Buffer,m_pc_CurrentBorderBrush,i_RunX,i_RunY);
         }
      }
   }




   m_ChangedRegion.SetXYXY(i_X1,i_Y1,i_X2,i_Y2);


} // DrawRectangle

//===========================================================================
//
//===========================================================================

void BufferPainter::BaseDrawLine (PixelBuffer *pc_Buffer,
                                  PainterBrush *pc_Brush,
                                  int i_StartX,
                                  int i_StartY,
                                  int i_EndX,
                                  int i_EndY,
                                  int i_LineWidth,
                                  RectRegion &c_ChangedRegion)
{

   float  f_DifferenceX;
   float  f_DifferenceY;
   float  f_Length;
   float  f_Width;
   int    i_DeltaX;
   int    i_DeltaY;
   int    i_PointX[4];
   int    i_PointY[4];
   int    i_RunningY;
   int    i_MaximumDifference;
   int    i_DeltaRight;
   int    i_DeltaLeft;

   // calculate the delta value
   i_DeltaX = i_EndX - i_StartX;
   i_DeltaY = i_EndY - i_StartY;


   // is it a vertical line ?
   if (i_DeltaX == 0)
   {

      int i_RunningX;
      int i_RunningY;

      // let startY smaller than end
      if (i_EndY < i_StartY)
      {
         Utilities::swap (i_StartY, i_EndY);
      }


      if (i_LineWidth == 1)
      {
         // set the modified region
         c_ChangedRegion.SetXYXY(i_StartX, i_StartY, i_EndX, i_EndY);

         // draw the vertical line
         for (i_RunningY = i_StartY; i_RunningY <= i_EndY; i_RunningY++)
         {
            DrawPixel(pc_Buffer,pc_Brush, i_StartX, i_RunningY);
         }
      }
      else // linewidth larger one
      {
         int i_RunningEndX;
         i_RunningX    = i_StartX   - i_LineWidth / 2;
         i_RunningEndX = i_RunningX + i_LineWidth - 1;
         c_ChangedRegion.SetXYXY( i_RunningX, i_StartY,
                                  i_RunningEndX, i_EndY);
         for (i_RunningY = i_StartY; i_RunningY <= i_EndY; i_RunningY++)
         {
            DrawHorizontalLine(pc_Buffer,
                               pc_Brush,
                               i_RunningX,
                               i_RunningEndX,
                               i_RunningY);
         }

      }

   }
   // is it a horizonal line
   else if (i_DeltaY == 0)
   {

      if (i_EndX < i_StartX)
      {
          Utilities::swap  ( i_StartX, i_EndX);
      }
 
      if (i_LineWidth == 1)
      {
         c_ChangedRegion.SetXYXY(i_StartX, i_StartY,
                                  i_EndX, i_EndY);
         DrawHorizontalLine(pc_Buffer,
                               pc_Brush,
                               i_StartX, i_EndX, i_StartY);
      }
      else
      {
         int i_Y;
         i_RunningY = i_StartY - i_LineWidth / 2;

         c_ChangedRegion.SetXYXY( i_StartX, i_RunningY,
                                  i_EndX, i_RunningY + i_LineWidth - 1);

         for (i_Y = 0; 
              i_Y < i_LineWidth;
              i_Y++, i_RunningY++)
         {
            DrawHorizontalLine(pc_Buffer,
                               pc_Brush,
                               i_StartX, 
                               i_EndX, 
                               i_RunningY);
         }
      }

   }

   // simple line with width 1
   else if (i_LineWidth == 1)

   {
      int i;
      c_ChangedRegion.SetXYXY(i_StartX, i_StartY,
                              i_EndX, i_EndY);
      i_MaximumDifference = abs (i_DeltaX);

      if (abs (i_DeltaY) > i_MaximumDifference)
      {
         i_MaximumDifference = abs (i_DeltaY);
      }

      for (i = 0; i <= i_MaximumDifference; i++)
      {
         int  i_XPosition;
         int  i_YPosition;
    
         i_XPosition = i_StartX + i_DeltaX * i / i_MaximumDifference;
         i_YPosition = i_StartY + i_DeltaY * i / i_MaximumDifference;
         DrawPixel (pc_Buffer,pc_Brush, i_XPosition, i_YPosition);
      }
   }

   // default case : not vertical, horizontal and width > 1
   else
   {

      int i_ClimbRight;
      int i_ClimbLeft;

      if (i_StartX > i_EndX)
      {
         Utilities::swap (i_StartX, i_EndX);
         Utilities::swap (i_StartY, i_EndY);
         i_DeltaX = -i_DeltaX;
         i_DeltaY = -i_DeltaY;
      }

      // calculate 4 points
      // point 0 upper most
      // point 1 left most
      // point 2 right most
      // point 3 lowest

      if (i_EndY > i_StartY)
      {
         f_Length = Utilities::Hypothenusis (i_DeltaX, i_DeltaY);
         f_Width = i_LineWidth / 2.0;
         f_DifferenceX = (float) f_Width *(float) i_DeltaY / (float) f_Length;
         f_DifferenceY = (float) f_Width *(float) i_DeltaX / (float) f_Length;

         i_PointX[0] = i_StartX + (int) (f_DifferenceX+0.5);
         i_PointY[0] = i_StartY - (int) (f_DifferenceY+0.5);
         i_PointX[1] = i_StartX - (int) (f_DifferenceX+0.5);
         i_PointY[1] = i_StartY + (int) (f_DifferenceY+0.5);
         i_PointX[2] = i_PointX[0] + i_DeltaX;
         i_PointY[2] = i_PointY[0] + i_DeltaY;
         i_PointX[3] = i_PointX[1] + i_DeltaX;
         i_PointY[3] = i_PointY[1] + i_DeltaY;
         i_DeltaRight = i_DeltaX;
         i_DeltaLeft  = i_DeltaY;
         i_ClimbRight = i_DeltaLeft / 2;
         i_ClimbLeft  = i_DeltaRight / 2;
      }
      else
      {
         f_Length = Utilities::Hypothenusis (i_DeltaX, i_DeltaY);
         f_Width = i_LineWidth / 2.0;
         f_DifferenceX = (float) f_Width * (-(float) i_DeltaY) / (float) f_Length;
         f_DifferenceY = (float) f_Width *   (float) i_DeltaX  / (float) f_Length;

         i_PointX[1] = i_StartX - (int) (f_DifferenceX+0.5);
         i_PointY[1] = i_StartY - (int) (f_DifferenceY+0.5);
         i_PointX[3] = i_StartX + (int) (f_DifferenceX+0.5);
         i_PointY[3] = i_StartY + (int) (f_DifferenceY+0.5);
         i_PointX[0] = i_PointX[1] + i_DeltaX;
         i_PointY[0] = i_PointY[1] + i_DeltaY;
         i_PointX[2] = i_PointX[3] + i_DeltaX;
         i_PointY[2] = i_PointY[3] + i_DeltaY;
         i_DeltaRight = -i_DeltaY;
         i_DeltaLeft  = i_DeltaX;
         i_ClimbRight = i_DeltaLeft / 2;
         i_ClimbLeft  = i_DeltaRight / 2;
      }

      {
         int i_ActualRightPosition;
         int i_ActualLeftPosition;
         int i_RunningY;

         i_ActualLeftPosition  = i_PointX[0];
         i_ActualRightPosition = i_PointX[0];

         // now calculate the extension for the uppermost row
         i_ClimbLeft -= i_DeltaLeft/2;
         while (i_ClimbLeft < 0)
         {
            i_ClimbLeft += i_DeltaRight;
            i_ActualLeftPosition--;
         }
         i_ClimbRight -= i_DeltaRight/2;
         while (i_ClimbRight < 0)
         {
            i_ClimbRight += i_DeltaLeft;
            i_ActualRightPosition++;
         }

         // clip at most left and right position
         if(i_ActualLeftPosition<i_PointX[1])
         {
            i_ActualLeftPosition=i_PointX[1];
         }

         if(i_ActualRightPosition>i_PointX[2])
         {
            i_ActualRightPosition=i_PointX[2];
         }            

         for (i_RunningY = i_PointY[0]; i_RunningY <= i_PointY[3]; i_RunningY++)
         {

            DrawHorizontalLine(pc_Buffer,
                               pc_Brush, 
                               i_ActualLeftPosition,
                               i_ActualRightPosition,
                               i_RunningY);

            // calculate the left startpoint of the drawing line
            if (i_RunningY < i_PointY[1])

            {
               i_ClimbLeft -= i_DeltaLeft;
               while (i_ClimbLeft < 0)
               {
                  i_ClimbLeft += i_DeltaRight;
                  i_ActualLeftPosition--;
               }
               if (i_ActualLeftPosition < i_PointX[1])
               {
                  i_ActualLeftPosition = i_PointX[1];
               }
            }
            else if (i_RunningY > i_PointY[1])

            {
               i_ClimbLeft -= i_DeltaRight;
               while (i_ClimbLeft <= 0)
               {
                  i_ClimbLeft += i_DeltaLeft;
                  i_ActualLeftPosition++;
               }
               if (i_ActualLeftPosition > i_PointX[3])
               {
                  i_ActualLeftPosition = i_PointX[3];
               }
            }
            else // point 1 reached
            {
               i_ActualLeftPosition = i_PointX[1];
               i_ClimbLeft = i_DeltaLeft / 2;
            }


            // calculate the right end of the line
            if (i_RunningY < i_PointY[2])
            {
               i_ClimbRight -= i_DeltaRight;
               while (i_ClimbRight < 0)
               {
                  i_ClimbRight += i_DeltaLeft;
                  i_ActualRightPosition++;
               }
               if (i_ActualRightPosition > i_PointX[2])
               {
                  i_ActualRightPosition = i_PointX[2];
               }
            }
            else if (i_RunningY > i_PointY[2])

            {
               i_ClimbRight -= i_DeltaLeft;
               while (i_ClimbRight < 0)
               {
                  i_ClimbRight += i_DeltaRight;
                  i_ActualRightPosition--;
               }
               if (i_ActualRightPosition < i_PointX[3])
               {
                  i_ActualRightPosition = i_PointX[3];
               }
            }
            else // point 2 reached
            {
               i_ActualRightPosition = i_PointX[2];
               i_ClimbRight = i_DeltaRight / 2;
            }
         }
      }

      c_ChangedRegion.SetXYXY (i_PointX[1], i_PointY[0],
                               i_PointX[2], i_PointY[3]);


   } // else ordinary line


} // BaseDrawLine

//===========================================================================
//
//===========================================================================

void BufferPainter::SetLineWidth(int i_LineWidth)
{
   m_i_LineWidth = i_LineWidth;
}

//===========================================================================
//
//===========================================================================

void BufferPainter::SetDrawInnerEnable(bool b_On)
{
   m_b_DrawInnerPart = b_On;

}  // SetDrawInnerEnable()

//===========================================================================
//
//===========================================================================

void BufferPainter::SetTransparencyEnable(bool b_on)
{
   m_b_DrawTransparent = b_on;
} // SetTransparencyEnable()

//===========================================================================
//
//===========================================================================

void BufferPainter::SetTransparencyFactor(int i_Factor)
{
   m_i_TransparencyFactor = i_Factor;
} // SetTransparencyFactor()


//===========================================================================
//
//===========================================================================

void BufferPainter::SetInnerColor(Pixel c_Color)
{
   m_InnerBrush.m_Color = c_Color;
}

//===========================================================================
//
//===========================================================================

void BufferPainter::SetBorderColor(Pixel c_Color)
{
   m_BorderBrush.m_Color = c_Color;
}

//===========================================================================
//
//===========================================================================

void BufferPainter::SetBrushShape(PixelBuffer *pc_BrushShape)
{
   if(m_BorderBrush.m_pc_BrushShape != NULL)
   {
      delete m_BorderBrush.m_pc_BrushShape;
      m_BorderBrush.m_pc_BrushShape = NULL;
   }
   m_BorderBrush.m_pc_BrushShape = pc_BrushShape;

   // create a brush shape for the white brush

   if(m_WhiteBrush.m_pc_BrushShape != NULL )
   {
      delete m_WhiteBrush.m_pc_BrushShape;
      m_WhiteBrush.m_pc_BrushShape = NULL;
   }

   if(pc_BrushShape != NULL)
   {
      m_WhiteBrush.m_pc_BrushShape = new PixelBuffer(pc_BrushShape);
   }

   // create a brush shape for the black brush
   if(m_BlackBrush.m_pc_BrushShape != NULL)
   {
      delete m_BlackBrush.m_pc_BrushShape;
      m_BlackBrush.m_pc_BrushShape = NULL;
   }
   if(pc_BrushShape != NULL)
   {
      m_BlackBrush.m_pc_BrushShape = new PixelBuffer(pc_BrushShape);
   }
} // SetBrushShape()

//===========================================================================
//
//===========================================================================

PixelBuffer *BufferPainter::GetBrushShape()
{
   return m_BorderBrush.m_pc_BrushShape;
} // GetBrushShape()

//===========================================================================
//
//===========================================================================

void BufferPainter::DrawBrush(PixelBuffer *pc_Buffer,int i_X, int i_Y)
{
   int i_RunX;
   int i_RunY;
   int i_Width;
   int i_Height;

   if(m_pc_CurrentBorderBrush->m_pc_BrushShape ==NULL)
   {
      return;
   }

   i_Width  = m_BorderBrush.m_pc_BrushShape->GetWidth();
   i_Height = m_BorderBrush.m_pc_BrushShape->GetHeight();

   for(i_RunY=0; i_RunY < i_Height; i_RunY ++)
   {
      for(i_RunX=0; i_RunX < i_Width; i_RunX ++)
      {
         // is the brush set for this pixel
         if( m_BorderBrush.m_pc_BrushShape->Get(i_RunX,i_RunY) > 0 )
         {
            DrawPixel(pc_Buffer,
                      m_pc_CurrentBorderBrush,
                      i_X+i_RunX,
                      i_Y+i_RunY);

         }

      } // for RunX
   } // for RunY


   m_ChangedRegion.SetXYWH(i_X,i_Y,i_Width,i_Height);

} // DrawBrush()

//===========================================================================
//
//===========================================================================

void BufferPainter::DrawCopyBrush(PixelBuffer *pc_Buffer,int i_X, int i_Y)
{
   RectRegion r_BufferRegion;
   int        i_RunX;
   int        i_RunY;
   int        i_Width;
   int        i_Height;

   if(m_pc_CurrentBorderBrush->m_pc_BrushShape ==NULL)
   {
      return;
   }

   r_BufferRegion.Set(pc_Buffer);
   
   i_Width  = m_BorderBrush.m_pc_BrushShape->GetWidth();
   i_Height = m_BorderBrush.m_pc_BrushShape->GetHeight();

   for(i_RunY=0; i_RunY < i_Height; i_RunY ++)
   {
      for(i_RunX=0; i_RunX < i_Width; i_RunX ++)
      {
         // is the brush set for this pixel
         if( m_BorderBrush.m_pc_BrushShape->Get(i_RunX,i_RunY) > 0 )
         {
            Pixel p_CopyPixel;
            int   i_IndexX;
            int   i_IndexY;

            i_IndexX = i_X+i_RunX + m_i_CopyDifferenceX;
            i_IndexY = i_Y+i_RunY + m_i_CopyDifferenceY;

            if(r_BufferRegion.PointInRegion(i_IndexX,i_IndexY))
            {
               p_CopyPixel = pc_Buffer->Get(i_IndexX,i_IndexY);
               DrawPixel(pc_Buffer,
                         p_CopyPixel,
                         i_X+i_RunX,
                         i_Y+i_RunY);
            }
         }

      } // for RunX
   } // for RunY


   m_ChangedRegion.SetXYWH(i_X,i_Y,i_Width,i_Height);

} // DrawBrush()

//===========================================================================
//
//===========================================================================

void BufferPainter::DrawAirBrush(PixelBuffer *pc_Buffer,int i_X, int i_Y)
{
   int i_RunX;
   int i_RunY;
   int i_Width;
   int i_Height;
   int i_MiddleX;
   int i_MiddleY;
   int i_DiffX;
   int i_DiffY;
   int i_DiffTotal;
   int i_MaxDiff;
   int i_MaxDiff4;
   int i_MaxDiff9;
   int i_MaxDiff25;
   int i_Faktor;

   if(m_pc_CurrentBorderBrush->m_pc_BrushShape ==NULL)
   {
      return;
   }

   i_Width  = m_BorderBrush.m_pc_BrushShape->GetWidth();
   i_Height = m_BorderBrush.m_pc_BrushShape->GetHeight();

   i_MiddleX = i_Width  / 2;
   i_MiddleY = i_Height / 2;
   i_MaxDiff = i_MiddleX*i_MiddleX + i_MiddleY*i_MiddleY;
   i_MaxDiff4  = i_MaxDiff/ 4;
   i_MaxDiff9  = i_MaxDiff/ 9;
   i_MaxDiff25 = i_MaxDiff/ 25;
   
   for(i_RunY=0; i_RunY < i_Height; i_RunY ++)
   {
      i_DiffY = i_MiddleY - i_RunY;
      i_DiffY *= i_DiffY;
      for(i_RunX=0; i_RunX < i_Width; i_RunX ++)
      {
         i_DiffX = i_MiddleX - i_RunX;
         i_DiffX *= i_DiffX;

         i_DiffTotal = i_DiffX + i_DiffY;
         
         // is the brush set for this pixel
         if( m_BorderBrush.m_pc_BrushShape->Get(i_RunX,i_RunY) > 0 )
         {
            Pixel p_Old;
            Pixel p_NewPixel;
            bool  b_Valid ;

            i_Faktor = 98 ;
            if(i_DiffTotal < i_MaxDiff4)
            {
               i_Faktor = 97 ;
               if(i_DiffTotal < i_MaxDiff9)
               {
                  i_Faktor = 96 ;
                  if(i_DiffTotal < i_MaxDiff25)
                  {
                     i_Faktor = 95 ;
                  }
               }
            }

            p_NewPixel = m_pc_CurrentBorderBrush->GetBrushPixel(i_X+i_RunX,i_Y+i_RunY,b_Valid);

            if(b_Valid)
            {
               // make a transparent color
   
               p_Old = pc_Buffer->Get(i_X+i_RunX,i_Y+i_RunY);

               p_NewPixel = ColorCalculation::TransparentAirBrushColor(p_Old,
                                                         p_NewPixel,
                                                        i_Faktor);
               pc_Buffer->Set(i_X+i_RunX,i_Y+i_RunY,p_NewPixel);

            }  // if brush ist set
         } // if shape

      } // for RunX
   } // for RunY


   m_ChangedRegion.SetXYWH(i_X,i_Y,i_Width,i_Height);

} // DrawBrush()

//===========================================================================
//
//===========================================================================

void BufferPainter::DrawMarkedPixel(PixelBuffer *pc_Buffer,
                        PixelBuffer *pc_MarkedBuffer,
                        RectRegion  *pc_ReducedRegion)
{
   int i_RunX;
   int i_RunY;

   int i_StartX;
   int i_StartY;
   int i_EndX;
   int i_EndY;

   int i_MarkStartX;
   int i_MarkStartY;
   int i_MarkEndX;
   int i_MarkEndY;

   if(pc_ReducedRegion == NULL)
   {
      i_StartX = 0;
      i_StartY = 0;
      i_EndX   = pc_Buffer->GetWidth()-1;
      i_EndY   = pc_Buffer->GetHeight()-1;
   }
   else
   {
      i_StartX = pc_ReducedRegion->GetStartX();
      i_StartY = pc_ReducedRegion->GetStartY();
      i_EndX   = pc_ReducedRegion->GetEndX();
      i_EndY   = pc_ReducedRegion->GetEndY();
   }

   // set mark to unused
   i_MarkStartX = 1000000;
   i_MarkStartY = 1000000;
   i_MarkEndX   = -1;
   i_MarkEndY   = -1;


   for(i_RunY= i_StartY; i_RunY<= i_EndY; i_RunY ++)
   {
      for(i_RunX= i_StartX; i_RunX<= i_EndX; i_RunX ++)
      {
         if(pc_MarkedBuffer->Get(i_RunX,i_RunY))
         {
            // remember the changed region
            if(i_RunX <i_MarkStartX)
            {
               i_MarkStartX = i_RunX;
            }
            if(i_RunY <i_MarkStartY)
            {
               i_MarkStartY = i_RunY;
            }

            if(i_RunX >i_MarkEndX)
            {
               i_MarkEndX = i_RunX;
            }
            if(i_RunY >i_MarkEndY)
            {
               i_MarkEndY = i_RunY;
            }
            DrawPixel(pc_Buffer,
                      m_pc_CurrentBorderBrush,
                      i_RunX,
                      i_RunY);

         } // if marked

      } // for X
   } // for y
   if(i_MarkStartX > i_MarkEndX)
   {
      // no pixel
      m_ChangedRegion.SetEmpty();
   }
   else
   {
      m_ChangedRegion.SetXYXY(i_MarkStartX,i_MarkStartY,i_MarkEndX,i_MarkEndX);
   }

} // DrawMarkedPixel



//===========================================================================
//
//===========================================================================

void BufferPainter::SetOneTimeDraw(bool b_On)
{
   m_b_UseOneTimeDrawBuffer = b_On;

} // SetOneTimeDraw()

//===========================================================================
//
//===========================================================================

void BufferPainter::DrawEllipse(PixelBuffer *pc_Buffer,
                             int i_CenterX, int i_CenterY,
                             int i_RadiusX, int i_RadiusY)
{
   float f_Factor;
   float f_InnerFactor;
   float f_WidthX;
   float f_InnerWidthX;
   int   i_RunY;

   int   i_InnerRadiusX;
   int   i_InnerRadiusY;
   int   i_Width;
   int   i_InnerWidth;


   if(    m_i_LineWidth ==0
      && !m_b_DrawInnerPart)
   {
      // draw nothing
      m_ChangedRegion.SetEmpty();
      return;
   }


   i_InnerRadiusX = (i_RadiusX-m_i_LineWidth);
   i_InnerRadiusY = (i_RadiusY-m_i_LineWidth);

   if(    i_InnerRadiusX <=0
       || i_InnerRadiusY <=0)
   {
      // only draw width the border color
      DrawEllipseSimple(pc_Buffer,
                             m_pc_CurrentBorderBrush,
                             i_CenterX, i_CenterY,
                             i_RadiusX, i_RadiusY);
      return;

   }
   else if( m_i_LineWidth ==0)
   {
      // only draw with inner Color
      DrawEllipseSimple(pc_Buffer,
                             m_pc_CurrentInnerBrush,
                             i_CenterX, i_CenterY,
                             i_RadiusX, i_RadiusY);
      return;
   }


   // line width >0 but inner is to see
   f_Factor =   (float)  (i_RadiusX * i_RadiusX)
              / ((float) (i_RadiusY * i_RadiusY));


   f_InnerFactor =     (float) (i_InnerRadiusX * i_InnerRadiusX)
                    / ((float) (i_InnerRadiusY * i_InnerRadiusY) );

   for(i_RunY= 1;i_RunY<= i_RadiusY; i_RunY ++)
   {

      f_WidthX = sqrt (( float) (f_Factor * (  i_RadiusY * i_RadiusY
                                             - i_RunY * i_RunY)));

      if(i_RunY < i_InnerRadiusY)
      {
         f_InnerWidthX = sqrt (( float) (f_InnerFactor * (  i_InnerRadiusY * i_InnerRadiusY
                                             - i_RunY * i_RunY)));

         i_Width      = (int) (f_WidthX +0.5);
         i_InnerWidth = (int) (f_InnerWidthX +0.5);
         if(f_WidthX >0)
         {
            // draw upper lines
            DrawHorizontalLine(pc_Buffer,
                            m_pc_CurrentBorderBrush,
                            i_CenterX-i_Width,i_CenterX-i_InnerWidth-1,
                            i_CenterY+i_RunY);

            if(m_b_DrawInnerPart)
            {
               DrawHorizontalLine(pc_Buffer,
                            m_pc_CurrentInnerBrush,
                            i_CenterX-i_InnerWidth,i_CenterX+i_InnerWidth,
                            i_CenterY+i_RunY);
            }
            DrawHorizontalLine(pc_Buffer,
                            m_pc_CurrentBorderBrush,
                            i_CenterX+i_InnerWidth+1,i_CenterX+i_Width,
                            i_CenterY+i_RunY);





            // draw lower line
            DrawHorizontalLine(pc_Buffer,
                            m_pc_CurrentBorderBrush,
                            i_CenterX-i_Width,i_CenterX-i_InnerWidth-1,
                            i_CenterY-i_RunY);

            if(m_b_DrawInnerPart)
            {
               DrawHorizontalLine(pc_Buffer,
                            m_pc_CurrentInnerBrush,
                            i_CenterX-i_InnerWidth,i_CenterX+i_InnerWidth,
                            i_CenterY-i_RunY);
            }
            DrawHorizontalLine(pc_Buffer,
                            m_pc_CurrentBorderBrush,
                            i_CenterX+i_InnerWidth+1,i_CenterX+i_Width,
                            i_CenterY-i_RunY);
         }

      }
      else // inner not part is visible at this line
      {
         i_Width = (int) (f_WidthX +0.5);
         if(i_Width >0)
         {
            // draw upper line
            DrawHorizontalLine(pc_Buffer,
                            m_pc_CurrentBorderBrush,
                            i_CenterX-i_Width,i_CenterX+i_Width,
                            i_CenterY+i_RunY);

            // draw lower line
            DrawHorizontalLine(pc_Buffer,
                            m_pc_CurrentBorderBrush,
                            i_CenterX-i_Width,i_CenterX+i_Width,
                            i_CenterY-i_RunY);
         }
      }
   } // for all lines

   // now draw the middle line
   f_WidthX = sqrt (( float) (f_Factor * (  i_RadiusY * i_RadiusY)  ));

   f_InnerWidthX = sqrt (( float) (f_InnerFactor * (  i_InnerRadiusY * i_InnerRadiusY)));

   i_Width = (int) (f_WidthX +0.5);
   i_InnerWidth = (int) (f_InnerWidthX +0.5);
   DrawHorizontalLine(pc_Buffer,
                            m_pc_CurrentBorderBrush,
                            i_CenterX-i_Width,i_CenterX-i_InnerWidth-1,
                            i_CenterY);
   if(m_b_DrawInnerPart)
   {
      DrawHorizontalLine(pc_Buffer,
                            m_pc_CurrentInnerBrush,
                            i_CenterX-i_InnerWidth,i_CenterX+i_InnerWidth,
                            i_CenterY);
   }
   DrawHorizontalLine(pc_Buffer,
                            m_pc_CurrentBorderBrush,
                            i_CenterX+i_InnerWidth+1,i_CenterX+i_Width,
                            i_CenterY);




   m_ChangedRegion.SetXYXY(i_CenterX-i_RadiusX,i_CenterY-i_RadiusY,
                           i_CenterX+i_RadiusX,i_CenterY+i_RadiusY);


} // DrawEllipse

//===========================================================================
//
//===========================================================================

void BufferPainter::DrawEllipseSimple(PixelBuffer *pc_Buffer,
                                      PainterBrush *pc_Brush,
                                      int i_CenterX, int i_CenterY,
                                      int i_RadiusX, int i_RadiusY)
{
   float f_Factor;
   float f_WidthX;
   int   i_RunY;
   int   i_Width;

   f_Factor =   (float)  (i_RadiusX * i_RadiusX)
              / ((float) (i_RadiusY * i_RadiusY));

   for(i_RunY= 1;i_RunY<= i_RadiusY; i_RunY ++)
   {

      f_WidthX = sqrt (( float) (f_Factor * (  i_RadiusY * i_RadiusY
                                             - i_RunY * i_RunY)));
      if(f_WidthX >0)
      {
         i_Width = (int) (f_WidthX +0.5);
         // draw upper line
         DrawHorizontalLine(pc_Buffer,
                            pc_Brush,
                            i_CenterX-i_Width,i_CenterX+i_Width,
                            i_CenterY+i_RunY);

         // draw lower line
         DrawHorizontalLine(pc_Buffer,
                            pc_Brush,
                            i_CenterX-i_Width,i_CenterX+i_Width,
                            i_CenterY-i_RunY);

      }
   } // for all lines

   // now draw the line in the middle
   f_WidthX = sqrt (( float) (f_Factor * (  i_RadiusY * i_RadiusY)));
   if(f_WidthX >0)
   {
      i_Width = (int) (f_WidthX +0.5);
      DrawHorizontalLine(pc_Buffer,
                            pc_Brush,
                            i_CenterX-i_Width,i_CenterX+i_Width,
                            i_CenterY);
   }


   m_ChangedRegion.SetXYXY(i_CenterX-i_RadiusX,i_CenterY-i_RadiusY,
                           i_CenterX+i_RadiusX,i_CenterY+i_RadiusY);


} // DrawEllipseSimple

//===========================================================================
// version with floating point
//===========================================================================

void BufferPainter::DrawEllipseSimple(PixelBuffer *pc_Buffer,
                                      PainterBrush *pc_Brush,
                                      float f_CenterX, float f_CenterY,
                                      float f_RadiusX, float f_RadiusY)
{
   float f_Factor;
   float f_WidthX;
   int   i_RunY;
   int   i_Width;

   f_Factor =   (float)  (f_RadiusX * f_RadiusX)
              / ((float) (f_RadiusY * f_RadiusY));

   for(i_RunY= 1;i_RunY<= f_RadiusY; i_RunY ++)
   {

      f_WidthX = sqrt (( float) (f_Factor * (  f_RadiusY * f_RadiusY
                                             - i_RunY * i_RunY)));
      if(f_WidthX >0)
      {
         i_Width = (int) (f_WidthX );
         // draw upper line
         DrawHorizontalLine(pc_Buffer,
                            pc_Brush,
                            (int)(f_CenterX-i_Width),
                            (int)(f_CenterX+i_Width),
                            (int) (f_CenterY+i_RunY));

         // draw lower line
         DrawHorizontalLine(pc_Buffer,
                            pc_Brush,
                            (int)(f_CenterX-i_Width),
                            (int)(f_CenterX+i_Width),
                            (int)(f_CenterY-i_RunY));

      }
   } // for all lines

   // now draw the line in the middle
   f_WidthX = sqrt (( float) (f_Factor * (  f_RadiusY * f_RadiusY)));
   if(f_WidthX >0)
   {
      i_Width = (int) (f_WidthX );
      DrawHorizontalLine(pc_Buffer,
                            pc_Brush,
                            (int)(f_CenterX-i_Width),
                            (int)(f_CenterX+i_Width),
                            (int)(f_CenterY));
   }


   m_ChangedRegion.SetXYXY((int)f_CenterX-f_RadiusX,(int)f_CenterY-f_RadiusY,
                           (int)f_CenterX+f_RadiusX,(int)f_CenterY+f_RadiusY);


} // DrawEllipseSimple


//===========================================================================
//
//===========================================================================

void BufferPainter::DrawFreeFigure(PixelBuffer *pc_Buffer,
                       PMemVector<PixelCoordinate> *pc_List,
                       bool b_ClosedFigure)
{
   PixelCoordinate c_Last;
   PixelCoordinate c_Actual;
   RectRegion      c_SingleRegion;
   RectRegion      c_TotalRegion;
   int             i_Size;
   int             i;
   int             i_StartCoordinateIndex;

   bool b_OneTimeDrawSave;

   // save an eventual active buffer
   b_OneTimeDrawSave =  m_b_UseOneTimeDrawBuffer;

   m_b_UseOneTimeDrawBuffer = true;
   pc_Buffer->CreateUsedPixelMask();
   pc_Buffer->UsedPixelMaskClean();


   i_Size = pc_List->GetCount();


   c_TotalRegion.SetEmpty();

   if(b_ClosedFigure)
   {
      i_StartCoordinateIndex = 0;
      c_Last = pc_List->Get(i_Size-1);
   }
   else
   {
      i_StartCoordinateIndex = 1 ;
      c_Last = pc_List->Get(0);
   }

   if(m_i_LineWidth>0)
   {

      // draw the first starting point
      DrawEllipseSimple(pc_Buffer,
             m_pc_CurrentBorderBrush,
             (float) c_Last.m_i_X  ,(float) c_Last.m_i_Y,
             (float)(m_i_LineWidth-0.5)/2.0 ,(float) (m_i_LineWidth-0.5)/2.0);

      c_SingleRegion = GetChangedRegion();

      c_TotalRegion.AddRegion(c_SingleRegion);

      
      for (i=i_StartCoordinateIndex;i<i_Size;i++)
      {
         int i_DiffX;
         int i_DiffY;
         // get two coordinates
         c_Actual = pc_List->Get(i);

// fprintf(stderr,"%d draw %d %d %d %d\n",i,c_Last.m_i_X  ,c_Last.m_i_Y, c_Actual.m_i_X,c_Actual.m_i_Y);

         i_DiffX= c_Last.m_i_X - c_Actual.m_i_X;
         if(i_DiffX <0)
         {
            i_DiffX = - i_DiffX;
         }
         i_DiffY= c_Last.m_i_Y - c_Actual.m_i_Y;
         if(i_DiffY <0)
         {
            i_DiffY = - i_DiffY;
         }

         // only draw a line, if the differences are large enough
         if(i_DiffX>1 || i_DiffY>1)
         {
            DrawLine(pc_Buffer,
                   c_Last.m_i_X  ,c_Last.m_i_Y,
                   c_Actual.m_i_X,c_Actual.m_i_Y);
         }

         DrawEllipseSimple(
                pc_Buffer,
                m_pc_CurrentBorderBrush,
                (float) c_Actual.m_i_X  ,(float) c_Actual.m_i_Y,
                (float)(m_i_LineWidth-0.5)/2.0 ,(float) (m_i_LineWidth-0.5)/2.0);

         c_SingleRegion = GetChangedRegion();

         c_TotalRegion.AddRegion(c_SingleRegion);

         c_Last = c_Actual;
      } // for all points
   } // if linewidth > 0

   m_ChangedRegion = c_TotalRegion;

   // now fill the inner part
   // can only draw inner, if figure is closed
   if(   b_ClosedFigure
      && m_b_DrawInnerPart)
   {
      BufferPainterEdges * pc_Edges;
      RectRegion           c_InnerRegion;
      int                  i_StartLine;
      int                  i_EndLine;
      int                  i_LastDirection;


      // calculate the maxiumum extension
      CalculateStartEndLine(pc_List,
                            i_StartLine,
                            i_EndLine);
      // create a structure for the edges
      pc_Edges = new BufferPainterEdges(i_StartLine,i_EndLine);

      // now draw all the line
      c_Last = pc_List->Get(0);

      i_LastDirection = CalculateInitialLastDirection(pc_List);

      for(i=1;i<i_Size;i++)
      {
         c_Actual = pc_List->Get(i);
         // draw line from last to actual
         CalculateEdgesOfLine (pc_Edges,
                    c_Last,
                    c_Actual,
                    i_LastDirection);

         c_Last = c_Actual;
      }

      // final connect the last point to the first one
      c_Actual = pc_List->Get(0);
      CalculateEdgesOfLine (pc_Edges,
                    c_Last,
                    c_Actual,
                    i_LastDirection);

      // now draw the inner figure
      DrawEdgesField(pc_Buffer,
                     m_pc_CurrentInnerBrush,
                     pc_Edges,
                     &c_InnerRegion);

      // after drawing delete the edges field
      delete pc_Edges;
      m_ChangedRegion.AddRegion(c_InnerRegion);
   } // if draw inner part

   // restore the original buffer
   //m_pc_OneTimeDrawBuffer   = pc_OneTimeDrawBufferSave;
   m_b_UseOneTimeDrawBuffer = b_OneTimeDrawSave ;


} // DrawFreeFigure()

//===========================================================================
//
//===========================================================================
void BufferPainter::CalculateStartEndLine(GeoCoordinateList *pc_List,
                 int &i_StartLine,
                 int &i_EndLine)
{
   GeoCoordinate c_Actual;
   int           i;
   int           i_Size;

   i_Size = pc_List->GetCount();

   if(i_Size <=0)
   {
      i_StartLine = -1;
      i_EndLine   = -1;
      return;
   }

   c_Actual = pc_List->Get(0);

   i_StartLine = (int) c_Actual.GetY();
   i_EndLine   = (int) c_Actual.GetY();

   for(i=1;i<i_Size ; i++)
   {
      int i_ActualY;
      
      c_Actual = pc_List->Get(i);
      i_ActualY =  (int) c_Actual.GetY();
      if(i_StartLine > i_ActualY)
      {
         i_StartLine = i_ActualY;
      }

      if(i_EndLine < i_ActualY)
      {
         i_EndLine = i_ActualY;
      }

   } // for i

} // CalculateStartEndLine()

void BufferPainter::CalculateStartEndLine(PMemVector<PixelCoordinate> *pc_List,
                 int &i_StartLine,
                 int &i_EndLine)
{
   PixelCoordinate c_Actual;
   int             i;
   int             i_Size;

   i_Size = pc_List->GetCount();

   if(i_Size <=0)
   {
      i_StartLine = -1;
      i_EndLine   = -1;
      return;
   }

   c_Actual = pc_List->Get(0);

   i_StartLine = c_Actual.m_i_Y;
   i_EndLine   = c_Actual.m_i_Y;

   for(i=1;i<i_Size ; i++)
   {
      c_Actual = pc_List->Get(i);

      if(i_StartLine > c_Actual.m_i_Y)
      {
         i_StartLine = c_Actual.m_i_Y;
      }

      if(i_EndLine < c_Actual.m_i_Y)
      {
         i_EndLine = c_Actual.m_i_Y;
      }

   } // for i
}// CalculateStartEndLine()

//===========================================================================
//
//===========================================================================

void BufferPainter::CalculateEdgesOfLine (BufferPainterEdges * pc_Edges,
                    GeoCoordinate c_From,
                    GeoCoordinate c_To,
                    int &LastDirection)
{
   PixelCoordinate p_From;
   PixelCoordinate p_To;

   p_From.m_i_X = (int) c_From.GetX();
   p_From.m_i_Y = (int) c_From.GetY();
   p_To.m_i_X   = (int) c_To.GetX();
   p_To.m_i_Y   = (int) c_To.GetY();

   CalculateEdgesOfLine(pc_Edges,p_From,p_To,LastDirection);
   
} // CalculateEdgesOfLine()

//===========================================================================
//
//===========================================================================


void BufferPainter::CalculateEdgesOfLine (BufferPainterEdges * pc_Edges,
                    PixelCoordinate c_From,
                    PixelCoordinate c_To,
                    int &i_LastDirection)
{
   int i_DiffX;
   int i_DiffY;
   int i_AbsolutDiffY;
   int i_BufferYIncrement;
   int i_YPosition;
   int i_XPosition;
   int i_StartIndex;
   int i_MaximumDifference;
   int i_LastYPosition;
   int i;


   /*
   fprintf(stderr,"Caluclate Edges of (%d/%d) - (%d/%d)\n",
        c_To.m_i_X,
        c_To.m_i_Y,
        c_From.m_i_X,
        c_From.m_i_Y);
   */
   
   i_DiffX = c_To.m_i_X  - c_From.m_i_X;
   i_DiffY = c_To.m_i_Y  - c_From.m_i_Y;
   if(i_DiffY == 0)
   {
      // horizontal line, do nothing
   }
   else
   {
      i_AbsolutDiffY = abs (i_DiffY);
      if (i_DiffY > 0)
      {
         i_BufferYIncrement = 1;
      }
      else
      {
         i_BufferYIncrement = -1;
      }
      i_YPosition = c_From.m_i_Y;

      // fprintf(stderr,"iLast = %d , current =%d\n",i_LastDirection,i_DiffY);

      if ((i_LastDirection < 0 && i_DiffY >= 0) ||
          (i_LastDirection > 0 && i_DiffY <= 0))
      {
         i_StartIndex = 0;
      }
      else
      {
         i_StartIndex = 1;
         i_YPosition += i_BufferYIncrement;


      }

      i_MaximumDifference = i_AbsolutDiffY;
      if (abs (i_DiffX) > i_MaximumDifference)
      {
         i_MaximumDifference = abs (i_DiffX);
      }

      i_LastYPosition = i_YPosition - i_BufferYIncrement;

      for (i=i_StartIndex ; i <= i_MaximumDifference; i++)
      {
         i_XPosition  = c_From.m_i_X + i_DiffX * i / i_MaximumDifference;
         i_YPosition  = c_From.m_i_Y + i_DiffY * i / i_MaximumDifference;
         if (   (i_BufferYIncrement > 0 && i_YPosition > i_LastYPosition)
             || (i_BufferYIncrement < 0 && i_YPosition < i_LastYPosition)
             )
         {
            pc_Edges->AddLinePoint (i_XPosition,i_YPosition);
            i_LastYPosition = i_YPosition;
         }

      } // for i

      i_LastDirection = i_DiffY;

   } // else not horizontal

} // CalculateEdgesOfLine();

//===========================================================================
//
//===========================================================================
int BufferPainter::CalculateInitialLastDirection(GeoCoordinateList *pc_List)
{
   GeoCoordinate c_First;
   GeoCoordinate c_Last;
   int           i_LastDirection;
   int           i_LastIndex;

   i_LastIndex = pc_List->GetCount()-1;

   c_First = pc_List->Get(0);
   c_Last  = pc_List->Get(i_LastIndex);

   i_LastDirection = (int)c_First.GetY() - (int) c_Last.GetY();

   // do we have a horizonalt line at the end??
   // then go back, until a direction is found
   while (   i_LastDirection == 0
          && i_LastIndex > 0)
   {
      i_LastIndex --;
      c_Last  = pc_List->Get(i_LastIndex);


      i_LastDirection   = (int) c_First.GetY() - (int) c_Last.GetY();

   }
   return i_LastDirection;

} // CalculateInitialLastDirection()


int BufferPainter::CalculateInitialLastDirection(PMemVector<PixelCoordinate> *pc_List)
{
   PixelCoordinate c_First;
   PixelCoordinate c_Last;
   int             i_LastDirection;
   int             i_LastIndex;


   i_LastIndex = pc_List->GetCount()-1;

   c_First = pc_List->Get(0);
   c_Last  = pc_List->Get(i_LastIndex);

   /*   
   {
      int i;
      for(i=0;i<=i_LastIndex;i++)
      {
         PixelCoordinate p;
         p  = pc_List->Get(i);

         fprintf(stderr,"%d is %d %d\n",i,p.m_i_X,p.m_i_Y);
      }
   }
   */
         
   i_LastDirection = c_First.m_i_Y - c_Last.m_i_Y;

   // do we have a horizonalt line at the end??
   // then go back, until a direction is found
   while (   i_LastDirection == 0
          && i_LastIndex > 0)
   {
      i_LastIndex --;
      c_Last  = pc_List->Get(i_LastIndex);


      i_LastDirection = c_First.m_i_Y - c_Last.m_i_Y;

   }

   return i_LastDirection;

} // CalculateInitialLastDirection()

//===========================================================================
//
//===========================================================================

void BufferPainter::DrawEdgesField(PixelBuffer *pc_Buffer,
                 PainterBrush *pc_Brush,
                 BufferPainterEdges * pc_Edges,
                 RectRegion *pc_ChangedRegion)
{
   int i_ActualLine;
   int i_StartLine;
   int i_EndLine;

   if(pc_ChangedRegion!= NULL)
   {
      pc_ChangedRegion->SetEmpty();
   }
   
   i_StartLine = pc_Edges->GetStartLine();
   i_EndLine   = pc_Edges->GetEndLine();

// fprintf(stderr,"DrawEdgesField %d %d\n",i_StartLine,i_EndLine);
   for(i_ActualLine = i_StartLine;
       i_ActualLine <= i_EndLine;
       i_ActualLine++)
   {
      BufferPainterEdgesLine * pc_Line;

      pc_Line = pc_Edges->GetLine(i_ActualLine);
      if(pc_Line != NULL)
      {
         int i;
         int i_Number;
         i_Number = pc_Line->GetNumber();
//fprintf(stderr,"%d edges in line %d\n",i_Number,i_ActualLine);

         for(i=0;i+1<i_Number; i+= 2)
         {
            int i_From;
            int i_To;
            i_From = pc_Line->GetPoint(i);
            i_To   = pc_Line->GetPoint(i+1);

            if(pc_ChangedRegion != NULL)
            {
               pc_ChangedRegion->AddPoint(i_From,i_ActualLine);
               pc_ChangedRegion->AddPoint(i_To  ,i_ActualLine);
            }               

            DrawHorizontalLine(pc_Buffer,
                                pc_Brush,
                                i_From,
                                i_To,
                                i_ActualLine);

         } // for i

      } // if Line != NULL

   } // for i_ActualLine

} // DrawEdgesField

//===========================================================================
//
//===========================================================================

void BufferPainter::SetMaskMode(bool b_EnableMode,bool b_White)
{
   if(b_EnableMode)
   {
      if(b_White)
      {
         m_pc_CurrentBorderBrush = &m_WhiteBrush;
         m_pc_CurrentInnerBrush  = &m_WhiteBrush;
      }
      else
      {
         m_pc_CurrentBorderBrush = &m_BlackBrush;
         m_pc_CurrentInnerBrush  = &m_BlackBrush;
      }
      m_b_MaskModeEnabled = true;
   }
   else
   {
      m_pc_CurrentBorderBrush = &m_BorderBrush;
      m_pc_CurrentInnerBrush  = &m_InnerBrush;
      m_b_MaskModeEnabled = false;
   }
} // SetMaskMode()

//===========================================================================
//
//===========================================================================

void BufferPainter::SetRecolorMode(bool b_EnableMode)
{
   m_BorderBrush.m_b_Recolor = b_EnableMode;
} // SetRecolorMode()
//===========================================================================
//
//===========================================================================

void BufferPainter::ResetBrushModes()
{
   m_BorderBrush.m_b_Recolor    = false;
   m_BorderBrush.m_b_Lightness  = false;
   m_BorderBrush.m_b_Saturation = false;
   m_BorderBrush.m_b_Filter     = false;

} // ResetBrushModes()

//===========================================================================
//
//===========================================================================

void BufferPainter::SetLightnessMode(bool b_EnableMode)
{
   m_BorderBrush.m_b_Lightness = b_EnableMode;
} // SetLightnessMode()

//===========================================================================
//
//===========================================================================

void BufferPainter::SetLightnessFactor(int i_Factor)
{
   m_BorderBrush.m_i_LightnessFactor = i_Factor;
} //SetLightnessFactor()

//===========================================================================
//
//===========================================================================

void BufferPainter::SetSaturationMode(bool b_EnableMode)
{
   m_BorderBrush.m_b_Saturation = b_EnableMode;
} // SetLightnessMode()

//===========================================================================
//
//===========================================================================

void BufferPainter::SetFilterMode(bool b_EnableMode)
{
   m_BorderBrush.m_b_Filter = b_EnableMode;
} // SetLightnessMode()

//===========================================================================
//
//===========================================================================

void BufferPainter::SetFilterWidth(int i_Width)
{
   m_i_FilterWidth = i_Width;
} // SetFilterWidth()

//===========================================================================
//
//===========================================================================

void BufferPainter::SetFilterNearDistanceActive(bool b_Active)
{
   m_b_FilterNearDistanceActive = b_Active;
} // SetFilterNearDistanceActive()

//===========================================================================
//
//===========================================================================

void BufferPainter::SetFilterNearDistance(int i_Distance)
{
   m_i_FilterNearDistance = i_Distance;
} // SetFilterNearDistance()

//===========================================================================
//
//===========================================================================

void BufferPainter::SetFilterAlgorithm(int i_Algorithm)
{
   m_i_FilterAlgorithm = i_Algorithm;
} // SetFilterAlgorithm()

//===========================================================================
//
//===========================================================================

void BufferPainter::SetInnerMotiveLayer(GraphicObjectLayer *pc_MotivLayer)
{
   m_InnerBrush.SetMotiveLayer(pc_MotivLayer);
}

//===========================================================================
//
//===========================================================================

void BufferPainter::SetBorderMotiveLayer(GraphicObjectLayer *pc_MotivLayer)
{
   m_BorderBrush.SetMotiveLayer(pc_MotivLayer);
}

//===========================================================================
//
//===========================================================================


void BufferPainter::SetInnerPattern(PixelBuffer * pc_Buffer)
{
   m_InnerBrush.SetPattern(pc_Buffer);
}

//===========================================================================
//
//===========================================================================

void BufferPainter::SetBorderPattern(PixelBuffer * pc_Buffer)
{
   m_BorderBrush.SetPattern(pc_Buffer);
}

//===========================================================================
//
//===========================================================================

void BufferPainter::SetInnerDrawMode(BrushMode i_BrushMode)
{
   m_InnerBrush.m_i_BrushMode = i_BrushMode;
}

//===========================================================================
//
//===========================================================================

void BufferPainter::SetBorderDrawMode(BrushMode i_BrushMode)
{
   m_BorderBrush.m_i_BrushMode = i_BrushMode;
}

//===========================================================================
//
//===========================================================================

void BufferPainter::SetSpeckleParameter(int i_SpeckleSize,
                                        int i_SpeckleDensity)
{
   m_i_SpeckleSize    = i_SpeckleSize;   
   m_i_SpeckleDensity = i_SpeckleDensity;
   if(m_i_SpeckleDensity <=0)
   {
      m_i_SpeckleDensity = 1;
   }
   
} // SetSpeckleParameter()

//===========================================================================
//
//===========================================================================

void BufferPainter::SetSmearParameter(int i_SmearStrength)
{
   m_i_SmearStrength    = i_SmearStrength;

} // SetSmearParameter()

//===========================================================================
//
//===========================================================================

void BufferPainter::DrawSpeckle(PixelBuffer *pc_Buffer, int i_X, int i_Y)
{
   int i_RunX;
   int i_RunY;
   int i_Width;
   int i_Height;

   if(m_pc_CurrentBorderBrush->m_pc_BrushShape ==NULL)
   {
      return;
   }

   i_Width  = m_BorderBrush.m_pc_BrushShape->GetWidth();
   i_Height = m_BorderBrush.m_pc_BrushShape->GetHeight();

   for(i_RunY=0; i_RunY < i_Height; i_RunY ++)
   {
      for(i_RunX=0; i_RunX < i_Width; i_RunX ++)
      {
         // is the brush set for this pixel
         if( m_BorderBrush.m_pc_BrushShape->Get(i_RunX,i_RunY) > 0 )
         {
            // now check, whether the densitity factor is set;
            int i_Random;
            int i_Size;
            int i_SizeHalf;
            int i_SizeSquare;
            int i_SpeckleX;
            int i_SpeckleY;

            // get random for the density
            i_Random = Utilities::Random( m_i_SpeckleDensity);
            if(i_Random != 0)
            {
               continue;
            }

            // get a random size
            i_Size       = Utilities::Random( m_i_SpeckleSize) +1;
            i_SizeHalf   = (i_Size /2)+1;
            i_SizeSquare = i_SizeHalf * i_SizeHalf;

            // check, whether it is complete inside !
            if( i_RunX - i_SizeHalf <0)
            {
               continue;
            }
            if( i_RunX + i_SizeHalf >= i_Width)
            {
               continue;
            }
            if( i_RunY - i_SizeHalf <0)
            {
               continue;
            }
            if( i_RunY + i_SizeHalf >= i_Height)
            {
               continue;
            }


            // now draw all the specklepoints
            for(i_SpeckleY = -i_SizeHalf;
                i_SpeckleY <= i_SizeHalf;
                i_SpeckleY ++)
            {
               for(i_SpeckleX = -i_SizeHalf;
                   i_SpeckleX <= i_SizeHalf;
                   i_SpeckleX ++)
               {
                  int i_Distance;
                  int i_DrawX;
                  int i_DrawY;
                  i_Distance =   i_SpeckleX*i_SpeckleX
                               + i_SpeckleY*i_SpeckleY;
                  if(i_Distance > i_SizeSquare)
                  {
                     continue;
                  }
                  i_DrawX = i_SpeckleX + i_RunX;
                  i_DrawY = i_SpeckleY + i_RunY;

                  if(   i_DrawX < 0
                     || i_DrawX >= i_Width)
                  {
                     continue;
                  }
                  if(   i_DrawY < 0
                     || i_DrawY >= i_Height)
                  {
                     continue;
                  }

                  DrawPixel(pc_Buffer,
                            m_pc_CurrentBorderBrush,
                            i_X+i_DrawX,
                            i_Y+i_DrawY);
                               
               } // for speckleX
            
            } // for speckleY
            
         } // if brush is set

      } // for RunX
   } // for RunY


   m_ChangedRegion.SetXYWH(i_X,i_Y,i_Width,i_Height);
} // DrawSpeckle()

//===========================================================================
//
//===========================================================================


void BufferPainter::DrawSmear(PixelBuffer *pc_Buffer,
                  int i_X,
                  int i_Y,
                  int i_VectorX,
                  int i_VectorY)
{
   RectRegion    r_DestinationRegion;
   float         f_VectorLength;
   float         f_RandomLength;
   int           i_Width;
   int           i_Height;
   int           i_RunX;
   int           i_RunY;
   int           i_PointImageX;
   int           i_PointImageY;
   int           i_PointRandomX;
   int           i_PointRandomY;
   int           i_InverseSmearStrength;

   i_InverseSmearStrength = 100 - m_i_SmearStrength;
   
   // create a buffer hold hold to new modified content
   // it has the same this as the brush. initial also copy the contents
   if( m_BorderBrush.m_pc_BrushShape == NULL)
   {
      return;
   }
         
   i_Width  = m_BorderBrush.m_pc_BrushShape->GetWidth();
   i_Height = m_BorderBrush.m_pc_BrushShape->GetHeight();

   if(m_pc_SmearCopy == NULL)
   {
      // buffer does not exist
       m_pc_SmearCopy = new PixelBuffer(i_Width,i_Height,pc_Buffer->GetDepth());
   }
   else if(     m_pc_SmearCopy->GetWidth() != i_Width
            ||  m_pc_SmearCopy->GetHeight() != i_Height  )
   {
      // buffer has wrong size
      
      delete m_pc_SmearCopy;
      m_pc_SmearCopy = new PixelBuffer(i_Width,i_Height,pc_Buffer->GetDepth());
   }
   
   

   r_DestinationRegion = pc_Buffer->GetRegion();

   f_VectorLength=Utilities::Hypothenusis(i_VectorX,i_VectorY); 

   for (i_RunY=0; i_RunY<i_Height;i_RunY ++)
   {
      for (i_RunX=0; i_RunX<i_Width;i_RunX ++)
      {
         Pixel p_Original;
         Pixel p_Random;
         Pixel p_New;
         int   i_TransparencyFactor;
         // work only on selected points
         if( m_BorderBrush.m_pc_BrushShape->Get(i_RunX,i_RunY) == 0)
         {
           continue;
         }
         // melt this point with a point back of the smear direction

         // calc original point
         i_PointImageX = i_RunX + i_X;
         i_PointImageY = i_RunY + i_Y;
         if( ! r_DestinationRegion.PointInRegion(i_PointImageX,i_PointImageY))
         {
            // destination outside -> disable this point

            continue;
            
         } // point outside

         p_Original = pc_Buffer->Get(i_PointImageX,i_PointImageY);

         // Get a random back point in direction of the vektor
         f_RandomLength = Utilities::Random((float) 5.0);
         f_RandomLength /= f_VectorLength;

         i_PointRandomX = (int) (i_VectorX* f_RandomLength);
         i_PointRandomY = (int) (i_VectorY* f_RandomLength);

         i_PointRandomX +=  i_RunX + i_X;
         i_PointRandomY +=  i_RunY + i_Y;

         if(! r_DestinationRegion.PointInRegion(i_PointRandomX,i_PointRandomY))
         {
            // random point outside, keep the original
            m_pc_SmearCopy->Set(i_RunX,i_RunY,p_Original);
            continue;
         }

         p_Random   = pc_Buffer->Get(i_PointRandomX,i_PointRandomY);
         
         // combine both points
         i_TransparencyFactor = Utilities::Random((int) m_i_SmearStrength) +i_InverseSmearStrength;
         p_New = ColorCalculation::TransparentColor(p_Original,
                                                    p_Random,
                                                    i_TransparencyFactor);
         m_pc_SmearCopy->Set(i_RunX,i_RunY,p_New);
      } // for RunX

   } // for RunY

   // now in a second run copy the new created buffer back into the image
   for (i_RunY=0; i_RunY<i_Height;i_RunY ++)
   {
      for (i_RunX=0; i_RunX<i_Width;i_RunX ++)
      {
         Pixel p_Point;
         // work only on selected points
         if(  m_BorderBrush.m_pc_BrushShape->Get(i_RunX,i_RunY) == 0 )
         {
             continue;
         }
         i_PointImageX = i_RunX + i_X;
         i_PointImageY = i_RunY + i_Y;

         if( ! r_DestinationRegion.PointInRegion(i_PointImageX,i_PointImageY))
         {
            // destination outside -> disable this point

            continue;

         } // point outside
         p_Point = m_pc_SmearCopy->Get(i_RunX,i_RunY);
         pc_Buffer->Set(i_PointImageX,i_PointImageY,p_Point);

      } // for RunX

   } // for RunY


   // set full area as changed region
   m_ChangedRegion.SetXYWH(i_X,i_Y,i_Width,i_Height);
} // DrawSmear()

//===========================================================================
//
//===========================================================================

void BufferPainter::SetCopyBrushParameter(int i_DifferenceX,
                                          int i_DifferenceY)
{
   m_i_CopyDifferenceX = i_DifferenceX;
   m_i_CopyDifferenceY = i_DifferenceY;
} // SetCopyBrushParameter()

//===========================================================================
//
//===========================================================================


void BufferPainter::DrawSpline(PixelBuffer *pc_Buffer,
                       GeoCoordinateList *pc_InputList,
                       bool b_ClosedFigure)
{
   GeoCoordinateList   * pc_InterpolationList;
   GeoCoordinateList   * pc_ResultList;
   CubicBezierSpline     c_Spline;
   GeoCoordinate         c_Last;
   GeoCoordinate         c_Actual;
   RectRegion            c_SingleRegion;
   RectRegion            c_TotalRegion;
   bool                  b_OneTimeDrawSave;
   int                   i_Size;
   int                   i_Line;
   int                   i_Count;
   int                   i_InterpolationCount;
   int                   i;
   int                   i_StartIndex;

   // first create a list of border points out of the point list

    // calculate the interpolation points
   pc_InterpolationList = CubicBezierSpline::CalculateInterpolationList(
                                                                 pc_InputList,
                                                                 0.0,
                                                                 0.0,
                                                                 !b_ClosedFigure);
   i_InterpolationCount = pc_InterpolationList->GetCount();
   i_Count = pc_InputList->GetCount();
   
   // draw a spline between all the sub-points
   for(i_Line=0;i_Line < i_InterpolationCount/3; i_Line++)
   {
      GeoCoordinate  c_Interpolation[4];
      int            j;
      int            i_Get;
      // get the for interpolation points for a single curve
      for(j=0;j<4;j++)
      {
         i_Get = j+i_Line*3;

         // at the end start at the beginning
         if(i_Get>=i_InterpolationCount)
         {
            i_Get = i_Get % i_InterpolationCount ;
         }

         c_Interpolation[j] = pc_InterpolationList->Get(i_Get);

      }  // for j
      if(i_Line==0)
      {
         c_Spline.StartCalculationSplinePoints(c_Interpolation,false);
      }
      else
      {
         c_Spline.StartCalculationSplinePoints(c_Interpolation,true);
      }
      


   } // for all points

   // free the list with interpolationPoints
   delete pc_InterpolationList;
   pc_ResultList = c_Spline.GetResultList();

  
   // save an eventual active buffer
   b_OneTimeDrawSave =  m_b_UseOneTimeDrawBuffer;

   m_b_UseOneTimeDrawBuffer = true;
   pc_Buffer->CreateUsedPixelMask();
   pc_Buffer->UsedPixelMaskClean();

   
   i_Size = pc_ResultList->GetCount();

   if(b_ClosedFigure)
   {
      i_StartIndex =0;
      c_Last = pc_ResultList->Get(i_Size-1);
   }
   else
   {
      i_StartIndex =1;
      c_Last = pc_ResultList->Get(0);
   }
   
   c_TotalRegion.SetEmpty();

   if(m_i_LineWidth>0)
   {

      // draw the first starting point
      DrawEllipseSimple(pc_Buffer,
             m_pc_CurrentBorderBrush,
             (float) c_Last.GetX()  ,(float) c_Last.GetY(),
             (float)(m_i_LineWidth-0.5)/2.0 ,(float) (m_i_LineWidth-0.5)/2.0);

      c_SingleRegion = GetChangedRegion();

      c_TotalRegion.AddRegion(c_SingleRegion);
      for (i=i_StartIndex;i<i_Size;i++)
      {
         int i_DiffX;
         int i_DiffY;
         // get two coordinates
         c_Actual = pc_ResultList->Get(i);

// fprintf(stderr,"%d draw %d %d %d %d\n",i,c_Last.m_i_X  ,c_Last.m_i_Y, c_Actual.m_i_X,c_Actual.m_i_Y);

         i_DiffX= (int) c_Last.GetX() - (int)c_Actual.GetX();
         if(i_DiffX <0)
         {
            i_DiffX = - i_DiffX;
         }
         i_DiffY= (int)c_Last.GetY() - (int)c_Actual.GetY();
         if(i_DiffY <0)
         {
            i_DiffY = - i_DiffY;
         }

         // only draw a line, if the differences are large enough
         if(i_DiffX>1 || i_DiffY>1)
         {
            DrawLine(pc_Buffer,
                   (int)c_Last.GetX()  ,(int)c_Last.GetY(),
                   (int)c_Actual.GetX(),(int)c_Actual.GetY());
         }
         DrawEllipseSimple(
                pc_Buffer,
                m_pc_CurrentBorderBrush,
                (float) c_Actual.GetX()  ,(float) c_Actual.GetY(),
                (float)(m_i_LineWidth-0.5)/2.0 ,(float) (m_i_LineWidth-0.5)/2.0);

         c_SingleRegion = GetChangedRegion();

         c_TotalRegion.AddRegion(c_SingleRegion);

         c_Last = c_Actual;
      } // for all points
   } // if linewidth > 0

   m_ChangedRegion = c_TotalRegion;

   // now fill the inner part
   // but only for closed figures
   if(   m_b_DrawInnerPart
      && b_ClosedFigure )
   {
      BufferPainterEdges * pc_Edges;
      RectRegion           c_InnerRegion;
      int                  i_StartLine;
      int                  i_EndLine;
      int                  i_LastDirection;


      // calculate the maxiumum extension
      CalculateStartEndLine(pc_ResultList,
                            i_StartLine,
                            i_EndLine);
      // create a structure for the edges
      pc_Edges = new BufferPainterEdges(i_StartLine,i_EndLine);

      // now draw all the line
      c_Last = pc_ResultList->Get(0);

      i_LastDirection = CalculateInitialLastDirection(pc_ResultList);

      for(i=1;i<i_Size;i++)
      {
         c_Actual = pc_ResultList->Get(i);
         // draw line from last to actual
         CalculateEdgesOfLine (pc_Edges,
                    c_Last,
                    c_Actual,
                    i_LastDirection);

         c_Last = c_Actual;
      }

      // connect the last with the first point
      c_Actual = pc_ResultList->Get(0);
      CalculateEdgesOfLine (pc_Edges,
                    c_Last,
                    c_Actual,
                    i_LastDirection);


      // now draw the inner figure
      DrawEdgesField(pc_Buffer,
                     m_pc_CurrentInnerBrush,
                     pc_Edges,
                     &c_InnerRegion);

      // after drawing delete the edges field
      delete pc_Edges;
      m_ChangedRegion.AddRegion(c_InnerRegion);
   } // if draw inner part

   // restore the original buffer
   //m_pc_OneTimeDrawBuffer   = pc_OneTimeDrawBufferSave;
   m_b_UseOneTimeDrawBuffer = b_OneTimeDrawSave ;

} // DrawFreeSpline()

//===========================================================================
//
//===========================================================================

void BufferPainter::SetRecolorParameter(Pixel p_From,
                                        Pixel p_To,
                                        int i_Distance)
{
   m_p_RecolorFromColor = p_From;
   m_p_RecolorToColor   = p_To;
   m_i_RecolorDistance  = i_Distance;

   ColorCalculation::PixelToHLS(m_p_RecolorFromColor,
         m_i_RecolorFromH,
         m_i_RecolorFromL,
         m_i_RecolorFromS
         );

   ColorCalculation::PixelToHLS(m_p_RecolorToColor ,
         m_i_RecolorToH,
         m_i_RecolorToL,
         m_i_RecolorToS
         );

} // SetRecolorParameter()

//===========================================================================
//
//===========================================================================

Pixel BufferPainter::CalculateRecolor(Pixel p_Old)
{
   Pixel p_New;
   int i_OldH;
   int i_OldL;
   int i_OldS;
   int i_Difference;
   int i_DiffH;
   int i_DiffL;
   int i_DiffS;
   int i_NewH;
   int i_NewL;
   int i_NewS;
   int i_L_Percent;
   int i_S_Percent;

   // get HLS representation of old color
   ColorCalculation::PixelToHLS(p_Old ,i_OldH,i_OldL,i_OldS);

   // first check, whether old Color is in range to original color
   i_Difference = i_OldH - m_i_RecolorFromH;
   i_Difference = abs(i_Difference);
   if(i_Difference >180)
   {
      i_Difference = 360 - i_Difference;
   }

   // i_Difference should be between 0 and 180.
   if(i_Difference > m_i_RecolorDistance)
   {   
      // differnce to large, dont change
      return p_Old;
   }

   // calculate differences old to from color
   i_DiffH= i_OldH - m_i_RecolorFromH;
   i_DiffL= i_OldL - m_i_RecolorFromL;
   i_DiffS= i_OldS - m_i_RecolorFromS;

   // get a precentage change value
   if( m_i_RecolorFromL >0)
   {
      i_L_Percent = i_DiffL*100 / m_i_RecolorFromL;
   }
   else
   {
      i_L_Percent = 0;
   }

   // get a precentage change value
   if( m_i_RecolorFromS >0)
   {
      i_S_Percent = i_DiffS*100 / m_i_RecolorFromS;
   }
   else
   {
      i_S_Percent = 0;
   }

   // now calculate the new color

   i_NewH = m_i_RecolorToH + i_DiffH;
   // make color within the circle
   if(i_NewH>=360)
   {
      i_NewH -= 360;
   }
   else if(i_NewH<0)
   {
      i_NewH += 360;
   }

   // use the percentage value
   i_NewL = (m_i_RecolorToL*(100+ i_L_Percent))  /100;
   if(i_NewL < 0)
   {
      i_NewL =0;
   }
   else if( i_NewL>100)
   {
      i_NewL = 100;
   }

   // use the percentage value
   i_NewS = (m_i_RecolorToS*(100+ i_S_Percent))  /100;
   if(i_NewS < 0)
   {
      i_NewS =0;
   }
   else if( i_NewS>100)
   {
      i_NewS = 100;
   }

   // get result pixel from hls
   p_New = ColorCalculation::HLSToPixel(i_NewH,i_NewL,i_NewS);
   return p_New;
   
} // CalculateRecolor()

//===========================================================================
//
//===========================================================================

