//---------------------------------------------------------------------------
// This sourcefile is created by Peter Jodda / peter@jodda.de
// It is under the GPL license.
// If you expand this file, please contact me, that I can integrate the
// enhancements in the original file.
//---------------------------------------------------------------------------
// Module: CorrectionBox
//---------------------------------------------------------------------------
// Purpose: dialog box for global color correction
//---------------------------------------------------------------------------

#include <stdio.h>
#include <stdlib.h>
#include <math.h>

#include <qlayout.h>
#include <qgrid.h>
#include <qlabel.h>
#include <qgroupbox.h>
#include <qvgroupbox.h>
#include <qhgroupbox.h>
#include <qfont.h>
#include <qfontmetrics.h>
#include <qcombobox.h>

#include "CorrectionBox.h"

#include "PixelBuffer.h"
#include "QtUtilities.h"
#include "ImageFrame.h"
#include "MessageServer.h"
#include "Utilities.h"
#include "Zoom.h"

//===========================================================================
//
//===========================================================================

CorrectionBox::CorrectionBox(QWidget *parent,
                             GlobalStorage *pc_Global) 
               : StandardDialog(parent,pc_Global),
                 Tracer("CorrectionBox")
{
   QHGroupBox   *ModeBox;
   QBoxLayout   *l_Layout;
   int           i;

   LoadPosition("CorrectionBox");
   
   TRACE << "Correction Started\n" ;

   setCaption(tr("Color Correction"));

   m_pc_GlobalStorage = pc_Global;
   m_b_Visible        = false;
   m_i_CurrentMode    = 0;

   // create a vertical layout
   l_Layout = new  QVBoxLayout( m_w_TopWidget,1);


   // allocate the histogram-vectors
   m_pc_HistogramBW    = new PixelVector(sizeof(int)*8,256);
   m_pc_HistogramRed   = new PixelVector(sizeof(int)*8,256);
   m_pc_HistogramGreen = new PixelVector(sizeof(int)*8,256);
   m_pc_HistogramBlue  = new PixelVector(sizeof(int)*8,256);


   // block for the mode
   {

      m_ModeWidget = new QButtonGroup(3,
                                    QGroupBox::Horizontal,
                                    tr("Type"),
                                    m_w_TopWidget);
      l_Layout->addWidget(m_ModeWidget);

      m_ModeLightnessButton  = new QRadioButton(tr("Lightness"),m_ModeWidget);
      m_ModeColorButton      = new QRadioButton(tr("Color"),m_ModeWidget);

      m_ModeLightnessButton->setChecked(1);

      m_RefreshHistogramButton = new QPushButton(tr("Refresh Histogram"),
                                                 m_ModeWidget);
      
      connect( m_ModeWidget,
              SIGNAL(clicked(int)),
              SLOT(ModeChanged(int)) );
      connect(m_RefreshHistogramButton,
              SIGNAL(clicked() ),
              SLOT(RefreshHistogramPressed()) );
   }


   // box for lighness
   m_LightnessBox = new QVGroupBox(tr("Lightness"),m_w_TopWidget);
   l_Layout->addWidget(m_LightnessBox);
   {
      m_pc_LightnessCorrection = new QUCorrectionCurve(m_LightnessBox,
                                                      m_pc_GlobalStorage);

      connect(m_pc_LightnessCorrection,
              SIGNAL(CurveChanged(unsigned char *)),
              SLOT(LightnessCurveChanged(unsigned char *)) );
   } // box for lightness


   // box for Color
   m_ColorBox = new QVGroupBox(tr("Color"),m_w_TopWidget);
   l_Layout->addWidget(m_ColorBox);
   {
      m_pc_RedCorrection = new QUCorrectionCurve(m_ColorBox,
                                                 m_pc_GlobalStorage,
                                                 QUCorrectionCurve::OptionSmall,
                                                 PIXEL_MAKE(0xff,0,0) );
      m_pc_GreenCorrection = new QUCorrectionCurve(m_ColorBox,
                                                 m_pc_GlobalStorage,
                                                 QUCorrectionCurve::OptionSmall,
                                                 PIXEL_MAKE(0,0xff,0));
      m_pc_BlueCorrection = new QUCorrectionCurve(m_ColorBox,
                                                 m_pc_GlobalStorage,
                                                 QUCorrectionCurve::OptionSmall,
                                                 PIXEL_MAKE(0,0,0xff));

      connect(m_pc_RedCorrection,
              SIGNAL(CurveChanged(unsigned char *)),
              SLOT(RedCurveChanged(unsigned char *)) );
      connect(m_pc_GreenCorrection,
              SIGNAL(CurveChanged(unsigned char *)),
              SLOT(GreenCurveChanged(unsigned char *)) );
      connect(m_pc_BlueCorrection,
              SIGNAL(CurveChanged(unsigned char *)),
              SLOT(BlueCurveChanged(unsigned char *)) );
   }

   // box for the mode
   ModeBox = new QHGroupBox(tr("Mode"),m_w_TopWidget);
   l_Layout->addWidget(ModeBox);
   {
      m_CorrectionVisibleCheck = new QCheckBox(tr("Visible"), ModeBox);

      m_ApplyButton = new QPushButton(tr("Apply"),  ModeBox);


      connect(m_CorrectionVisibleCheck,
              SIGNAL(toggled(bool)),
              SLOT( VisibleChecked(bool) ) );
      connect(m_ApplyButton,
              SIGNAL(clicked() ),
              SLOT(ApplyPressed()) );
   }

   SetSensitivity();

   // add message receiver for disable mode messages
   m_pc_GlobalStorage->m_pc_MessageServer->AddMessageReceiver((MessageCallbackFunction) OnMessageReceiveCB,
                                                       (void *) this,
                                                       "ImageNewBuffer");

   m_pc_GlobalStorage->m_pc_MessageServer->AddMessageReceiver((MessageCallbackFunction)OnMessageReceiveCB,
                                                       (void *) this,
                                                       "ImageResize");
   // initital lightness in on, color is off
   m_LightnessBox->show();
   m_ColorBox    ->hide();

   // set a 1 - to 1 conversion
   for(i=0;i<256;i++)
   {
      m_uca_CorrectionRed      [i] = (unsigned char) i;
      m_uca_CorrectionGreen    [i] = (unsigned char) i;
      m_uca_CorrectionBlue     [i] = (unsigned char) i;
      m_uca_CorrectionLightness[i] = (unsigned char) i;

   }
} // CorrectionBox()

//===========================================================================
//
//===========================================================================

CorrectionBox::~CorrectionBox()
{
   // delete the histogram pixelvectors
   if(m_pc_HistogramBW != NULL)
   {
      delete m_pc_HistogramBW;
      m_pc_HistogramBW = NULL;
   }
   if(m_pc_HistogramRed !=NULL)
   {
      delete m_pc_HistogramRed;
      m_pc_HistogramRed = NULL;
   }
   if(m_pc_HistogramGreen != NULL)
   {
      delete m_pc_HistogramGreen;
      m_pc_HistogramGreen = NULL;
   }
   if(m_pc_HistogramBlue != NULL)
   {
      delete m_pc_HistogramBlue;
      m_pc_HistogramBlue = NULL;
   }

} // ~CorrectionBox()

//-----------------------------------------------------------------------------
//
//-----------------------------------------------------------------------------

void CorrectionBox::OnMessageReceive(QString qs_MessageType,
                                 void *p_Data)
{
   TRACE << "Message Received member this = " << (int) this << "\n";

   if(qs_MessageType.compare("ImageResize") ==0)
   {
      Disable();
   }
   if(qs_MessageType.compare("ImageNewBuffer") ==0)
   {
      Disable();
   }
} // OnMessageReceive()

//===========================================================================
//
//===========================================================================


void CorrectionBox::SetSensitivity()
{
} // SetSensitivity()

//===========================================================================
//
//===========================================================================

void CorrectionBox::LightnessCurveChanged(unsigned char *uc_CorrectionCurve)
{
   TRACE << "Lighness Curve Changed\n" ;

   // copy the curve also into a local buffer
   memcpy(m_uca_CorrectionLightness,uc_CorrectionCurve,256);

   // copy the correction curve into the zoom object
   m_pc_GlobalStorage->m_pc_Zoom->SetCorrectionCurve(m_uca_CorrectionLightness,
                           m_uca_CorrectionLightness,
                           m_uca_CorrectionLightness);
   // force a redraw
   if(m_b_Visible)
   {
      m_pc_GlobalStorage->m_pc_ImageFrame->RedrawAll();
   }

} // LightnessCurveChanged()

//===========================================================================
//
//===========================================================================

void CorrectionBox::RedCurveChanged(unsigned char *uc_CorrectionCurve)
{
   TRACE << "Red Curve Changed\n" ;

   // copy the curve also into a local buffer
   memcpy(m_uca_CorrectionRed,uc_CorrectionCurve,256);

   // copy the correction curve into the zoom object
   m_pc_GlobalStorage->m_pc_Zoom->SetCorrectionCurve(m_uca_CorrectionRed,
                           m_uca_CorrectionGreen,
                           m_uca_CorrectionBlue);
   // force a redraw
   if(m_b_Visible)
   {
      m_pc_GlobalStorage->m_pc_ImageFrame->RedrawAll();
   }

} // RedCurveChanged()
//===========================================================================
//
//===========================================================================

void CorrectionBox::GreenCurveChanged(unsigned char *uc_CorrectionCurve)
{
   TRACE << "Green Curve Changed\n" ;

   // copy the curve also into a local buffer
   memcpy(m_uca_CorrectionGreen,uc_CorrectionCurve,256);

   // copy the correction curve into the zoom object
   m_pc_GlobalStorage->m_pc_Zoom->SetCorrectionCurve(m_uca_CorrectionRed,
                           m_uca_CorrectionGreen,
                           m_uca_CorrectionBlue);
   // force a redraw
   if(m_b_Visible)
   {
      m_pc_GlobalStorage->m_pc_ImageFrame->RedrawAll();
   }

} // GreenCurveChanged()
//===========================================================================
//
//===========================================================================

void CorrectionBox::BlueCurveChanged(unsigned char *uc_CorrectionCurve)
{
   TRACE << "Blue Curve Changed\n" ;

   // copy the curve also into a local buffer
   memcpy(m_uca_CorrectionBlue,uc_CorrectionCurve,256);

   // copy the correction curve into the zoom object
   m_pc_GlobalStorage->m_pc_Zoom->SetCorrectionCurve(m_uca_CorrectionRed,
                           m_uca_CorrectionGreen,
                           m_uca_CorrectionBlue);
   // force a redraw
   if(m_b_Visible)
   {
      m_pc_GlobalStorage->m_pc_ImageFrame->RedrawAll();
   }

} // BlueCurveChanged()
//===========================================================================
//
//===========================================================================

void CorrectionBox::ApplyPressed()
{
   TRACE << "Apply pressed\n" ;

   // apply the correction to the image
   if(m_i_CurrentMode == 0)
   {
      m_pc_GlobalStorage->m_pc_MainImage->ConvertColors(m_uca_CorrectionLightness,
                           m_uca_CorrectionLightness,
                           m_uca_CorrectionLightness);
   }
   else if(m_i_CurrentMode == 1)
   {
      m_pc_GlobalStorage->m_pc_MainImage->ConvertColors(m_uca_CorrectionRed,
                           m_uca_CorrectionGreen,
                           m_uca_CorrectionBlue);
   }


   // force a redraw
   m_pc_GlobalStorage->m_pc_ImageFrame->RedrawAll();
   m_CorrectionVisibleCheck->setChecked(0);

   // tell that the image is changed
   m_pc_GlobalStorage->m_pc_MessageServer->SendMessage(0,
                                                 this,
                                                 "ImageChanged");

   // also update the histogram buffers                                                 
   GetHistogram();
   TransferHistogramIntoBuffer();
} // ApplyPressed()

//===========================================================================
//
//===========================================================================

void CorrectionBox::VisibleChecked(bool b_On)
{
   m_b_Visible = b_On;

   // tell state to zoom
   if(m_b_Visible)
   {
      m_pc_GlobalStorage->m_pc_Zoom->SetCorrectionEnabled(true);
   }
   else
   {
      m_pc_GlobalStorage->m_pc_Zoom->SetCorrectionEnabled(false);
   }
   // force a redraw
   m_pc_GlobalStorage->m_pc_ImageFrame->RedrawAll();
} // VisibleChecked()

//===========================================================================
//
//===========================================================================

void CorrectionBox::Disable()
{
   if(m_b_Visible)
   {
      // disable the button
      m_CorrectionVisibleCheck->setChecked(0);
      // disable it in the zoom
      m_pc_GlobalStorage->m_pc_Zoom->SetCorrectionEnabled(false);

      m_b_Visible = false;
      // force a redraw
      m_pc_GlobalStorage->m_pc_ImageFrame->RedrawAll();
   } // if was visible

} // Disable

//===========================================================================
//
//===========================================================================

void CorrectionBox::ModeChanged(int i_NewMode)
{
   m_i_CurrentMode = i_NewMode;

   // switch from color to lightness and vice versa
   if(m_i_CurrentMode ==0)
   {
      // to lightness
      m_LightnessBox->show();
      m_ColorBox    ->hide();
      m_pc_GlobalStorage->m_pc_Zoom->SetCorrectionCurve(m_uca_CorrectionLightness,
                           m_uca_CorrectionLightness,
                           m_uca_CorrectionLightness);
   }
   else if (m_i_CurrentMode ==1)
   {
      // to color
      m_LightnessBox->hide();
      m_ColorBox    ->show();
      m_pc_GlobalStorage->m_pc_Zoom->SetCorrectionCurve(m_uca_CorrectionRed,
                           m_uca_CorrectionGreen,
                           m_uca_CorrectionBlue);
   }

   // let the widget resize to the new layout
   ResizeDisplay();

   // apply the new curves in the image
   if(m_b_Visible)
   {
      m_pc_GlobalStorage->m_pc_ImageFrame->RedrawAll();
   } // if visible

   // also update the histogram buffers
   GetHistogram();
   TransferHistogramIntoBuffer();

} // ModeChanged()

//===========================================================================
//
//===========================================================================

void CorrectionBox::showEvent(QShowEvent *)
{
   GetHistogram();
   TransferHistogramIntoBuffer();
}  // showEvent()

//===========================================================================
//
//===========================================================================

void CorrectionBox::hideEvent(QHideEvent *)
{
   // if hidden the disable the visible mode to normal

   Disable();
   
}  // hideEvent()

//===========================================================================
//
//===========================================================================

void CorrectionBox::GetHistogram()
{
   m_pc_GlobalStorage->m_pc_MainImage->HistogramBW(m_pc_HistogramBW);
   m_pc_GlobalStorage->m_pc_MainImage->HistogramRGB(m_pc_HistogramRed,
                                                    m_pc_HistogramGreen,
                                                    m_pc_HistogramBlue);
   
} // GetHistogram()

//===========================================================================
//
//===========================================================================

void CorrectionBox::TransferHistogramIntoBuffer()
{
   m_pc_LightnessCorrection->SetHistogramIntoBuffer(m_pc_HistogramBW);
   m_pc_RedCorrection     ->SetHistogramIntoBuffer(m_pc_HistogramRed);
   m_pc_GreenCorrection   ->SetHistogramIntoBuffer(m_pc_HistogramGreen);
   m_pc_BlueCorrection    ->SetHistogramIntoBuffer(m_pc_HistogramBlue);

   m_pc_LightnessCorrection->Redraw();
   m_pc_RedCorrection->Redraw();
   m_pc_GreenCorrection->Redraw();
   m_pc_BlueCorrection->Redraw();

} // TransferHistogramIntoBuffer()

//===========================================================================
//
//===========================================================================

void CorrectionBox::RefreshHistogramPressed()
{
   GetHistogram();
   TransferHistogramIntoBuffer();
   
} // RefreshHistogramPressed()

//===========================================================================
//
//===========================================================================

int CorrectionBox::WriteFile(PericoFileWriter *pc_FileWriter,
                         QString qs_Name)
{
   pc_FileWriter->StartStructure(qs_Name,"CorrectionBox");

   pc_FileWriter->AddValue("CurrentMode",m_i_CurrentMode);

   m_pc_LightnessCorrection->WriteFile(pc_FileWriter,"LightnessCorrection");
   m_pc_RedCorrection->WriteFile(pc_FileWriter,      "RedCorrection");
   m_pc_GreenCorrection->WriteFile(pc_FileWriter,    "GreenCorrection");
   m_pc_BlueCorrection->WriteFile(pc_FileWriter,     "BlueCorrection");
   
   pc_FileWriter->FinishStructure();

   // everything is OK
   return 0;
} // WriteFile

//===========================================================================
//
//===========================================================================

int CorrectionBox::LoadFile(PericoFileWriter *pc_FileWriter)
{

   PERICO_READ_STRUCTURE_START

   if(qs_Name.compare("CurrentMode") == 0)
   {
      m_i_CurrentMode = atoi(qs_Value);
   }
   else if(    qs_Name.compare("LightnessCorrection") == 0
            && qs_Value.compare("QUCorrectionCurve") == 0
            && b_IsStartStructure)
   {
      m_pc_LightnessCorrection->LoadFile(pc_FileWriter);
   }
   else if(    qs_Name.compare("RedCorrection") == 0
            && qs_Value.compare("QUCorrectionCurve") == 0
            && b_IsStartStructure)
   {
      m_pc_RedCorrection->LoadFile(pc_FileWriter);
   }
   else if(    qs_Name.compare("GreenCorrection") == 0
            && qs_Value.compare("QUCorrectionCurve") == 0
            && b_IsStartStructure)
   {
      m_pc_GreenCorrection->LoadFile(pc_FileWriter);
   }
   else if(    qs_Name.compare("BlueCorrection") == 0
            && qs_Value.compare("QUCorrectionCurve") == 0
            && b_IsStartStructure)
   {
      m_pc_BlueCorrection->LoadFile(pc_FileWriter);
   }

   PERICO_READ_STRUCTURE_END

   // set to Display
   if(m_i_CurrentMode ==0)
   {
      m_ModeLightnessButton->setChecked(true);
      m_ModeColorButton    ->setChecked(false);
   }
   else
   {
      m_ModeLightnessButton->setChecked(false);
      m_ModeColorButton    ->setChecked(true);
   }
   ModeChanged(m_i_CurrentMode);

   // everything is OK
   return 0;
} // LoadFile

//===========================================================================
//
//===========================================================================

