//---------------------------------------------------------------------------
// This sourcefile is created by Peter Jodda / peter@jodda.de
// It is under the GPL license.
// If you expand this file, please contact me, that I can integrate the
// enhancements in the original file.
//---------------------------------------------------------------------------
// Module:  CubicSpline
//---------------------------------------------------------------------------
// Purpose:
//---------------------------------------------------------------------------

#include "CubicSpline.h"

//===========================================================================
//
//===========================================================================

CubicBezierSpline::CubicBezierSpline()
{
   m_b_OpenCurve = false;
}

//===========================================================================
//
//===========================================================================

CubicBezierSpline::~CubicBezierSpline()
{

}

//===========================================================================
//
//===========================================================================

void CubicBezierSpline::CalculateSubPoints(float f_TLeft,
               GeoCoordinate c_Left,
               float f_TRight,
               GeoCoordinate c_Right
               )
{
   float f_Difference;

   f_Difference = c_Left.DifferenceTo(c_Right);
   if(  f_Difference>1)
   {
      // calculate points in between
      GeoCoordinate c_Middle;
      float         f_TMiddle;

      f_TMiddle = (f_TLeft + f_TRight) /2.0;
      c_Middle  = CalculatePointAt(f_TMiddle);

      // recursive call before
      CalculateSubPoints( f_TLeft,
                          c_Left,
                          f_TMiddle,
                          c_Middle
                        );
                        
      // add this point
      m_l_SubPoints.AddElement(c_Middle);      
      
      // recursive call after
      CalculateSubPoints( f_TMiddle,
                          c_Middle,
                          f_TRight,
                          c_Right
                        );


   }

} // CalculateSubPoints()

//===========================================================================
//
//===========================================================================

GeoCoordinate CubicBezierSpline::CalculatePointAt(float f_T)
{
   GeoCoordinate c_Result;
   float f_X;
   float f_Y;
   float f_OneMinusT;
   
   f_OneMinusT = (1-f_T);    

   f_X =    f_OneMinusT*f_OneMinusT*f_OneMinusT*m_c_InterpolationPoints[0].GetX();
   f_X += 3*f_OneMinusT*f_OneMinusT*       f_T *m_c_InterpolationPoints[1].GetX();
   f_X += 3*f_OneMinusT*       f_T *       f_T *m_c_InterpolationPoints[2].GetX();
   f_X +=          f_T *       f_T *       f_T *m_c_InterpolationPoints[3].GetX();

   f_Y =    f_OneMinusT*f_OneMinusT*f_OneMinusT*m_c_InterpolationPoints[0].GetY();
   f_Y += 3*f_OneMinusT*f_OneMinusT*       f_T *m_c_InterpolationPoints[1].GetY();
   f_Y += 3*f_OneMinusT*       f_T *       f_T *m_c_InterpolationPoints[2].GetY();
   f_Y +=          f_T *       f_T *       f_T *m_c_InterpolationPoints[3].GetY();

   c_Result.SetXY(f_X,f_Y);

   
   return c_Result;
} // CalculatePointAt()

//===========================================================================
//
//===========================================================================

void CubicBezierSpline::StartCalculationSplinePoints(GeoCoordinate c_Points[4],
                           bool b_Append )
{
   GeoCoordinate c_Start;
   GeoCoordinate c_End;
   int           i;
   for(i=0;i<4;i++)
   {
      m_c_InterpolationPoints[i] = c_Points[i];
   }

   // clean all points from the list
   if(! b_Append)
   {
      m_l_SubPoints.Clean();
   }
   
   c_Start = CalculatePointAt(0);
   c_End   = CalculatePointAt(1);

   // add first point
   m_l_SubPoints.AddElement(c_Start);

   // calculate all points in between
   CalculateSubPoints( 0.0,
                       c_Start,
                       1.0,
                       c_End
                      );

   // add last point
   m_l_SubPoints.AddElement(c_End);

} // StartCalculationSplinePoints()


//===========================================================================
//
//===========================================================================

GeoCoordinateList *CubicBezierSpline::GetResultList()
{
   return &m_l_SubPoints;
} // GetResultList()

//===========================================================================
//
//===========================================================================

GeoCoordinateList *CubicBezierSpline::CalculateInterpolationList(
         GeoCoordinateList *pc_OriginalList,
         float f_OffsetX,
         float f_OffsetY,
         bool  b_OpenCurve
         )
{
   GeoCoordinateList   * pc_InterpolationList;
   GeoCoordinate         c_Actual;
   GeoCoordinate         c_Last;
   GeoCoordinate         c_Next;
   int                   i;
   int                   i_Count;

   i_Count = pc_OriginalList->GetCount();

   if(b_OpenCurve)
   {
      i_Count = pc_OriginalList->GetCount();

      pc_InterpolationList = new GeoCoordinateList(3*(i_Count+1) +1);

      
      c_Last   = pc_OriginalList->Get(0);
      c_Last.Transform(f_OffsetX,f_OffsetY);
      c_Actual = pc_OriginalList->Get(0);
      c_Actual.Transform(f_OffsetX,f_OffsetY);

      pc_InterpolationList->Set(0,c_Actual);

      // <= maeans on point more
      for(i=0;i<=i_Count;i++)
      {
         GeoCoordinate c_Point;

         if(i+1<i_Count)
         {
            c_Next = pc_OriginalList->Get(i+1) ;
         }
         else
         {
            c_Next =  pc_OriginalList->Get(i_Count-1);
         }
         c_Next.Transform(f_OffsetX,f_OffsetY);

         c_Point.SetX( 1.0/3.0*( 2.0*c_Last.GetX() + c_Actual.GetX()));
         c_Point.SetY( 1.0/3.0*( 2.0*c_Last.GetY() + c_Actual.GetY()));
         pc_InterpolationList->Set((i)*3+1    ,c_Point);

         c_Point.SetX( 1.0/3.0*( c_Last.GetX() + 2.0*c_Actual.GetX()));
         c_Point.SetY( 1.0/3.0*( c_Last.GetY() + 2.0*c_Actual.GetY()));
         pc_InterpolationList->Set((i)*3 +2 ,c_Point);

         c_Point.SetX( 1.0/6.0*( c_Last.GetX() + 4.0*c_Actual.GetX() + c_Next.GetX()));
         c_Point.SetY( 1.0/6.0*( c_Last.GetY() + 4.0*c_Actual.GetY() + c_Next.GetY()));

         pc_InterpolationList->Set((i)*3+3 ,c_Point);


         // prepare for the next loop
         c_Last   = c_Actual;
         c_Actual = c_Next;
      } // for i_Count

   }
   else // closed curve
   {
      i_Count = pc_OriginalList->GetCount();

      pc_InterpolationList = new GeoCoordinateList(3*i_Count);

      c_Last   = pc_OriginalList->Get(i_Count-1);
      c_Last.Transform(f_OffsetX,f_OffsetY);
      c_Actual = pc_OriginalList->Get(0);
      c_Actual.Transform(f_OffsetX,f_OffsetY);

      for(i=0;i<i_Count;i++)
      {
         GeoCoordinate c_Point;
         int           i_LastDestination;
      
         if(i+1<i_Count)
         {
            c_Next = pc_OriginalList->Get(i+1) ;
            i_LastDestination = i*3+3;
         }
         else
         {
            c_Next =  pc_OriginalList->Get(0);
            i_LastDestination =0;
         }
         c_Next.Transform(f_OffsetX,f_OffsetY);

         c_Point.SetX( 1.0/3.0*( 2.0*c_Last.GetX() + c_Actual.GetX()));
         c_Point.SetY( 1.0/3.0*( 2.0*c_Last.GetY() + c_Actual.GetY()));
         pc_InterpolationList->Set(i*3+1    ,c_Point);

         c_Point.SetX( 1.0/3.0*( c_Last.GetX() + 2.0*c_Actual.GetX()));
         c_Point.SetY( 1.0/3.0*( c_Last.GetY() + 2.0*c_Actual.GetY()));
         pc_InterpolationList->Set(i*3 +2 ,c_Point);

         c_Point.SetX( 1.0/6.0*( c_Last.GetX() + 4.0*c_Actual.GetX() + c_Next.GetX()));
         c_Point.SetY( 1.0/6.0*( c_Last.GetY() + 4.0*c_Actual.GetY() + c_Next.GetY()));

         pc_InterpolationList->Set(i_LastDestination ,c_Point);


         // prepare for the next loop
         c_Last   = c_Actual;
         c_Actual = c_Next;
      } // for i_Count

   }
   return pc_InterpolationList;
} // CalculateInterpolationList

//===========================================================================
//
//===========================================================================


void CubicBezierSpline::SetSplineStyle(bool b_Open)
{
   m_b_OpenCurve = b_Open;
} // SetSplineStyle()

//===========================================================================
//
//===========================================================================

