//---------------------------------------------------------------------------
// This sourcefile is created by Peter Jodda / peter@jodda.de
// It is under the GPL license.
// If you expand this file, please contact me, that I can integrate the
// enhancements in the original file.
//---------------------------------------------------------------------------
// Module: EffectBox
//---------------------------------------------------------------------------
// Purpose: Dialog to select the main Effect options.
//---------------------------------------------------------------------------

#include <stdio.h>
#include <stdlib.h>
#include <math.h>

#include <qlayout.h>
#include <qgrid.h>
#include <qlabel.h>
#include <qgroupbox.h>
#include <qvgroupbox.h>
#include <qhgroupbox.h>
#include <qfont.h>
#include <qfontmetrics.h>
#include <qcombobox.h>
#include <qgrid.h>


#include "EffectBox.h"

#include "PixelBuffer.h"
#include "QtUtilities.h"
#include "ImageFrame.h"
#include "MessageServer.h"
#include "Selection.h"
#include "BufferPainter.h"
#include "MouseEvent.h"
#include "Utilities.h"

#include "QULabelSlider.h"
#include "ApplicationStorage.h"
#include "MainFrame.h"
#include "MotiveBox.h"

#include "Effects.h"
#include "QUIconButton.h"

//============================================================================
//
//============================================================================

EffectBox::EffectBox(QWidget *parent,
                        GlobalStorage *pc_Global,
                        MainFrame *pc_MainFrame) :
                StandardDialog(parent,pc_Global),
                Tracer("EffectBox")
{
   QBoxLayout    *l_Layout;

   QHGroupBox   *ActivationBox;

   LoadPosition("EffectBox");
   setCaption(tr("Effect"));
   TRACE << "Effect Started\n" ;

   m_pc_MainFrame         = pc_MainFrame;

   // settings for comic
   m_b_ComicPreBoost      = false;
   m_i_ComicAlgorithmus   = 0;
   m_i_ComicLineDetection = 20;
   m_b_ComicLineEnabled   = true;

   // settings for mosaic
   m_i_MosaicElementSize  = 5;
   m_i_MosaicForm         = MosaicFormRound;
   m_b_MosaicVarySize     = true;
   m_i_MosaicShineFactor  = 0;

   // settings for color reduce
   m_i_ReduceNumberOfColors  = 10;
   m_i_ReduceLeastDifference = 15;

   m_p_BlackWhiteFromColor         = PIXEL_MAKE(0,0,0);
   m_p_BlackWhiteToColor           = PIXEL_MAKE(255,255,255);
   m_b_BlackWhiteOpticalCorrection = false;
   m_b_BlackWhitePure              = false;
   

   l_Layout = new  QVBoxLayout( m_w_TopWidget,1);

   // --- box for activation
   ActivationBox = new QHGroupBox(tr("Effect Operation"),m_w_TopWidget);
   ActivationBox->setInsideSpacing(3);
   ActivationBox->setColumns(1);
   
   ActivationBox->setInsideMargin(ActivationBox->insideMargin()*2);
  
   
   l_Layout->addWidget(ActivationBox);
   {
      // buttons to establish effect mode

      m_ComicButton         = new QUIconButton(ActivationBox,
                                                "Comic",
                                                tr("Comic"));

      m_MosaicButton         = new QUIconButton(ActivationBox,
                                                "Mosaic",
                                                tr("Mosaic"));

      m_ReduceButton         = new QUIconButton(ActivationBox,
                                                "ColorReduce",
                                                tr("Reduce Color"));

      m_BlackWhiteButton         = new QUIconButton(ActivationBox,
                                                "BlackWhite",
                                                tr("Black/White Color Change"));

      connect( m_ComicButton ,
               SIGNAL(clicked()),
               SLOT( ActivateComic() ) );
      connect( m_MosaicButton ,
               SIGNAL(clicked()),
               SLOT( ActivateMosaic() ) );
      connect( m_ReduceButton ,
               SIGNAL(clicked()),
               SLOT( ActivateReduce() ) );
      connect( m_BlackWhiteButton ,
               SIGNAL(clicked()),
               SLOT( ActivateBlackWhite() ) );
   }


   // box for comic style
   m_ComicModeBox = new QVGroupBox(tr("Comic Options"),m_w_TopWidget);
   l_Layout->addWidget(m_ComicModeBox);
   {

      m_ComicLineDetectionEnableCheck     = new QCheckBox(tr("Line Detection"),
                                     m_ComicModeBox);
      m_ComicLineDetectionEnableCheck->setChecked(m_b_ComicLineEnabled);

      m_ComicLineDectionLevel = new QULabelSlider(m_ComicModeBox,
                                    QULabelSlider::LabelStyleAutomatic |
                                          QULabelSlider::LabelStyleEdit
                                       ,
                                    1,100,
                                    m_i_ComicLineDetection,
                                    tr("Line Detection Level")
                                    );

      m_ComicPreCalculationBoostCheck     = new QCheckBox(tr("Pre Calculation Boost"),
                                     m_ComicModeBox);
      m_ComicPreCalculationBoostCheck->setChecked(m_b_ComicPreBoost);



      m_ComicColorAlgorithmusBox = new QButtonGroup(3,
                                    QGroupBox::Vertical,
                                    tr("Calculation Mode"),
                                    m_ComicModeBox);

      m_ComicAlgCopy = new QRadioButton(tr("Copy"),
                                m_ComicColorAlgorithmusBox);
      m_ComicAlgFlatten = new QRadioButton(tr("Flatten"),
                                m_ComicColorAlgorithmusBox);
      m_ComicAlgBoost = new QRadioButton(tr("Boost"),
                                m_ComicColorAlgorithmusBox);
      m_ComicAlgFlattenAndBoost = new QRadioButton(tr("Flatten and Boost"),
                                m_ComicColorAlgorithmusBox);

      m_ComicAlgCopy->setChecked(1);

      m_ComicExecuteButton = new QPushButton(tr("Do comic Effect"),
                                        m_ComicModeBox);

                                     
      connect( m_ComicLineDetectionEnableCheck,
                  SIGNAL(toggled(bool)),
                  SLOT( ComicLineDetectionToggled(bool )) );

      connect( m_ComicLineDectionLevel,
               SIGNAL(valueChanged(int)),
               SLOT( ComicLineDetectionChanged(int )) );


      connect( m_ComicColorAlgorithmusBox,
               SIGNAL(clicked(int)),
               SLOT(ComicAlgorithmusChanged(int)) );

      connect( m_ComicPreCalculationBoostCheck,
                  SIGNAL(toggled(bool)),
                  SLOT( ComicPreCalculationBoostCheckModeToggled(bool )) );

      connect( m_ComicExecuteButton,
                  SIGNAL(clicked()),
                  SLOT( ComicDoEffect()) );

                  
                  
   } // box for comic style

   // box for comic style
   m_MosaicModeBox = new QVGroupBox(tr("Mosaic Options"),m_w_TopWidget);
   l_Layout->addWidget(m_MosaicModeBox);
   {
      QButtonGroup *FormBox;
      
      FormBox = new QButtonGroup(3,
                                    QGroupBox::Horizontal,
                                    tr("Form"),
                                    m_MosaicModeBox);

      m_MosaicFormRoundButton  = new QRadioButton(tr("Round"),
                                             FormBox);
      m_MosaicFormSquareButton = new QRadioButton(tr("Square"),
                                             FormBox);
      if(m_i_MosaicForm == MosaicFormRound)
      {
         m_MosaicFormRoundButton->setChecked(true);
      }
      else
      {
         m_MosaicFormSquareButton->setChecked(true);

      }

      m_MosaicElementSizeSlider = new QULabelSlider(m_MosaicModeBox,
                                    QULabelSlider::LabelStyleAutomatic |
                                          QULabelSlider::LabelStyleEdit
                                       ,
                                    1,10,
                                    m_i_MosaicElementSize,
                                    tr("Mosaic Element Size")
                                    );
      m_MosaicVarySizeCheck = new QCheckBox(tr("Vary in Size"),
                                     m_MosaicModeBox);
      m_MosaicVarySizeCheck->setChecked(m_b_MosaicVarySize);
      m_MosaicShineSlider = new QULabelSlider(m_MosaicModeBox,
                                    QULabelSlider::LabelStyleAutomatic |
                                          QULabelSlider::LabelStyleEdit
                                       ,
                                    0,10,
                                    m_i_MosaicShineFactor,
                                    tr("Shine through")
                                    );

      m_MosaicExecuteButton = new QPushButton(tr("Do mosaic Effect"),
                                        m_MosaicModeBox);

      connect( FormBox,
               SIGNAL(clicked(int)),
               SLOT(MosaicFormChanged(int)) );

      connect( m_MosaicElementSizeSlider,
               SIGNAL(valueChanged(int)),
               SLOT( MosaicSizeChanged(int )) );

      connect( m_MosaicVarySizeCheck,
                  SIGNAL(toggled(bool)),
                  SLOT( MosaicVarySizeToggled(bool )) );

      connect( m_MosaicShineSlider,
               SIGNAL(valueChanged(int)),
               SLOT( MosaicShineFactorChanged(int )) );

      connect( m_MosaicExecuteButton,
                  SIGNAL(clicked()),
                  SLOT( MosaicDoEffect()) );



   } // box for mosaic effect

   // box for reduce style

   m_ReduceModeBox = new QVGroupBox(tr("Reduce Options"),m_w_TopWidget);
   l_Layout->addWidget(m_ReduceModeBox);
   {
      m_ReduceColorNumberSlider = new QULabelSlider(m_ReduceModeBox,
                                    QULabelSlider::LabelStyleAutomatic |
                                          QULabelSlider::LabelStyleEdit
                                       ,
                                    2,100,
                                    m_i_ReduceNumberOfColors,
                                    tr("Number of Colors")
                                    );
      m_ReduceLeastDifferenceSlider = new QULabelSlider(m_ReduceModeBox,
                                    QULabelSlider::LabelStyleAutomatic |
                                          QULabelSlider::LabelStyleEdit
                                       ,
                                    1,50,
                                    m_i_ReduceLeastDifference,
                                    tr("Least Color Difference")
                                    );

      m_ReduceExecuteButton = new QPushButton(tr("Reduce Colors"),
                                        m_ReduceModeBox);
      connect( m_ReduceColorNumberSlider,
               SIGNAL(valueChanged(int)),
               SLOT( ReduceColorNumberChanged(int )) );

      connect( m_ReduceLeastDifferenceSlider,
               SIGNAL(valueChanged(int)),
               SLOT( ReduceLeastDifferenceChanged(int )) );

      connect( m_ReduceExecuteButton,
                  SIGNAL(clicked()),
                  SLOT( ReduceDoEffect()) );



   } // reduce box

   // box for black white effect

   m_BlackWhiteModeBox = new QVGroupBox(tr("Black/White"),m_w_TopWidget);
   l_Layout->addWidget(m_BlackWhiteModeBox);
   {

      m_BlackWhiteFromColorWidget = new QUColorDragLabeledWidget(m_BlackWhiteModeBox,
                                                 tr("From Color"),
                                                 QUColorDragWidget::ModePipette
                                               | QUColorDragWidget::ModeEnableColorselectBox,
                                               m_pc_GlobalStorage,
                                               pc_MainFrame->GetColorSelectBox()  );
      
      m_BlackWhiteToColorWidget = new QUColorDragLabeledWidget(m_BlackWhiteModeBox,
                                                 tr("To Color"),
                                                 QUColorDragWidget::ModePipette
                                               | QUColorDragWidget::ModeEnableColorselectBox,
                                               m_pc_GlobalStorage,
                                               pc_MainFrame->GetColorSelectBox()  );

      m_BlackWhiteFromColorWidget->SetLabelWidth(10);
      m_BlackWhiteToColorWidget->SetLabelWidth(10);

      m_BlackWhiteFromColorWidget->SetColor(m_p_BlackWhiteFromColor);                                                                  
      m_BlackWhiteToColorWidget->SetColor(m_p_BlackWhiteToColor);

      m_BlackWhiteOpticalCheck =  new QCheckBox(tr("Optical Correction"),
                                     m_BlackWhiteModeBox);
      m_BlackWhiteOpticalCheck->setChecked(m_b_BlackWhiteOpticalCorrection);

      
      m_BlackWhitePureCheck =  new QCheckBox(tr("Reduce to pure Colors (Random Dithering)"),
                                     m_BlackWhiteModeBox);
      m_BlackWhitePureCheck->setChecked(m_b_BlackWhitePure);


      m_BlackWhiteExecuteButton = new QPushButton(tr("Reduce to Black/White"),
                                        m_BlackWhiteModeBox);


      connect( m_BlackWhiteFromColorWidget,
                  SIGNAL(ColorChanged(Pixel)),
                  SLOT( BlackWhiteFromColorChanged(Pixel)) );

      connect( m_BlackWhiteToColorWidget,
                  SIGNAL(ColorChanged(Pixel)),
                  SLOT( BlackWhiteToColorChanged(Pixel)) );

      connect( m_BlackWhiteOpticalCheck,
                  SIGNAL(toggled(bool)),
                  SLOT( BlackWhiteCorrectionChecked(bool)) );

      connect( m_BlackWhitePureCheck,
                  SIGNAL(toggled(bool)),
                  SLOT( BlackWhitePureChecked(bool)) );

      connect( m_BlackWhiteExecuteButton,
                  SIGNAL(clicked()),
                  SLOT( BlackWhiteDoEffect()) );



   } // black white

   
   SetSensitivity();

   // initial show elements for effects -> comic

   SetWidgetToMode(EffectComic);
   
} // PaintBox()

//-----------------------------------------------------------------------------
//
//-----------------------------------------------------------------------------

EffectBox::~EffectBox()
{

} // ~EffectBox()

//-----------------------------------------------------------------------------
//
//-----------------------------------------------------------------------------


void EffectBox::ActivateComic()
{
   SetWidgetToMode(EffectComic);

} // ActivateComic()

//-----------------------------------------------------------------------------
//
//-----------------------------------------------------------------------------

void EffectBox::ActivateMosaic()
{
   SetWidgetToMode(EffectMosaic);
} // ActivateMosaic()

//-----------------------------------------------------------------------------
//
//-----------------------------------------------------------------------------

void EffectBox::ActivateReduce()
{
   SetWidgetToMode(EffectColorReduce);
} // ActivateReduce()

//-----------------------------------------------------------------------------
//
//-----------------------------------------------------------------------------

void EffectBox::ActivateBlackWhite()
{
   SetWidgetToMode(EffectBlackWhite);
} // ActivateBlackWhite()

//-----------------------------------------------------------------------------
//
//-----------------------------------------------------------------------------

void EffectBox::OnMessageReceive(QString qs_MessageType,
                                 void *p_Data)
{
   TRACE << "Message Received member this = " << (int) this << "\n";

   if(qs_MessageType.compare("ImageResize") ==0)
   {
   }
   if(qs_MessageType.compare("ImageNewBuffer") ==0)
   {
   }
} // OnMessageReceive()


//===========================================================================
//
//===========================================================================

#ifdef UNUSED
void *EffectBox::SelectionEventArrived(void *pc_Data)
{
   SelectionCallbackData *cb;
   cb = (SelectionCallbackData *) pc_Data;
   if(cb->i_CallbackReason == SelectionCallbackData:: ReasonOperationDisabled)
   {
      // clean up unused structures
      return NULL;
   }
   if( !cb->i_CallbackReason == SelectionCallbackData:: ReasonSelection)
   {
      return NULL;
   }
   // from now only real selections


   m_i_MouseEventType = m_pc_GlobalStorage->m_pc_Selection->GetMouseEventType();
   if(m_i_MouseEventType == MouseEvent::EventStart)
   {
      m_b_LastPoint = false;
   }

   // perform different actions depending on the graphic mode
   switch(m_i_CurrentGraphikMode)
   {
   default:
      break;
   } // switch

   // at mouse end sometimes the transparency mask hast to be cleared
   if(m_i_MouseEventType == MouseEvent::EventEnd)
   {
      m_b_LastPoint = false;

      if(m_i_TransparencyClearMode == ClearModeAutomatic)
      {
         TransparencyMaskClear();
      }
   }
   else if(m_i_MouseEventType == MouseEvent::EventStart
           || m_i_MouseEventType == MouseEvent::EventDo)
   {
      m_b_LastPoint = true;
      m_pc_GlobalStorage->m_pc_Selection->GetSelectionPoint(m_i_LastX,m_i_LastY);
   }

   return NULL;
} // SelectionEventArrived()

//===========================================================================
//
//===========================================================================


void *PaintBox::SelectionEventArrivedCB(void *Thisdata,void *pc_Data)
{
   PaintBox *pc_This = (PaintBox *) Thisdata;

   return pc_This->SelectionEventArrived(pc_Data);

} // SelectionEventArrivedCB()

#endif // unused

//===========================================================================
//
//===========================================================================

void EffectBox::SetSensitivity()
{
   m_ComicLineDectionLevel->setEnabled(m_b_ComicLineEnabled);
   m_ComicPreCalculationBoostCheck->setEnabled(m_b_ComicLineEnabled);
} // SetSensitivity()


//===========================================================================
//
//===========================================================================

void EffectBox::SetWidgetToMode(int i_Mode)
{
   m_i_CurrentGraphikMode = i_Mode;

   // first disable all
   m_ComicModeBox ->hide();
   m_ComicButton->SetSelected(false);
   m_MosaicModeBox ->hide();
   m_MosaicButton->SetSelected(false);
   m_ReduceModeBox ->hide();
   m_ReduceButton->SetSelected(false);
   m_BlackWhiteModeBox ->hide();
   m_BlackWhiteButton->SetSelected(false);

   // now enable, what is needed
   switch(i_Mode)
   {
   case EffectComic:
      m_ComicModeBox ->show();
      m_ComicButton->SetSelected(true);
      break;
   case EffectMosaic:
      m_MosaicModeBox ->show();
      m_MosaicButton->SetSelected(true);
      break;
   case EffectColorReduce:
      m_ReduceModeBox ->show();
      m_ReduceButton->SetSelected(true);
      break;
   case EffectBlackWhite:
      m_BlackWhiteModeBox ->show();
      m_BlackWhiteButton->SetSelected(true);
      break;
   } // switch mode

   ResizeDisplay();
   SetSensitivity();

} // SetWidgetToMode()

//===========================================================================
//
//===========================================================================

int EffectBox::WriteFile(PericoFileWriter *pc_FileWriter,
                         QString qs_Name)
{
   pc_FileWriter->StartStructure(qs_Name,"EffectBox");

   pc_FileWriter->AddValue("ComicPreBoost"      , m_b_ComicPreBoost);
   pc_FileWriter->AddValue("ComicAlgorithmus"   , m_i_ComicAlgorithmus);
   pc_FileWriter->AddValue("ComicLineDectection", m_i_ComicLineDetection);
   pc_FileWriter->AddValue("ComicLineDectectionEnabed", m_b_ComicLineEnabled);

   pc_FileWriter->AddValue("MosaicElementSize",m_i_MosaicElementSize);
   pc_FileWriter->AddValue("MosaicForm"       ,m_i_MosaicForm);
   pc_FileWriter->AddValue("MosaicVarySize"   ,m_b_MosaicVarySize);
   pc_FileWriter->AddValue("MosaicShineFactor",m_i_MosaicShineFactor);

   pc_FileWriter->AddValue("ReduceNumberColors"   ,m_i_ReduceNumberOfColors);
   pc_FileWriter->AddValue("ReduceLeastDifference",m_i_ReduceLeastDifference);

   pc_FileWriter->AddValue("BlackWhiteFromColor"  ,(int)m_p_BlackWhiteFromColor);
   pc_FileWriter->AddValue("BlackWhiteToColor"    ,(int)m_p_BlackWhiteToColor);
   pc_FileWriter->AddValue("BlackWhiteCorrection" ,m_b_BlackWhiteOpticalCorrection);

   pc_FileWriter->FinishStructure();

   // everything is OK
   return 0;
} // WriteFile

//===========================================================================
//
//===========================================================================

int EffectBox::LoadFile(PericoFileWriter *pc_FileWriter)
{

   PERICO_READ_STRUCTURE_START

   if(   qs_Name.compare("ComicPreBoost")==0)
   {
      m_b_ComicPreBoost = PericoFileWriter::atob(qs_Value);
   }
   else if(   qs_Name.compare("ComicAlgorithmus")==0)
   {
      m_i_ComicAlgorithmus = atoi(qs_Value);
   }
   else if(   qs_Name.compare("ComicLineDectection")==0)
   {
      m_i_ComicLineDetection = atoi(qs_Value);
   }
   else if(   qs_Name.compare("ComicLineDectectionEnabed")==0)
   {
      m_b_ComicLineEnabled = PericoFileWriter::atob(qs_Value);
   }

   else if(   qs_Name.compare("MosaicElementSize")==0)
   {
      m_i_MosaicElementSize = atoi(qs_Value);
   }
   else if(   qs_Name.compare("MosaicForm")==0)
   {
      m_i_MosaicForm = atoi(qs_Value);
   }
   else if(   qs_Name.compare("MosaicVarySize")==0)
   {
      m_b_MosaicVarySize = PericoFileWriter::atob(qs_Value);
   }
   else if(   qs_Name.compare("MosaicShineFactor")==0)
   {
      m_i_MosaicShineFactor = atoi(qs_Value);
   }
   
   else if(   qs_Name.compare("ReduceNumberColors")==0)
   {
      m_i_ReduceNumberOfColors = atoi(qs_Value);
   }
   else if(   qs_Name.compare("ReduceLeastDifference")==0)
   {
      m_i_ReduceLeastDifference = atoi(qs_Value);
   }

   else if(   qs_Name.compare("BlackWhiteFromColor")==0)
   {
      m_p_BlackWhiteFromColor = (Pixel) atoi(qs_Value);
   }
   else if(   qs_Name.compare("BlackWhiteToColor")==0)
   {
      m_p_BlackWhiteToColor = (Pixel) atoi(qs_Value);
   }
   else if(   qs_Name.compare("BlackWhiteCorrection")==0)
   {
      m_b_BlackWhiteOpticalCorrection = PericoFileWriter::atob(qs_Value);
   }

   PERICO_READ_STRUCTURE_END

   // set to Display
   ComicSetEffectAlgorithmusWidget( m_i_ComicAlgorithmus);

   m_ComicPreCalculationBoostCheck->setChecked(m_b_ComicPreBoost);

   m_ComicLineDectionLevel->SetValue( m_i_ComicLineDetection);
   m_ComicLineDetectionEnableCheck->setChecked(m_b_ComicLineEnabled);


   m_MosaicElementSizeSlider->SetValue(m_i_MosaicElementSize);
   if(m_i_MosaicForm == MosaicFormRound)
   {
      m_MosaicFormRoundButton->setChecked(true);
   }
   else
   {
      m_MosaicFormSquareButton->setChecked(true);
   }
   m_MosaicVarySizeCheck->setChecked(m_b_MosaicVarySize);
   m_MosaicShineSlider->SetValue(m_i_MosaicShineFactor);
   
   m_ReduceColorNumberSlider->SetValue(m_i_ReduceNumberOfColors);
   m_ReduceLeastDifferenceSlider->SetValue(m_i_ReduceLeastDifference);
      
   m_BlackWhiteFromColorWidget->SetColor(m_p_BlackWhiteFromColor);
   m_BlackWhiteToColorWidget  ->SetColor(m_p_BlackWhiteToColor);
   m_BlackWhiteOpticalCheck   ->setChecked(m_b_BlackWhiteOpticalCorrection);

   // finaly set all sensitive modes of GUI elements
   SetSensitivity();
   // everything is OK
   return 0;
} // LoadFile

//===========================================================================
//
//===========================================================================

void EffectBox::ComicLineDetectionToggled(bool b_On)
{
   m_b_ComicLineEnabled = b_On;
   SetSensitivity();
} // ComicLineDetectionToggled()

//===========================================================================
//
//===========================================================================

void EffectBox::ComicPreCalculationBoostCheckModeToggled(bool b_On)
{
   m_b_ComicPreBoost = b_On;
} // ComicPreCalculationBoostCheckModeToggled()

//===========================================================================
//
//===========================================================================

void EffectBox::ComicAlgorithmusChanged(int i_Mode)
{
   m_i_ComicAlgorithmus = i_Mode;
} // ComicAlgorithmusChanged()

//===========================================================================
//
//===========================================================================

void EffectBox::ComicLineDetectionChanged(int i_Value)
{
   m_i_ComicLineDetection = i_Value;
} // ComicLineDetectionChanged()

//===========================================================================
//
//===========================================================================

void EffectBox::ComicDoEffect()
{
   PixelBuffer *pc_Image;
   Effects      c_Effects;
   double       d_Level;

   int          i_PreAlgoritm;
   int          i_Algorithm;
   int          i_LineDetection;

   pc_Image = m_pc_GlobalStorage->m_pc_MainImage;

   // make from 0..100 a range from 50 to 22000 exponential
   d_Level = (m_i_ComicLineDetection / 16.66) + 4.0;
   d_Level = exp(d_Level);
   i_LineDetection = (int) d_Level;

   c_Effects.SetComicBlackLineLevel(m_b_ComicLineEnabled,
                                   i_LineDetection);

   if(m_b_ComicPreBoost)
   {
      i_PreAlgoritm =Effects::ComicPrePixelActionColorBoost;
   }
   else
   {
      i_PreAlgoritm =Effects::ComicPrePixelActionCopy;
   }
   
   switch(m_i_ComicAlgorithmus)
   {
   case 0:
      i_Algorithm = Effects::ComicColorActionCopy;
      break;
   case 1:
      i_Algorithm = Effects::ComicColorActionFlatten;
      break;
   case 2:
      i_Algorithm = Effects::ComicColorActionColorBoost;
      break;
   case 3:
      i_Algorithm = Effects::ComicColorActionFlattenAndBoost;
      break;
   default:
      i_Algorithm = Effects::ComicColorActionCopy;
      break;
   }
   c_Effects.SetComicAlgorithmus(i_Algorithm,i_PreAlgoritm);

   // set cursor in waiting mode
   QtUtility::StartWaitingCursor();

   c_Effects.DoComicEffect(pc_Image);
   // now get the image

   // force a global redraw
   m_pc_GlobalStorage->m_pc_ImageFrame->RedrawAll();

   // send image changed message for undo processing
   m_pc_GlobalStorage->m_pc_MessageServer->SendMessage(0,
                                                       this,
                                                       "ImageChanged");
   // set cursor in normal mode
   QtUtility::EndWaitingCursor();

      
} // ComicDoEffect()

//===========================================================================
//
//===========================================================================

void EffectBox::ComicSetEffectAlgorithmusWidget(int i_Mode)
{
   switch(i_Mode)
   {
   case 0:
      m_ComicAlgCopy            ->setChecked(1);
      break;
   case 1:
      m_ComicAlgFlatten         ->setChecked(1);
      break;
   case 2:
      m_ComicAlgBoost           ->setChecked(1);
      break;
   case 3:
      m_ComicAlgFlattenAndBoost ->setChecked(1);
      break;
   }
        
} // SetEffectAlgorithmus()

//===========================================================================
//
//===========================================================================

void EffectBox::MosaicFormChanged(int i_Mode)
{
   if(i_Mode == 0)
   {
      m_i_MosaicForm = MosaicFormRound;
   }
   else if(i_Mode == 1)
   {
      m_i_MosaicForm = MosaicFormSquare;
   }
   
} // MosaicFormChanged()

//===========================================================================
//
//===========================================================================

void EffectBox::MosaicSizeChanged(int i_Value)
{
   m_i_MosaicElementSize = i_Value;
} // MosaicSizeChanged()

//===========================================================================
//
//===========================================================================

void EffectBox::MosaicVarySizeToggled(bool b_On)
{
   m_b_MosaicVarySize = b_On;
} // MosaicVarySizeToggled

//===========================================================================
//
//===========================================================================

void EffectBox::MosaicShineFactorChanged(int i_Value)
{
   m_i_MosaicShineFactor = i_Value;
} // MosaicSizeChanged()

//===========================================================================
//
//===========================================================================

void EffectBox::MosaicDoEffect()
{
   PixelBuffer *pc_Image;
   Effects      c_Effects;

   pc_Image = m_pc_GlobalStorage->m_pc_MainImage;

   // transfer all settings into the effect object
   if( m_i_MosaicForm == MosaicFormRound)
   {
      c_Effects.SetMosaicForm(Effects::MosaicFormRound);
   }
   else if( m_i_MosaicForm == MosaicFormSquare)
   {
      c_Effects.SetMosaicForm(Effects::MosaicFormSquare);
   }
   c_Effects.SetMosaicShineThrough(m_i_MosaicShineFactor);
   c_Effects.SetMosaicSize(m_i_MosaicElementSize,m_b_MosaicVarySize);

   // set cursor in waiting mode
   QtUtility::StartWaitingCursor();

   c_Effects.DoMosaicEffect(pc_Image);

   // force a global redraw
   m_pc_GlobalStorage->m_pc_ImageFrame->RedrawAll();

   // send image changed message for undo processing
   m_pc_GlobalStorage->m_pc_MessageServer->SendMessage(0,
                                                       this,
                                                       "ImageChanged");
   // set cursor in normal mode
   QtUtility::EndWaitingCursor();


} // MosaicDoEffect()

//===========================================================================
//
//===========================================================================

void EffectBox::ReduceColorNumberChanged(int i_Value)
{
   m_i_ReduceNumberOfColors = i_Value;
} // ReduceColorNumberChanged()

//===========================================================================
//
//===========================================================================


void EffectBox::ReduceLeastDifferenceChanged(int i_Value)
{
   m_i_ReduceLeastDifference = i_Value;
} // ReduceLeastDifferenceChanged()

//===========================================================================
//
//===========================================================================

void EffectBox::ReduceDoEffect()
{
   PixelBuffer *pc_Image;
   Effects      c_Effects;

   pc_Image = m_pc_GlobalStorage->m_pc_MainImage;

   // transfer all settings into the effect object
   c_Effects.SetColorReduceLeastDifference(m_i_ReduceLeastDifference);

   // set cursor in waiting mode
   QtUtility::StartWaitingCursor();

   c_Effects.DoColorReduce(pc_Image,m_i_ReduceNumberOfColors);

   // force a global redraw
   m_pc_GlobalStorage->m_pc_ImageFrame->RedrawAll();

   // send image changed message for undo processing
   m_pc_GlobalStorage->m_pc_MessageServer->SendMessage(0,
                                                       this,
                                                       "ImageChanged");
   // set cursor in normal mode
   QtUtility::EndWaitingCursor();


} // ReduceDoEffect()

//===========================================================================
//
//===========================================================================

void EffectBox::BlackWhiteDoEffect()
{
   PixelBuffer *pc_Image;
   Effects      c_Effects;

   pc_Image = m_pc_GlobalStorage->m_pc_MainImage;

   // transfer all settings into the effect object
   c_Effects.SetBlackWhiteColors(m_p_BlackWhiteFromColor,
                                 m_p_BlackWhiteToColor);

   c_Effects.SetBlackWhiteOpticalCorrection(m_b_BlackWhiteOpticalCorrection);

   c_Effects.SetBlackWhitePureMode(m_b_BlackWhitePure);
                                   
   // set cursor in waiting mode
   QtUtility::StartWaitingCursor();

   c_Effects.DoBlackWhiteEffect(pc_Image);

   // force a global redraw
   m_pc_GlobalStorage->m_pc_ImageFrame->RedrawAll();

   // send image changed message for undo processing
   m_pc_GlobalStorage->m_pc_MessageServer->SendMessage(0,
                                                       this,
                                                       "ImageChanged");
   // set cursor in normal mode
   QtUtility::EndWaitingCursor();


} // BlackWhiteDoEffect()

//===========================================================================
//
//===========================================================================

void EffectBox::BlackWhiteCorrectionChecked( bool b_On)
{
   m_b_BlackWhiteOpticalCorrection = b_On;
} // BlackWhiteCorrectionChecked()

//===========================================================================
//
//===========================================================================

void EffectBox::BlackWhiteFromColorChanged(Pixel c_Color)
{
   m_p_BlackWhiteFromColor = c_Color;
} // BlackWhiteFromColorChanged()

//===========================================================================
//
//===========================================================================

void EffectBox::BlackWhiteToColorChanged(Pixel c_Color)
{
   m_p_BlackWhiteToColor = c_Color;
} // BlackWhiteToColorChanged()

//===========================================================================
//
//===========================================================================



void EffectBox::BlackWhitePureChecked(bool b_on)
{
   m_b_BlackWhitePure = b_on;
} // BlackWhitePureChecked()

//===========================================================================
//
//===========================================================================



