//---------------------------------------------------------------------------
// This sourcefile is created by Peter Jodda / peter@jodda.de
// It is under the GPL license.
// If you expand this file, please contact me, that I can integrate the
// enhancements in the original file.
//---------------------------------------------------------------------------
// Module: FileLoader.c
//---------------------------------------------------------------------------
// Purpose: file load and write functions for the Perico program
//---------------------------------------------------------------------------
// file load and write functions for the Perico program

#include <sys/stat.h>
#include <sys/types.h>


#include <qfiledialog.h>

#include "FileLoader.h"
#include "PixelBuffer.h"
#include "ApplicationStorage.h"
#include "PericoFileWriter.h"
#include "MainFrame.h"
#include "BaseData.h"
#include "BaseDataStrings.h"
#include "FilePreview.h"


extern "C"
{
#include <jpeglib.h>
#include <gif_lib.h>
#include <tiffio.h>
#include <png.h>
#include <libexif/exif-data.h>
}

#define JPEG_FILTER "Jpeg (*.jpg )"
#define TIFF_FILTER "Tiff (*.tif )"
#define PNG_FILTER  "Png (*.png )"
#define BMP_FILTER  "True Color Bmp (*.bmp )"
#define PERICO_FILTER  "Perico (*.perico )"
#define ALL_FILTER  "Images (*.jpg *.gif *.tif *.png *.bmp *.perico)"

//===========================================================================
//
//===========================================================================

FileLoaderOptions::FileLoaderOptions()
{
   b_HasSelectionMask    = false;
   b_HasMainFramePointer = false;
   m_pc_MainFrame        = NULL;
   m_b_ExifInfoValid     = false;
   
} // FileLoaderOptions()

//===========================================================================
//
//===========================================================================

int FileLoaderOptions::GetNumberOfExifLines()
{
   return m_sl_ExifInfo.count();
} // GetNumberOfExifLines()

//===========================================================================
//
//===========================================================================

void FileLoaderOptions::ClearExif()
{
   m_sl_ExifInfo.clear();
   m_b_ExifInfoValid     = false;
} // ClearExif()

//===========================================================================
//
//===========================================================================

QString FileLoaderOptions::GetExifLine(unsigned int i_Index)
{
   if(m_sl_ExifInfo.count()>i_Index)
   {
      QString qs_Result;
      qs_Result = m_sl_ExifInfo[ i_Index ];
      return qs_Result;
   }
   else // not a valid line
   {
      return "";
   }
} // GetExifLine
     
//===========================================================================
//
//===========================================================================

void FileLoaderOptions::AddExifLine(QString qs_Line)
{
   m_sl_ExifInfo.append(qs_Line);
   m_b_ExifInfoValid = true;
} // AddExifLine()

//===========================================================================
//
//===========================================================================

bool FileLoaderOptions::IsExifInfoValid()
{
   return m_b_ExifInfoValid;
} // IsExifInfoValid()

//===========================================================================
//
//===========================================================================

QStringList *FileLoaderOptions::GetExifInfo()
{
   return &m_sl_ExifInfo;
} // GetExifInfo()

//===========================================================================
//
//===========================================================================


FileLoader::FileLoader(QWidget *MainWidget)
{
   m_MainWidget            = MainWidget;
   m_pc_CurrentLoadOptions = NULL;

   m_i_MaximumIconWidth    = 128;
   m_i_MaximumIconHeight   = 128;

}  // FileLoader()

//===========================================================================
//
//===========================================================================

FileLoader::~FileLoader()
{
}  // ~FileLoader()

//===========================================================================
//
//===========================================================================

QString FileLoader::AskForFilename(bool b_OpenMode,QString qs_DefaultName)
{
   QString qs_ReturnValue;
   QString qs_DefaultDirectory;
   
   qs_ReturnValue ="";

   if(b_OpenMode)
   {
      QFileDialog *w_FileDialog;
      FilePreview *pc_Preview;
      int         i_ExecReturn;

      pc_Preview = new FilePreview(m_MainWidget,this);
      w_FileDialog = new QFileDialog(m_MainWidget);
      w_FileDialog->setContentsPreviewEnabled( TRUE );
      w_FileDialog->setContentsPreview( pc_Preview, pc_Preview );
      w_FileDialog->setPreviewMode( QFileDialog::Contents );
      
      w_FileDialog->setCaption(QObject::tr("Perico: Load a File"));

      w_FileDialog->addFilter(JPEG_FILTER );
      w_FileDialog->addFilter(PNG_FILTER );
      w_FileDialog->addFilter(TIFF_FILTER );
      w_FileDialog->addFilter(BMP_FILTER );
      w_FileDialog->addFilter(PERICO_FILTER );
      w_FileDialog->addFilter(ALL_FILTER );

      if( ! g_Application->m_qs_LoadDirectory.isEmpty() )
      {
         w_FileDialog->setDir(g_Application->m_qs_LoadDirectory);
      }
      else
      {
         qs_DefaultDirectory = GetImageFileDirectory();
         if(!qs_DefaultDirectory.isEmpty())
         {
            w_FileDialog->setDir(qs_DefaultDirectory);
         }
      }

      // ask for a name
      i_ExecReturn = w_FileDialog->exec();

      // if abort, then return an empty filename
      if(i_ExecReturn == 0)
      {
         // delete the allocated dialog
         delete w_FileDialog;
         return qs_ReturnValue;
      }

      // get the filename
      qs_ReturnValue = w_FileDialog->selectedFile();

      // save the current path
      g_Application->m_qs_LoadDirectory = w_FileDialog->dirPath();

      // delete the allocated dialog
      delete w_FileDialog;

   }
   else // save mode
   {

      QFileDialog *w_FileDialog;
      QString     qs_SelectedFile;
      QString     qs_SelectedFilter;
      QString     qs_SelectedEnding;
      QString     qs_ActualEnding;
      int         i_ExecReturn;

      w_FileDialog = new QFileDialog(m_MainWidget);
      
      w_FileDialog->setCaption(QObject::tr("Perico: Save a File"));

      w_FileDialog->addFilter(JPEG_FILTER );
      w_FileDialog->addFilter(PNG_FILTER );
      w_FileDialog->addFilter(TIFF_FILTER );
      w_FileDialog->addFilter(PERICO_FILTER );
      w_FileDialog->setMode(QFileDialog::AnyFile);

      // if we have a default name, then try to set into
      // that directory
      if( !qs_DefaultName.isEmpty() )
      {
         QString qs_Extension;
         qs_Extension = GetExtension(qs_DefaultName);

         if( ! qs_Extension.isEmpty())
         {
            // remove extension from the name in the box
            QString qs_NameWithoutExtension;
            int i_ExtensionLength;
            int i_TotalLength;
            i_ExtensionLength = qs_Extension.length();
            i_TotalLength     = qs_DefaultName.length();
            // -1, to also remove the point
            qs_NameWithoutExtension = qs_DefaultName.left(i_TotalLength- i_ExtensionLength - 1);

            w_FileDialog->setSelection(qs_NameWithoutExtension);
            
         }
         else
         {
            // set the selection to the filename
            w_FileDialog->setSelection(qs_DefaultName);
         }

         // try to set the extension
         if(qs_Extension.compare("jpg") == 0)
         {
            w_FileDialog->setSelectedFilter( JPEG_FILTER );
         }
         else if(qs_Extension.compare("tiff") == 0)
         {
            w_FileDialog->setSelectedFilter( TIFF_FILTER );
         }
         else if(qs_Extension.compare("png") == 0)
         {
            w_FileDialog->setSelectedFilter( PNG_FILTER );
         }
         else if(qs_Extension.compare("perico") == 0)
         {
            w_FileDialog->setSelectedFilter( PERICO_FILTER );
         }

      }
      else
      {
         if( ! g_Application->m_qs_LoadDirectory.isEmpty() )
         {
            w_FileDialog->setDir(g_Application->m_qs_LoadDirectory);
         }
         else
         {            
            qs_DefaultDirectory = GetImageFileDirectory();
            if(!qs_DefaultDirectory.isEmpty())
            {
               w_FileDialog->setDir(qs_DefaultDirectory);
            }
         }
      }

      // ask for a file
      i_ExecReturn = w_FileDialog->exec();

      // if abort, then return an empty filename
      if(i_ExecReturn == 0)
      {
         // delete the allocated dialog
         delete w_FileDialog;
         return qs_ReturnValue;
      }
      // now get back the current filter an the filename

      qs_SelectedFile = w_FileDialog->selectedFile();
      qs_SelectedFilter = w_FileDialog->selectedFilter();

      // get the current selected ending
      if(qs_SelectedFilter.compare( JPEG_FILTER ) ==0)
      {
         qs_SelectedEnding = "jpg";
      }
      else if(qs_SelectedFilter.compare( TIFF_FILTER ) ==0)
      {
         qs_SelectedEnding = "tif";
      }
      else if(qs_SelectedFilter.compare( PNG_FILTER ) ==0)
      {
         qs_SelectedEnding = "png";
      }
      else if(qs_SelectedFilter.compare( BMP_FILTER ) ==0)
      {
         qs_SelectedEnding = "bmp";
      }
      else if(qs_SelectedFilter.compare( PERICO_FILTER ) ==0)
      {
         qs_SelectedEnding = "perico";
      }
      else
      {
         // default is tiff
         qs_SelectedEnding = "perico";
      }

      // look whether the file has an ending
      qs_ActualEnding = GetExtension(qs_SelectedFile);


      // are file and selected ending the same?
      if(qs_ActualEnding.compare(qs_SelectedEnding) != 0)
      {
         // no, then
         // put selected ending behing name
         qs_SelectedFile = qs_SelectedFile+"."+qs_SelectedEnding;
      }

      // delete the allocated dialog
      delete w_FileDialog;

      // return the filename
      qs_ReturnValue = qs_SelectedFile;

   } // save mode

   return qs_ReturnValue;

} // AskForFilename()

//===========================================================================
//
//===========================================================================


PixelBuffer *FileLoader::LoadFile(QString qs_Filename,FileLoaderOptions *pc_Options )
{
   PixelBuffer *pc_NewBuffer;
   QString      qs_Extension;
   
   m_pc_CurrentLoadOptions = pc_Options;

   pc_NewBuffer = NULL;

   if(pc_Options != NULL)
   {
      pc_Options->b_HasSelectionMask=false;
   }
   
   qs_Extension = GetExtension(qs_Filename);

   if(qs_Extension.compare("jpg") == 0)
   {
      pc_NewBuffer = LoadFileJpeg(qs_Filename);
   }
   else if(qs_Extension.compare("gif") == 0)
   {
      pc_NewBuffer = LoadFileGif(qs_Filename);
   }
   else if(qs_Extension.compare("tif") == 0)
   {
      pc_NewBuffer = LoadFileTif(qs_Filename);
   }
   else if(qs_Extension.compare("png") == 0)
   {
      pc_NewBuffer = LoadFilePng(qs_Filename);
      if(   pc_NewBuffer != NULL
         && pc_Options   != NULL
         && pc_NewBuffer->GetSelectionMask() != NULL)
      {
         if(pc_NewBuffer->IsSelectionMaskUsed())
         {
            pc_Options->b_HasSelectionMask=true;
         }
      }
   }
   else if(qs_Extension.compare("bmp") == 0)
   {
      pc_NewBuffer = LoadFileBmp(qs_Filename);
   }
   else if(qs_Extension.compare("perico") == 0)
   {
      if(   pc_Options != NULL
         && pc_Options->b_HasMainFramePointer
         && pc_Options->m_pc_MainFrame != NULL)
      {
         pc_NewBuffer = LoadFilePerico(qs_Filename,pc_Options);
         
      }
      else
      {
         pc_NewBuffer = LoadFilePerico(qs_Filename);
         if(pc_Options != NULL)
         {
            pc_Options->b_HasSelectionMask=true;
         }
      }
   }

   return pc_NewBuffer;

} // LoadFile()

//===========================================================================
//
//===========================================================================

PixelBuffer *FileLoader::LoadFilePng(QString qs_Filename)
{
   unsigned char *pc_ReadBuffer;
   PixelBuffer   *pc_NewImage ;
   PixelBuffer   *pc_SelectionMask;
   png_structp    pc_Png;
   png_infop      pc_PngInfo;

   FILE          *fp_Infile;
   int            i_Width;
   int            i_Height;
   int            i_Bitdepth;
   int            i_ColorType;
   int            i_Y;
   int            i_X;

   // open the infile
   fp_Infile = fopen(qs_Filename, "rb");
   if(fp_Infile == NULL)
   {
      return NULL;
   }

   pc_NewImage = NULL;

   // create the png struct
   pc_Png = png_create_read_struct(PNG_LIBPNG_VER_STRING,  0, 0, 0);
   if(pc_Png == NULL)
   {
      fclose(fp_Infile);
      return NULL;
   }
   
   // create the info struct
   pc_PngInfo = png_create_info_struct(pc_Png);
   if(pc_PngInfo == NULL)
   {
      fclose(fp_Infile);
      return NULL;
   }

   // attach infile to the io
   png_init_io(pc_Png, fp_Infile);
   
   // read header of the file
   png_read_info(pc_Png, pc_PngInfo);

   i_Width     = pc_PngInfo->width;
   i_Height    = pc_PngInfo->height;
   i_ColorType = pc_PngInfo->color_type;
   i_Bitdepth  = pc_PngInfo->bit_depth;


   // need a valid image size
   if(   i_Width  <=0
      || i_Height <=0 )
   {
      png_destroy_read_struct(&pc_Png, &pc_PngInfo, 0);
      fclose(fp_Infile);
      return NULL;
   }

   // need RGB or RGB_ALPHA
   if(   i_ColorType != PNG_COLOR_TYPE_RGB_ALPHA
      && i_ColorType != PNG_COLOR_TYPE_RGB)
   {
      png_destroy_read_struct(&pc_Png, &pc_PngInfo, 0);
      fclose(fp_Infile);
      return NULL;
   }

   // allocate a new buffer for the pictrue
   pc_NewImage =  new PixelBuffer(i_Width,i_Height,BITNUMBER_LONG);
   if(pc_NewImage == NULL)
   {
      png_destroy_read_struct(&pc_Png, &pc_PngInfo, 0);
      fclose(fp_Infile);
      return NULL;
   }
   pc_SelectionMask = pc_NewImage->GetSelectionMask();

   // allocate a buffer for one line
   pc_ReadBuffer = new unsigned char [i_Width *4];

   // loop for reading the image
   for(i_Y = 0; i_Y < i_Height; i_Y ++)
   {
      unsigned char *pc_In;

      png_read_row(pc_Png, pc_ReadBuffer, 0);
      pc_In = pc_ReadBuffer;
      for(i_X=0;i_X < i_Width;i_X ++)
      {
         unsigned int i_Red;
         unsigned int i_Green;
         unsigned int i_Blue;
         Pixel p;

         i_Red   = (unsigned int) *pc_In++;
         i_Green = (unsigned int) *pc_In++;
         i_Blue  = (unsigned int) *pc_In++;

         p = PIXEL_MAKE(i_Red,i_Green,i_Blue);

         pc_NewImage->SetRaw(i_X,i_Y,p);
         if( i_ColorType == PNG_COLOR_TYPE_RGB_ALPHA)
         {
            Pixel p_Mask;
            p_Mask  = (unsigned int) *pc_In++;
            if(p_Mask != 0)
            {
               pc_SelectionMask->SetRaw(i_X,i_Y,1);       
            }
         } // if alpha type

      } // for i_X
   } // for i_Y
   
   delete pc_ReadBuffer;

   // destroy the png structures
   png_read_end(pc_Png, 0);
   png_destroy_read_struct(&pc_Png, &pc_PngInfo, 0);

   // close the file
   fclose(fp_Infile);

   // return the new image
   return pc_NewImage;
} // LoadFilePng()

//===========================================================================
//
//===========================================================================

PixelBuffer *FileLoader::LoadFileTif(QString qs_Filename)
{
   TIFFRGBAImage  img;
   PixelBuffer   *pc_NewImage ;
   TIFF          *tif;
   char           emsg[1024];

   pc_NewImage = NULL;

   tif = TIFFOpen(qs_Filename, "r");

   if (tif == NULL)
   {
      return NULL;
   }
   
   if (TIFFRGBAImageBegin(&img, tif, 0, emsg))
   {
      uint32  * raster;
      size_t    npixels;

      npixels = img.width * img.height;
      raster = (uint32*) _TIFFmalloc(npixels * sizeof (uint32));

      if (raster != NULL)
      {
         if (TIFFRGBAImageGet(&img, raster, img.width, img.height))
         {
            unsigned int * pui_Source;
            int            i_X;
            int            i_Y;

            pc_NewImage =  new PixelBuffer(img.width,img.height,BITNUMBER_LONG);
            pui_Source   = (unsigned int *) raster;

            // copy all pixel into the new buffer
            for(i_Y = img.height-1;i_Y >=0; i_Y --)
            {
               for(i_X = 0; i_X< (int) img.width; i_X ++)
               {
			      unsigned int ui_Value;
                  unsigned int i_Red;
                  unsigned int i_Green;
                  unsigned int i_Blue;
                  Pixel        p;

                  ui_Value = *pui_Source++;
					 
                  i_Red   = TIFFGetR(ui_Value);
                  i_Green = TIFFGetG(ui_Value);
                  i_Blue  = TIFFGetB(ui_Value);
                     
                  p= PIXEL_MAKE(i_Red,i_Green,i_Blue);
                  pc_NewImage->SetRaw(i_X,i_Y,p);
               } // for i_X
            } // for i_Y
         }
         _TIFFfree(raster);
      }
      TIFFRGBAImageEnd(&img);
   }
   
   TIFFClose(tif);

   return pc_NewImage;
} // LoadFileTif()

//===========================================================================
//
//===========================================================================

int FileLoader::LoadFileTiff(QString qs_Filename,PixelBuffer *pc_Destination)
{
   TIFFRGBAImage  img;
   TIFF          *tif;
   char           emsg[1024];
   int            i_Depth;


   i_Depth = pc_Destination->GetDepth();

   tif = TIFFOpen(qs_Filename, "r");
   if (tif == NULL)
   {
      return -1;
   }

   if(pc_Destination->GetDepth()>= 24)
   {
      if (TIFFRGBAImageBegin(&img, tif, 0, emsg))
      {
         uint32  * raster;
         size_t    npixels;

         npixels = img.width * img.height;
         raster = (uint32*) _TIFFmalloc(npixels * sizeof (uint32));

         if (raster != NULL)
         {
            if (TIFFRGBAImageGet(&img, raster, img.width, img.height))
            {
               unsigned int * pui_Source;
               int            i_X;
               int            i_Y;

               pui_Source   = (unsigned int *) raster;

               // copy all pixel into the new buffer
               for(i_Y = img.height-1;i_Y >=0; i_Y --)
               {
                  for(i_X = 0; i_X< (int) img.width; i_X ++)
                  {
				     unsigned int ui_Value;
                     unsigned int i_Red;
                     unsigned int i_Green;
                     unsigned int i_Blue;
                     Pixel        p;

                     ui_Value = *pui_Source++;
					 
                     i_Red   = TIFFGetR(ui_Value);
                     i_Green = TIFFGetG(ui_Value);
                     i_Blue  = TIFFGetB(ui_Value);
                     
                     p= PIXEL_MAKE(i_Red,i_Green,i_Blue);
                     pc_Destination->SetRaw(i_X,i_Y,p);
                  } // for i_X
               } // for i_Y
            }
            _TIFFfree(raster);
         }
         TIFFRGBAImageEnd(&img);
      }
   }
   else if(pc_Destination->GetDepth() == 8)
   {
      if (TIFFRGBAImageBegin(&img, tif, 0, emsg))
      {
         uint32  * raster;
         size_t    npixels;

         npixels = img.width * img.height;
         raster = (uint32*) _TIFFmalloc(npixels * sizeof (uint32));

         if (raster != NULL)
         {
            if (TIFFRGBAImageGet(&img, raster, img.width, img.height))
            {
               unsigned char* pc_Source;
               int            i_X;
               int            i_Y;

               pc_Source   = (unsigned char *) raster;

               // copy all pixel into the new buffer
               for(i_Y = img.height-1;i_Y >=0; i_Y --)
               {
                  for(i_X = 0; i_X< (int) img.width; i_X ++)
                  {
                     Pixel        p;

                     p   = (unsigned int) *pc_Source ;
                     pc_Source += 4;
                     pc_Destination->SetRaw(i_X,i_Y,p);
                  } // for i_X
               } // for i_Y
            }
            _TIFFfree(raster);
         }
         TIFFRGBAImageEnd(&img);
      }
   }   
   else if(pc_Destination->GetDepth() == 1)
   {
      if (TIFFRGBAImageBegin(&img, tif, 0, emsg))
      {
         uint32  * raster;
         size_t    npixels;

         npixels = img.width * img.height;
         raster = (uint32*) _TIFFmalloc(npixels * sizeof (uint32));

         if (raster != NULL)
         {
            if (TIFFRGBAImageGet(&img, raster, img.width, img.height))
            {
               unsigned char* pc_Source;
               int            i_X;
               int            i_Y;

               pc_Source   = (unsigned char *) raster;

               // copy all pixel into the new buffer
               for(i_Y = img.height-1;i_Y >=0; i_Y --)
               {
                  for(i_X = 0; i_X< (int) img.width; i_X ++)
                  {
                     Pixel        p;
                     
                     p   = (unsigned int) *pc_Source ;
                     pc_Source += 4;
                     if(p>0 )
                     {
                        p= 1;
                     }
                     else
                     {
                        p=0;
                     }
                     pc_Destination->SetRaw(i_X,i_Y,p);
                  } // for i_X
               } // for i_Y
            }
            _TIFFfree(raster);
         }
         TIFFRGBAImageEnd(&img);
      }
   }
   
   TIFFClose(tif);

   // everything is OK
   return 0;
} // LoadFileTiff()

//===========================================================================
//
//===========================================================================

PixelBuffer *FileLoader::LoadFileGif(QString qs_Filename)
{
   ColorMapObject *pc_ColorMap;
   GifPixelType   *pc_PixelSource;
   GifColorType   *Colors;
   GifFileType    * pc_GifFile;
   PixelBuffer    * pc_NewImage ;
   SavedImage     * pc_Image;
   int            i_Width;
   int            i_Height;
   int            i_Y;

   // open the gif file
   pc_GifFile = DGifOpenFileName( qs_Filename);
   if(pc_GifFile == NULL)
   {
      return NULL;
   }

   // get dimensions
   i_Width   = pc_GifFile->SWidth;
   i_Height  = pc_GifFile->SHeight;

   // allocate a buffer for the new image
   pc_NewImage = new PixelBuffer(i_Width,i_Height,BITNUMBER_LONG);

   // read in the complete file
   DGifSlurp(pc_GifFile);

   // get the first image (in case of a multiimage file
   pc_Image = pc_GifFile->SavedImages;

   // get the colormap
   pc_ColorMap = pc_GifFile->SColorMap;
   if( pc_ColorMap == NULL)
   {
      // get map of the single image
      pc_ColorMap = pc_Image->ImageDesc.ColorMap;
   }

   // get pointer to the single colors
   Colors = pc_ColorMap->Colors;

   // get pointer to the rasterbits
   pc_PixelSource = (GifPixelType *) pc_Image->RasterBits;

   // now copy the file into the new pixelbuffer
   if(  pc_Image-> ImageDesc.Interlace != 0
     )
   {
      // interlaced lines
      for(i_Y =0; i_Y < i_Height; i_Y += 8)
      {
         SetGifLine(i_Y,pc_PixelSource,pc_NewImage,Colors);
      }
      for(i_Y =4; i_Y < i_Height; i_Y += 8)
      {
         SetGifLine(i_Y,pc_PixelSource,pc_NewImage,Colors);
      }
      for(i_Y =2; i_Y < i_Height; i_Y += 4)
      {
         SetGifLine(i_Y,pc_PixelSource,pc_NewImage,Colors);
      }
      for(i_Y =1; i_Y < i_Height; i_Y += 2)
      {
         SetGifLine(i_Y,pc_PixelSource,pc_NewImage,Colors);
      }

   }
   else // not interlaced
   {
      for(i_Y =0; i_Y < i_Height; i_Y++)
      {
         SetGifLine(i_Y,pc_PixelSource,pc_NewImage,Colors);

      } // for all lines
   }

   // close the input file
   DGifCloseFile(pc_GifFile);

   // return the result
   return pc_NewImage;

} // LoadFileGif()

//===========================================================================
//
//===========================================================================

void FileLoader::SetGifLine(int i_Line,
                    unsigned char * &pc_Source,
                    PixelBuffer* pc_Destination,
                    GifColorType *Colors)
{
   int i_X;
   int i_Width = pc_Destination->GetWidth();

   for(i_X=0 ; i_X < i_Width; i_X ++)
   {
      int i_Red;
      int i_Green;
      int i_Blue;
      unsigned int i_Index;

      Pixel p_Color;
      i_Index = (unsigned int) *pc_Source++;

      i_Red   = Colors[i_Index].Red;
      i_Green = Colors[i_Index].Green;
      i_Blue  = Colors[i_Index].Blue;

      p_Color =PIXEL_MAKE(i_Red,i_Green,i_Blue);

      //fprintf(stderr,"%d %x\n",i_X,ReadBuffer[i_X]);
      pc_Destination->Set(i_X,i_Line,p_Color);
   }

} // SetGifLine()

//===========================================================================
//
//===========================================================================


PixelBuffer *FileLoader::LoadFileJpeg(QString qs_Filename)
{
   int i_OpenResult;
   int i_Width;
   int i_Height;
   int y;

   PixelBuffer * pc_NewImage = NULL;

   i_OpenResult = JpegSetFilenameForLoad(qs_Filename,
                                        i_Width,
                                        i_Height);

   if(    i_OpenResult <0
       || i_Width  <=0
       || i_Height <=0
      )
   {
      return NULL;
   }
   
   pc_NewImage = new PixelBuffer(i_Width,i_Height,BITNUMBER_LONG);

   for(y=0; y< i_Height;y++)
   {
      unsigned char *puc_Line;

      puc_Line = JpegReadLine();
      if(puc_Line == NULL)
      {
         break;
      }

      if(jpegInfo.out_color_space == JCS_GRAYSCALE)
      {
         int x;

         // put line into the pixelbuffer
         for(x=0; x < i_Width; x++)
         {
            unsigned char r,g,b;
            Pixel p_Pixel;
            r = (unsigned char) *puc_Line;
            g = (unsigned char) *puc_Line;
            b = (unsigned char) *puc_Line++;
            p_Pixel = PIXEL_MAKE(r,g,b) ;
            pc_NewImage->SetRaw(x,y,p_Pixel);
         }
      }
      else // not grayscale // assume rgb
      {
         int x;

         // put line into the pixelbuffer
         for(x=0; x < i_Width; x++)
         {
            unsigned char r,g,b;
            Pixel p_Pixel;
            r = (unsigned char) *puc_Line++;
            g = (unsigned char) *puc_Line++;
            b = (unsigned char) *puc_Line++;
            p_Pixel = PIXEL_MAKE(r,g,b) ;
            pc_NewImage->SetRaw(x,y,p_Pixel);
         }
      }
   } // for all lines

   JpegClose();

   LoadExifData(qs_Filename);
   
   return pc_NewImage;

} // LoadFile()

//===========================================================================
//
//===========================================================================

PixelBuffer *FileLoader::LoadFilePerico(QString qs_Filename,
                     FileLoaderOptions *pc_Options )
{
   PericoFileWriter c_FileWriter;
   PixelBuffer      *pc_MainImage;

   pc_MainImage = NULL;
   
   GetIconforFile(qs_Filename);
   
   c_FileWriter.OpenForRead(qs_Filename);

   {
      QString qs_Name;
      QString qs_Value;
      bool    b_IsStartStructure;
      bool    b_IsEndStructure;
      int     i_GetValueStatus;
      while(true)
      {
         i_GetValueStatus = c_FileWriter.GetValue(qs_Name,
                                  qs_Value,
                                  b_IsStartStructure,
                                  b_IsEndStructure);
      
         if(i_GetValueStatus !=0)
         {
            break;
         }

         if(    qs_Name.compare("MainImage") == 0
             && qs_Value.compare("PixelBuffer") == 0
             && b_IsStartStructure)
         {
            
            // read a main image
            pc_MainImage = new PixelBuffer(&c_FileWriter);
         }
         else if(    pc_Options != NULL 
                  && pc_Options->b_HasMainFramePointer
                  && pc_Options->m_pc_MainFrame != NULL
                  && qs_Name.compare("MainFrame") == 0
                  && qs_Value.compare("MainFrame") == 0
                  && b_IsStartStructure)
         {
            pc_Options->m_pc_MainFrame->LoadFile(&c_FileWriter);
         }
         else if(b_IsEndStructure)
         {
            break;
         }
         else if(b_IsStartStructure)
         {
            c_FileWriter.SkipStructure();
         }
         
      } // while true
      
   } // block for read in

   c_FileWriter.Close();

   return pc_MainImage;
   
} // LoadFilePerico()

//===========================================================================
//
//===========================================================================


int FileLoader::JpegSetFilenameForLoad(QString qs_Filename,
                                       int &i_Width,
                                       int &i_Height)
{
   int  i_RowStride;

   jpegInfo.err =jpeg_std_error(&jpegError);

   jpeg_create_decompress(&jpegInfo);
     
   m_JpegFile = fopen(qs_Filename,"r");

   // file open success?
   if(m_JpegFile == NULL)
   {
      i_Width  = -1;
      i_Height = -1;
      return -1;
   }

   jpeg_stdio_src(&jpegInfo,m_JpegFile);
   
   /*===================================*/
   
   jpeg_read_header(&jpegInfo,TRUE);


   jpeg_start_decompress(&jpegInfo);

   i_RowStride = jpegInfo.output_width * jpegInfo.output_components;

   m_JpegOutputLine = (*jpegInfo.mem->alloc_sarray)
                       ((j_common_ptr) &jpegInfo, 
                       JPOOL_IMAGE,
                       i_RowStride,
                       1);


   i_Width  = jpegInfo.image_width;
   i_Height = jpegInfo.image_height;

   return 0;

} // JpegSetFilenameForLoad()

//===========================================================================
//
//===========================================================================

void FileLoader::JpegClose()
{

   jpeg_finish_decompress(&jpegInfo);

   jpeg_destroy_decompress(&jpegInfo);

   if(m_JpegFile != NULL)
   {
      fclose(m_JpegFile);
      m_JpegFile=NULL;
   }

} // JpegClose()

//===========================================================================
//
//===========================================================================

unsigned char * FileLoader::JpegReadLine()
{
   unsigned char *outbuffer;

   jpeg_read_scanlines(&jpegInfo,m_JpegOutputLine,1);
   outbuffer = (unsigned char *) *(m_JpegOutputLine);

   return outbuffer;

} // JpegReadLine()


//===========================================================================
//
//===========================================================================

int FileLoader::SaveFile(QString qs_Filename,
                PixelBuffer *pc_Image,
                FileLoaderOptions *pc_Options)
{
   QString qs_Extension;
   int     i_Result;

   m_pc_CurrentLoadOptions = pc_Options;
   
   qs_Extension = GetExtension(qs_Filename);
   // fprintf(stderr,"extension = >%s<\n",(const char *) qs_Extension);


   if(qs_Extension.compare("jpg") == 0)
   {
      i_Result = SaveFileJpeg(qs_Filename,pc_Image);
   }
   else if(qs_Extension.compare("tif") == 0)
   {
      i_Result = SaveFileTiff(qs_Filename,pc_Image);
   }
   else if(qs_Extension.compare("png") == 0)
   {
      i_Result = SaveFilePng(qs_Filename,pc_Image);
   }
   else if(qs_Extension.compare("perico") == 0)
   {
      i_Result = SaveFilePerico(qs_Filename,pc_Image,pc_Options);
   }

   

   return 0;
} // SaveFile

//===========================================================================
//
//===========================================================================

int FileLoader::SaveFilePerico(QString qs_Filename,
                PixelBuffer *pc_Image,
                FileLoaderOptions *pc_Options )
{
   PericoFileWriter c_PericoFile;
   QString          qs_IconFilename;
   
   c_PericoFile.OpenForWrite(qs_Filename);

   if(pc_Options != NULL
      && pc_Options->b_HasMainFramePointer
      && pc_Options->m_pc_MainFrame!= NULL)
   {
      pc_Image->WriteFile(&c_PericoFile,"MainImage");
      pc_Options->m_pc_MainFrame->WriteFile(&c_PericoFile,"MainFrame");
   }
   else
   {
      pc_Image->WriteFile(&c_PericoFile,"MainImage");
   }

   SaveIconFile(&c_PericoFile,pc_Image);

   c_PericoFile.Close();

   
   return 0;
} // SaveFilePerico()

//===========================================================================
//
//===========================================================================

int FileLoader::SaveFileJpeg(QString qs_Filename,PixelBuffer *pc_Image)
{
   struct jpeg_compress_struct jpegWriteInfo;
   unsigned char              *pc_Line;
   FILE                       *fp_Outfile;
   bool                       b_Success;
   int                        i_Y;
   int                        i_X;
   int                        i_Width;
   int                        i_JpegQuality;

   
   // create the output file
   fp_Outfile = fopen(qs_Filename,"w");
   if(fp_Outfile == NULL)
   {
      return -1;
   }

   // get the jpeg quality
   if(g_Application->m_pc_BaseData== NULL)
   {
      b_Success = false;
   }
   else
   {
      b_Success = g_Application->m_pc_BaseData->GetValue(
                                  BASEDATA_JPEG_QUALITY,
                                  i_JpegQuality);
   }
   if(!b_Success)
   {
      i_JpegQuality = 97;
   }


   
   // clean up the write parameter buffer
   memset(&jpegWriteInfo,0,sizeof(jpegWriteInfo));

   jpegWriteInfo.err = jpeg_std_error(&jpegError);

   jpeg_create_compress(&jpegWriteInfo);

   jpeg_stdio_dest(&jpegWriteInfo, fp_Outfile);


   i_Width = pc_Image->GetWidth();
   jpegWriteInfo.image_width      = i_Width;
   jpegWriteInfo.image_height     = pc_Image->GetHeight();
   jpegWriteInfo.input_components = 3;
   jpegWriteInfo.in_color_space   = JCS_RGB;

   jpeg_set_defaults(&jpegWriteInfo);

   jpeg_set_quality(&jpegWriteInfo,i_JpegQuality,TRUE);

   jpeg_start_compress(&jpegWriteInfo,TRUE);


   // now write all lines
   pc_Line = new unsigned char[ (i_Width+1)*3];

   for(i_Y=0; i_Y < pc_Image->GetHeight(); i_Y ++)
   {
      unsigned char * pc_InLine;
      JSAMPROW        row_pointer[1];

      pc_InLine = pc_Line;
      for(i_X=0; i_X < i_Width; i_X ++)
      {
         Pixel p_Pixel;
         p_Pixel = pc_Image->Get(i_X,i_Y);
         *pc_InLine ++ = PIXEL_RED(p_Pixel);
         *pc_InLine ++ = PIXEL_GREEN(p_Pixel);
         *pc_InLine ++ = PIXEL_BLUE(p_Pixel);

      }
      row_pointer[0] = (JSAMPROW) pc_Line;
      jpeg_write_scanlines(&jpegWriteInfo,row_pointer,1);
   }

   delete pc_Line;


   // finish compression
   jpeg_finish_compress(&jpegWriteInfo);

   fclose(fp_Outfile);

   jpeg_destroy_compress(&jpegWriteInfo);

   return 0;
   
} // SaveFileJpeg

//===========================================================================
//
//===========================================================================

int FileLoader::SaveFileTiff(QString qs_Filename,PixelBuffer *pc_Image)
{
   unsigned char * puc_Outbuffer;
   TIFF          * p_TiffOut;
   int             i_Outbytes;
   int             i_Y;
   int             i_X;
   int             i_Width;
   int             i_Height;
   int             i_Depth;


   p_TiffOut=TIFFOpen(qs_Filename,"w");

   if(p_TiffOut ==NULL)
   {
      return -1;
   }


   i_Width  = pc_Image->GetWidth();
   i_Height = pc_Image->GetHeight();
   i_Depth  = pc_Image->GetDepth();

   if(i_Depth >=24)
   {
      // True Color image
      TIFFSetField( p_TiffOut, TIFFTAG_SAMPLESPERPIXEL, 3);
      TIFFSetField( p_TiffOut, TIFFTAG_BITSPERSAMPLE, 8);
      TIFFSetField ( p_TiffOut, TIFFTAG_PHOTOMETRIC, PHOTOMETRIC_RGB);
   }
   else if( i_Depth ==8 )
   {
      // Byte Image
      TIFFSetField( p_TiffOut, TIFFTAG_SAMPLESPERPIXEL, 1);
      TIFFSetField( p_TiffOut, TIFFTAG_BITSPERSAMPLE, 8);
      TIFFSetField ( p_TiffOut, TIFFTAG_PHOTOMETRIC, PHOTOMETRIC_MINISBLACK);
   }
   else if( i_Depth ==1 )
   {
      // Bit Image
      TIFFSetField( p_TiffOut, TIFFTAG_SAMPLESPERPIXEL, 1);
      TIFFSetField( p_TiffOut, TIFFTAG_BITSPERSAMPLE, 1);
      TIFFSetField ( p_TiffOut, TIFFTAG_PHOTOMETRIC, PHOTOMETRIC_MINISBLACK);
   }
      

   TIFFSetField( p_TiffOut, TIFFTAG_IMAGELENGTH, i_Height);
   TIFFSetField( p_TiffOut, TIFFTAG_IMAGEWIDTH, i_Width);

   TIFFSetField( p_TiffOut, TIFFTAG_ORIENTATION, 1);
   TIFFSetField ( p_TiffOut, TIFFTAG_SUBFILETYPE, 0);

   // does not work because of patents
   // TIFFSetField ( out, TIFFTAG_COMPRESSION, COMPRESSION_LZW);
   TIFFSetField ( p_TiffOut, TIFFTAG_PLANARCONFIG, PLANARCONFIG_CONTIG);
   TIFFSetField ( p_TiffOut, TIFFTAG_ROWSPERSTRIP, 1);


   i_Outbytes = TIFFScanlineSize(p_TiffOut);

   puc_Outbuffer = (unsigned char *) _TIFFmalloc( i_Outbytes );

   if(i_Depth >=24)
   {
      for(i_Y =0; i_Y < i_Height; i_Y ++)
      {
         unsigned char * pc_Inout;

         pc_Inout = puc_Outbuffer;

         for(i_X =0; i_X < i_Width; i_X ++)
         {
            Pixel p;

            p = pc_Image->Get(i_X,i_Y);
            *pc_Inout++ = PIXEL_RED(p);
            *pc_Inout++ = PIXEL_GREEN(p);
            *pc_Inout++ = PIXEL_BLUE(p);

         }

         TIFFWriteScanline(p_TiffOut, puc_Outbuffer, i_Y, 0 );

      } // for i_Y
   }
   else if(i_Depth ==8)
   {
      for(i_Y =0; i_Y < i_Height; i_Y ++)
      {
         unsigned char * pc_Inout;

         pc_Inout = puc_Outbuffer;

         for(i_X =0; i_X < i_Width; i_X ++)
         {
            Pixel p;

            p = pc_Image->Get(i_X,i_Y);
            *pc_Inout++ = (unsigned char) p;

         }

         TIFFWriteScanline(p_TiffOut, puc_Outbuffer, i_Y, 0 );

      } // for i_Y
   }
   else if(i_Depth ==1)
   {
      for(i_Y =0; i_Y < i_Height; i_Y ++)
      {
         unsigned char * pc_Inout;

         // clean up the memory first
         memset(puc_Outbuffer,0, i_Outbytes );
         pc_Inout = puc_Outbuffer;

         for(i_X =0; i_X < i_Width; i_X ++)
         {
            Pixel p;
            p = pc_Image->Get(i_X,i_Y);

            // is pixel set
            if(p>0)
            {
               unsigned char uc_Mask;
               int           i_Bit;
               int           i_Byte;
               i_Byte  = i_X /8;
               i_Bit   = i_X %8;
               uc_Mask = 0x80>> i_Bit;

               puc_Outbuffer[i_Byte] |= uc_Mask;
            } // pixel set

         } // for i_X

         TIFFWriteScanline(p_TiffOut, puc_Outbuffer, i_Y, 0 );

      } // for i_Y
   } // else DEPTH = 1


   TIFFClose(p_TiffOut);

   _TIFFfree(puc_Outbuffer);

   return 0;

} // SaveFileTiff

//===========================================================================
//
//===========================================================================

int FileLoader::SaveFilePng(QString qs_Filename,PixelBuffer *pc_Image)
{
   unsigned char * puc_Outbuffer;
   PixelBuffer   * pc_Mask;

   png_structp     pc_Png;
   png_infop       pc_PngInfo;
   FILE          * fp_Outfile;
   bool            b_MaskUsed;
   int             i_Y;
   int             i_X;
   int             i_Width;
   int             i_Height;

   pc_Mask    = pc_Image->GetSelectionMask();
   b_MaskUsed = pc_Image->IsSelectionMaskUsed();

   // open the outfile;
   fp_Outfile = fopen(qs_Filename, "wb");
   if(fp_Outfile == NULL)
   {
      return -1;
   }

   // get image dimensions
   i_Width  = pc_Image->GetWidth();
   i_Height = pc_Image->GetHeight();

  
   // create the write structure
   pc_Png = png_create_write_struct(PNG_LIBPNG_VER_STRING, NULL, NULL, NULL);
   if(pc_Png ==NULL)
   {
      fclose(fp_Outfile);
      return -1;
   }

   // create the info structure
   pc_PngInfo = png_create_info_struct(pc_Png);
   if(pc_PngInfo ==NULL)
   {
      png_destroy_write_struct(&pc_Png, NULL);
      fclose(fp_Outfile);
      return -1;
   }

   // attach the outfile
   png_init_io(pc_Png, fp_Outfile);

   if(   pc_Image->GetSelectionMaskActive()
      || b_MaskUsed)
   {
      // write the header information
      png_set_IHDR(pc_Png,
                pc_PngInfo,
                i_Width,
                i_Height,
                8,
                PNG_COLOR_TYPE_RGB_ALPHA,
                PNG_INTERLACE_NONE,
                PNG_COMPRESSION_TYPE_BASE,
                PNG_FILTER_TYPE_BASE);

      png_write_info(pc_Png, pc_PngInfo);


      // create buffer for a single line
      puc_Outbuffer = new unsigned char[i_Width *4];

      // write all lines
      for(i_Y =0; i_Y < i_Height; i_Y ++)
      {
         unsigned char * pc_Inout;

         pc_Inout = puc_Outbuffer;

         for(i_X =0; i_X < i_Width; i_X ++)
         {
            Pixel p;
            Pixel p_Mask;

            p_Mask = pc_Mask->Get(i_X,i_Y);
            if(p_Mask)
            {
               p_Mask = 0xff;
            }
            else
            {
               p_Mask = 0x00;
            }

            p = pc_Image->Get(i_X,i_Y);
            *pc_Inout++ = PIXEL_RED(p);
            *pc_Inout++ = PIXEL_GREEN(p);
            *pc_Inout++ = PIXEL_BLUE(p);
            *pc_Inout++ = p_Mask;

         }
         // write line
         png_write_row(pc_Png, (png_byte *) puc_Outbuffer);

      } // for i_Y
   }
   else
   {
      // write the header information
      png_set_IHDR(pc_Png,
                pc_PngInfo,
                i_Width,
                i_Height,
                8,
                PNG_COLOR_TYPE_RGB,
                PNG_INTERLACE_NONE,
                PNG_COMPRESSION_TYPE_BASE,
                PNG_FILTER_TYPE_BASE);

      png_write_info(pc_Png, pc_PngInfo);


      // create buffer for a single line
      puc_Outbuffer = new unsigned char[i_Width *4];

      // write all lines
      for(i_Y =0; i_Y < i_Height; i_Y ++)
      {
         unsigned char * pc_Inout;

         pc_Inout = puc_Outbuffer;

         for(i_X =0; i_X < i_Width; i_X ++)
         {
            Pixel p;

            p = pc_Image->Get(i_X,i_Y);
            *pc_Inout++ = PIXEL_RED(p);
            *pc_Inout++ = PIXEL_GREEN(p);
            *pc_Inout++ = PIXEL_BLUE(p);

         }
         // write line
         png_write_row(pc_Png, (png_byte *) puc_Outbuffer);

      } // for i_Y

   } // else selection mask not active

   delete puc_Outbuffer;

   // clean up the png structure
   png_write_end(pc_Png, pc_PngInfo);
   png_destroy_write_struct(&pc_Png, &pc_PngInfo);

   // close the outfile
   fclose(fp_Outfile);
   
   return 0;

} // SaveFilePng();

//===========================================================================
// get extension without the point
//===========================================================================

QString FileLoader::GetExtension(QString qs_Name)
{
   QString qs_Result;
   int     i_PointPosition;

   // set empty
   qs_Result= "";

   i_PointPosition = qs_Name.findRev('.');
   if(i_PointPosition <=0)
   {
      return qs_Result;
   }

   // get only extension
   qs_Result = qs_Name.mid(i_PointPosition+1);

   // make it lowercase
   qs_Result = qs_Result.lower();

   return qs_Result;
} // GetExtension()

//===========================================================================
//
//===========================================================================

// get value a pointer as an int (LSB first / intel)

static int GetInt(void *pv_P)
{
   unsigned char *puc_P;
   unsigned int  i_Return;

   puc_P = (unsigned char *) pv_P;

   i_Return  = ((unsigned int) (*puc_P++));
   i_Return |= ((unsigned int) (*puc_P++))<<8;
   i_Return |= ((unsigned int) (*puc_P++))<<16;
   i_Return |= ((unsigned int) (*puc_P++))<<24;
   
   return (int) i_Return;

} // GetInt()
   
//===========================================================================
//
//===========================================================================

// structures for the bmpfile
typedef struct{
   unsigned char m_uc_SignatureB;
   unsigned char m_uc_SignatureM;
   unsigned char m_uc_FLen[4];
   unsigned char m_uc_Dummy[4];
   unsigned char m_uc_Offset[4];
} BMPHEADER;

typedef struct{
   unsigned char m_uc_ISize[4];
   unsigned char m_uc_XMax[4];
   unsigned char m_uc_YMax[4];
   unsigned char m_uc_Planes[2];
   unsigned char m_uc_BitsPerPixel[2];
   unsigned char m_uc_Compress[4];
   unsigned char m_uc_XSize[4];
   unsigned char m_uc_Hdpi[4];
   unsigned char m_uc_Vdpi[4];
   unsigned char m_uc_Cols[4];
   unsigned char m_uc_Colsi[4];
} BMPINFO;


PixelBuffer *FileLoader::LoadFileBmp(QString qs_Filename)
{
   PixelBuffer *pc_NewImage;
   BMPHEADER    c_BmpHeader;
   BMPINFO      c_BmpInfoHeader;
   FILE        *fp_Infile;
   int          i_Read;
   int          i_Width ;
   int          i_Height;
   int          i_BitsPerPixel;

   pc_NewImage = NULL;

   // open the file for reading
   fp_Infile = fopen(qs_Filename,"r");
   if(fp_Infile == NULL)
   {
      return NULL;
   }

   // read the header
   i_Read = fread(&c_BmpHeader,1,sizeof(c_BmpHeader), fp_Infile );
   if(i_Read != sizeof(c_BmpHeader))
   {
      fclose(fp_Infile);
      return NULL;
   }
   
   // read the info header
   i_Read = fread(&c_BmpInfoHeader,1,sizeof(c_BmpInfoHeader), fp_Infile );
   if(i_Read != sizeof(c_BmpInfoHeader))
   {
      fclose(fp_Infile);
      return NULL;
   }

   // check for the signature
   if ( !(     (c_BmpHeader.m_uc_SignatureB == 'B')
           &&  (c_BmpHeader.m_uc_SignatureM == 'M') )
      )
   {
      fclose(fp_Infile);
      return NULL;
   }
      
   i_Width  = GetInt(&c_BmpInfoHeader.m_uc_XMax);
   i_Height = GetInt(&c_BmpInfoHeader.m_uc_YMax);


   i_BitsPerPixel =   (c_BmpInfoHeader.m_uc_BitsPerPixel[0])
                    + (c_BmpInfoHeader.m_uc_BitsPerPixel[1]<<8);

   // only load true color pictures
   if( i_BitsPerPixel == 24)
   {
      unsigned char *puc_LineBuffer;
      int            i_LoadOffset;
      int            i_RunY;
      int            i_LineWidth;

      
      i_LoadOffset = GetInt(& c_BmpHeader.m_uc_Offset);
      fseek( fp_Infile,i_LoadOffset,SEEK_SET);

      // allocate a new buffer for the pictrue
      pc_NewImage =  new PixelBuffer(i_Width,i_Height,BITNUMBER_LONG);

      if(pc_NewImage == NULL)
      {
         fclose(fp_Infile);
         return NULL;
      }

      // allocate a buffer to read a line
      i_LineWidth = 3*i_Width;

      // let it be a multiple of 4
      i_LineWidth = (i_LineWidth+3) & 0xfffffffc;
      
      puc_LineBuffer = new unsigned char[i_LineWidth];
      if( puc_LineBuffer == NULL)
      {
         delete pc_NewImage;
         fclose(fp_Infile);
         return NULL;
      }

      // read all lines
      for(i_RunY=0;i_RunY<i_Height;i_RunY++)
      {
         unsigned char * puc_InBuffer;
         int            i_RunX;

         // read single line
         i_Read = fread(puc_LineBuffer,1,i_LineWidth, fp_Infile );
         if(i_Read != i_LineWidth)
         {
            fclose(fp_Infile);
            delete puc_LineBuffer;
            delete pc_NewImage;
            return NULL;
         }

         
         // copy line into the image
         puc_InBuffer = puc_LineBuffer;
         for(i_RunX=0;i_RunX < i_Width; i_RunX ++)
         {
            unsigned char uc_Red;
            unsigned char uc_Green;
            unsigned char uc_Blue;
            Pixel p_Pixel;

            // get rgb values
            uc_Blue   = *puc_InBuffer++;
            uc_Green  = *puc_InBuffer++;
            uc_Red    = *puc_InBuffer++;

            // make a pixel
            p_Pixel  = PIXEL_MAKE(uc_Red,uc_Green,uc_Blue);

            // put into image
            pc_NewImage->Set(i_RunX,i_Height-(i_RunY+1),p_Pixel);
         } // for RunX
         
      } // for runY

      // delete the linebuffer
      delete puc_LineBuffer;

   } // if ==24 bits per pixel


   // close file
   fclose(fp_Infile);

   // return the new image
   return pc_NewImage;

} // LoadFileBmp()

//===========================================================================
//
//===========================================================================

QString FileLoader::GetImageFileDirectory()
{
   QString qs_Return;
   bool    b_Success;

   // empty string for nothing found
   qs_Return = "";
   
   if(g_Application->m_pc_BaseData== NULL)
   {
      // no base data -> nothing found
      return qs_Return;
   }

   // try to load the default directory
   b_Success = g_Application->m_pc_BaseData->GetValue(
                                  BASEDATA_IMAGE_FILE_DIRECTORY,
                                  qs_Return);
   // not found??
   if(!b_Success)
   {
      return "";
   }

   // return the found name
   return qs_Return;
   
} //GetImageFileDirectory()

//===========================================================================
//
//===========================================================================

int FileLoader::SaveIconFile(PericoFileWriter *pc_FileWriter,
                 PixelBuffer *pc_Image)
{
   QString qs_IconFilename;
   PixelBuffer *pc_Icon;

   pc_Icon = pc_Image->CreateIconFromBuffer(m_i_MaximumIconWidth,
                                            m_i_MaximumIconHeight);

   qs_IconFilename = pc_FileWriter->GetPathFilename("Icon.tif");

   SaveFileTiff(qs_IconFilename,pc_Icon);

   delete pc_Icon ;
   
   return 0;
} // SaveIconFile()

//===========================================================================
//
//===========================================================================

PixelBuffer *FileLoader::GetIconforFile(QString qs_Filename)
{
   PixelBuffer *pc_ReturnImage;
   QString qs_Extension;
   QString qs_IconFile;
   PericoFileWriter c_FileWriter;

   qs_Extension = GetExtension(qs_Filename);
   if(qs_Extension.compare("jpg") == 0)
   {
      return GetIconforJpegFile(qs_Filename);
   }
   if(qs_Extension.compare("tif") == 0)
   {
      return GetIconforTifFile(qs_Filename);
   }
   if(qs_Extension.compare("gif") == 0)
   {
      return GetIconforGifFile(qs_Filename);
   }
   if(qs_Extension.compare("png") == 0)
   {
      return GetIconforPngFile(qs_Filename);
   }
   else if(qs_Extension.compare("perico") != 0)
   {
      //fprintf(stderr,"GetIconforFile - no Perico file\n");
      return NULL;
   }

   qs_IconFile = c_FileWriter.GetIconFileForFile(qs_Filename);

   //fprintf(stderr,"GetIconforFile - qs_IconFile=<%s>\n",
   //      (const char *) qs_IconFile);
   if(qs_IconFile.isEmpty())
   {
      c_FileWriter.DeleteTempIconFile();
      return NULL;
   }

   pc_ReturnImage = LoadFileTif(qs_IconFile);
   // fprintf(stderr,"Image = %x\n",pc_ReturnImage);
   c_FileWriter.DeleteTempIconFile();
   
   return pc_ReturnImage;
} // GetIconforFile()

//===========================================================================
// get a name out of the filename
// remove the extension and all path components
//===========================================================================


QString FileLoader::GetPureName(QString qs_Filename)
{
   QString qs_Return ="";
   int     i_FindResult;

   // look for a final slash
   i_FindResult = qs_Filename.findRev('/');
   if(i_FindResult < 0)
   {
      qs_Return = qs_Filename;
   }
   else
   {
      qs_Return = qs_Filename.mid(i_FindResult+1);
   }

   // next step: find a dot
   
   i_FindResult = qs_Return.findRev('.');
   if(i_FindResult > 0)
   {
      qs_Return = qs_Return.left(i_FindResult);
   }

   return qs_Return;
} // GetPureName

//===========================================================================
//
//===========================================================================


int FileLoader::MakeDirectory(QString qs_Name)
{
   int i_Result;
   // create it, in case of non existent
   i_Result = mkdir((const char *) qs_Name,
            0777);
   return i_Result;
   
} // MakeDirectory()

//===========================================================================
//
//===========================================================================


void FileLoader::ExifContentForeachEntryFunction (ExifEntry *pc_Entry,
                                                   void *vp_UserData)
{
   FileLoader * pc_This;
   const char * cp_Info;
   QString      qs_Name;
   QString      qs_Info;
   char         c_Buffer[2000];

   pc_This= (FileLoader *) vp_UserData;
   
   // fprintf(stderr," ExifContentForeachEntryFunction %x\n",pc_Entry);

   cp_Info = exif_entry_get_value(pc_Entry ,c_Buffer,sizeof(c_Buffer));
   if(cp_Info == NULL)
   {
      return;
   }
   qs_Info = cp_Info;

   qs_Name        =  exif_tag_get_name(pc_Entry->tag);
   // fprintf(stderr,"N=<%s> V=<%s>\n", (const char *)  qs_Name, (const char *)  qs_Info);

   pc_This->AddExifData(qs_Name,qs_Info);
         
} // ExifContentForeachEntryFunction()

//===========================================================================
//
//===========================================================================

void FileLoader::ExifDataForeachContentFunction (ExifContent *pc_Content,
                                                 void *vp_UserData)
{
   // fprintf(stderr," ExifDataForeachContentFunc\n");

   exif_content_foreach_entry (pc_Content,
                                 ExifContentForeachEntryFunction,
                                 vp_UserData);
} // ExifDataForeachContentFunction()

//===========================================================================
//
//===========================================================================

int  FileLoader::LoadExifData(QString qs_Filename)
{
   ExifData * pc_TheExif;

   // no structure available to store the exif data
   if(m_pc_CurrentLoadOptions == NULL)
   {
      return -1;
   }
   
   pc_TheExif = exif_data_new_from_file (qs_Filename);

   if(pc_TheExif == NULL)
   {
      return -1;
   }

   //exif_data_dump(pc_TheExif);

   exif_data_foreach_content (pc_TheExif,
            ExifDataForeachContentFunction,
            (void *) this);
            
   
   exif_data_free(pc_TheExif);
   
   return 0;
} // LoadExifData()

//===========================================================================
//
//===========================================================================

void FileLoader::AddExifData(QString qs_Name,QString qs_Value)
{
   QString qs_Total;

   if(m_pc_CurrentLoadOptions == NULL)
   {
      return;
   }
   
   qs_Total = qs_Name +" = "+qs_Value;
   m_pc_CurrentLoadOptions->AddExifLine(qs_Total);   

} // AddExitData()

//===========================================================================
//
//===========================================================================

PixelBuffer *FileLoader::GetIconforJpegFile(QString qs_Filename)
{
   PixelBuffer *pc_NewImage;
   int i_OriginalWidth;
   int i_OriginalHeight;
   int i_OpenResult;
   int i_Y;
   

   i_OpenResult = JpegSetFilenameForLoad(qs_Filename,
                                       i_OriginalWidth,
                                       i_OriginalHeight);

   if(i_OpenResult <0)
   {
      return NULL;
   }
   if(    i_OriginalWidth  < m_i_MaximumIconWidth
      &&  i_OriginalHeight < m_i_MaximumIconHeight)
   {
      // image is so small, that no reduction is needed
      
      pc_NewImage = new PixelBuffer(i_OriginalWidth,i_OriginalHeight,BITNUMBER_LONG);

      for(i_Y=0; i_Y< i_OriginalHeight;i_Y++)
      {
         unsigned char *puc_Line;

         puc_Line = JpegReadLine();
         if(puc_Line == NULL)
         {
            break;
         }

         if(jpegInfo.out_color_space == JCS_GRAYSCALE)
         {
            int x;

            // put line into the pixelbuffer
            for(x=0; x < i_OriginalWidth; x++)
            {
               unsigned char r,g,b;
               Pixel p_Pixel;
               r = (unsigned char) *puc_Line;
               g = (unsigned char) *puc_Line;
               b = (unsigned char) *puc_Line++;
               p_Pixel = PIXEL_MAKE(r,g,b) ;
               pc_NewImage->SetRaw(x,i_Y,p_Pixel);
            }
         }
         else // not grayscale // assume rgb
         {
            int x;

            // put line into the pixelbuffer
            for(x=0; x < i_OriginalWidth; x++)
            {
               unsigned char r,g,b;
               Pixel p_Pixel;
               r = (unsigned char) *puc_Line++;
               g = (unsigned char) *puc_Line++;
               b = (unsigned char) *puc_Line++;
               p_Pixel = PIXEL_MAKE(r,g,b) ;
               pc_NewImage->SetRaw(x,i_Y,p_Pixel);
            }
         }
      } // for all lines

      JpegClose();

      return pc_NewImage;
   } // if too small
   else
   {
      // calculate a zoom faktor;
      int i_ZoomX;
      int i_ZoomY;
      int i_DestinationX;
      int i_DestinationY;

      i_ZoomX = i_OriginalWidth  / m_i_MaximumIconWidth;
      if( i_OriginalWidth % m_i_MaximumIconWidth != 0)
      {
         i_ZoomX += 1;
      }
      
      i_ZoomY = i_OriginalHeight  / m_i_MaximumIconHeight;
      if( i_OriginalHeight % m_i_MaximumIconHeight != 0)
      {
         i_ZoomY += 1;
      }

      // put in zoomx the largest faktor
      if(i_ZoomX < i_ZoomY)
      {
         i_ZoomX = i_ZoomY;
      }

      pc_NewImage = new PixelBuffer(i_OriginalWidth/i_ZoomX,
                                    i_OriginalHeight/i_ZoomX,
                                    BITNUMBER_LONG);

      i_DestinationY =0;
      for(i_Y=0; i_Y< i_OriginalHeight;i_Y++)
      {
         unsigned char *puc_Line;

         puc_Line = JpegReadLine();
         if( ( i_Y % i_ZoomX) != 0 )
         {
            continue;
         }
         
         if(puc_Line == NULL)
         {
            break;
         }

         i_DestinationX =0;
         if(jpegInfo.out_color_space == JCS_GRAYSCALE)
         {
            int x;

            // put line into the pixelbuffer
            for(x=0; x < i_OriginalWidth; x++)
            {
               unsigned char r,g,b;
               Pixel p_Pixel;
               r = (unsigned char) *puc_Line;
               g = (unsigned char) *puc_Line;
               b = (unsigned char) *puc_Line++;
               if( ( x % i_ZoomX) != 0 )
               {
                  continue;
               }
               p_Pixel = PIXEL_MAKE(r,g,b) ;
               pc_NewImage->SetRaw(i_DestinationX,i_DestinationY,p_Pixel);
               i_DestinationX++;
            }
         }
         else // not grayscale // assume rgb
         {
            int x;

            // put line into the pixelbuffer
            for(x=0; x < i_OriginalWidth; x++)
            {
               unsigned char r,g,b;
               Pixel p_Pixel;
               r = (unsigned char) *puc_Line++;
               g = (unsigned char) *puc_Line++;
               b = (unsigned char) *puc_Line++;
               if( ( x % i_ZoomX) != 0 )
               {
                  continue;
               }
               p_Pixel = PIXEL_MAKE(r,g,b) ;
               pc_NewImage->SetRaw(i_DestinationX,i_DestinationY,p_Pixel);
               i_DestinationX++;
            }
         }
         i_DestinationY++;
         
      } // for all lines

      JpegClose();

      return pc_NewImage;
      
      
   }

   return NULL;
} // GetIconforJpegFile()

//===========================================================================
//
//===========================================================================

PixelBuffer *FileLoader::GetIconforTifFile(QString qs_Filename)
{
   PixelBuffer *pc_Image;
   PixelBuffer *pc_Icon;

   pc_Image = LoadFileTif(qs_Filename);
   if(pc_Image == NULL)
   {
      return NULL;
   }
   pc_Icon = pc_Image->CreateIconFromBuffer(m_i_MaximumIconWidth,
                                            m_i_MaximumIconHeight);

   delete pc_Image;
   return pc_Icon;
} // GetIconforTifFile()

//===========================================================================
//
//===========================================================================

PixelBuffer *FileLoader::GetIconforGifFile(QString qs_Filename)
{
   PixelBuffer *pc_Image;
   PixelBuffer *pc_Icon;

   pc_Image = LoadFileGif(qs_Filename);
   if(pc_Image == NULL)
   {
      return NULL;
   }
   pc_Icon = pc_Image->CreateIconFromBuffer(m_i_MaximumIconWidth,
                                            m_i_MaximumIconHeight);

   delete pc_Image;
   return pc_Icon;
} // GetIconforGifFile()

//===========================================================================
//
//===========================================================================



PixelBuffer *FileLoader::GetIconforPngFile(QString qs_Filename)
{
   PixelBuffer *pc_Image;
   PixelBuffer *pc_Icon;

   pc_Image = LoadFilePng(qs_Filename);
   if(pc_Image == NULL)
   {
      return NULL;
   }
   pc_Icon = pc_Image->CreateIconFromBuffer(m_i_MaximumIconWidth,
                                            m_i_MaximumIconHeight);

   delete pc_Image;
   return pc_Icon;
} // GetIconforPngFile()

//===========================================================================
//
//===========================================================================





