//---------------------------------------------------------------------------
// This sourcefile is created by Peter Jodda / peter@jodda.de
// It is under the GPL license.
// If you expand this file, please contact me, that I can integrate the
// enhancements in the original file.
//---------------------------------------------------------------------------
// Module: FilterBox.c
//---------------------------------------------------------------------------
// Purpose: dialog for all filter settings.
//---------------------------------------------------------------------------

#include <stdio.h>
#include <stdlib.h>
#include <math.h>

#include <qlayout.h>
#include <qgrid.h>
#include <qlabel.h>
#include <qgroupbox.h>
#include <qvgroupbox.h>
#include <qhgroupbox.h>
#include <qfont.h>
#include <qfontmetrics.h>
#include <qcombobox.h>

#include "FilterBox.h"

#include "PixelBuffer.h"
#include "QtUtilities.h"
#include "ImageFrame.h"
#include "MessageServer.h"
#include "Utilities.h"

#include "QULabelSlider.h"
#include "ApplicationStorage.h"

#include "MainFrame.h"

//-----------------------------------------------------------------------------
//
//-----------------------------------------------------------------------------

FilterBox::FilterBox(QWidget *parent,
                        GlobalStorage *pc_Global,
                        MainFrame *pc_MainFrame) :
                StandardDialog(parent,pc_Global),
                Tracer("FilterBox")
{
   QBoxLayout    *l_Layout;


   LoadPosition("FilterBox");
   
   setCaption(tr("Filter"));
   TRACE << "Filter Started\n" ;

   m_i_FilterWidth          = 1;
   m_i_NearDistance         = 30;
   m_b_NearEnabled          = false;

   m_b_OnlyOneColorActive   = false;
   m_i_OnlyOneColorDistance = 30;
   m_p_OnlyOneColor         = 0x564855;


   m_pc_GlobalStorage = pc_Global;

   m_pc_Filter      = new Filter(pc_Global);
   m_pc_Xenohandler = new XenoHandler();
   m_pqsl_XenoFilterNames = m_pc_Xenohandler->GetKernalNames();

   l_Layout = new  QVBoxLayout( m_w_TopWidget,1);


   m_ActivationBox = new QHGroupBox(tr("Filter Operation"),m_w_TopWidget);
   m_ActivationBox->setInsideSpacing(3);
   m_ActivationBox->setColumns(1);

   m_ActivationBox->setInsideMargin(m_ActivationBox->insideMargin()*2);

   l_Layout->addWidget(m_ActivationBox);


   {
      // buttons to establish effect mode

      m_RegularModeButton  = new QUIconButton(m_ActivationBox,
                                                "Filter",
                                                tr("Regular"));

      m_XenoModeButton         = new QUIconButton(m_ActivationBox,
                                                "Xenomorph",
                                                tr("Xenomorph"));


      connect( m_RegularModeButton ,
               SIGNAL(clicked()),
               SLOT( ActivateRegular() ) );
      connect( m_XenoModeButton ,
               SIGNAL(clicked()),
               SLOT( ActivateXeno() ) );
   }

   
   // box for Filter style
   m_FilterStyleBox = new QVGroupBox(tr("Filter Style"),m_w_TopWidget);
   l_Layout->addWidget(m_FilterStyleBox);

   {
      m_FilterWidthSlider = new QULabelSlider(m_FilterStyleBox,
                                       QULabelSlider::LabelStyleAutomatic |
                                          QULabelSlider::LabelStyleEdit
                                       ,
                                       0,15,
                                       m_i_FilterWidth,
                                       tr("Width"),6);

      connect( m_FilterWidthSlider,
               SIGNAL(valueChanged(int)),
               SLOT( FilterWidthChanged(int )) );


   } // box for Filter Settings

   m_OnlyOneColorBox = new QVGroupBox(tr("Filter only one Color"),m_w_TopWidget);
   l_Layout->addWidget(m_OnlyOneColorBox);
   {

      m_OnlyOneColorCheck = new QCheckBox(tr("Active"),
                                              m_OnlyOneColorBox);

      m_OnlyOneColorButton = new QUColorDragLabeledWidget(m_OnlyOneColorBox,
                                               tr("Color to filter"),
                                                 QUColorDragWidget::ModePipette
                                               | QUColorDragWidget::ModeEnableColorselectBox,
                                               m_pc_GlobalStorage,
                                               pc_MainFrame->GetColorSelectBox());
      m_OnlyOneColorButton->SetColor(m_p_OnlyOneColor);

      m_OnlyOneColorDistanceSlider = new QULabelSlider(m_OnlyOneColorBox,
                                       QULabelSlider::LabelStyleAutomatic |
                                          QULabelSlider::LabelStyleEdit
                                       ,
                                       0,100,
                                       m_i_OnlyOneColorDistance,
                                       tr("Distance"),
                                       6);
      connect( m_OnlyOneColorCheck ,
               SIGNAL(toggled(bool)),
               SLOT( FilterOnlyOneClicked(bool) ) );
      connect( m_OnlyOneColorButton,
               SIGNAL(ColorChanged(Pixel)) ,
               SLOT( OnlyOneColorChanged(Pixel ) )
             );
      connect( m_OnlyOneColorDistanceSlider,
               SIGNAL(valueChanged(int)),
               SLOT( OnlyOneDistanceChanged(int )) );


   } // box for nearColor selection

   
   m_NearBox = new QVGroupBox(tr("Extended Option"),m_w_TopWidget);
   l_Layout->addWidget(m_NearBox);
   {

      m_OnlyFilterNearColorsCheck = new QCheckBox(tr("Include only near colors"),
                                              m_NearBox);

      m_NearDistanceSlider = new QULabelSlider(m_NearBox,
                                       QULabelSlider::LabelStyleAutomatic |
                                          QULabelSlider::LabelStyleEdit
                                       ,
                                       0,100,
                                       m_i_NearDistance,
                                       tr("Distance"),
                                       6);
      connect( m_OnlyFilterNearColorsCheck ,
               SIGNAL(toggled(bool)),
               SLOT( FilterOnlyNearClicked(bool) ) );
      connect( m_NearDistanceSlider,
               SIGNAL(valueChanged(int)),
               SLOT( NearDistanceChanged(int )) );


   } // box for nearColor selection

   // --- box for activation
   m_RegularActivationBox = new QHGroupBox(tr("Perform Filter"),m_w_TopWidget);
   l_Layout->addWidget(m_RegularActivationBox);
   {
      // buttons to establish painting mode

      m_MiddleButton  = new QPushButton(tr("Middle"),m_RegularActivationBox);
      m_MedianButton  = new QPushButton(tr("Median"),m_RegularActivationBox);
      m_SharpButton   = new QPushButton(tr("Sharp"),m_RegularActivationBox);

      connect( m_MiddleButton ,
               SIGNAL(clicked()),
               SLOT( ActivateMiddle() ) );

      connect( m_MedianButton ,
               SIGNAL(clicked()),
               SLOT( ActivateMedian() ) );
      connect( m_SharpButton ,
               SIGNAL(clicked()),
               SLOT( ActivateSharp() ) );
   }

   // now the part for the xeno-filters
   m_XenoParameterBox = new QVGroupBox(tr("Xenomorph Filters"),
                                        m_w_TopWidget);
                                        
   l_Layout->addWidget(m_XenoParameterBox);
   {
      QStringList::iterator it_StringIterator;

      /*
      m_XenoKernelSelectionCombo = new QComboBox(false,
                                                  m_XenoParameterBox);

      */

      m_FilterTreePath = new QUTreePath(m_XenoParameterBox);
      m_FilterTreePath->setMinimumWidth(
               g_Application->m_pc_QtUtility->GetMaximalFontWidth()*30
               );
      m_FilterTreePath->setMinimumHeight(
               g_Application->m_pc_QtUtility->GetMaximalFontHeight()*20
               );



      // now add all the filters
      for(  it_StringIterator = m_pqsl_XenoFilterNames->begin() ;
            it_StringIterator != m_pqsl_XenoFilterNames->end() ;
            it_StringIterator ++)
      {
         QString qs_Name;
         qs_Name = * it_StringIterator;
         // m_XenoKernelSelectionCombo->insertItem(qs_Name);
         m_FilterTreePath->AddPath(qs_Name);

      }

      m_FilterTreePath->ViewList(tr("Filters"));

      connect( m_FilterTreePath ,
               SIGNAL(SelectionHasChanged(QString)),
               SLOT( XenoSelectionHasChanged(QString) ) );

      m_XenoStartButton = new QPushButton(tr("Engage"),
                                          m_XenoParameterBox);
      
      m_XenoStartButton->setEnabled(false);
      connect( m_XenoStartButton ,
               SIGNAL(clicked()),
               SLOT( ActivateXenoFilter() ) );

   } // // xeno box

   ActivateRegular(); 

} // FilterBox()

//-----------------------------------------------------------------------------
//
//-----------------------------------------------------------------------------


FilterBox::~FilterBox()
{

} // ~FilterBox()

//-----------------------------------------------------------------------------
//
//-----------------------------------------------------------------------------

void FilterBox::OnMessageReceive(QString qs_MessageType,
                                 void *p_Data)
{
   TRACE << "Message Received member this = " << (int) this << "\n";

   if(qs_MessageType.compare("ImageResize") ==0)
   {
   }
   if(qs_MessageType.compare("ImageNewBuffer") ==0)
   {
   }
} // OnMessageReceive()


//===========================================================================
//
//===========================================================================

void FilterBox::FilterWidthChanged(int i_Width)
{

   m_i_FilterWidth = i_Width;

} // FilterWidthChanged()

//===========================================================================
//
//===========================================================================

void FilterBox::ActivateMiddle()
{
   // initialize structures
   PrepareFiltering(Filter::FilterMiddle);

   // do filtering
   m_pc_Filter->DoFilter();

   // cleanup, redraw and undo
   EndFiltering();
} // ActivateMiddle()

//===========================================================================
//
//===========================================================================

void FilterBox::ActivateMedian()
{
   // initialize structures
   PrepareFiltering(Filter::FilterMedian);

   // do filtering
   m_pc_Filter->DoFilter();

   // cleanup, redraw and undo
   EndFiltering();
} // ActivateMedian()

//===========================================================================
//
//===========================================================================

void FilterBox::ActivateSharp()
{
   // initialize structures
   PrepareFiltering(Filter::FilterSharp);

   // do filtering
   m_pc_Filter->DoFilter();

   // cleanup, redraw and undo
   EndFiltering();
} // ActivateMedian()

//===========================================================================
//
//===========================================================================

void FilterBox::PrepareFiltering(int i_FilterType)
{
   // set cursor in waiting mode
   QtUtility::StartWaitingCursor();

   // transfer all stati into filter object
   m_pc_Filter->SetFilterType(i_FilterType);
   m_pc_Filter->SetFilterWidth(m_i_FilterWidth);
   m_pc_Filter->SetOriginalBuffer(m_pc_GlobalStorage->m_pc_MainImage);
   m_pc_Filter->SetNearDistance(m_i_NearDistance);
   m_pc_Filter->SetNearMode(m_b_NearEnabled);
   m_pc_Filter->SetOnlyOneMode(m_b_OnlyOneColorActive);
   m_pc_Filter->SetOnlyOneColor(m_p_OnlyOneColor);
   m_pc_Filter->SetOnlyOneColorDistance(m_i_OnlyOneColorDistance);

   m_qs_XenoFilterName = m_FilterTreePath->GetSelected();

} // PrepareFiltering

//===========================================================================
//
//===========================================================================

void FilterBox::EndFiltering()
{
   // force a global redraw
   m_pc_GlobalStorage->m_pc_ImageFrame->RedrawAll();

   // send image changed message for undo processing
   m_pc_GlobalStorage->m_pc_MessageServer->SendMessage(0,
                                                       this,
                                                       "ImageChanged");
   // cursor back to normal
   QtUtility::EndWaitingCursor();
} // EndFiltering()

//===========================================================================
//
//===========================================================================

void FilterBox::NearDistanceChanged(int i_Distance)
{
   m_i_NearDistance = i_Distance;
} // NearDistanceChanged()

//===========================================================================
//
//===========================================================================

void FilterBox::FilterOnlyNearClicked(bool b_On)
{
   m_b_NearEnabled = b_On;
} // FilterOnlyNearClicked()

//===========================================================================
//
//===========================================================================

void FilterBox::OnlyOneDistanceChanged(int i_Distance)
{
   m_i_OnlyOneColorDistance = i_Distance;
} // OnlyOneDistanceChanged()

//===========================================================================
//
//===========================================================================

void FilterBox::FilterOnlyOneClicked(bool b_On)
{
   m_b_OnlyOneColorActive = b_On;
} // FilterOnlyOneClicked()

//===========================================================================
//
//===========================================================================

void FilterBox::OnlyOneColorChanged(Pixel p_NewColor)
{
   m_p_OnlyOneColor = p_NewColor;
} // OnlyOneColorChanged()

//===========================================================================
//
//===========================================================================

int FilterBox::WriteFile(PericoFileWriter *pc_FileWriter,
                         QString qs_Name)
{
   pc_FileWriter->StartStructure(qs_Name,"FilterBox");

   pc_FileWriter->AddValue("FilterWidth",m_i_FilterWidth);
   pc_FileWriter->AddValue("NearDistance",m_i_NearDistance);
   pc_FileWriter->AddValue("NearEnabled",m_b_NearEnabled);
   pc_FileWriter->AddValue("OnlyOneColorActive",m_b_OnlyOneColorActive);
   pc_FileWriter->AddValue("OnlyOneColorDistance",m_i_OnlyOneColorDistance);
   pc_FileWriter->AddValue("OnlyOneColor",(int) m_p_OnlyOneColor);

   pc_FileWriter->FinishStructure();

   // everything is OK
   return 0;
} // WriteFile

//===========================================================================
//
//===========================================================================

int FilterBox::LoadFile(PericoFileWriter *pc_FileWriter)
{

   PERICO_READ_STRUCTURE_START

   if(qs_Name.compare("FilterWidth") == 0)
   {
      m_i_FilterWidth = atoi(qs_Value);
   }
   else if(qs_Name.compare("NearDistance") == 0)
   {
      m_i_NearDistance = atoi(qs_Value);
   }
   else if(qs_Name.compare("NearEnabled") == 0)
   {
      m_b_NearEnabled = PericoFileWriter::atob(qs_Value);
   }
   else if(qs_Name.compare("OnlyOneColorActive") == 0)
   {
      m_b_OnlyOneColorActive = PericoFileWriter::atob(qs_Value);
   }
   else if(qs_Name.compare("OnlyOneColorDistance") == 0)
   {
      m_i_OnlyOneColorDistance = atoi(qs_Value);
   }
   else if(qs_Name.compare("OnlyOneColor") == 0)
   {
      m_p_OnlyOneColor = (Pixel) atoi(qs_Value);
   }

   PERICO_READ_STRUCTURE_END

   // set to Display
   m_FilterWidthSlider->SetValue(m_i_FilterWidth);
   
   m_OnlyFilterNearColorsCheck->setChecked(m_b_NearEnabled);
   m_NearDistanceSlider->SetValue(m_i_NearDistance);

   m_OnlyOneColorCheck->setChecked(m_b_OnlyOneColorActive);
   m_OnlyOneColorDistanceSlider->SetValue(m_i_OnlyOneColorDistance);
   m_OnlyOneColorButton->SetColor(m_p_OnlyOneColor);
   
   // everything is OK
   return 0;
} // LoadFile

//===========================================================================
//
//===========================================================================

void FilterBox::ActivateXenoFilter()
{
   // initialize structures
   // -1 means other filter type
   PrepareFiltering(Filter::FilterNothing);
   

   // do filtering
   m_pc_Xenohandler->ProcessImage(m_pc_GlobalStorage->m_pc_MainImage,
                                  m_qs_XenoFilterName);
   
   // cleanup, redraw and undo
   EndFiltering();

} // ActivateXenoFilter()

//===========================================================================
//
//===========================================================================

void FilterBox::XenoSelectionHasChanged(QString qs_Name)
{
   if(qs_Name.isEmpty())
   {
      m_XenoStartButton->setEnabled(false);
   }
   else
   {
      m_XenoStartButton->setEnabled(true);
   }

} // XenoSelectionHasChanged()

//===========================================================================
//
//===========================================================================


void FilterBox::ActivateRegular()
{
   SetWidgetToMode(ModeRegular);

}

//===========================================================================
//
//===========================================================================

void FilterBox::ActivateXeno()
{
   SetWidgetToMode(ModeXeno);

}

//===========================================================================
//
//===========================================================================

void FilterBox::SetWidgetToMode(int i_Mode)
{
   m_i_Mode = i_Mode;

   // first hide everything
   m_RegularModeButton->SetSelected(false);
   m_FilterStyleBox->hide();
   m_OnlyOneColorBox->hide();
   m_NearBox->hide();
   m_RegularActivationBox->hide();
   m_XenoModeButton->SetSelected(false);
   m_XenoParameterBox->hide();

   switch(m_i_Mode)
   {
   case ModeRegular:
      m_FilterStyleBox->show();
      m_OnlyOneColorBox->show();
      m_NearBox->show();
      m_RegularActivationBox->show();
      m_RegularModeButton->SetSelected(true);
      break;
   case ModeXeno:
      m_XenoParameterBox->show();
      m_XenoModeButton->SetSelected(true);
      break;
   }      

   ResizeDisplay();
   // SetSensitivity();

} // SetWidgetToMode()


//===========================================================================
//
//===========================================================================

