//---------------------------------------------------------------------------
// This sourcefile is created by Peter Jodda / peter@jodda.de
// It is under the GPL license.
// If you expand this file, please contact me, that I can integrate the
// enhancements in the original file.
//---------------------------------------------------------------------------
// Module: Geometry.c
//---------------------------------------------------------------------------
// Purpose: basic mathematical geometric functions
//---------------------------------------------------------------------------

#include <math.h>

#include "Utilities.h"
#include "Geometry.h"
#include "PList.h"

//============================================================================
//
//============================================================================

double Geometry::AngleNormalize(double d_RadianAngle)
{
   if( d_RadianAngle < 0)
   {
      d_RadianAngle += (2* M_PI);
   }
   else if(d_RadianAngle > (2* M_PI) )
   {
      d_RadianAngle -= (2* M_PI);
   }
   return d_RadianAngle;
} // AngleNormalize()

//============================================================================
//
//============================================================================

double Geometry::AngleRadianToDegree(double d_RadianAngle)
{
  return d_RadianAngle * 180.0 / M_PI;
} // AngleRadianToDegree()

//============================================================================
//
//============================================================================

double Geometry::AngleDegreeToRadian(double d_DegreeAngle)
{
  return d_DegreeAngle * M_PI / 180.0;

} // AngleDegreeToRadian()

//============================================================================
//
//============================================================================


GeoCoordinate::GeoCoordinate()
{
   m_f_X = 0.0;
   m_f_Y = 0.0;
} // GeoCoordinate()

//============================================================================
//
//============================================================================

GeoCoordinate::GeoCoordinate(float f_X, float f_Y)
{
   m_f_X = f_X;
   m_f_Y = f_Y;
} // GeoCoordinate()

//============================================================================
//
//============================================================================


GeoCoordinate::~GeoCoordinate()
{
} // ~GeoCoordinate()


//============================================================================
//
//============================================================================

void GeoCoordinate::SetXY(float f_X, float f_Y)
{
   m_f_X = f_X;
   m_f_Y = f_Y;
}

//============================================================================
//
//============================================================================

void GeoCoordinate::SetX(float f_X)
{
   m_f_X = f_X;
}

//============================================================================
//
//============================================================================

void GeoCoordinate::SetY( float f_Y)
{
   m_f_Y = f_Y;
}

//============================================================================
//
//============================================================================

float GeoCoordinate::GetX( )
{
   return m_f_X;
}

//============================================================================
//
//============================================================================

float GeoCoordinate::GetY( )
{
   return m_f_Y;
}

//============================================================================
//
//============================================================================

void GeoCoordinate::Transform(float f_DifferenceX, float f_DifferenceY)
{
   m_f_X += f_DifferenceX;
   m_f_Y += f_DifferenceY;

} // Transform()

//============================================================================
//
//============================================================================

void GeoCoordinate::Multiply(float f_FaktorX, float f_FaktorY)
{
   m_f_X *= f_FaktorX;
   m_f_Y *= f_FaktorY;

} // Multiply()

//============================================================================
//
//============================================================================



float GeoCoordinate::DifferenceTo(GeoCoordinate c_Other)
{
   float f_DiffX;
   float f_DiffY;

   f_DiffX = m_f_X - c_Other.m_f_X;
   f_DiffY = m_f_Y - c_Other.m_f_Y;

   return (float) sqrt(f_DiffX*f_DiffX + f_DiffY*f_DiffY);

} // DifferenceTo

//============================================================================
//
//============================================================================



float GeoCoordinate::DifferenceTo(float f_X, float f_Y)
{
   float f_DiffX;
   float f_DiffY;

   f_DiffX = m_f_X - f_X;
   f_DiffY = m_f_Y - f_Y;

   return (float) sqrt(f_DiffX*f_DiffX + f_DiffY*f_DiffY);

} // DifferenceTo


//============================================================================
//
//============================================================================

float GeoCoordinate::AngleTo(float f_X, float f_Y)
{
   float f_DiffX;
   float f_DiffY;

   float f_Angle;

   f_DiffX = m_f_X - f_X;
   f_DiffY = m_f_Y - f_Y;

   // f_Diff is point relative to (0/0)

   if(   f_DiffX==0.0
      && f_DiffY==0.0 )
   {
      // is the center point
      f_Angle = 0.0;
   }
   else // real angle
   {
      f_Angle = atan2((double)f_DiffX,(double) f_DiffY);
   }
   if( f_Angle < 0.0)
   {
      f_Angle += 2* M_PI;
   }

   return f_Angle;

} // AngleTo


//============================================================================
//
//============================================================================

float GeoCoordinate::AngleTo(GeoCoordinate c_Other)
{
   return AngleTo(c_Other.m_f_X, c_Other.m_f_Y);
} // AngleTo()

//============================================================================
//
//============================================================================

void GeoCoordinate::printf()
{
   fprintf(stderr,"%f/%f ",m_f_X,m_f_Y);
} // printf()

//============================================================================
//
//============================================================================

GeoCoordinateList::GeoCoordinateList()
{
   m_i_NumberInArray = 10;
   m_CoordinateList.Resize(m_i_NumberInArray);
   // initial no elements
   m_i_NumberOfElements=0;
} // GeoCoordinateList()

//============================================================================
//
//============================================================================

GeoCoordinateList::GeoCoordinateList(int i_InitialSize)
{
   m_i_NumberInArray = i_InitialSize;
   m_CoordinateList.Resize(m_i_NumberInArray);
   // initial no elements
   m_i_NumberOfElements=i_InitialSize;
} // GeoCoordinateList()

//============================================================================
//
//============================================================================


int GeoCoordinateList::GetCount()
{
   return m_i_NumberOfElements;
} // GetCount()

//============================================================================
//
//============================================================================

void GeoCoordinateList::Clean()
{
   m_i_NumberOfElements=0;
} // Clean()

//============================================================================
//
//============================================================================

int GeoCoordinateList::AddElement(GeoCoordinate c_Element)
{

   if(m_i_NumberOfElements +1 >= m_i_NumberInArray)
   {
      m_i_NumberInArray *=2;
      m_CoordinateList.Resize(m_i_NumberInArray);
   }
   
   m_CoordinateList.Set(m_i_NumberOfElements,c_Element);

   m_i_NumberOfElements ++;

   return m_i_NumberOfElements;
}  // AddElement()

//============================================================================
//
//============================================================================

void GeoCoordinateList::Set(int i_Index,GeoCoordinate c_Element)
{
   if(   i_Index <0
      || i_Index >= m_i_NumberOfElements)
   {
      return;
   }
   m_CoordinateList.Set(i_Index, c_Element);

} // Set()
   
//============================================================================
//
//============================================================================

int GeoCoordinateList::InsertElement(GeoCoordinate c_Element, int i_Position)
{
   int i;

   if(m_i_NumberOfElements +1 >= m_i_NumberInArray)
   {
      m_i_NumberInArray *=2;
      m_CoordinateList.Resize(m_i_NumberInArray);
   }

   if(i_Position> m_i_NumberOfElements)
   {
      return AddElement(c_Element);
   }

   // copy all elements behind it one position to the end
   for(i = m_i_NumberOfElements;
       i > i_Position;
       i--)
   {
      m_CoordinateList.Set(i, m_CoordinateList.Get(i - 1));
   }

   m_CoordinateList.Set(i_Position,c_Element);
   m_i_NumberOfElements ++;

   return i_Position;

}  // InsertElement()

//============================================================================
//
//============================================================================

void GeoCoordinateList::RemoveElement(int i_Index)
{
   int i;
   
   if(   i_Index < 0
      || i_Index >= m_i_NumberOfElements)
   {
      return;
   }

   // copy all elements one to the front
   for(i = i_Index;
       i < m_i_NumberOfElements -1;
       i++)
   {
      m_CoordinateList.Set(i,m_CoordinateList.Get(i + 1));
   }

   m_i_NumberOfElements --;

} // RemoveElement()

//============================================================================
//
//============================================================================

void GeoCoordinateList::Transform(float f_DifferenceX, float f_DifferenceY)
{
   int i;
   for(i=0; i< m_i_NumberOfElements; i++)
   {
      m_CoordinateList.Get(m_i_NumberOfElements)
                                 .Transform(f_DifferenceX,f_DifferenceY);
   }
} // Transform()

//============================================================================
//
//============================================================================

GeoCoordinate GeoCoordinateList::Get(int i_Index)
{
   if (   i_Index <0
       || i_Index >= m_i_NumberOfElements)
   {
      return GeoCoordinate(0,0);
   }
   return m_CoordinateList.Get(i_Index);

} // Get()

//============================================================================
//
//============================================================================

GeoLine::GeoLine()
{
   m_Start.SetXY(0.0,0.0);
   m_End  .SetXY(0.0,0.0);
} // GeoLine()

//============================================================================
//
//============================================================================

GeoLine::GeoLine(float f_StartX,float f_StartY,float f_EndX,float f_EndY)
{
   m_Start.SetXY(f_StartX,f_StartY);
   m_End.SetXY  (f_EndX  ,f_EndY  );
} // GeoLine()

//============================================================================
//
//============================================================================

GeoLine::GeoLine(GeoCoordinate c_Start,GeoCoordinate c_End)
{
   m_Start=c_Start;
   m_End  =c_End;
} // GeoLine()

//============================================================================
//
//============================================================================

GeoCoordinate GeoLine::GetStartPoint()
{
   return  m_Start;
} // GetStartPoint()

//============================================================================
//
//============================================================================


GeoCoordinate GeoLine::GetEndPoint()
{
   return  m_End;
} // GetEndPoint()

//============================================================================
//
//============================================================================

void GeoLine::Set(float f_StartX,float f_StartY,float f_EndX,float f_EndY)
{
   m_Start.SetXY(f_StartX,f_StartY);
   m_End.SetXY  (f_EndX  ,f_EndY  );
} // GeoLine()

//============================================================================
//
//============================================================================

void GeoLine::Set(GeoCoordinate c_Start,GeoCoordinate c_End)
{
   m_Start=c_Start;
   m_End  =c_End;
} // GeoLine()


//============================================================================
//
//============================================================================

void GeoLine::Transform(float f_DifferenceX, float f_DifferenceY)
{
   m_Start.Transform(f_DifferenceX,f_DifferenceY);
   m_End  .Transform(f_DifferenceX,f_DifferenceY);
} // Transform()

//============================================================================
// check, whether two lines cut. If they cut, then deliver the
// cutting point.
//============================================================================

bool GeoLine::CutsLine(GeoLine c_SecondLine, GeoCoordinate &c_CutPoint)
{
   float f_FirstDiffX;
   float f_FirstDiffY;
   float f_SecondDiffX;
   float f_SecondDiffY;
   float f_CommonDiffX;
   float f_CommonDiffY;

   float f_DetermineFirst;
   float f_DetermineSecond;
   float f_DetermineCommon;

   float f_CuttingFactorFirst;
   float f_CuttingFactorSecond;
   
   // Calculate differences

   // make first negative, for the normalform
   f_FirstDiffX= - (m_End.GetX() - m_Start.GetX());
   f_FirstDiffY= - (m_End.GetY() - m_Start.GetY());

   f_SecondDiffX= c_SecondLine.m_End.GetX() - c_SecondLine.m_Start.GetX();
   f_SecondDiffY= c_SecondLine.m_End.GetY() - c_SecondLine.m_Start.GetY();

   f_CommonDiffX = m_Start.GetX() - c_SecondLine.m_Start.GetX();
   f_CommonDiffY = m_Start.GetY() - c_SecondLine.m_Start.GetY();

   f_DetermineCommon =    CalcDeterminante(f_FirstDiffX ,f_SecondDiffX,
                                           f_FirstDiffY ,f_SecondDiffY);
   f_DetermineFirst  =  - CalcDeterminante(f_SecondDiffX,f_CommonDiffX,
                                           f_SecondDiffY,f_CommonDiffY);
   f_DetermineSecond =    CalcDeterminante(f_FirstDiffX ,f_CommonDiffX,
                                           f_FirstDiffY ,f_CommonDiffY);

   if(f_DetermineCommon == 0.0)
   {
      // parallel lines
      // TODO: make test whether the linie lay over each other
      return false;
   }
   
   f_CuttingFactorFirst  = f_DetermineFirst  / f_DetermineCommon;
   f_CuttingFactorSecond = f_DetermineSecond / f_DetermineCommon;

   if(   f_CuttingFactorFirst < 0.0
      || f_CuttingFactorFirst > 1.0
      || f_CuttingFactorSecond < 0.0
      || f_CuttingFactorSecond > 1.0
      )
   {
      // lines dont cut;
      return false;
   }

   // lines cut, now calculate the cutting point


   c_CutPoint.SetX( m_Start.GetX() + f_CuttingFactorFirst * ( -f_FirstDiffX ) );
   c_CutPoint.SetY( m_Start.GetY() + f_CuttingFactorFirst * ( -f_FirstDiffY ) );

   return true;

} // CutsLine

//============================================================================
//
//============================================================================

bool GeoLine::CutsLineExtended(GeoLine c_SecondLine, GeoCoordinate &c_CutPoint,
                              float *pf_CuttingFactorFirst,
                              float *pf_CuttingFactorSecond
                              )
{
   float f_FirstDiffX;
   float f_FirstDiffY;
   float f_SecondDiffX;
   float f_SecondDiffY;
   float f_CommonDiffX;
   float f_CommonDiffY;

   float f_DetermineFirst;
   float f_DetermineSecond;
   float f_DetermineCommon;

   float f_CuttingFactorFirst;
   float f_CuttingFactorSecond;

   // Calculate differences

   // make first negative, for the normalform
   f_FirstDiffX= - (m_End.GetX() - m_Start.GetX());
   f_FirstDiffY= - (m_End.GetY() - m_Start.GetY());

   f_SecondDiffX= c_SecondLine.m_End.GetX() - c_SecondLine.m_Start.GetX();
   f_SecondDiffY= c_SecondLine.m_End.GetY() - c_SecondLine.m_Start.GetY();

   f_CommonDiffX = m_Start.GetX() - c_SecondLine.m_Start.GetX();
   f_CommonDiffY = m_Start.GetY() - c_SecondLine.m_Start.GetY();

   f_DetermineCommon =    CalcDeterminante(f_FirstDiffX ,f_SecondDiffX,
                                           f_FirstDiffY ,f_SecondDiffY);
   f_DetermineFirst  =  - CalcDeterminante(f_SecondDiffX,f_CommonDiffX,
                                           f_SecondDiffY,f_CommonDiffY);
   f_DetermineSecond =    CalcDeterminante(f_FirstDiffX ,f_CommonDiffX,
                                           f_FirstDiffY ,f_CommonDiffY);

   if(f_DetermineCommon == 0.0)
   {
      // parallel lines
      // TODO: make test whether the linie lay over each other
      
      return false;
   }

   f_CuttingFactorFirst  = f_DetermineFirst  / f_DetermineCommon;
   f_CuttingFactorSecond = f_DetermineSecond / f_DetermineCommon;

   if( pf_CuttingFactorFirst!= NULL)
   {
      *pf_CuttingFactorFirst =  f_CuttingFactorFirst;
   }
   if( pf_CuttingFactorSecond!= NULL)
   {
      *pf_CuttingFactorSecond =  f_CuttingFactorSecond;
   }
   
   //  calculate the cutting point

   c_CutPoint.SetX( m_Start.GetX() + f_CuttingFactorFirst * ( -f_FirstDiffX ) );
   c_CutPoint.SetY( m_Start.GetY() + f_CuttingFactorFirst * ( -f_FirstDiffY ) );

   return true;

} // CutsLine

//============================================================================
//
//============================================================================

float GeoLine::CalcDeterminante(float x1,float y1,float x2,float y2)
{
   // classic determinante in 2 dimension
   return (x1*y2) - (y1 * x2);
} // CalcDeterminante()

//============================================================================
// false: nothing is in
// true: there is something in. this line is in c_ResultLine
//============================================================================


bool GeoLine::ClipInRectRegion(RectRegion c_Region,GeoLine &c_ResultLine)
{
   float f_StartX;
   float f_StartY;
   float f_EndX;
   float f_EndY;
   bool  b_StartInRegion;
   bool  b_EndInRegion;

   if(m_Start.GetY() == m_End.GetY())
   {
      return ClipInRectRegionHorizontal(c_Region,c_ResultLine);
   }

   b_StartInRegion = c_Region.PointInRegion(m_Start);
   b_EndInRegion   = c_Region.PointInRegion(m_End);

   if(b_StartInRegion && b_EndInRegion)
   {
      // both are in, so result is entire line
      c_ResultLine = *this;
      return true;
   }

   // at least there ist one cutting point.
   // get coordiantes of the region
   f_StartX = c_Region.GetStartXFloat();
   f_StartY = c_Region.GetStartYFloat();
   f_EndX   = c_Region.GetEndXFloat();
   f_EndY   = c_Region.GetEndYFloat();


   if( b_StartInRegion)
   {
      // only starting point is inside
      GeoCoordinate  c_CuttingPoint;
      GeoLine        c_BorderLine;

      c_ResultLine.m_Start = m_Start;

      // upper Line
      c_BorderLine.Set(f_StartX,f_StartY,f_EndX,f_StartY);
      if( CutsLine(c_BorderLine,c_CuttingPoint) )
      {
         c_ResultLine.m_End = c_CuttingPoint;

         return true;
      }

      // right Line
      c_BorderLine.Set(f_EndX,f_StartY,f_EndX,f_EndY);
      if( CutsLine(c_BorderLine,c_CuttingPoint) )
      {
         c_ResultLine.m_End = c_CuttingPoint;

         return true;
      }

      // bottom Line
      c_BorderLine.Set(f_StartX,f_EndY,f_EndX,f_EndY);
      if( CutsLine(c_BorderLine,c_CuttingPoint) )
      {
         c_ResultLine.m_End = c_CuttingPoint;

         return true;
      }

      // left Line
      c_BorderLine.Set(f_StartX,f_StartY,f_StartX,f_EndY);
      if( CutsLine(c_BorderLine,c_CuttingPoint) )
      {
         c_ResultLine.m_End = c_CuttingPoint;

         return true;
      }

   }
   else if( b_EndInRegion)
   {
      // only starting point is inside
      GeoCoordinate  c_CuttingPoint;
      GeoLine        c_BorderLine;

      c_ResultLine.m_End = m_End;

      // upper Line
      c_BorderLine.Set(f_StartX,f_StartY,f_EndX,f_StartY);
      if( CutsLine(c_BorderLine,c_CuttingPoint) )
      {
         c_ResultLine.m_Start = c_CuttingPoint;

         return true;
      }

      // right Line
      c_BorderLine.Set(f_EndX,f_StartY,f_EndX,f_EndY);
      if( CutsLine(c_BorderLine,c_CuttingPoint) )
      {
         c_ResultLine.m_Start = c_CuttingPoint;

         return true;
      }

      // bottom Line
      c_BorderLine.Set(f_StartX,f_EndY,f_EndX,f_EndY);
      if( CutsLine(c_BorderLine,c_CuttingPoint) )
      {
         c_ResultLine.m_Start = c_CuttingPoint;

         return true;
      }

      // left Line
      c_BorderLine.Set(f_StartX,f_StartY,f_StartX,f_EndY);
      if( CutsLine(c_BorderLine,c_CuttingPoint) )
      {
         c_ResultLine.m_Start = c_CuttingPoint;

         return true;
      }

   }
   else // both point out of the region
   {


      // only starting point is inside
      GeoCoordinate  c_CuttingPoint;
      GeoLine        c_BorderLine;

      int            i_CutsFound;
      
      i_CutsFound =0;

      // upper Line
      c_BorderLine.Set(f_StartX,f_StartY,f_EndX,f_StartY);
      if( CutsLine(c_BorderLine,c_CuttingPoint) )
      {
         if(i_CutsFound ==0)
         {
            c_ResultLine.m_Start = c_CuttingPoint;
            i_CutsFound ++;
         }
         else
         {
            c_ResultLine.m_End    = c_CuttingPoint;
            return true;

         }

      }

      // right Line
      c_BorderLine.Set(f_EndX,f_StartY,f_EndX,f_EndY);
      if( CutsLine(c_BorderLine,c_CuttingPoint) )
      {
         if(i_CutsFound ==0)
         {
            c_ResultLine.m_Start = c_CuttingPoint;
            i_CutsFound ++;
         }
         else
         {
            c_ResultLine.m_End    = c_CuttingPoint;
            return true;

         }
      }

      // bottom Line
      c_BorderLine.Set(f_StartX,f_EndY,f_EndX,f_EndY);
      if( CutsLine(c_BorderLine,c_CuttingPoint) )
      {
         if(i_CutsFound ==0)
         {
            c_ResultLine.m_Start = c_CuttingPoint;
            i_CutsFound ++;
         }
         else
         {
            c_ResultLine.m_End    = c_CuttingPoint;
            return true;

         }
      }

      // left Line
      c_BorderLine.Set(f_StartX,f_StartY,f_StartX,f_EndY);
      if( CutsLine(c_BorderLine,c_CuttingPoint) )
      {
         if(i_CutsFound ==0)
         {
            c_ResultLine.m_Start = c_CuttingPoint;
            i_CutsFound ++;
         }
         else
         {
            c_ResultLine.m_End    = c_CuttingPoint;
            return true;

         }
      }

   } // else both endpoints are outside

   // everything of the line is outside
   return false;
} // ClipInRectRegion()



//============================================================================
// false: nothing is in
// true: there is something in. this line is in c_ResultLine
// line is horizontal, starty and endy are same
//============================================================================


bool GeoLine::ClipInRectRegionHorizontal(RectRegion c_Region,
                                         GeoLine &c_ResultLine)
{
   float f_Y;

   float f_StartX;
   float f_EndX;
   float f_RegionStartX;
   float f_RegionEndX;

   f_Y = m_Start.GetY();

   if(    f_Y < c_Region.GetStartYFloat()
       || f_Y > c_Region.GetEndYFloat()
     )
   {
      // line is upper or lower
      return false;
   }

   f_StartX = m_Start.GetX();
   f_EndX   = m_End.GetX();
   

   // make sure, start x is smaller than end
   if(f_StartX > f_EndX)
   {
      Utilities::swap(f_StartX, f_EndX);
   }

   // get dimensions of the region
   f_RegionStartX = c_Region.GetStartXFloat();
   f_RegionEndX   = c_Region.GetEndXFloat();


   if(f_StartX > f_RegionEndX)
   {
      // total line is right of the box
      return false;
   }
   if(f_EndX < f_RegionStartX)
   {
      // total line is left of the box
      return false;
   }

   // line is overlapping the region
   // clip if necessary

   if(f_StartX < f_RegionStartX)
   {
      f_StartX = f_RegionStartX;
   }
   if(f_EndX > f_RegionEndX)
   {
      f_EndX = f_RegionEndX;
   }

   // set the resultin line
   c_ResultLine.Set(f_StartX,f_Y,f_EndX,f_Y);

   return true;
} // ClipInRectRegionHorizontal()


//============================================================================
// false: nothing is in
// true: there is something in. this line is in c_ResultLine
// line is vertical, startx and endx are same
//============================================================================


bool GeoLine::ClipInRectRegionVertical(RectRegion c_Region,
                                       GeoLine &c_ResultLine)
{
   float f_X;

   float f_StartY;
   float f_EndY;
   float f_RegionStartY;
   float f_RegionEndY;

   f_X = m_Start.GetX();

   if(    f_X < c_Region.GetStartXFloat()
       || f_X > c_Region.GetEndXFloat()
     )
   {
      // line is left or right
      return false;
   }

   f_StartY = m_Start.GetY();
   f_EndY   = m_End.GetY();


   // make sure, start x is smaller than end
   if(f_StartY > f_EndY)
   {
      Utilities::swap(f_StartY, f_EndY);
   }

   // get dimensions of the region
   f_RegionStartY = c_Region.GetStartYFloat();
   f_RegionEndY   = c_Region.GetEndYFloat();


   if(f_StartY > f_RegionEndY)
   {
      // total line is below the box
      return false;
   }
   if(f_EndY < f_RegionStartY)
   {
      // total line is above the box
      return false;
   }

   // line is overlapping the region
   // clip if necessary

   if(f_StartY < f_RegionStartY)
   {
      f_StartY = f_RegionStartY;
   }
   if(f_EndY > f_RegionEndY)
   {
      f_EndY = f_RegionEndY;
   }

   // set the resulting line
   c_ResultLine.Set(f_X,f_StartY,f_X,f_EndY);

   return true;
} // ClipInRectRegionVertical()


//============================================================================
//
//============================================================================

void GeoLine::Get( float &f_StartX,
                   float &f_StartY,
                   float &f_EndX,
                   float &f_EndY)
{
   f_StartX = m_Start.GetX();
   f_StartY = m_Start.GetY();
   f_EndX   = m_End.GetX();
   f_EndY   = m_End.GetY();
} // Get()

//============================================================================
//
// make a perpendicular (senkrechte) of the point to the line.
// look, whether the curring point cuts the line
//============================================================================


double GeoLine::PerpendicularOfPointCutsAt(GeoCoordinate c_Point,
                                           GeoCoordinate &c_CutPoint)
{
   GeoCoordinate c_NewPoint;
   GeoCoordinate c_Cut;
   GeoLine       c_Perpendicular;
   double        d_X;
   double        d_Y;
   bool          b_Cuts;
   float         f_CuttingFactorFirst;
   
   d_X = m_End.GetX() - m_Start.GetX();
   d_Y = m_End.GetY() - m_Start.GetY();

   // subtract crosswise
   c_NewPoint.SetX(c_Point.GetX() - d_Y);
   c_NewPoint.SetY(c_Point.GetY() + d_X);

   // make the perpendicular
   c_Perpendicular.Set(c_Point,c_NewPoint);   

   b_Cuts = CutsLineExtended(c_Perpendicular,
                             c_CutPoint,
                             &f_CuttingFactorFirst,
                             NULL);
   
   return f_CuttingFactorFirst;
   
} // PerpendicularOfPointCutsAt()

//============================================================================
//
//============================================================================

bool GeoLine::IsOnSameSideOfLine(GeoCoordinate c_Point1, GeoCoordinate c_Point2)
{
  float f_DiffX;
  float f_DiffY;
  float f_Point1X;
  float f_Point2X;
  float f_Point1Y;
  float f_Point2Y;
  float f_DeterminePoint1;
  float f_DeterminePoint2;
  bool  b_Result;

  f_DiffX=m_End.GetX() - m_Start.GetX();
  f_DiffY=m_End.GetY() - m_Start.GetY();

  f_Point1X = c_Point1.GetX() - m_Start.GetX();
  f_Point1Y = c_Point1.GetY() - m_Start.GetY();
  f_Point2X = c_Point2.GetX() - m_Start.GetX();
  f_Point2Y = c_Point2.GetY() - m_Start.GetY();


  f_DeterminePoint1=CalcDeterminante(f_Point1X,f_DiffX,f_Point1Y,f_DiffY);
  f_DeterminePoint2=CalcDeterminante(f_Point2X,f_DiffX,f_Point2Y,f_DiffY);


  if(   f_DeterminePoint1>= 0.0
     && f_DeterminePoint2>= 0.0)
  {
    b_Result = true;
  }
  
  else if(   f_DeterminePoint2<=0.0
     && f_DeterminePoint1<=0.0)
  {
    b_Result = true;
  }
  else
  {
     b_Result = false;
  }
/*
  fprintf(stderr," same Side of (%f %f) - (%f %f) of ( %f %f) and (%f %f) is %d\n",
               m_Start.GetX(),m_Start.GetY(),
               m_End.GetX(),m_End.GetY(),
               c_Point1.GetX(),c_Point1.GetY(),
               c_Point2.GetX(),c_Point2.GetY(),
               b_Result);
   fprintf(stderr,"det1 = %f, det2 = %f\n",f_DeterminePoint1,f_DeterminePoint2);               
*/  
  return b_Result;

} // IsOnSameSideOfLine()

//============================================================================
//
//============================================================================

