//---------------------------------------------------------------------------
// This sourcefile is created by Peter Jodda / peter@jodda.de
// It is under the GPL license.
// If you expand this file, please contact me, that I can integrate the
// enhancements in the original file.
//---------------------------------------------------------------------------
// Module: GraphicObjectLayer
//---------------------------------------------------------------------------
// Purpose: layer to hold graphic objects
//---------------------------------------------------------------------------

#include "GraphicObject.h"
#include "GraphicObjectList.h"

#include "GraphicObjectLayer.h"

//===========================================================================
//
//===========================================================================

GraphicObjectLayer::GraphicObjectLayer(QString qs_LayerName)
{
   m_qs_LayerName = qs_LayerName;

   m_b_IsActive = true;

   // allocate the list with all objects
   m_Objects         = new GraphicObjectList(true);

   // allocate the list for the selected objects
   // objects are same pointers into the all objects list!
   // so no autodelete
   m_SelectedObjects = new GraphicObjectList(false);

   // layer is visible by default
   m_b_IsVisible = true;

} // GraphicObjectLayer()

//===========================================================================
//
//===========================================================================

GraphicObjectLayer::GraphicObjectLayer(PericoFileWriter *pc_FileWriter)
{
   m_qs_LayerName = ""; // empty

   m_b_IsActive = true;

   // allocate the list with all objects
   m_Objects         = new GraphicObjectList(true);

   // allocate the list for the selected objects
   // objects are same pointers into the all objects list!
   // so no autodelete
   m_SelectedObjects = new GraphicObjectList(false);

   // layer is visible by default
   m_b_IsVisible = true;

   LoadFile(pc_FileWriter);

} // GraphicObjectLayer()

//===========================================================================
//
//===========================================================================

GraphicObjectLayer::~GraphicObjectLayer()
{
   // first delete all objects
   delete m_Objects;
   
} // ~GraphicObjectLayer()

//===========================================================================
//
//===========================================================================

void GraphicObjectLayer::AddObject(GraphicObject *pc_NewObject)
{
   m_Objects->AddObject(pc_NewObject);

} // AddObject()

//===========================================================================
//
//===========================================================================

int  GraphicObjectLayer::GetObjectCount()
{
   return m_Objects->GetCount();

} // GetObjectGount()

//===========================================================================
//
//===========================================================================

GraphicObject *GraphicObjectLayer::GetObject(int i_Index)
{
   return m_Objects->GetObject(i_Index);

} // GetObject()

//===========================================================================
//
//===========================================================================


bool GraphicObjectLayer::IsVisible()
{
   return m_b_IsVisible;

} // IsVisible()

//===========================================================================
//
//===========================================================================


void GraphicObjectLayer::SetVisible(bool b_Visible)
{
   m_b_IsVisible = b_Visible;

} // SetVisible()

//===========================================================================
//
//===========================================================================

GraphicObject *GraphicObjectLayer::SearchObjectAt(float f_PositionX,
                                                  float f_PositionY,
                                 GraphicObject *pc_StartObject)
{
   GraphicObject * pc_Actual;
   RectRegion      c_ObjectSize;
   int i_NumberOfObjects;
   int i;

   i_NumberOfObjects = m_Objects->GetCount();

   if(pc_StartObject == NULL)
   {
      for(i=0;i<i_NumberOfObjects;i++)
      {
         pc_Actual = m_Objects->GetObject(i);
         if(    pc_Actual->IsSelectable()
             && pc_Actual->IsVisible()    )
         {
            c_ObjectSize = pc_Actual->GetSize();
            if( c_ObjectSize.PointInRegion(f_PositionX,f_PositionY))
            {
               // found an object
               return pc_Actual;
            }
         } // if visible and selectable

      } // for i;

   }
   else // startobject available
   {
      // search from the object to the last, and then from the start
      GraphicObject *pc_Before;
      int            i_ObjectIndex;

      // unused
      i_ObjectIndex = -1;
      pc_Before     = NULL;

      for(i=0;i <i_NumberOfObjects; i++)
      {
         pc_Actual = m_Objects->GetObject(i);
         if(pc_Actual == pc_StartObject)
         {
            i_ObjectIndex =i;
            continue;
         }
         
         // object must be selectable and visible
         if(    !pc_Actual->IsSelectable()
             || !pc_Actual->IsVisible()    )
         {
            continue;
         }

         c_ObjectSize = pc_Actual->GetSize();
         if( c_ObjectSize.PointInRegion(f_PositionX,f_PositionY))
         {
            if(i_ObjectIndex <0)
            {
               // before object
               if(pc_Before == NULL)
               {
                  pc_Before = pc_Actual;
               }
            }
            else
            {
               // found an object behind
               return pc_Actual;
            }
         } // if point in region

      } // for i;

      // found nothing behinde
      if(pc_Before != NULL)
      {
         // but something before.
         return pc_Before;
      }

   } // else startobject available

   // nothing found
   return NULL;
} // SearchObjectAt()

//===========================================================================
//
//===========================================================================

QString GraphicObjectLayer::GetName()
{
   return m_qs_LayerName;
} // GetName()

//===========================================================================
//
//===========================================================================

GraphicObject *GraphicObjectLayer::GetLastSelectedObject()
{
   GraphicObject *pc_LastObject;
   int            i_NumberOfObjects;
   int            i;

   pc_LastObject = NULL;

   i_NumberOfObjects = GetObjectCount();
   for (i=0; i< i_NumberOfObjects; i++)
   {
      GraphicObject *pc_Actual;

      pc_Actual =  GetObject(i);
      if(pc_Actual -> IsSelected())
      {
         pc_LastObject = pc_Actual;
      } // if is selected
   } // for i

   return pc_LastObject;
} // GetLastSelectedObject()

//===========================================================================
//
//===========================================================================

int GraphicObjectLayer::GetNumberOfSelectedObjects(GraphicObject * & pc_FirstObject)
{
   int  i_NumberOfObjects;
   int  i_SelectedObjects;
   int  i;

   i_SelectedObjects = 0;

   i_NumberOfObjects = GetObjectCount();

   pc_FirstObject    = NULL;

   for (i=0; i< i_NumberOfObjects; i++)
   {
      GraphicObject *pc_Actual;

      pc_Actual =  GetObject(i);
      if(pc_Actual -> IsSelected())
      {
         i_SelectedObjects += 1;

         if(pc_FirstObject == NULL)
         {
            pc_FirstObject = pc_Actual;
         }
      } // if is selected
   } // for i

   return i_SelectedObjects;

} // GetNumberOfSelectedObjects()

//===========================================================================
//
//===========================================================================

GraphicObject *GraphicObjectLayer::GetNextSelectedObject(
                               GraphicObject *pc_PreviousObject)
{
   bool b_TakeNext;
   int  i_NumberOfObjects;
   int  i;

   i_NumberOfObjects = GetObjectCount();
   b_TakeNext = false;

   // go through all objects
   for (i=0; i< i_NumberOfObjects; i++)
   {
      GraphicObject *pc_Actual;

      pc_Actual =  GetObject(i);

      // is object selected?
      if(pc_Actual -> IsSelected())
      {
         // signal to take it set??
         if(b_TakeNext)
         {
            // the return it
            return pc_Actual;
         }

         // is it the previous, then signal to take the next
         if(pc_Actual == pc_PreviousObject)
         {
            b_TakeNext = true;
         }
      } // if is selected
   } // for i

   // nothing more found
   return NULL;

} // GetNextSelectedObject()

//===========================================================================
//
//===========================================================================
bool GraphicObjectLayer::CursorOverSelectedObjects(int i_CursorX, int i_CursorY)
{
   RectRegion  c_ObjectSize;
   int         i_NumberOfObjects;
   int         i;


   i_NumberOfObjects = GetObjectCount();

   for (i=0; i< i_NumberOfObjects; i++)
   {
      GraphicObject *pc_Actual;

      pc_Actual =  GetObject(i);
      if(pc_Actual -> IsSelected())
      {
         c_ObjectSize = pc_Actual->GetSize();
         if(c_ObjectSize.PointInRegion(i_CursorX,i_CursorY))
         {
            return true;
         }

      } // if is selected

   } // for i

   return false;

} // CursorOverSelectedObjects()

//===========================================================================
//
//===========================================================================

void GraphicObjectLayer::MoveSelectedObjects(int i_DifferenceX,
                         int i_DifferenceY,
                         RectRegion &c_ChangedRegion)
{
   bool b_FirstObject;
   
   b_FirstObject = true;

   RectRegion  c_ObjectSize;
   int         i_NumberOfObjects;
   int         i;

   // get number of objects
   i_NumberOfObjects = GetObjectCount();

   // go through all objects
   for (i=0; i< i_NumberOfObjects; i++)
   {
      GraphicObject *pc_Actual;

      pc_Actual =  GetObject(i);

      // but move only selected objects
      if(pc_Actual -> IsSelected())
      {
         // add region of unmoved object
         c_ObjectSize = pc_Actual->GetSize();
         if(b_FirstObject)
         {
            b_FirstObject = false;
            c_ChangedRegion = c_ObjectSize;
         }
         else
         {
            c_ChangedRegion.AddRegion(c_ObjectSize);
         }

         pc_Actual->MoveOffset((float) i_DifferenceX,(float) i_DifferenceY);

         // add region of moved object
         c_ObjectSize = pc_Actual->GetSize();
         c_ChangedRegion.AddRegion(c_ObjectSize);

      } // if is selected

   } // for i



} // MoveSelectedObjects()

//===========================================================================
//
//===========================================================================

void GraphicObjectLayer::DeleteObject(GraphicObject *pc_ObjectToDelete)
{
   // remove object from the list
   m_Objects->DeleteObject(pc_ObjectToDelete);

} // DeleteObject()

//===========================================================================
//
//===========================================================================

void GraphicObjectLayer::RemoveAllObjects()
{
   // remove object from the list
   m_Objects->RemoveAllObjects();

} // RemoveAllObjects()

//===========================================================================
//
//===========================================================================

void GraphicObjectLayer::RemoveObject(GraphicObject *pc_ObjectToDelete)
{
   // remove object from the list
   m_Objects->RemoveObject(pc_ObjectToDelete);

} // RemoveObject()

//===========================================================================
//
//===========================================================================

void GraphicObjectLayer::DeleteAllObjects()
{
   // remove object from the list
   m_Objects->DeleteAllObjects();

} // DeleteAllObjects()

//===========================================================================
//
//===========================================================================

void GraphicObjectLayer::SetObjectOrderBottom(GraphicObject *pc_Object)
{
   m_Objects->SetObjectOrderBottom(pc_Object);
}  // SetObjectOrderBottom()

//===========================================================================
//
//===========================================================================

void GraphicObjectLayer::SetObjectOrderTop(GraphicObject *pc_Object)
{
   m_Objects->SetObjectOrderTop(pc_Object);
} // SetObjectOrderTop()

//===========================================================================
//
//===========================================================================

Pixel GraphicObjectLayer::GetPixelAt(int i_X, int i_Y, bool &b_Found)
{
   GraphicObject * pc_Actual;
   Pixel           p_ReturnPixel;
   int             i;
   int             i_Count;
   
   b_Found = false;

   i_Count = m_Objects->GetCount();
   
   for(i=0;i<i_Count;i++)
   {
      pc_Actual = m_Objects->GetObject(i);
      if(pc_Actual->GetObjectType() != GraphicObject::TypeImage)
      {
         continue;
      }
      // now we have an image
      p_ReturnPixel = pc_Actual->GetPixelAt(i_X,i_Y,b_Found);
      if(b_Found)
      {
         return p_ReturnPixel;
      }
      
   } // for i
     
   
   return 0;
   
} // GetPixelAt()

//===========================================================================
//
//===========================================================================

int GraphicObjectLayer::WriteFile(PericoFileWriter *pc_FileWriter,
                                  QString qs_Name)
{
   int i;
   int i_Count;
   
   pc_FileWriter->StartStructure(qs_Name,"GraphicObjectLayer");

   pc_FileWriter->AddValue("LayerName",m_qs_LayerName);

   i_Count = m_Objects->GetCount();
   for(i=0;i<i_Count;i++)
   {
      GraphicObject *pc_Object;
      QString        qs_ObjectName;

      pc_Object = m_Objects->GetObject(i);
      qs_ObjectName.sprintf("Object%d",i);

      pc_Object->WriteFile(pc_FileWriter,qs_ObjectName);

      
   }
   
   pc_FileWriter->FinishStructure();


   // everything is OK
   return 0;
   
} // WriteFile()

//===========================================================================
//
//===========================================================================

int GraphicObjectLayer::LoadFile(PericoFileWriter *pc_FileWriter)
{

   PERICO_READ_STRUCTURE_START

   if( qs_Name.compare("LayerName") == 0)
   {
      m_qs_LayerName = qs_Value;
   }
   else if(     qs_Value.compare("GraphicObject") == 0
            && b_IsStartStructure)
   {
      // ignore the name of this attribute,
      // simply add it to the list
      GraphicObject *pc_Object;
      pc_Object = new GraphicObject(pc_FileWriter);
      AddObject(pc_Object);
   }
   else if(     qs_Value.compare("GraphicObjectEllipse") == 0
            && b_IsStartStructure)
   {
      // ignore the name of this attribute,
      // simply add it to the list
      GraphicObject *pc_Object;
      pc_Object = new GraphicObjectEllipse(pc_FileWriter);
      AddObject(pc_Object);
   }
   else if(     qs_Value.compare("GraphicObjectImage") == 0
            && b_IsStartStructure)
   {
      // ignore the name of this attribute,
      // simply add it to the list
      GraphicObject *pc_Object;
      pc_Object = new GraphicObjectImage(pc_FileWriter);
      AddObject(pc_Object);
   }
   else if(     qs_Value.compare("GraphicObjectLine") == 0
            && b_IsStartStructure)
   {
      // ignore the name of this attribute,
      // simply add it to the list
      GraphicObject *pc_Object;
      pc_Object = new GraphicObjectLine(pc_FileWriter);
      AddObject(pc_Object);
   }
   else if(     qs_Value.compare("GraphicObjectPoint") == 0
            && b_IsStartStructure)
   {
      // ignore the name of this attribute,
      // simply add it to the list
      GraphicObject *pc_Object;
      pc_Object = new GraphicObjectPoint(pc_FileWriter);
      AddObject(pc_Object);
   }
   else if(     qs_Value.compare("GraphicObjectRectangle") == 0
            && b_IsStartStructure)
   {
      // ignore the name of this attribute,
      // simply add it to the list
      GraphicObject *pc_Object;
      pc_Object = new GraphicObjectRectangle(pc_FileWriter);
      AddObject(pc_Object);
   }

   PERICO_READ_STRUCTURE_END

   return 0;
} // GraphicObjectLayer()

//===========================================================================
//
//===========================================================================





