//---------------------------------------------------------------------------
// This sourcefile is created by Peter Jodda / peter@jodda.de
// It is under the GPL license.
// If you expand this file, please contact me, that I can integrate the
// enhancements in the original file.
//---------------------------------------------------------------------------
// Module: GraphikBox.c
//---------------------------------------------------------------------------
// Purpose: Graphik dialog box
//---------------------------------------------------------------------------

#include <stdio.h>
#include <stdlib.h>

#include <qlayout.h>
#include <qgrid.h>
#include <qlabel.h>
#include <qgroupbox.h>
#include <qvgroupbox.h>
#include <qhgroupbox.h>
#include <qfont.h>
#include <qfontmetrics.h>


#include "GraphikBox.h"

#include "PixelBuffer.h"
#include "QtUtilities.h"
#include "ImageFrame.h"
#include "MessageServer.h"
#include "Selection.h"
#include "BufferPainter.h"

#include "QULabelSlider.h"
#include "MaskSettings.h"
#include "MainFrame.h"
#include "MotiveBox.h"
#include "MouseEvent.h"


#define GRAPHIKBOXLAYER "GraphicBox"

//-----------------------------------------------------------------------------
//
//-----------------------------------------------------------------------------

GraphikBox::GraphikBox(QWidget *parent,
                        GlobalStorage *pc_Global,
                        MainFrame *pc_MainFrame) :
                StandardDialog(parent,pc_Global),
                Tracer("GraphikBox")
{
   QBoxLayout   *l_Layout;

   QHGroupBox   *ActivationBox;
   QVGroupBox   *LineBox;
   QVGroupBox   *TransparencyBox;

   LoadPosition("GraphikBox");
   
   l_Layout = new  QVBoxLayout( m_w_TopWidget,1);

   // -- box with the activation buttons
   ActivationBox = new QHGroupBox(tr("Graphics"),m_w_TopWidget);
   ActivationBox->setColumns(2);
   ActivationBox->setInsideSpacing(3);
   l_Layout->addWidget(ActivationBox);

   m_pc_MainFrame     = pc_MainFrame;
   m_pc_GlobalStorage = pc_Global;

   setCaption(tr("Draw Graphic"));
   
   m_i_LineWidth         = 1;
   m_b_DrawInner         = false;
   m_b_TransparencyOn    = false;
   m_i_TransparencyFactor= 50;

   m_i_ConstructMode             = ConstructModeOff;
   m_b_ContructIgnoreRedraw      = false;
   m_i_ConstructAddPointPosition = -1;
   m_pc_ContructEditedObject     = NULL;
   
   // create a new layer
   m_pc_ObjectLayer= m_pc_GlobalStorage->m_pc_ObjectMain
                                 ->CreateLayer(GRAPHIKBOXLAYER);
   m_pc_ObjectLayer->SetVisible(false);
   m_pc_RubberLine = new GraphicObjectLine(0,0,0,0);
   m_pc_ObjectLayer->AddObject(m_pc_RubberLine);
   m_pc_RubberStartPoint = NULL;
   m_pc_RubberLine->SetVisible(false);
   m_pc_ContructEditedSpline     = new GraphicObjectSpline();
   m_pc_ContructEditedSpline->SetVisible(false);
   m_pc_ContructEditedSpline->SetSelectable(false);
   m_pc_ObjectLayer->AddObject(m_pc_ContructEditedSpline);


   m_pc_BufferPainter = new BufferPainter();
   m_pc_BufferPainter->SetLineWidth(m_i_LineWidth);
   m_pc_BufferPainter->SetDrawInnerEnable(false);
   m_pc_BufferPainter->SetTransparencyEnable(false);
   m_pc_BufferPainter->SetTransparencyFactor(50);


   {
      // buttons to establish painting mode

      m_PointButton     = new QUIconButton(ActivationBox,
                                            "GraphPoint",
                                            tr("Point"));
      m_LineButton      = new QUIconButton(ActivationBox,
                                            "GraphLine",
                                            tr("Line"));
      m_QuadratButton   = new QUIconButton(ActivationBox,
                                            "GraphQuadrat",
                                             tr("Quadrat"));
      m_RectangleButton = new QUIconButton(ActivationBox,
                                            "GraphRectangle",
                                            tr("Rectangle"));
      m_CircleButton    = new QUIconButton(ActivationBox,
                                            "GraphCircle",
                                            tr("Circle"));
      m_EllipseButton   = new QUIconButton(ActivationBox,
                                            "GraphEllipse",
                                            tr("Ellipse"));
      m_FreeDrawButton   = new QUIconButton(ActivationBox,
                                             "GraphFree",
                                             tr("Free Hand")
                                             );
      m_ConstructButton = new QUIconButton(ActivationBox,
                                             "GraphConstruct",
                                             tr("Complex")
                                             );

      connect( m_PointButton ,
               SIGNAL(clicked()),
               SLOT( ActivatePoint() ) );
      connect( m_LineButton,
               SIGNAL(clicked()),
               SLOT( ActivateLine()) );
      connect( m_QuadratButton,
               SIGNAL(clicked()),
               SLOT( ActivateQuadrat()) );
      connect( m_RectangleButton,
               SIGNAL(clicked()),
               SLOT( ActivateRectangle()) );
      connect( m_EllipseButton,
               SIGNAL(clicked()),
               SLOT( ActivateEllipse() ) );
      connect( m_CircleButton, 
               SIGNAL(clicked()),
               SLOT( ActivateCircle() ) );
      connect( m_FreeDrawButton, 
               SIGNAL(clicked()), 
               SLOT( ActivateFreeDraw() ) );
      connect( m_ConstructButton,
               SIGNAL(clicked()),
               SLOT( ActivateConstruct() ) );
   }

   // box with the line styles
   LineBox = new QVGroupBox(tr("Line Style"),m_w_TopWidget);
   l_Layout->addWidget(LineBox);

   {
      m_LineWidthSlider = new QULabelSlider(LineBox,
                                       QULabelSlider::LabelStyleAutomatic |
                                          QULabelSlider::LabelStyleEdit
                                       ,
                                       0,30,
                                       m_i_LineWidth,
                                       tr("Line Width"),6);

      m_LineColorWidget = new QUColorOrPattern(LineBox,
                                               m_pc_GlobalStorage,
                                               pc_MainFrame->GetColorSelectBox()
                                               );
      m_LineColorWidget->SetColor(0x00ff00);
      

      connect( m_LineWidthSlider,
               SIGNAL(valueChanged(int)),
               SLOT( LineThicknessChanged(int )) );

      connect( m_LineColorWidget,
               SIGNAL(ColorChanged(Pixel)) ,
               SLOT( LineColorChanged(Pixel ) )
             );
      connect( m_LineColorWidget,
               SIGNAL(ImageDropped(PixelBuffer *)) ,
               SLOT( LinePatternChanged(PixelBuffer * ) )
             );
      connect( m_LineColorWidget,
               SIGNAL(ModeChange(BrushMode)) ,
               SLOT( LineModeChanged(BrushMode ) )
             );

   } // box for line style


   // -- box for the inner style
   m_InnerBox = new QVGroupBox(tr("Inner Style"),m_w_TopWidget);
   l_Layout->addWidget(m_InnerBox);

   {
      // inner is off
      m_DrawInnerEnable = new QCheckBox(tr("Draw Inner"),m_InnerBox);

      m_InnerColorWidget = new QUColorOrPattern(m_InnerBox,
                                               m_pc_GlobalStorage,
                                               pc_MainFrame->GetColorSelectBox());
      m_InnerColorWidget->SetColor(0x044ff);

      connect( m_DrawInnerEnable,
               SIGNAL(toggled(bool)) ,
               SLOT( DrawInnerChanged(bool) )
             );

      connect( m_InnerColorWidget,
               SIGNAL(ColorChanged(Pixel)) ,
               SLOT( InnerColorChanged(Pixel ) )
             );
      connect( m_InnerColorWidget,
               SIGNAL(ImageDropped(PixelBuffer *)) ,
               SLOT( InnerPatternChanged(PixelBuffer * ) )
             );
      connect( m_InnerColorWidget,
               SIGNAL(ModeChange(BrushMode)) ,
               SLOT( InnerModeChanged(BrushMode ) )
             );

   } // box for inner settings

   // box for transparency draw mode
   TransparencyBox = new QVGroupBox(tr("Transparency"),m_w_TopWidget);
   l_Layout->addWidget(TransparencyBox);

   {
      // inner is off
      m_TransparencyEnable = new QCheckBox(tr("Enable"),TransparencyBox);

      m_TransparencySlider = new QULabelSlider(TransparencyBox,
                                       QULabelSlider::LabelStyleAutomatic |
                                          QULabelSlider::LabelStyleEdit
                                       ,
                                       0,100,
                                       m_i_TransparencyFactor,
                                       tr("Factor"),6);

      connect( m_TransparencyEnable,
               SIGNAL(toggled(bool)) ,
               SLOT( TransparencyEnableChanged(bool) )
             );
      connect( m_TransparencySlider,
               SIGNAL(valueChanged(int)),
               SLOT( TransparencyFactorChanged(int )) );

   } // block for tranparency box


   // final settings
   m_pc_BufferPainter->SetBorderColor(m_LineColorWidget->GetColor());
   m_pc_BufferPainter->SetInnerColor(m_InnerColorWidget->GetColor());

   m_ConstructBox = new QVGroupBox(tr("Construction"),m_w_TopWidget);
   l_Layout->addWidget(m_ConstructBox);
   {
      QHGroupBox *w_PolygonBox;
      QHGroupBox *w_PointBox;
      QWidget    *w_StateCheckWidget;
      QHBoxLayout *w_StateCheckBox;
      
      w_PolygonBox = new QHGroupBox(tr("Polygon"),m_ConstructBox);
      m_ConstructNewPolygonButton = new QPushButton(tr("Start new"),w_PolygonBox);
      m_ConstructDrawIntoButton = new QPushButton(tr("Draw into Image"),w_PolygonBox);
      
      w_PointBox = new QHGroupBox(tr("Points"),m_ConstructBox);
      m_ConstructEditPointButton   = new QPushButton(tr("edit"),w_PointBox);
      m_ConstructAddPointButton    = new QPushButton(tr("add"),w_PointBox);
      m_ConstructDeletePointButton = new QPushButton(tr("delete"),w_PointBox);

      w_StateCheckWidget = new QWidget(m_ConstructBox);
      w_StateCheckBox=new QHBoxLayout(w_StateCheckWidget,2);
      m_ConstructClosedFigureCheck = new QCheckBox(tr("Closed Figure"),w_StateCheckWidget);      
      m_ConstructClosedFigureCheck->setChecked(true);
      
      m_ConstructSplineCheck = new QCheckBox(tr("Spline"),w_StateCheckWidget);
      w_StateCheckBox->addWidget(m_ConstructClosedFigureCheck);
      w_StateCheckBox->addWidget(m_ConstructSplineCheck);
      
      
      connect( m_ConstructNewPolygonButton,
               SIGNAL(clicked()) ,
               SLOT( ConstructNewPolygon() )
             );
      connect( m_ConstructDrawIntoButton,
               SIGNAL(clicked()) ,
               SLOT( ConstructDrawPolygon() )
             );
      connect( m_ConstructEditPointButton,
               SIGNAL(clicked()) ,
               SLOT(ConstructEditPoint() )
             );
      connect( m_ConstructAddPointButton,
               SIGNAL(clicked()) ,
               SLOT(ConstructAddPoint() ) 
             );
      connect( m_ConstructDeletePointButton,
               SIGNAL(clicked()) ,
               SLOT( ConstructDeletePoint() )
             );
      connect( m_ConstructClosedFigureCheck,
               SIGNAL(toggled(bool)) ,
               SLOT( ConstructClosedFigureChanged(bool) )
             );
      connect( m_ConstructSplineCheck,
               SIGNAL(toggled(bool)) ,
               SLOT( ConstructSplineChanged(bool) )
             );

   } // Construction box

   
   m_InnerBox->hide();

   SetModeTo(GraphikNone);
   
} // GraphikBox()

//-----------------------------------------------------------------------------
//
//-----------------------------------------------------------------------------

GraphikBox::~GraphikBox()
{

} // ~GraphikBox()

//-----------------------------------------------------------------------------
//
//-----------------------------------------------------------------------------


void GraphikBox::ActivatePoint()
{
   m_pc_GlobalStorage->m_pc_Selection->ActivateSelection(Selection::SelectionPoint,
                                 tr("Point"),
                                 Callback(SelectionEventArrivedCB,this) );

   SetModeTo(GraphikPoint);
} // ActivatePoint()

//-----------------------------------------------------------------------------
//
//-----------------------------------------------------------------------------

void GraphikBox::ActivateLine()
{
   m_pc_GlobalStorage->m_pc_Selection->ActivateSelection(Selection::SelectionLine,
                                                  tr("Line"),
                                                  Callback(SelectionEventArrivedCB,this));

   SetModeTo(GraphikLine);

} // ActivateLine()

//-----------------------------------------------------------------------------
//
//-----------------------------------------------------------------------------

void GraphikBox::ActivateRectangle()
{
   m_pc_GlobalStorage->m_pc_Selection->ActivateSelection(Selection::SelectionRectangle,
                                                  tr("Rectangle"),
                                                  Callback(SelectionEventArrivedCB,this));

   SetModeTo(GraphikRectangle);

} // ActivateRectangle()

//-----------------------------------------------------------------------------
//
//-----------------------------------------------------------------------------

void GraphikBox::ActivateQuadrat()
{
   m_pc_GlobalStorage->m_pc_Selection->ActivateSelection(Selection::SelectionQuadrat,
                                                  tr("Quadrat"),
                                                  Callback(SelectionEventArrivedCB,this));

   SetModeTo(GraphikQuadrat);

} // ActivateQuadrat()

//-----------------------------------------------------------------------------
//
//-----------------------------------------------------------------------------

void GraphikBox::ActivateEllipse()
{
   m_pc_GlobalStorage->m_pc_Selection->ActivateSelection(Selection::SelectionEllipse,
                                                  tr("Ellipse"),
                                                  Callback(SelectionEventArrivedCB,this));

   SetModeTo(GraphikEllipse);

} // ActivateEllipse()

//-----------------------------------------------------------------------------
//
//-----------------------------------------------------------------------------

void GraphikBox::ActivateCircle()
{
   m_pc_GlobalStorage->m_pc_Selection->ActivateSelection(Selection::SelectionCircle,
                                                  tr("Circle"),
                                                  Callback(SelectionEventArrivedCB,this));

   SetModeTo(GraphikCircle);

} // ActivateCircle()

//===========================================================================
//
//===========================================================================

void GraphikBox::ActivateFreeDraw()
{
   m_pc_GlobalStorage->m_pc_Selection->ActivateSelection(Selection::SelectionFreeDraw,
                                                  tr("Free"),
                                                  Callback(SelectionEventArrivedCB,this));

   SetModeTo(GraphikFreeDraw);

} // ActivateFreeDraw()

//===========================================================================
//
//===========================================================================

void GraphikBox::ActivateConstruct()
{

   SetModeTo(GraphikConstruct);
   if(   m_pc_CurvePoints.count()>0
      || m_pc_CurvePolygons.count() > 0)
   {
      // already points there
      ConstructSwitchToMode(ConstructModeModify);
   }
   else
   {
      
      // first create points
      ConstructSwitchToMode(ConstructModeAdd);
   }
   

} // ActivateConstruct()

//===========================================================================
//
//===========================================================================

void GraphikBox::OnMessageReceive(QString qs_MessageType,
                                 void *p_Data)
{
//fprintf(stderr,"Message Received member this = %x\n",this);
   if(qs_MessageType.compare("ImageResize") ==0)
   {
   }
   if(qs_MessageType.compare("ImageNewBuffer") ==0)
   {
   }
} // OnMessageReceive()


//===========================================================================
//
//===========================================================================


void *GraphikBox::SelectionEventArrived(void *pc_Data)
{
//fprintf(stderr,"SelectionEventArrived\n");
   SelectionCallbackData *cb;
   cb = (SelectionCallbackData *) pc_Data;
   if(cb->i_CallbackReason == SelectionCallbackData:: ReasonOperationDisabled)
   {
      // clean up unused structures
      if(m_i_CurrentGraphikMode == GraphikConstruct)
      {
         ConstructSwitchToMode(ConstructModeOff);
      }
   }
   else if(cb->i_CallbackReason == SelectionCallbackData:: ReasonSelection)
   {

      switch(m_i_CurrentGraphikMode)
      {
      case GraphikPoint:
         DrawPoint();
         break;
      case GraphikLine:
         DrawLine();
         break;
      case GraphikRectangle:
         DrawRectangle();
         break;
      case GraphikQuadrat:
         DrawQuadrat();
         break;
      case GraphikEllipse:
         DrawEllipse();
         break;
      case GraphikCircle:
         DrawCircle();
         break;
      case GraphikFreeDraw:
         DrawFreeDraw();
         break;
      case GraphikConstruct:
         DrawConstruct();
         break;
      default:
         break;
      }
   } // else reasonselection

   return NULL;
} // SelectionEventArrived()


//===========================================================================
//
//===========================================================================


void *GraphikBox::SelectionEventArrivedCB(void *Thisdata,void *pc_Data)
{
   GraphikBox *pc_This = (GraphikBox *) Thisdata;

   return pc_This->SelectionEventArrived(pc_Data);

} // SelectionEventArrivedCB()

//===========================================================================
//
//===========================================================================


void GraphikBox::DrawPoint()
{
   RectRegion c_ChangedRegion;
   int        i_X;
   int        i_Y;

   m_pc_GlobalStorage->m_pc_Selection->GetSelectionPoint(i_X,i_Y);


   m_pc_BufferPainter->DrawPoint(GetDestinationBuffer(),i_X,i_Y);

   c_ChangedRegion = m_pc_BufferPainter->GetChangedRegion();


   m_pc_GlobalStorage->m_pc_ImageFrame->RedrawBufferRegion(c_ChangedRegion);

   m_pc_GlobalStorage->m_pc_MessageServer->SendMessage(0,
                                                       this,
                                                       "ImageChanged");

} // DrawPoint()

//===========================================================================
//
//===========================================================================


void GraphikBox::DrawLine()
{
   RectRegion c_ChangedRegion;
   int        i_X1;
   int        i_Y1;
   int        i_X2;
   int        i_Y2;

   m_pc_GlobalStorage->m_pc_Selection->GetSelectionLine(i_X1,i_Y1,i_X2,i_Y2);


   if(   i_X1 == i_X2
      && i_Y1 == i_Y2
     )
   {
      // start end end are the same point. forget the line
      return;
   }


   m_pc_BufferPainter->DrawLine(GetDestinationBuffer(),
             i_X1,i_Y1,
             i_X2,i_Y2);


   c_ChangedRegion = m_pc_BufferPainter->GetChangedRegion();


   m_pc_GlobalStorage->m_pc_ImageFrame->RedrawBufferRegion(c_ChangedRegion);

   m_pc_GlobalStorage->m_pc_MessageServer->SendMessage(0,
                                                       this,
                                                       "ImageChanged");
} // DrawLine()

//===========================================================================
//
//===========================================================================


void GraphikBox::DrawRectangle()
{
   RectRegion c_ChangedRegion;
   int        i_X1;
   int        i_Y1;
   int        i_X2;
   int        i_Y2;

   m_pc_GlobalStorage->m_pc_Selection->GetSelectionRectangle(i_X1,i_Y1,i_X2,i_Y2);

   m_pc_BufferPainter->DrawRectangle(GetDestinationBuffer(),
             i_X1,i_Y1,
             i_X2,i_Y2);

   c_ChangedRegion = m_pc_BufferPainter->GetChangedRegion();

   m_pc_GlobalStorage->m_pc_ImageFrame->RedrawBufferRegion(c_ChangedRegion);

   m_pc_GlobalStorage->m_pc_MessageServer->SendMessage(0,
                                                       this,
                                                       "ImageChanged");
} // DrawRectangle()

//===========================================================================
//
//===========================================================================

void GraphikBox::DrawQuadrat()
{
   RectRegion c_ChangedRegion;
   int        i_X1;
   int        i_Y1;
   int        i_X2;
   int        i_Y2;

   m_pc_GlobalStorage->m_pc_Selection->GetSelectionQuadrat(i_X1,i_Y1,i_X2,i_Y2);

   m_pc_BufferPainter->DrawRectangle(GetDestinationBuffer(),
             i_X1,i_Y1,
             i_X2,i_Y2);

   c_ChangedRegion = m_pc_BufferPainter->GetChangedRegion();

   m_pc_GlobalStorage->m_pc_ImageFrame->RedrawBufferRegion(c_ChangedRegion);

   m_pc_GlobalStorage->m_pc_MessageServer->SendMessage(0,
                                                       this,
                                                       "ImageChanged");
} // DrawQuadrat()

//===========================================================================
//
//===========================================================================

void GraphikBox::DrawEllipse()
{
   RectRegion c_ChangedRegion;
   int        i_X1;
   int        i_Y1;
   int        i_X2;
   int        i_Y2;

   m_pc_GlobalStorage->m_pc_Selection->GetSelectionEllipse(i_X1,i_Y1,i_X2,i_Y2);

   m_pc_BufferPainter->DrawEllipse(GetDestinationBuffer(),
             i_X1,i_Y1,
             i_X2,i_Y2);



   c_ChangedRegion = m_pc_BufferPainter->GetChangedRegion();


   m_pc_GlobalStorage->m_pc_ImageFrame->RedrawBufferRegion(c_ChangedRegion);

   m_pc_GlobalStorage->m_pc_MessageServer->SendMessage(0,
                                                       this,
                                                       "ImageChanged");
} // DrawEllipse()

//===========================================================================
//
//===========================================================================


void GraphikBox::DrawCircle()
{
   RectRegion c_ChangedRegion;
   int        i_X1;
   int        i_Y1;
   int        i_Radius;

   m_pc_GlobalStorage->m_pc_Selection->GetSelectionCircle(i_X1,i_Y1,i_Radius);

   m_pc_BufferPainter->DrawEllipse(GetDestinationBuffer(),
             i_X1,i_Y1,
             i_Radius, i_Radius);



   c_ChangedRegion = m_pc_BufferPainter->GetChangedRegion();


   m_pc_GlobalStorage->m_pc_ImageFrame->RedrawBufferRegion(c_ChangedRegion);

   m_pc_GlobalStorage->m_pc_MessageServer->SendMessage(0,
                                                       this,
                                                       "ImageChanged");
} // DrawCircle()

//===========================================================================
//
//===========================================================================


void GraphikBox::DrawFreeDraw()
{
   PMemVector<PixelCoordinate> *pc_List;
   RectRegion      c_ChangedRegion;
   int i_Size;

   pc_List = m_pc_GlobalStorage->m_pc_Selection->GetSelectionPointList();
   if(pc_List == NULL)
   {
      return;
   }
   i_Size = pc_List->GetCount();

   if(i_Size ==0)
   {
      return;
   }
   m_pc_BufferPainter->DrawFreeFigure(GetDestinationBuffer(),
                       pc_List);

   c_ChangedRegion = m_pc_BufferPainter->GetChangedRegion();

   m_pc_GlobalStorage->m_pc_ImageFrame->RedrawBufferRegion(c_ChangedRegion);

   m_pc_GlobalStorage->m_pc_MessageServer->SendMessage(0,
                                                       this,
                                                       "ImageChanged");
} // DrawFreeDraw()

//===========================================================================
//
//===========================================================================

void GraphikBox::LineThicknessChanged(int i_Width)
{
   m_pc_BufferPainter->SetLineWidth(i_Width);
   m_i_LineWidth = i_Width;

} // LineThicknessChanged()

//===========================================================================
//
//===========================================================================

void GraphikBox::DrawInnerChanged(bool b_On)
{
   m_b_DrawInner = b_On;
   m_pc_BufferPainter->SetDrawInnerEnable(b_On);
}  // DrawInnerChanged()

//===========================================================================
//
//===========================================================================

void GraphikBox::TransparencyEnableChanged(bool b_On)
{
   m_b_TransparencyOn = b_On;
   m_pc_BufferPainter->SetTransparencyEnable(b_On);
} // ransparencyEnableChanged

//===========================================================================
//
//===========================================================================

void GraphikBox::TransparencyFactorChanged(int i_Factor)
{
   m_i_TransparencyFactor = i_Factor;
   m_pc_BufferPainter->SetTransparencyFactor(i_Factor);
} // TransparencyFactorChanged()

//===========================================================================
//
//===========================================================================

void GraphikBox::LineColorChanged(Pixel c_Color)
{
   m_pc_BufferPainter->SetBorderColor(c_Color);
} // LineColorChanged()

//===========================================================================
//
//===========================================================================

void GraphikBox::InnerColorChanged(Pixel c_Color)
{
   m_pc_BufferPainter->SetInnerColor(c_Color);
} // InnerColorChanged()

//===========================================================================
//
//===========================================================================

PixelBuffer *GraphikBox::GetDestinationBuffer()
{
   PixelBuffer *pc_Destination;
   
   pc_Destination = m_pc_GlobalStorage->m_pc_MainImage;
   if( m_pc_GlobalStorage->m_pc_MaskSettings->GetPaintMode() == MaskSettings::MaskPaintModeInSafety)
   {
      pc_Destination = pc_Destination->GetSafetyMask();
      if(m_pc_GlobalStorage->m_pc_MaskSettings->GetDraw() == MaskSettings::MaskPaintDrawExtend)
      {
         m_pc_BufferPainter->SetMaskMode(true,true);
      }
      else
      {
         m_pc_BufferPainter->SetMaskMode(true,false);
      }
   }
   else if( m_pc_GlobalStorage->m_pc_MaskSettings->GetPaintMode() == MaskSettings::MaskPaintModeInSelection)
   {
      pc_Destination = pc_Destination->GetSelectionMask();
      if(m_pc_GlobalStorage->m_pc_MaskSettings->GetDraw() == MaskSettings::MaskPaintDrawExtend)
      {
         m_pc_BufferPainter->SetMaskMode(true,true);
      }
      else
      {
         m_pc_BufferPainter->SetMaskMode(true,false);
      }
   }
   else
   {
      m_pc_BufferPainter->SetMaskMode(false,false);
   }

   return pc_Destination;
} // GetDestinationBuffer()

//===========================================================================
//
//===========================================================================

void GraphikBox::LinePatternChanged(PixelBuffer *pc_Buffer)
{
   m_pc_BufferPainter->SetBorderPattern(pc_Buffer);
} // LinePatternChanged()

//===========================================================================
//
//===========================================================================

void GraphikBox::LineModeChanged(BrushMode i_BrushMode)
{
   if(i_BrushMode == SelectMotive)
   {
      // get the motiveLayer
      GraphicObjectLayer *pc_Motive;
      pc_Motive = m_pc_MainFrame->m_pc_MotiveBox->GetMotiveLayer();
      m_pc_BufferPainter->SetBorderMotiveLayer(pc_Motive);
   }
   m_pc_BufferPainter->SetBorderDrawMode(i_BrushMode);
}  // LineModeChanged()

//===========================================================================
//
//===========================================================================

void GraphikBox::InnerPatternChanged(PixelBuffer *pc_Buffer)
{
   m_pc_BufferPainter->SetInnerPattern(pc_Buffer);
} // InnerPatternChanged()

//===========================================================================
//
//===========================================================================

void GraphikBox::InnerModeChanged(BrushMode i_BrushMode)
{
   if(i_BrushMode == SelectMotive)
   {
      // get the motiveLayer
      GraphicObjectLayer *pc_Motive;
      pc_Motive = m_pc_MainFrame->m_pc_MotiveBox->GetMotiveLayer();
      m_pc_BufferPainter->SetInnerMotiveLayer(pc_Motive);
   }
   m_pc_BufferPainter->SetInnerDrawMode(i_BrushMode);
}  // InnerModeChanged()

//===========================================================================
//
//===========================================================================

int GraphikBox::WriteFile(PericoFileWriter *pc_FileWriter,
                         QString qs_Name)
{
   pc_FileWriter->StartStructure(qs_Name,"GraphikBox");

   pc_FileWriter->AddValue("LineWidth", m_i_LineWidth);

   m_LineColorWidget->WriteFile(pc_FileWriter,"LineColorWidget");

   pc_FileWriter->AddValue("DrawInner", m_b_DrawInner);
   m_InnerColorWidget->WriteFile(pc_FileWriter,"InnerColorWidget");

   pc_FileWriter->AddValue("TransparencyOn", m_b_TransparencyOn);
   pc_FileWriter->AddValue("TransparencyFactor", m_i_TransparencyFactor);

   // save the polygons
   ConstructWriteFile(pc_FileWriter);

   pc_FileWriter->FinishStructure();

   // everything is OK
   return 0;
} // WriteFile

//===========================================================================
//
//===========================================================================

int GraphikBox::LoadFile(PericoFileWriter *pc_FileWriter)
{

   PERICO_READ_STRUCTURE_START

   if(qs_Name.compare("LineWidth") == 0)
   {
      m_i_LineWidth = atoi(qs_Value);
   }
   else if(    qs_Name.compare("LineColorWidget")==0
            && qs_Value.compare("QUColorOrPattern")==0
            && b_IsStartStructure)
   {
      m_LineColorWidget->LoadFile(pc_FileWriter);
   }
   else if(qs_Name.compare("DrawInner") == 0)
   {
      m_b_DrawInner = PericoFileWriter::atob(qs_Value);
   }
   else if(   qs_Name.compare("InnerColorWidget")==0
           && qs_Value.compare("QUColorOrPattern")==0
           && b_IsStartStructure)
   {
      m_InnerColorWidget->LoadFile(pc_FileWriter);
   }
   else if(qs_Name.compare("TransparencyOn") == 0)
   {
      m_b_TransparencyOn = PericoFileWriter::atob(qs_Value);
   }
   else if(qs_Name.compare("TransparencyFactor") == 0)
   {
      m_i_TransparencyFactor = atoi(qs_Value);
   }
   else if(qs_Name.compare("PolygonList") == 0)
   {
      ConstructLoadFile(pc_FileWriter);
   }

   PERICO_READ_STRUCTURE_END

   // set to Display

   m_LineWidthSlider->SetValue(m_i_LineWidth);
   LineThicknessChanged(m_i_LineWidth);


   m_DrawInnerEnable->setChecked(m_b_DrawInner);
   DrawInnerChanged(m_b_DrawInner);

   m_TransparencyEnable->setChecked(m_b_TransparencyOn);
   m_TransparencySlider->SetValue(m_i_TransparencyFactor);
   TransparencyEnableChanged(m_b_TransparencyOn);
   TransparencyFactorChanged(m_i_TransparencyFactor);
   
   // everything is OK
   return 0;
} // LoadFile

//===========================================================================
//
//===========================================================================


void GraphikBox::SetModeTo(int i_Mode)
{
   bool b_LayerWasVisible;
   bool b_NewLayerVisibility;
   
   b_LayerWasVisible    = m_pc_ObjectLayer->IsVisible();
   b_NewLayerVisibility = false;

   if(isHidden())
   {
      return;
   }
   
   // initial deselect all buttons
   m_PointButton     ->SetSelected(false);
   m_LineButton      ->SetSelected(false);
   m_RectangleButton ->SetSelected(false);
   m_QuadratButton   ->SetSelected(false);
   m_EllipseButton   ->SetSelected(false);
   m_CircleButton    ->SetSelected(false);
   m_FreeDrawButton  ->SetSelected(false);
   m_ConstructButton ->SetSelected(false);

   // most used variant
   m_InnerBox->show();
   m_ConstructBox->hide();

   m_i_CurrentGraphikMode = i_Mode;
   switch(m_i_CurrentGraphikMode)
   {
   case GraphikNone:
      break;
   case GraphikPoint:
      m_InnerBox->hide();
      m_PointButton    ->SetSelected(true);
      break;
   case GraphikLine:
      m_InnerBox->hide();
      m_LineButton     ->SetSelected(true);
      break;
   case GraphikRectangle:
      m_RectangleButton->SetSelected(true);
      break;
   case GraphikQuadrat:
      m_QuadratButton  ->SetSelected(true);
      break;
   case GraphikEllipse:
      m_EllipseButton  ->SetSelected(true);
      break;
   case GraphikCircle:
      m_CircleButton   ->SetSelected(true);
      break;
   case GraphikFreeDraw:
      m_FreeDrawButton ->SetSelected(true);
      break;
   case GraphikConstruct:
      m_ConstructBox->show();
      m_ConstructButton ->SetSelected(true);
      b_NewLayerVisibility = true;
      ConstructSetSensitivity();

      break;

   }
   ResizeDisplay();

   // has layer visiblity changed?
   if(b_LayerWasVisible != b_NewLayerVisibility)
   {
      m_pc_GlobalStorage->m_pc_ObjectMain->SetVisible(b_NewLayerVisibility);
      m_pc_ObjectLayer->SetVisible(b_NewLayerVisibility);
      m_pc_GlobalStorage->m_pc_ImageFrame->RedrawAll();
      
   }

   
} // SetModeTo()

//===========================================================================
// event entry function when in drawing construction
//===========================================================================


void GraphikBox::DrawConstruct()
{
   GraphicObjectPoint *pc_NewPoint;
   GraphicObjectLine  *pc_NewLine;
   RectRegion          c_ChangedRegion;
   RectRegion          c_SplineRegion;
   int                 i_X;
   int                 i_Y;
   int                 i_MouseEventType;
   int                 i_CurrentPoint;

   i_MouseEventType = m_pc_GlobalStorage->m_pc_Selection->GetMouseEventType();

   if(i_MouseEventType ==MouseEvent::EventOver)
   {
      // draw a line from to last point to the current position
      i_CurrentPoint = m_pc_CurvePoints.count();
      if(   i_CurrentPoint>0
         && m_pc_RubberStartPoint != NULL )
      {
         RectRegion c_Old;
         RectRegion c_New;
         float      f_X;
         float      f_Y;

         m_pc_RubberStartPoint->GetPosition(f_X,f_Y);

         c_Old = m_pc_RubberLine->GetSize();
         m_pc_GlobalStorage->m_pc_Selection->GetSelectionPoint(i_X,i_Y);
         m_pc_RubberLine->SetStartPoint(f_X,f_Y);
         m_pc_RubberLine->SetEndPoint(i_X,i_Y); 
         m_pc_RubberLine->SetVisible(true);

         c_New = m_pc_RubberLine->GetSize();

         c_New.AddRegion(c_Old);
         
         c_ChangedRegion = c_New;
      }
   }
   else if(i_MouseEventType == MouseEvent::EventStart)
   {
      // create a new point

      // get the coordinate for the point
      m_pc_GlobalStorage->m_pc_Selection->GetSelectionPoint(i_X,i_Y);

      i_CurrentPoint = m_pc_CurvePoints.count();
      
      if(i_CurrentPoint ==0)
      {
         // first point
         pc_NewPoint = new GraphicObjectPoint(i_X,i_Y);
         m_pc_CurvePoints.append(pc_NewPoint);
         
         m_pc_ObjectLayer->AddObject(pc_NewPoint);
         c_ChangedRegion = pc_NewPoint->GetSize();
         m_pc_RubberStartPoint = pc_NewPoint;
      }
      else // previous points available
      {
         GraphicObjectPoint *pc_LastPoint;
         RectRegion          c_Last;
         float               f_LastPositionX;
         float               f_LastPositionY;

         if( m_i_ConstructAddPointPosition == -1)
         {
            // insert at the end
            pc_LastPoint =m_pc_CurvePoints.last();

            pc_LastPoint->GetPosition( f_LastPositionX,f_LastPositionY);

            if(   i_X == f_LastPositionX
               && i_Y == f_LastPositionY
               )
            {
               // double point at the same position
               // finish the line
               GraphicObjectPoint *pc_FirstPoint;
               float               f_FirstPositionX;
               float               f_FirstPositionY;

               pc_FirstPoint =m_pc_CurvePoints.first();
               pc_FirstPoint->GetPosition( f_FirstPositionX,f_FirstPositionY);
            
               // create a line from last to current
               pc_NewLine = new GraphicObjectLine(f_LastPositionX,
                                            f_LastPositionY,
                                            f_FirstPositionX,
                                            f_FirstPositionY);
               // line is never selecteble
               pc_NewLine->SetSelectable(false);
               m_pc_CurveLines.append(pc_NewLine);
               m_pc_ObjectLayer->AddObject(pc_NewLine);
               m_pc_RubberStartPoint = NULL;

               c_ChangedRegion.SetXYXY(f_LastPositionX,
                                    f_LastPositionY,
                                    f_FirstPositionX,
                                    f_FirstPositionY);

               ConstructSwitchToMode(ConstructModeModify);
            }
            else
            {
               // add a new point
         
               pc_NewPoint = new GraphicObjectPoint(i_X,i_Y);
               m_pc_CurvePoints.append(pc_NewPoint);

               m_pc_ObjectLayer->AddObject(pc_NewPoint);
               c_ChangedRegion = pc_NewPoint->GetSize();

               // create a line from last to current
               pc_NewLine = new GraphicObjectLine(f_LastPositionX,
                                            f_LastPositionY,
                                            i_X,
                                            i_Y);
               pc_NewLine->SetSelectable(false);
               m_pc_CurveLines.append(pc_NewLine);
               m_pc_ObjectLayer->AddObject(pc_NewLine);
               // combine both regions
               c_Last = pc_LastPoint->GetSize();
               c_ChangedRegion.AddRegion(c_Last);
               m_pc_RubberStartPoint = pc_NewPoint;
            }
         } // add at the end
         else
         {
            // insert at the middle
            pc_LastPoint =m_pc_CurvePoints.at(m_i_ConstructAddPointPosition);

            pc_LastPoint->GetPosition( f_LastPositionX,f_LastPositionY);

            if(   i_X == f_LastPositionX
               && i_Y == f_LastPositionY
               )
            {
               // double point at the same position
               // finish the line
               GraphicObjectPoint *pc_FirstPoint;
               GraphicObjectLine  *pc_CurrentLine;
               
               float               f_FirstPositionX;
               float               f_FirstPositionY;
               int                 i_NextIndex;

               i_NextIndex = m_i_ConstructAddPointPosition+1;
               if(i_NextIndex >= (int) m_pc_CurvePoints.count())
               {
                  i_NextIndex =0;
               }
               pc_FirstPoint =m_pc_CurvePoints.at(i_NextIndex);
               pc_FirstPoint->GetPosition( f_FirstPositionX,f_FirstPositionY);

               pc_CurrentLine= m_pc_CurveLines.at(m_i_ConstructAddPointPosition);
               // create a line from last to current
               pc_CurrentLine->SetStartPoint(f_LastPositionX,f_LastPositionY);
               pc_CurrentLine->SetEndPoint(f_FirstPositionX,f_FirstPositionY);
               
               m_pc_RubberStartPoint = NULL;

               c_ChangedRegion.SetXYXY(f_LastPositionX,
                                    f_LastPositionY,
                                    f_FirstPositionX,
                                    f_FirstPositionY);

               ConstructSwitchToMode(ConstructModeModify);
               // disable the inner mode
               m_i_ConstructAddPointPosition = -1;
            }
            else
            {
               // add a new point
               GraphicObjectLine  *pc_CurrentLine;

               pc_NewPoint = new GraphicObjectPoint(i_X,i_Y);
               m_pc_CurvePoints.insert(m_i_ConstructAddPointPosition+1,
                                       pc_NewPoint);

               m_pc_ObjectLayer->AddObject(pc_NewPoint);
               c_ChangedRegion = pc_NewPoint->GetSize();


               // get line at the current position
               pc_CurrentLine= m_pc_CurveLines.at(m_i_ConstructAddPointPosition);
               // set this line from last to current
               pc_CurrentLine->SetEndPoint(i_X,i_Y);

               // create a new line
               // set this line emtpy at the last point
               pc_NewLine = new GraphicObjectLine(i_X,
                                            i_Y,
                                            i_X,
                                            i_Y);
               pc_NewLine->SetSelectable(false);
               m_pc_CurveLines.insert(m_i_ConstructAddPointPosition+1,pc_NewLine);
               m_pc_ObjectLayer->AddObject(pc_NewLine);

               // combine both regions
               c_Last = pc_LastPoint->GetSize();
               c_ChangedRegion.AddRegion(c_Last);
               m_pc_RubberStartPoint = pc_NewPoint;

               // increase the actual isnert position
               m_i_ConstructAddPointPosition += 1;
            }
         } // else add in the middle
      } // else previous points available
      
   }  // else event start

   c_SplineRegion = ConstructReCalcConstructSpline();
   c_ChangedRegion.AddRegion(c_SplineRegion);
   
   // adjust visiblity of the last line
   ConstructSetLastLinesVisible();

   m_pc_GlobalStorage->m_pc_ImageFrame->RedrawBufferRegion(c_ChangedRegion);

} // DrawConstruct()

//===========================================================================
//
//===========================================================================

void GraphikBox::ConstructNewPolygon()
{
   ConstructPointsToPolygon();
   ConstructDeleteCurvePoints();
   m_i_ConstructAddPointPosition = -1; // at the end

   ConstructSwitchToMode(ConstructModeAdd);
            
} // ConstructNewPolygon()

//===========================================================================
//
//===========================================================================

void GraphikBox::ConstructDrawPolygon()
{
   PMemVector<PixelCoordinate>   pc_List;
   GraphicObjectMultipoint      *pc_Multipoint;
   PixelCoordinate               c_ActualBufferPosition;
   RectRegion                    c_ChangedRegion;
   RectRegion                    c_TotalChangedRegion;
   bool                          b_ClosedFigure;
   int                           i_Count;
   int                           i_Polygons;
   int                           i_ActualPolygon;
   int                           i_ActualPoint;

   // only draw in modify mode
   if(m_i_ConstructMode != ConstructModeModify)
   {
      return;
   }

   // get number of polygons   
   i_Polygons = m_pc_CurvePolygons.count();
   if(i_Polygons <=0)
   {
      return;
   }

   c_TotalChangedRegion.SetEmpty();

   // for all polygons
   for(i_ActualPolygon=0;i_ActualPolygon<i_Polygons;i_ActualPolygon++)
   {
      GeoCoordinateList *pc_Coordinates;

      pc_Multipoint = m_pc_CurvePolygons.at( i_ActualPolygon);
      b_ClosedFigure = pc_Multipoint->IsClosedFigure();
      
      pc_Coordinates = pc_Multipoint->GetCoordinateList();

      // get number of points
      i_Count = pc_Coordinates->GetCount();
      if(i_Count <2)
      {
         continue;
      }

      // go through all points
      for( i_ActualPoint=0; i_ActualPoint < i_Count; i_ActualPoint++)
      {
         GeoCoordinate c_Coordinate;
         c_Coordinate = pc_Coordinates->Get(i_ActualPoint);
         c_ActualBufferPosition.m_i_X =(int) c_Coordinate.GetX();
         c_ActualBufferPosition.m_i_Y =(int) c_Coordinate.GetY();
         pc_List.AddTail(c_ActualBufferPosition);
      } // for all points

      // draw coordinate list with the bufferpainter

      if(pc_Multipoint->GetObjectType() == GraphicObject::TypePolygon)
      {
         m_pc_BufferPainter->DrawFreeFigure(GetDestinationBuffer(),
                          &pc_List,
                          b_ClosedFigure);
      }
      else if(pc_Multipoint->GetObjectType() == GraphicObject::TypeSpline)
      {
         m_pc_BufferPainter->DrawSpline(GetDestinationBuffer(),
                          pc_Coordinates,
                          b_ClosedFigure);
      }

      // combine changed region to a total one
      c_ChangedRegion = m_pc_BufferPainter->GetChangedRegion();
      c_TotalChangedRegion.AddRegion(c_ChangedRegion);

      // remove all points
      pc_List.Clear();
      

   } // for all polygons

   m_pc_GlobalStorage->m_pc_ImageFrame->RedrawBufferRegion(c_TotalChangedRegion);

   m_pc_GlobalStorage->m_pc_MessageServer->SendMessage(0,
                                                       this,
                                                       "ImageChanged");

} // ConstructDrawPolygon()

//===========================================================================
//
//===========================================================================

void GraphikBox::ConstructAddPoint()
{
   GraphicObjectPoint           *pc_Point;
   GraphicObjectLine            *pc_Line;
   GraphicObject                *pc_Object;
   GeoCoordinate                 c_Coordinate;
   int                           i_SelectedCount;
   int                           i_SelectedIndex;

   // are we in the correct mode?
   if(   m_i_ConstructMode == ConstructModeAdd
      || m_i_ConstructMode == ConstructModeAddMiddle)
   {
      // allready in add mode, so do nothing
      return;
   }
   
   // get the selected point
   i_SelectedCount = m_pc_ObjectLayer->GetNumberOfSelectedObjects(pc_Object);

   if(pc_Object->GetObjectType() != GraphicObject::TypePoint)
   {
      return;
   }

   pc_Point =(GraphicObjectPoint *) pc_Object;

   // get the index of the selected point
   i_SelectedIndex = m_pc_CurvePoints.find(pc_Point);
   if( i_SelectedIndex<0)
   {
      return;
   }

   // set add position on the selected index
   m_i_ConstructAddPointPosition = i_SelectedIndex;
   // rubber begin is the selected point
   m_pc_RubberStartPoint         = pc_Point;
   pc_Line = m_pc_CurveLines.at(m_i_ConstructAddPointPosition);

   // make empty line by setting endpoint to startpoint
   if(pc_Line ==NULL)
   {
      return;
   }
   c_Coordinate = pc_Line->GetStartPoint();
   pc_Line->SetEndPoint(c_Coordinate);

   // enter the construct add mode. this enables the events
   ConstructSwitchToMode(ConstructModeAddMiddle);
   
   // adjust visiblity of the last line
   ConstructSetLastLinesVisible();

} // ConstructAddPoint()

//===========================================================================
//
//===========================================================================

void GraphikBox::ConstructDeletePoint()
{
   QPtrList<GraphicObjectPoint > pc_SelectedPoints;
   QPtrList<GraphicObjectMultipoint > pc_SelectedMultipoints;
   GraphicObjectPolygon           *pc_Polygon;
   GraphicObjectSpline           *pc_Spline;
   GraphicObjectPoint           *pc_Point;
   GraphicObject                *pc_Object;
   int                           i_SelectedCount;
   int                           i_Index;

   
   if(   m_i_ConstructMode == ConstructModeAdd
      || m_i_ConstructMode == ConstructModeAddMiddle)
   {
      // do not delete points in add mode
      return;
   }

   // get the selected point
   i_SelectedCount = m_pc_ObjectLayer->GetNumberOfSelectedObjects(pc_Object);

   // build up a list with all selected points
   while(pc_Object != NULL)
   {
      if(pc_Object->GetObjectType() == GraphicObject::TypePoint)
      {
         pc_Point =(GraphicObjectPoint *) pc_Object;
         pc_SelectedPoints.append(pc_Point);

      }
      else if(pc_Object->GetObjectType() == GraphicObject::TypePolygon)
      {
         pc_Polygon =(GraphicObjectPolygon *) pc_Object;
         pc_SelectedMultipoints.append(pc_Polygon);

      }
      else if(pc_Object->GetObjectType() == GraphicObject::TypeSpline)
      {
         pc_Spline =(GraphicObjectSpline *) pc_Object;
         pc_SelectedMultipoints.append(pc_Spline);

      }
      pc_Object = m_pc_ObjectLayer->GetNextSelectedObject(pc_Object);
   }

   // first delete the list of selected polygons
   // now go through the list and remove the points
   while(pc_SelectedMultipoints.count() >0)
   {
      // get first point, and remove it from the list
      GraphicObjectMultipoint *pc_Object;
      pc_Object = pc_SelectedMultipoints.getFirst();
      pc_SelectedMultipoints.removeFirst();

      ConstructRemoveObject(pc_Object);

   }
   
   // now go through the list and remove the points
   while(pc_SelectedPoints.count() >0)
   {
      GraphicObjectLine * pc_LineBefore;
      GraphicObjectLine * pc_LineAfter;
      GeoCoordinate       c_Endpoint;
      int                 i_LineIndexBefore;
      
      
      if(m_pc_CurvePoints.count()<3)
      {
         // useless to delete 2 points
         continue;
      }

      // get first point, and remove it from the list
      pc_Point = pc_SelectedPoints.getFirst();
      pc_SelectedPoints.removeFirst();

      
      // find the index in the list
      i_Index = m_pc_CurvePoints.find(pc_Point);
      if(i_Index == -1)
      {
         // should not occur)
         continue;
      }
      // now we have to get the line index-1 and index
      i_LineIndexBefore = i_Index -1;
      if(i_LineIndexBefore <0)
      {
         i_LineIndexBefore = m_pc_CurveLines.count()-1;
      }
      // get both lines attached at this point
      pc_LineBefore = m_pc_CurveLines.at(i_LineIndexBefore);
      pc_LineAfter  = m_pc_CurveLines.at(i_Index);

      // the endpoint of the 2nd will be the endpoint of the first
      c_Endpoint = pc_LineAfter->GetEndPoint();
      pc_LineBefore->SetEndPoint(c_Endpoint);

      // remove point and line from layer
      m_pc_ObjectLayer->RemoveObject(pc_LineAfter);
      m_pc_ObjectLayer->RemoveObject(pc_Point);

      // remove point and line from object lists
      m_pc_CurvePoints.remove(i_Index);
      m_pc_CurveLines .remove(i_Index);

      // delete point and line
      delete pc_LineAfter;
      delete pc_Point;

   } // while selected points
   
   // adjust visiblity of the last line
   ConstructSetLastLinesVisible();

   ConstructReCalcConstructSpline();
   // redraw image
   m_pc_GlobalStorage->m_pc_ImageFrame->RedrawAll();
   
   ConstructSetSensitivity();

} // ConstructDeletePoint()

//===========================================================================
//
//===========================================================================

void GraphikBox::ConstructDeleteCurvePoints()
{

   // delete all points
   while(m_pc_CurvePoints.count()>0)
   {
      GraphicObjectPoint *pc_Point;
      pc_Point = m_pc_CurvePoints.getFirst();
      m_pc_ObjectLayer->RemoveObject(pc_Point);
      delete pc_Point;

      m_pc_CurvePoints.removeFirst();
      
   }

   // delete all lines
   while(m_pc_CurveLines.count()>0)
   {
      GraphicObjectLine *pc_Line;
      pc_Line = m_pc_CurveLines.getFirst();
      m_pc_ObjectLayer->RemoveObject(pc_Line);
      delete pc_Line;

      m_pc_CurveLines.removeFirst();


   }
   m_pc_RubberStartPoint= NULL;
   m_pc_RubberLine->SetVisible(false);
   m_i_ConstructAddPointPosition = -1;
   
   // adjust visiblity of the last line
   ConstructSetLastLinesVisible();

   ConstructSetSensitivity();
} // ConstructDeleteCurvePoints()

//===========================================================================
//
//===========================================================================

void GraphikBox::ConstructSwitchToMode(int i_NewMode)
{

   switch(i_NewMode)
   {
   case ConstructModeOff:
      m_pc_ObjectLayer->SetVisible(false);
      ConstructPointsToPolygon();
      ConstructReCalcConstructSpline();
      if( ! m_b_ContructIgnoreRedraw )
      {
         m_pc_GlobalStorage->m_pc_ImageFrame->RedrawAll();
      }
      break;
         
   case ConstructModeAdd:
      m_b_ContructIgnoreRedraw = true;
      m_pc_RubberStartPoint = NULL;
      m_pc_RubberLine->SetVisible(false);
      m_pc_GlobalStorage->m_pc_ObjectMain->ActivateInput(false);
      m_pc_GlobalStorage->m_pc_ObjectMain
                                 ->DeselectAllObjects();
      // disable the callback for graphic object changes
      m_pc_GlobalStorage->m_pc_ObjectMain->SetEventCallback(
                              Callback(NULL,NULL));
      m_pc_GlobalStorage->m_pc_Selection->ActivateSelection(
                        Selection::SelectionStream,
                           tr("Construct complex Figures"),
                          Callback(SelectionEventArrivedCB,this));
      m_pc_ObjectLayer->SetVisible(true);
      m_b_ContructIgnoreRedraw = false;
      ConstructReCalcConstructSpline();
      m_pc_GlobalStorage->m_pc_ImageFrame->RedrawAll();
      m_i_ConstructAddPointPosition = -1;      
      break;

   case ConstructModeAddMiddle:
      // same as add, but without setting the rubberpoint
      m_b_ContructIgnoreRedraw = true;
      // set rubber line before the first event to
      // a position outside the image
      m_pc_RubberLine->SetPosition(-1,-1,-1,-1);
      m_pc_RubberLine->SetVisible(true);
      m_pc_GlobalStorage->m_pc_ObjectMain->ActivateInput(false);
      // disable the callback for graphic object changes
      m_pc_GlobalStorage->m_pc_ObjectMain->SetEventCallback(
                              Callback(NULL,NULL));
      m_pc_GlobalStorage->m_pc_Selection->ActivateSelection(
                        Selection::SelectionStream,
                           tr("Construct complex Figures"),
                          Callback(SelectionEventArrivedCB,this));
      m_pc_ObjectLayer->SetVisible(true);
      m_b_ContructIgnoreRedraw = false;
      ConstructReCalcConstructSpline();
      m_pc_GlobalStorage->m_pc_ImageFrame->RedrawAll();

      break;

   case ConstructModeModify:

      // set the callback for graphic object changes
      m_b_ContructIgnoreRedraw = true;
      m_pc_GlobalStorage->m_pc_ObjectMain->SetEventCallback(
                              Callback(GraphicObjectCallbackCB,this));
      m_pc_RubberStartPoint = NULL;
      m_pc_RubberLine->SetVisible(false);
      m_pc_GlobalStorage->m_pc_ObjectMain->SetActiveLayer(GRAPHIKBOXLAYER);
      m_pc_GlobalStorage->m_pc_ObjectMain->ActivateInput(true);
      m_pc_ObjectLayer->SetVisible(true);
      m_b_ContructIgnoreRedraw = false;
      ConstructReCalcConstructSpline();
      m_pc_GlobalStorage->m_pc_ImageFrame->RedrawAll();
      break;

   default:
      break;
   }
   m_i_ConstructMode = i_NewMode;
   ConstructSetSensitivity();

} // ConstructSwitchToMode()

//===========================================================================
//
//===========================================================================

void GraphikBox::GraphicObjektCallback(GraphicObjectMainEvent *pc_CallbackData)
{
   // fprintf(stderr,"GraphicObjektCallback\n");
   if(pc_CallbackData->i_CallbackType ==GraphicObjectMainEvent::SelectionDisabled)
   {
      ConstructSwitchToMode(ConstructModeOff);
   }
   
   if(pc_CallbackData->i_CallbackType ==GraphicObjectMainEvent::Moved)
   {
      GraphicObjectPoint *pc_Point;
      GraphicObjectLine  *pc_LineBefore;
      GraphicObjectLine  *pc_LineAfter;
      GeoCoordinate       c_Old;
      RectRegion          c_ChangedRegion;
      RectRegion          c_AddRegion;
      RectRegion          c_SplineRegion;
      float               f_NewX;
      float               f_NewY;
      int                 i_Index;
      int                 i_LineIndexBefore;

      // get the moved object
      pc_Point = (GraphicObjectPoint *) pc_CallbackData->pc_Object;

      // find the index in the list
      i_Index = m_pc_CurvePoints.find(pc_Point);
      if(i_Index == -1)
      {
         // should not occur)
         return;
      }
      // now we have to get the line index-1 and index
      i_LineIndexBefore = i_Index -1;
      if(i_LineIndexBefore <0)
      {
         i_LineIndexBefore = m_pc_CurveLines.count()-1;
      }
      // get both lines attached at this point
      pc_LineBefore = m_pc_CurveLines.at(i_LineIndexBefore);
      pc_LineAfter  = m_pc_CurveLines.at(i_Index);

      c_Old = pc_LineBefore->GetEndPoint();

      // set the new object point also into the lines
      pc_Point->GetPosition(f_NewX,f_NewY);
      pc_LineBefore->SetEndPoint(f_NewX,f_NewY);
      pc_LineAfter ->SetStartPoint(f_NewX,f_NewY);

      // calculate the total changed area
      c_ChangedRegion = pc_LineBefore->GetSize();
      c_AddRegion     = pc_LineAfter->GetSize();
      c_ChangedRegion.AddRegion(c_AddRegion);
      c_ChangedRegion.AddPoint((int)c_Old.GetX(),(int)c_Old.GetY());
      
      // adjust visiblity of the last line
      ConstructSetLastLinesVisible();

      c_SplineRegion = ConstructReCalcConstructSpline();
      c_ChangedRegion.AddRegion(c_SplineRegion);

      m_pc_GlobalStorage->m_pc_ImageFrame->RedrawBufferRegion(c_ChangedRegion);
 
   } // if moved

   ConstructSetSensitivity();
} // GraphicObjektCallback()

void *GraphikBox::GraphicObjectCallbackCB(void *thisData,
                                          void *GraphicObjectCallbackData)
{
   GraphikBox *pc_This;

   pc_This = (GraphikBox *) thisData;

   pc_This->GraphicObjektCallback((GraphicObjectMainEvent *) GraphicObjectCallbackData);

   return NULL;

} // GraphicObjectCallbackCB

//===========================================================================
//
//===========================================================================

void GraphikBox::ConstructSetSensitivity()
{

   switch(m_i_ConstructMode)
   {
   case ConstructModeOff:
      // nothing is possible
      m_ConstructEditPointButton  ->setEnabled(false);
      m_ConstructAddPointButton   ->setEnabled(false);
      m_ConstructDeletePointButton->setEnabled(false);
      m_ConstructNewPolygonButton ->setEnabled(true);
      if(m_pc_CurvePoints.count() >0)
      {
         m_ConstructDrawIntoButton   ->setEnabled(true);
      }
      else
      {
         m_ConstructDrawIntoButton   ->setEnabled(false);
      }
      m_ConstructClosedFigureCheck->setEnabled(false);
      m_ConstructSplineCheck->setEnabled(false);
      break;
      
   case ConstructModeAddMiddle:
   case ConstructModeAdd:
      // add and add middle have the save settings

      // in add more also draw and new polygon are disabled
      m_ConstructNewPolygonButton ->setEnabled(false);
      m_ConstructDrawIntoButton   ->setEnabled(false);

      // disable the add and the delete button
      m_ConstructEditPointButton  ->setEnabled(false);
      m_ConstructAddPointButton   ->setEnabled(false);
      m_ConstructDeletePointButton->setEnabled(false);
      
      m_ConstructClosedFigureCheck->setEnabled(true);
      m_ConstructSplineCheck->setEnabled(true);
      break;
      
   case ConstructModeModify:
      // check the number of selected objects
      GraphicObject *pc_Object;
      int            i_SelectedCount;
      
      i_SelectedCount = m_pc_ObjectLayer->GetNumberOfSelectedObjects(pc_Object);

      // add only, if exact one is selected
      if(   i_SelectedCount==1
         && pc_Object->GetObjectType() == GraphicObject::TypePoint
         )
      {
         m_ConstructAddPointButton   ->setEnabled(true);
      }
      else
      {
         m_ConstructAddPointButton   ->setEnabled(false);
      }

      // delete, if one or more are selected and at least
      // 2 will remain
      if(   i_SelectedCount>=1
          && (m_pc_CurvePoints.count() - i_SelectedCount  >=2) )
      {
         m_ConstructDeletePointButton   ->setEnabled(true);
      }
      else
      {
         m_ConstructDeletePointButton   ->setEnabled(false);
      }

      // new is possible      
      m_ConstructNewPolygonButton ->setEnabled(true);
      // draw only, if there are points
      if(m_pc_CurvePolygons.count() >0)
      {
         m_ConstructDrawIntoButton   ->setEnabled(true);
      }
      else
      {
         m_ConstructDrawIntoButton   ->setEnabled(false);
      }

      if(   (    i_SelectedCount >0
              && (   pc_Object->GetObjectType() == GraphicObject::TypePolygon
                  || pc_Object->GetObjectType() == GraphicObject::TypeSpline)
                 )
         || (m_pc_CurvePoints.count() >0)
         )
      {
         m_ConstructEditPointButton   ->setEnabled(true);

      }
      else
      {
         m_ConstructEditPointButton   ->setEnabled(false);

      }
      if(m_pc_CurvePoints.count() >0)
      {
         m_ConstructClosedFigureCheck->setEnabled(true);
         m_ConstructSplineCheck->setEnabled(true);
      }
      else
      {
         m_ConstructClosedFigureCheck->setEnabled(false);
         m_ConstructSplineCheck->setEnabled(false);
      }
         
      break;
   default:
      break;
   }

} // ConstructSetSensitivity()


//===========================================================================
// create out of all construction points a polygon
//===========================================================================

void GraphikBox::ConstructPointsToPolygon()
{
   GraphicObjectMultipoint *pc_Object;
   GraphicObjectPolygon *pc_Polygon;
   GraphicObjectSpline  *pc_Spline;
   bool                  b_IsSpline;
   int                   i;
   int                   i_PointCount;

   i_PointCount = (int) m_pc_CurvePoints.count();

   b_IsSpline = m_ConstructSplineCheck->isChecked();

   // not enough points to create a polygon?
   if(i_PointCount<2)
   {
      ConstructDeleteCurvePoints();
      m_pc_ContructEditedObject = NULL;
      return;
   }

   pc_Object = NULL;
   
   if(m_pc_ContructEditedObject ==NULL)
   {
      // we have a new object
      if( b_IsSpline)
      {
         pc_Spline = new GraphicObjectSpline();
         m_pc_ObjectLayer->AddObject(pc_Spline);
         m_pc_CurvePolygons.append(pc_Spline);
         pc_Object = pc_Spline;
      }
      else
      {
         pc_Polygon = new GraphicObjectPolygon();
         m_pc_ObjectLayer->AddObject(pc_Polygon);
         m_pc_CurvePolygons.append(pc_Polygon);
         pc_Object = pc_Polygon;
      }
   }
   else // change an existing object
   {
      if(m_pc_ContructEditedObject->GetObjectType()== GraphicObject::TypePolygon)
      {
         if(b_IsSpline)
         {
            // remove polygon and replace by spline
            ConstructRemoveObject(m_pc_ContructEditedObject);
            m_pc_ContructEditedObject = NULL;

            pc_Spline = new GraphicObjectSpline();
            m_pc_ObjectLayer->AddObject(pc_Spline);
            m_pc_CurvePolygons.append(pc_Spline);
            pc_Object = pc_Spline;
         }
         else
         {
            // replace an existing polygon
            pc_Polygon = (GraphicObjectPolygon*) m_pc_ContructEditedObject;
            pc_Polygon->RemoveAllCoordinates();
            pc_Polygon->SetSelectable(true);
            pc_Polygon->SetColor(0x000000);
            pc_Object = pc_Polygon;
         }
      }
      else if(m_pc_ContructEditedObject->GetObjectType()== GraphicObject::TypeSpline)
      {
         if(b_IsSpline)
         {
            // replace an existing spline
            pc_Spline = (GraphicObjectSpline*) m_pc_ContructEditedObject;
            pc_Spline->RemoveAllCoordinates();
            pc_Spline->SetSelectable(true);
            pc_Spline->SetColor(0x000000);
            pc_Object = pc_Spline;
         }
         else
         {
            // remove spline and replace by polygon
            ConstructRemoveObject(m_pc_ContructEditedObject);
            m_pc_ContructEditedObject = NULL;

            pc_Polygon = new GraphicObjectPolygon();
            m_pc_ObjectLayer->AddObject(pc_Polygon);
            m_pc_CurvePolygons.append(pc_Polygon);
            pc_Object = pc_Polygon;
         }
         // replace existing spline
      }
      
   } // else change existing object

   if(pc_Object ==NULL)
   {
      // should not occur
      return;
   }
   // move all points into the polygon
   for(i=0;i<(int)m_pc_CurvePoints.count();i++)
   {
      GraphicObjectPoint *pc_Point;
      GeoCoordinate       c_Coordinate;
      pc_Point = m_pc_CurvePoints.at(i);
      c_Coordinate = pc_Point->GetPosition();

      pc_Object->AddCoordinate(c_Coordinate);
      
   }

   // set closed state depending on the checkbox
   if( m_ConstructClosedFigureCheck->isChecked() )
   {
      pc_Object->SetClosedFigure(true);
   }
   else
   {
      pc_Object->SetClosedFigure(false);
   }

   // delete all construction help points and lines
   ConstructDeleteCurvePoints();
   ConstructReCalcConstructSpline();
   
   m_pc_ContructEditedObject = NULL;

} // ConstructPointsToPolygon()

//===========================================================================
// move all construction points into an polygon
//===========================================================================


void GraphikBox::ContructPolygonToPoints(GraphicObjectMultipoint *pc_Object)
{
   GeoCoordinateList * pc_CoordinateList;
   GeoCoordinate       c_Current;
   GeoCoordinate       c_Next;
   int                 i_Count;
   int                 i;
      
   m_pc_ContructEditedObject = pc_Object;
   if(pc_Object == NULL)
   {
      return ;
   }
   ConstructDeleteCurvePoints();

   pc_CoordinateList = m_pc_ContructEditedObject->GetCoordinateList();
   i_Count = pc_CoordinateList->GetCount();
   // should not occur
   if(i_Count <=0)
   {
      return;
   }
   
   for(i=0;i<i_Count;i++)
   {
      GraphicObjectPoint *pc_Point;
      GraphicObjectLine  *pc_Line;
      c_Current = pc_CoordinateList->Get(i);
      c_Next    = pc_CoordinateList->Get( (i+1) %i_Count);

      pc_Point = new GraphicObjectPoint(c_Current);
      m_pc_CurvePoints.append(pc_Point);

      m_pc_ObjectLayer->AddObject(pc_Point);

      // create a line from last to current
      pc_Line = new GraphicObjectLine(c_Current,c_Next);
      pc_Line->SetSelectable(false);
      m_pc_CurveLines.append(pc_Line);
      m_pc_ObjectLayer->AddObject(pc_Line);

   }
   m_pc_ContructEditedObject->SetSelectable(false);
   m_pc_ContructEditedObject->SetColor(0x777777);

   if(pc_Object->IsClosedFigure() )
   {
      m_ConstructClosedFigureCheck->setChecked(true);
      
   }
   else
   {
      m_ConstructClosedFigureCheck->setChecked(false);
   }
   if(pc_Object->GetObjectType() == GraphicObject::TypePolygon)
   {
      m_ConstructSplineCheck->setChecked(false);
   }
   else if(pc_Object->GetObjectType() == GraphicObject::TypeSpline)
   {
      m_ConstructSplineCheck->setChecked(true);
   }

   ConstructReCalcConstructSpline();
   // adjust visiblity of the last line
   ConstructSetLastLinesVisible();      
} // ContructPolygonToPoints()

//===========================================================================
//
//===========================================================================

void GraphikBox::ConstructEditPoint()
{
   GraphicObject *pc_Object;
   int            i_SelectedCount;


   pc_Object =NULL;

   i_SelectedCount = m_pc_ObjectLayer->GetNumberOfSelectedObjects(pc_Object);
   if(   i_SelectedCount ==1
      && pc_Object != NULL
      && (    pc_Object->GetObjectType() == GraphicObject::TypePolygon
           || pc_Object->GetObjectType() == GraphicObject::TypeSpline
         )
     )
   {
      if(   m_pc_CurvePoints.count() >0
         && m_pc_ContructEditedObject != NULL)
      {
         // another polygon is just edited.
         // finish it
         ConstructPointsToPolygon();
      }
      
      ContructPolygonToPoints((GraphicObjectPolygon *)pc_Object);
      // prepare Polygon into single points
      m_pc_GlobalStorage->m_pc_ObjectMain
                                 ->DeselectAllObjects();
      m_pc_GlobalStorage->m_pc_ImageFrame->RedrawAll();
   }
   else
   {
      if (m_pc_CurvePoints.count() >0)
      {
         // create a polygon out of the points
         ConstructPointsToPolygon();
         ConstructSwitchToMode(ConstructModeModify);
         m_pc_GlobalStorage->m_pc_ObjectMain
                                 ->DeselectAllObjects();
         m_pc_GlobalStorage->m_pc_ImageFrame->RedrawAll();
         
      }
   }
   
} // ConstructEditPoint()

//===========================================================================
//
//===========================================================================

void GraphikBox::ConstructDeleteAllPolygons()
{
   // clean edit pointer
   m_pc_ContructEditedObject = NULL;

   // for all polygons
   while(m_pc_CurvePolygons.count() >0)
   {
      GraphicObjectMultipoint *pc_Object;
      pc_Object = m_pc_CurvePolygons.getFirst();

      // remove object from layer
      m_pc_ObjectLayer->RemoveObject(pc_Object);

      // remove from internal list
      m_pc_CurvePolygons.removeFirst();

      // finaly delete the object itself
      delete pc_Object;

   } // while polygons available

   
} // ConstructDeleteAllPolygons()

//===========================================================================
//
//===========================================================================


int GraphikBox::ConstructWriteFile(PericoFileWriter *pc_FileWriter)
{
   int i_Count;
   int i_ActualPolygon;

   i_Count = (int) m_pc_CurvePolygons.count();
   pc_FileWriter->StartStructure("PolygonList","PolygonList");

   pc_FileWriter->AddValue("NumberOfObjects", i_Count);

   for(i_ActualPolygon=0;i_ActualPolygon<i_Count;i_ActualPolygon++)
   {
      GraphicObjectMultipoint *pc_Object;
      pc_Object = m_pc_CurvePolygons.at(i_ActualPolygon);
      if(pc_Object->GetObjectType() == GraphicObject::TypePolygon)
      {
         pc_Object->WriteFile(pc_FileWriter,"Polygon");
      }
      else if(pc_Object->GetObjectType() == GraphicObject::TypeSpline)
      {
         pc_Object->WriteFile(pc_FileWriter,"Spline");
      }
      
   } // for all polygons
   
   pc_FileWriter->FinishStructure();

   // everything is OK
   return 0;

} // ConstructWriteFile()

//===========================================================================
//
//===========================================================================

int GraphikBox::ConstructLoadFile(PericoFileWriter *pc_FileWriter)
{
   // first clean up internal storage
   ConstructDeleteCurvePoints();
   ConstructDeleteAllPolygons();


   PERICO_READ_STRUCTURE_START
   if(qs_Name.compare("NumberOfObjects") == 0)
   {
      // currently ignore it
   }
   else if(    qs_Name.compare("Polygon")==0
            && b_IsStartStructure)
   {
      GraphicObjectPolygon *pc_Polygon;

      pc_Polygon = new GraphicObjectPolygon(pc_FileWriter);

      if( pc_Polygon!= NULL )
      {
         // reset default color and accessibility
         pc_Polygon->SetColor(0x000000);
         pc_Polygon->SetSelectable(true);

         // add object to list and layer
         m_pc_CurvePolygons.append(pc_Polygon);
         m_pc_ObjectLayer->AddObject(pc_Polygon);
      }
      
   }
   else if(    qs_Name.compare("Spline")==0
            && b_IsStartStructure)
   {
      GraphicObjectSpline *pc_Spline;

      pc_Spline = new GraphicObjectSpline(pc_FileWriter);

      if( pc_Spline!= NULL )
      {
         // reset default color and accessibility
         pc_Spline->SetColor(0x000000);
         pc_Spline->SetSelectable(true);

         // add object to list and layer
         m_pc_CurvePolygons.append(pc_Spline);
         m_pc_ObjectLayer->AddObject(pc_Spline);
      }

   }

   PERICO_READ_STRUCTURE_END

   // everything is OK
   return 0;

} // ConstructLoadFile()

//===========================================================================
//
//===========================================================================

void GraphikBox::ConstructSetLastLinesVisible()
{
   GraphicObjectLine *pc_LastLine;
   int i_LinesCount;

   i_LinesCount = m_pc_CurveLines.count();

   // no lines available;
   if(i_LinesCount <=0)
   {
      return;
   }

   // make a previos line visible every time
   if(i_LinesCount >1)
   {
      GraphicObjectLine *pc_PreviousLine;
      pc_PreviousLine = m_pc_CurveLines.at(i_LinesCount-2);
      pc_PreviousLine->SetVisible(true);
   }

   // the last line depends on the checkbox
   pc_LastLine = m_pc_CurveLines.at(i_LinesCount-1);

   if( m_i_ConstructMode== ConstructModeAdd)
   {
      // in the add mode the last line is visbile
      pc_LastLine->SetVisible(true);
   }
   else if( m_i_ConstructMode== ConstructModeAddMiddle)
   {
      //in the add middle mode, the last line depends on the check box
      if(m_ConstructClosedFigureCheck->isChecked())
      {
         pc_LastLine->SetVisible(true);
      }
      else
      {
         pc_LastLine->SetVisible(false);
      }
   }
   else // modify mode or off
   {
      // status depends on the checkbox
      if(m_ConstructClosedFigureCheck->isChecked())
      {
         pc_LastLine->SetVisible(true);
      }
      else
      {
         pc_LastLine->SetVisible(false);
      }
   }      
   
} // ConstructSetLastLinesVisible()

//===========================================================================
//
//===========================================================================

void GraphikBox::ConstructClosedFigureChanged(bool b_On)
{
   // change visibility of the last line
   ConstructSetLastLinesVisible();

   // and redraw all 
   m_pc_GlobalStorage->m_pc_ImageFrame->RedrawAll();
   
} // ConstructClosedFigureChanged()

//===========================================================================
//
//===========================================================================

void GraphikBox::ConstructSplineChanged(bool b_On)
{
   // change visibility of the last line
   ConstructReCalcConstructSpline();

   // and redraw all
   m_pc_GlobalStorage->m_pc_ImageFrame->RedrawAll();

} // ConstructClosedFigureChanged()

//===========================================================================
//
//===========================================================================

RectRegion GraphikBox::ConstructReCalcConstructSpline()
{
   RectRegion c_Return;
   int        i_Count;
   int        i;

   c_Return.SetEmpty();
   if( ! m_ConstructSplineCheck->isChecked() )
   {
      m_pc_ContructEditedSpline->SetVisible(false);
      return c_Return;
   }

   m_pc_ContructEditedSpline->SetVisible(true);
   m_pc_ContructEditedSpline->RemoveAllCoordinates();

   // now rebuild the object from the current points
   i_Count = m_pc_CurvePoints.count();

   for(i=0;i<i_Count;i++)
   {
      GraphicObjectPoint *pc_Point;
      GeoCoordinate      c_Coordinate;
      pc_Point = m_pc_CurvePoints.at(i);

      c_Coordinate = pc_Point->GetPosition();

      m_pc_ContructEditedSpline->AddCoordinate(c_Coordinate);
      
   }

   // get open/close state from current checkbox state
   if(m_ConstructClosedFigureCheck->isChecked())
   {
      m_pc_ContructEditedSpline->SetClosedFigure(true);
   }
   else
   {
      m_pc_ContructEditedSpline->SetClosedFigure(false);
   }

   c_Return = m_pc_ContructEditedSpline->GetSize();
   
   return c_Return;
} // ConstructReCalcConstructSpline()

//===========================================================================
//
//===========================================================================

void GraphikBox::ConstructRemoveObject(GraphicObject *pc_Object)
{
   int i_Index;
   if(pc_Object ==NULL)
   {
      return;
   }

   // remove it from the layer
   m_pc_ObjectLayer->RemoveObject(pc_Object);

   // find the index in the list
   i_Index = m_pc_CurvePolygons.find((GraphicObjectMultipoint *) pc_Object);
   if(i_Index == -1)
   {
      // should not occur)
      return;
   }
   m_pc_CurvePolygons.remove(i_Index);

   // finaly delete the polygon itself
   delete pc_Object;

} // ConstructRemoveObject()

//===========================================================================
//
//===========================================================================

void GraphikBox::showEvent(QShowEvent *)
{
   SetModeTo(m_i_CurrentGraphikMode);

} // showEvent();

//===========================================================================
//
//===========================================================================




