//---------------------------------------------------------------------------
// This sourcefile is created by Peter Jodda / peter@jodda.de
// It is under the GPL license.
// If you expand this file, please contact me, that I can integrate the
// enhancements in the original file.
//---------------------------------------------------------------------------
// Module: PaintBox
//---------------------------------------------------------------------------
// Purpose: Dialog to select the main painting options.
//---------------------------------------------------------------------------

#include <stdio.h>
#include <stdlib.h>
#include <math.h>

#include <qlayout.h>
#include <qgrid.h>
#include <qlabel.h>
#include <qgroupbox.h>
#include <qvgroupbox.h>
#include <qhgroupbox.h>
#include <qfont.h>
#include <qfontmetrics.h>
#include <qcombobox.h>
#include <qgrid.h>


#include "PaintBox.h"
#include "BrushMode.h"

#include "PixelBuffer.h"
#include "QtUtilities.h"
#include "ImageFrame.h"
#include "MessageServer.h"
#include "Selection.h"
#include "BufferPainter.h"
#include "MouseEvent.h"
#include "Utilities.h"

#include "QUPixelButton.h"
#include "QULabelSlider.h"
#include "BufferFill.h"
#include "MaskSettings.h"
#include "ApplicationStorage.h"
#include "MainFrame.h"
#include "MotiveBox.h"
#include "LineIterator.h"
#include "GraphicObjectPoint.h"
#include "Zoom.h"
#include "IconLibrary.h"
#include "FileLoader.h"
#include "Utilities.h"

#define PAINTBOXLAYER "PaintBox"
//============================================================================
//
//============================================================================

PaintBox::PaintBox(QWidget *parent,
                        GlobalStorage *pc_Global,
                        MainFrame *pc_MainFrame) :
                StandardDialog(parent,pc_Global),
                Tracer("PaintBox")
{
   QBoxLayout    *l_Layout;

   QHGroupBox   *ActivationBox;

   LoadPosition("PaintBox");
   setCaption(tr("Paint"));
   TRACE << "Paint Started\n" ;

   m_pc_MainFrame               = pc_MainFrame;
   m_b_TransparencyEnabled      = false;
   m_i_CurrentThickness         = 5;
   m_i_TransparencyClearMode    = ClearModeAutomatic;
   m_b_UseTransparencyMask      = false;
   m_p_FillDistanceColor        = 0x808080;
   m_b_FillFromStartPoint       = true;
   m_b_FillFromColor            = false;
   m_i_FillDistance             = 0;
   m_i_LightnessFactor          = 0;
   m_i_FilterWidth              = 1;
   m_b_FilterNearDistanceActive = false;
   m_i_FilterNearDistance       = 30;
   m_i_FilterAlgorithm          = BufferPainter::FilterAlgorithmMiddle;
   m_i_SpeckleSize              = 4;
   m_i_SpeckleDensity           = 100;
   m_i_SpeckleDensityMaximum    = 5000;
   m_i_SmearStrength            = 10;
   m_p_RecolorFromColor         = PIXEL_MAKE(12,244,66);
   m_p_RecolorToColor           = PIXEL_MAKE(250,100,190);
   m_i_RecolorDistance          = 10;
   m_i_BrushShapeStyle          = BrushShapeCircle;
   m_pc_BrushShapeFreeBuffer    = NULL;
   InitBrushShapeFree();
   m_BrushShapeUpdateConnection.SetParentCallback(
               Callback(FreeBrushPictureConnectionEventArrivedCB,this));



   // create a new layer
   m_pc_ObjectLayer= m_pc_GlobalStorage->m_pc_ObjectMain->CreateLayer(PAINTBOXLAYER);
   m_pc_CopyBrushOriginPoint = new GraphicObjectPoint( 0,0);
   m_pc_ObjectLayer->AddObject(m_pc_CopyBrushOriginPoint);
   m_pc_ObjectLayer->SetVisible(false);

   // create a buffer painter and do some basic settings
   m_pc_BufferPainter = new BufferPainter();
   m_pc_BufferPainter->SetLineWidth(1);
   m_pc_BufferPainter->SetDrawInnerEnable(false);
   m_pc_BufferPainter->SetTransparencyEnable(false);
   m_pc_BufferPainter->SetTransparencyFactor(50);
   CreateBrushShape(m_i_CurrentThickness);
   m_pc_BufferPainter->SetFilterWidth(m_i_FilterWidth);
   m_pc_BufferPainter->SetFilterNearDistanceActive(m_b_FilterNearDistanceActive);
   m_pc_BufferPainter->SetFilterNearDistance(m_i_FilterNearDistance);
   m_pc_BufferPainter->SetFilterAlgorithm(m_i_FilterAlgorithm);
   m_pc_BufferPainter->SetRecolorParameter(m_p_RecolorFromColor,
                                           m_p_RecolorToColor,
                                           m_i_RecolorDistance);

   l_Layout = new  QVBoxLayout( m_w_TopWidget,1);

   // --- box for activation
   ActivationBox = new QHGroupBox(tr("Paint Operation"),m_w_TopWidget);
   ActivationBox->setColumns(2);
   ActivationBox->setInsideSpacing(3);

//   ActivationBox->setInsideMargin(ActivationBox->insideMargin()*2);


   l_Layout->addWidget(ActivationBox);
   {
      // buttons to establish painting mode

      m_BrushButton         = new QUIconButton(ActivationBox,
                                                "Brush",
                                                tr("Brush"));
      m_FillButton          = new QUIconButton(ActivationBox,
                                                "Fill",
                                                tr("Fill"));
      m_LightnessButton     = new QUIconButton(ActivationBox,
                                                 "Sun",
                                                 tr("Lightness"));
      m_SaturationButton    = new QUIconButton(ActivationBox,
                                                "Saturation",
                                                tr("Saturation"));
      m_FilterButton        = new QUIconButton(ActivationBox,
                                                "Filter",
                                                tr("Filter"));
      m_SpeckleButton       = new QUIconButton(ActivationBox,
                                                "Speckle",
                                                tr("Speckle"));
      m_AirBrushButton      = new QUIconButton(ActivationBox,
                                                "Airbrush",
                                                tr("Airbrush"));
      m_SmearButton         = new QUIconButton(ActivationBox,
                                                "Smear",
                                                tr("Smear"));
      m_CopyButton          = new QUIconButton(ActivationBox,
                                                "CopyBrush",
                                                tr("Copy Brush"));
      m_RecolorButton       = new QUIconButton(ActivationBox,
                                                "Recolor",
                                                tr("Change Color"));

      connect( m_BrushButton ,
               SIGNAL(clicked()),
               SLOT( ActivateBrush() ) );
      connect( m_FillButton ,
               SIGNAL(clicked()),
               SLOT( ActivateFill() ) );
      connect( m_LightnessButton ,
               SIGNAL(clicked()),
               SLOT( ActivateLightness() ) );
      connect( m_SaturationButton ,
               SIGNAL(clicked()),
               SLOT( ActivateSaturation() ) );
      connect( m_FilterButton ,
               SIGNAL(clicked()),
               SLOT( ActivateFilter() ) );
      connect( m_SpeckleButton ,
               SIGNAL(clicked()),
               SLOT( ActivateSpeckle() ) );
      connect( m_AirBrushButton ,
               SIGNAL(clicked()),
               SLOT( ActivateAirBrush() ) );
      connect( m_SmearButton ,
               SIGNAL(clicked()),
               SLOT( ActivateSmear() ) );
      connect( m_CopyButton ,
               SIGNAL(clicked()),
               SLOT( ActivateCopy() ) );
      connect( m_RecolorButton ,
               SIGNAL(clicked()),
               SLOT( ActivateRecolor() ) );
   }


   // box for brush style
   m_BrushSizeModeBox = new QVGroupBox(tr("Brush Style"),m_w_TopWidget);
   l_Layout->addWidget(m_BrushSizeModeBox);

   {
      m_BrushThicknessSlider = new QULabelSlider(m_BrushSizeModeBox,
                                       QULabelSlider::LabelStyleAutomatic |
                                          QULabelSlider::LabelStyleEdit
                                       ,
                                       0,30,
                                       m_i_CurrentThickness,
                                       tr("Brush Size"),
                                       6);

      m_b_BrushContinuous     = true;
      m_BrushContinuousEnable = new QCheckBox(tr("Continous"),m_BrushSizeModeBox);
      m_BrushContinuousEnable->setChecked(m_b_BrushContinuous);

      connect( m_BrushThicknessSlider,
               SIGNAL(valueChanged(int)),
               SLOT( ThicknessChanged(int )) );

      connect( m_BrushContinuousEnable,
               SIGNAL(toggled(bool)) ,
               SLOT( ContinuousEnableChanged(bool) )
             );

   } // box for brush style

   // --- box for brush shape style
   m_BrushShapeModeBox = new QHGroupBox(tr("Brush Shape"),m_w_TopWidget);
   m_BrushShapeModeBox->setColumns(1);
   m_BrushShapeModeBox->setInsideMargin(m_BrushShapeModeBox->insideMargin()*2);
   m_BrushShapeModeBox->setInsideSpacing(3);
   l_Layout->addWidget(m_BrushShapeModeBox);
   {
      // buttons to set brush style
      m_BrushShapeCircleButton = new QUIconButton(m_BrushShapeModeBox,
                                                "BrushStyleCircle",
                                                tr("Circle"));
      m_BrushShapeSquareButton = new QUIconButton(m_BrushShapeModeBox,
                                                "BrushStyleSquare",
                                                tr("Square"));
      m_BrushShapeVerticalButton = new QUIconButton(m_BrushShapeModeBox,
                                                "BrushStyleVertical",
                                                tr("Vertical"));
      m_BrushShapeHorizontalButton = new QUIconButton(m_BrushShapeModeBox,
                                                "BrushStyleHorizontal",
                                                tr("Horizontal"));
      m_BrushShapeNWSEButton = new QUIconButton(m_BrushShapeModeBox,
                                                "BrushStyleNWSE",
                                                tr("Diagonal Down"));
      m_BrushShapeSWNEButton = new QUIconButton(m_BrushShapeModeBox,
                                                "BrushStyleSWNE",
                                                tr("Diagonal Up"));
      m_BrushShapeFreeButton = new QUSelectIconButton(m_BrushShapeModeBox,
                                                "BrushStyleFree",
                                                tr("Free Figure"),
                                                QUIconButton::ModeImageReceiver
                                                | QUIconButton::ModeContextMenu
                                                );
      m_BrushShapeFreeButton->AddContextString(tr("Edit"));
      m_BrushShapeFreeButton->AddContextString(tr("Save"));
      m_BrushShapeFreeButton->AddContextString(tr("Load"));

      connect( m_BrushShapeCircleButton ,
               SIGNAL(clicked()),
               SLOT( BrushShapeCircleSelected() ) );
      connect( m_BrushShapeSquareButton ,
               SIGNAL(clicked()),
               SLOT( BrushShapeSquareSelected() ) );
      connect( m_BrushShapeVerticalButton ,
               SIGNAL(clicked()),
               SLOT( BrushShapeVerticalSelected() ) );
      connect( m_BrushShapeHorizontalButton ,
               SIGNAL(clicked()),
               SLOT( BrushShapeHorizontalSelected() ) );
      connect(  m_BrushShapeNWSEButton,
               SIGNAL(clicked()),
               SLOT( BrushShapeNWSESelected() ) );
      connect( m_BrushShapeSWNEButton ,
               SIGNAL(clicked()),
               SLOT( BrushShapeSWNESelected() ) );
      connect( m_BrushShapeFreeButton ,
               SIGNAL(clicked()),
               SLOT( BrushShapeFreeSelected() ) );

      // special connections for the free button
      connect( m_BrushShapeFreeButton ,
               SIGNAL(ImageDropped(PixelBuffer *)),
               SLOT( BrushShapeFreeImageDropped(PixelBuffer *) ) );
      connect( m_BrushShapeFreeButton ,
               SIGNAL(ContextMenuSelected(int)),
               SLOT( BrushShapeFreeMenuSelected(int) ) );
   }

   // box for brush color
   m_BrushColorModeBox = new QVGroupBox(tr("Color"),m_w_TopWidget);
   l_Layout->addWidget(m_BrushColorModeBox);

   {
      m_BrushColorWidget = new QUColorOrPattern(m_BrushColorModeBox,
                                               m_pc_GlobalStorage,
                                               pc_MainFrame->GetColorSelectBox());
      m_BrushColorWidget->SetColor(0x335577);

      connect( m_BrushColorWidget,
               SIGNAL(ColorChanged(Pixel)) ,
               SLOT( BrushColorChanged(Pixel ) )
             );
      connect( m_BrushColorWidget,
               SIGNAL(ImageDropped(PixelBuffer *)) ,
               SLOT( BrushPatternChanged(PixelBuffer *) )
             );
      connect( m_BrushColorWidget,
               SIGNAL(ModeChange(BrushMode)) ,
               SLOT( BrushModeChanged(BrushMode) )
             );

   } // box for Color style

   // -- box for transparency
   m_TransparencyBox = new QVGroupBox(tr("Transparency"),m_w_TopWidget);
   l_Layout->addWidget(m_TransparencyBox);

   {
      // inner is off
      m_i_TransparencyFactor = 50;
      m_TransparencyEnable = new QCheckBox(tr("Enable"),m_TransparencyBox);
      m_TransparencyEnable->setChecked(m_b_TransparencyEnabled);

      m_TransparencySlider = new QULabelSlider(m_TransparencyBox,
                                       QULabelSlider::LabelStyleAutomatic |
                                          QULabelSlider::LabelStyleEdit
                                       ,
                                       0,100,
                                       m_i_TransparencyFactor,
                                       tr("Factor"),
                                       6);
      m_TransparencyMaskUseCheck = new QCheckBox(tr("Use Mask"),
                                                  m_TransparencyBox);

      connect( m_TransparencyEnable,
               SIGNAL(toggled(bool)) ,
               SLOT( TransparencyEnableChanged(bool) )
             );
      connect( m_TransparencySlider,
               SIGNAL(valueChanged(int)),
               SLOT( TransparencyFactorChanged(int )) );

      connect( m_TransparencyMaskUseCheck,
               SIGNAL(toggled(bool)) ,
               SLOT( TransparencyMaskUseCheckChanged(bool)) );

   } // block for tranparency box

   // -- box for Mask clearing mode
   m_MaskClearModeBox = new QHGroupBox(tr(" Mask Clear Mode"),m_w_TopWidget);
   l_Layout->addWidget(m_MaskClearModeBox);

   {
      m_TransparencyClearCombo = new QComboBox(false,
                                               m_MaskClearModeBox,
                                               "TransparencyMode");
      m_TransparencyClearCombo->insertItem(tr("manual"),0);
      m_TransparencyClearCombo->insertItem(tr("automatic"),1);
      m_TransparencyClearCombo->setCurrentItem(m_i_TransparencyClearMode);
      m_ClearTransparencyMaskButton = new QPushButton(tr("Clear"),m_MaskClearModeBox);

      connect( m_TransparencyClearCombo,
               SIGNAL(activated(int)),
               SLOT( TransparencyAutomaticChanged(int )) );
      connect( m_ClearTransparencyMaskButton ,
               SIGNAL(clicked()),
               SLOT( TransparencyMaskClear() ) );

   } // block for mask clearing mode

   // -- box for fill mode
   m_FillModeBox = new QVGroupBox(tr("Fill Mode"),m_w_TopWidget);
   l_Layout->addWidget(m_FillModeBox);
   {
      QWidget *GridWidget;
      QGridLayout *grid;
      m_FillDistanceSlider = new QULabelSlider(m_FillModeBox,
                                               QULabelSlider::LabelStyleAutomatic |
                                                 QULabelSlider::LabelStyleEdit
                                               ,
                                               0,200,
                                               m_i_FillDistance,
                                               tr("Color Distance"),
                                               6);

      new QLabel(tr("Distance from"),m_FillModeBox);
      GridWidget = new QWidget(m_FillModeBox);
      grid = new QGridLayout( GridWidget, 2, 3, 10 );

      m_FillModeFromOrigin = new QRadioButton(tr("Startpoint"),GridWidget);
      m_FillModeFromColor  = new QRadioButton(tr("Color"),GridWidget);

      m_FillDistanceColor = new QUColorDragWidget(GridWidget,
                                               QUColorDragWidget::ModePipette,
                                               m_pc_GlobalStorage);
      m_FillDistanceColor->SetColor(m_p_FillDistanceColor);

      grid->addWidget(m_FillModeFromOrigin,0,1);
      grid->addWidget(m_FillModeFromColor,1,1);
      grid->addWidget(m_FillDistanceColor,1,2);
      grid->addColSpacing(0,g_Application->m_pc_QtUtility->GetMaximalFontWidth());
      grid->setColStretch(1,10);
      grid->setColStretch(2,2);

      m_FillModeFromOrigin ->setChecked(true);
      m_FillModeFromColor  ->setChecked(false);

      connect( m_FillModeFromOrigin,
               SIGNAL(toggled(bool)) ,
               SLOT( FillDistanceFromStartPoint(bool) )
             );
      connect( m_FillModeFromColor,
               SIGNAL(toggled(bool)) ,
               SLOT( FillDistanceFromColor(bool) )
             );
      connect( m_FillDistanceSlider,
               SIGNAL(valueChanged(int)),
               SLOT( FillDistanceChanged(int )) );
      connect( m_FillDistanceColor,
               SIGNAL(ColorChanged(Pixel)) ,
               SLOT( FillColorChanged(Pixel ) )
             );



   } // block for the fill mode


   // -- box for lightness
   m_LightnessModeBox = new QVGroupBox(tr("Value Modification"),
                                       m_w_TopWidget);
   l_Layout->addWidget(m_LightnessModeBox);
   {
       m_LightnessSlider = new QULabelSlider(m_LightnessModeBox,
                                             QULabelSlider::LabelStyleAutomatic |
                                               QULabelSlider::LabelStyleEdit
                                             ,
                                             -50,
                                             50,
                                             m_i_LightnessFactor,
                                             tr("Change"),
                                             6);
      connect( m_LightnessSlider,
               SIGNAL(valueChanged(int)),
               SLOT( LightnessFactorChanged(int )) );

   } // lighness box


   // -- box for Filter mode
   m_FilterModeBox = new QVGroupBox(tr("Filter"),m_w_TopWidget);
   l_Layout->addWidget(m_FilterModeBox);
   {
      QButtonGroup * m_AlgorithmBox;
      m_FilterWidthSlider = new QULabelSlider(m_FilterModeBox,
                                              QULabelSlider::LabelStyleAutomatic |
                                               QULabelSlider::LabelStyleEdit
                                              ,
                                              1,10,
                                              m_i_FilterWidth,
                                              tr("Width"),
                                              6);
      m_FilterDistanceActiveCheck = new QCheckBox(tr("Filter only near colors"),
                                            m_FilterModeBox);

      m_FilterDistanceSlider = new QULabelSlider(m_FilterModeBox,
                                              QULabelSlider::LabelStyleAutomatic |
                                                 QULabelSlider::LabelStyleEdit
                                              ,
                                              1,100,
                                              m_i_FilterNearDistance,
                                              tr("Distance"),
                                              6);

      m_AlgorithmBox = new QButtonGroup(3,
                                    QGroupBox::Horizontal,
                                    tr("Filter Algorithm"),
                                    m_FilterModeBox);

      m_FilterMiddleButton  = new QRadioButton(tr("Middle"),m_AlgorithmBox);
      m_FilterMedianButton  = new QRadioButton(tr("Median"),m_AlgorithmBox);
      m_FilterSharpButton   = new QRadioButton(tr("Sharp"),m_AlgorithmBox);

      switch(m_i_FilterAlgorithm)
      {
      case BufferPainter::FilterAlgorithmMiddle:
         m_FilterMiddleButton->setChecked(1);
         break;
      case BufferPainter::FilterAlgorithmMedian:
         m_FilterMedianButton->setChecked(1);
         break;
      case BufferPainter::FilterAlgorithmSharp:
         m_FilterSharpButton->setChecked(1);
         break;
      default:
         break;
      }
      connect( m_AlgorithmBox,
               SIGNAL(clicked(int)),
               SLOT(FilterAlgorithmChanged(int)) );



      connect( m_FilterWidthSlider,
               SIGNAL( valueChanged(int)),
               SLOT(   FilterWidthChanged(int )) );

      connect( m_FilterDistanceActiveCheck,
               SIGNAL( toggled(bool)) ,
               SLOT(   FilterDistanceEnableChanged(bool) ) );

      connect( m_FilterDistanceSlider,
               SIGNAL( valueChanged(int)),
               SLOT(   FilterDistanceChanged(int )) );

   } // filter mode box

   // box for speckle
   m_SpeckleModeBox = new QVGroupBox(tr("Speckle"),m_w_TopWidget);
   l_Layout->addWidget(m_SpeckleModeBox);
   {
      m_SpeckleSizeSlider = new QULabelSlider(m_SpeckleModeBox,
                                              QULabelSlider::LabelStyleAutomatic |
                                                 QULabelSlider::LabelStyleEdit
                                              ,
                                              1,20,
                                              m_i_SpeckleSize,
                                              tr("Size"),
                                              6);
      m_SpeckleDensitySlider = new QULabelSlider(m_SpeckleModeBox,
                                              QULabelSlider::LabelStyleAutomatic |
                                                QULabelSlider::LabelStyleEdit
                                              ,
                                              1,m_i_SpeckleDensityMaximum,
                                              m_i_SpeckleDensity,
                                              tr("Density"),
                                              6);
      connect( m_SpeckleSizeSlider,
               SIGNAL( valueChanged(int)),
               SLOT(   SpeckleSizeChanged(int )) );
      connect( m_SpeckleDensitySlider,
               SIGNAL( valueChanged(int)),
               SLOT(   SpeckleDensityChanged(int )) );
      
   }

   // box for smear
   m_SmearModeBox = new QVGroupBox(tr("Smear"),m_w_TopWidget);
   l_Layout->addWidget(m_SmearModeBox);
   {
      m_SmearStrengthSlider = new QULabelSlider(m_SmearModeBox,
                                              QULabelSlider::LabelStyleAutomatic |
                                                 QULabelSlider::LabelStyleEdit
                                              ,
                                              1,40,
                                              m_i_SmearStrength,
                                              tr("Strength"),
                                              6);
      connect( m_SmearStrengthSlider,
               SIGNAL( valueChanged(int)),
               SLOT(   SmearStrengthChanged(int )) );

   }

   // box for Copybrush
   m_CopyBrushModeBox = new QVGroupBox(tr("Copy Brush"),m_w_TopWidget);
   l_Layout->addWidget(m_CopyBrushModeBox);
   {
      m_CopyBrushResetButton = new QPushButton(
                  tr("New Origin Point"),m_CopyBrushModeBox);

      connect( m_CopyBrushResetButton ,
               SIGNAL(clicked()),
               SLOT( CopyBrushNewOrigin() ) );

   }

   // box for Recolor
   m_RecolorModeBox = new QVGroupBox(tr("Change Color"),m_w_TopWidget);
   l_Layout->addWidget(m_RecolorModeBox);
   {
      QHBoxLayout *l_ColorLayout;
      QWidget     *w_ColorBox;

      w_ColorBox = new QWidget(m_RecolorModeBox);

      l_ColorLayout = new QHBoxLayout(w_ColorBox);
      l_ColorLayout->addWidget( new QLabel(tr("From"),w_ColorBox));
      m_RecolorFromColor = new QUColorDragWidget(w_ColorBox,
                                               QUColorDragWidget::ModePipette
                                               | QUColorDragWidget::ModeEnableColorselectBox,
                                               m_pc_GlobalStorage,
                                               0,0,
                                               pc_MainFrame->GetColorSelectBox());
      l_ColorLayout->addWidget(m_RecolorFromColor);

      l_ColorLayout->addWidget( new QtUtilSpacer(true,false,w_ColorBox));
      
                                                     
      l_ColorLayout->addWidget( new QLabel(tr("To")  ,w_ColorBox));
      m_RecolorToColor = new QUColorDragWidget(w_ColorBox,
                                               QUColorDragWidget::ModePipette
                                               | QUColorDragWidget::ModeEnableColorselectBox,
                                               m_pc_GlobalStorage,
                                               0,0,
                                               pc_MainFrame->GetColorSelectBox());
      l_ColorLayout->addWidget(m_RecolorToColor);

      m_RecolorDistanceSlider = new QULabelSlider(m_RecolorModeBox,
                                               QULabelSlider::LabelStyleAutomatic |
                                                 QULabelSlider::LabelStyleEdit
                                               ,
                                               1,30,
                                               m_i_RecolorDistance,
                                               tr("Color Distance"),
                                               6);
      
      m_RecolorFromColor->SetColor(m_p_RecolorFromColor);
      m_RecolorToColor  ->SetColor(m_p_RecolorToColor);

      connect( m_RecolorFromColor,
               SIGNAL(ColorChanged(Pixel)) ,
               SLOT( RecolorFromColorChanged(Pixel ) )
             );

      connect( m_RecolorToColor,
               SIGNAL(ColorChanged(Pixel)) ,
               SLOT( RecolorToColorChanged(Pixel ) )
             );
      connect( m_RecolorDistanceSlider,
               SIGNAL(valueChanged(int)),
               SLOT( RecolorDistanceChanged(int )) );

   }   // Recolor block

   // final settings
   m_pc_BufferPainter->SetBorderColor(m_BrushColorWidget->GetColor());
   m_pc_BufferPainter->SetInnerColor(m_BrushColorWidget->GetColor());

   SetFreeBrushShapeIcon(m_pc_BrushShapeFreeBuffer);
   
   SetSensitivity();

   // initial show elements for painting / brush
   m_TransparencyBox   ->show();
   m_FillModeBox       ->hide();
   m_LightnessModeBox  ->hide();
   m_FilterModeBox     ->hide();
   m_SpeckleModeBox    ->hide();
   m_SmearModeBox      ->hide();
   m_CopyBrushModeBox  ->hide();
   m_RecolorModeBox    ->hide();

} // PaintBox()

//-----------------------------------------------------------------------------
//
//-----------------------------------------------------------------------------

PaintBox::~PaintBox()
{

} // ~PaintBox()

//-----------------------------------------------------------------------------
//
//-----------------------------------------------------------------------------


void PaintBox::ActivateBrush()
{
   // enable the valid subwidgets
   SetWidgetToMode(PaintBrush);
   
   m_pc_GlobalStorage->m_pc_Selection->ActivateSelection(Selection::SelectionStream,
                                                  tr("Brush"),
                                                  Callback(SelectionEventArrivedCB,this));

   // used one bit deep mask
   m_pc_GlobalStorage->m_pc_MainImage->CreateUsedPixelMask(1,false);

} // ActivateBrush()

//-----------------------------------------------------------------------------
//
//-----------------------------------------------------------------------------

void PaintBox::ActivateFill()
{
   // enable the valid subwidgets
   SetWidgetToMode(PaintFill);

   m_pc_GlobalStorage->m_pc_Selection->ActivateSelection(Selection::SelectionPoint,
                                                  tr("Fill"),
                                                  Callback(SelectionEventArrivedCB,this));

   // used one bit deep mask
   m_pc_GlobalStorage->m_pc_MainImage->CreateUsedPixelMask(1,false);

} // ActivateFill()

//-----------------------------------------------------------------------------
//
//-----------------------------------------------------------------------------
void PaintBox::ActivateLightness()
{
   // enable the valid subwidgets
   SetWidgetToMode(PaintLightness);

   m_pc_GlobalStorage->m_pc_Selection->ActivateSelection(Selection::SelectionStream,
                                                  tr("Lightness"),
                                                  Callback(SelectionEventArrivedCB,this));

   // used one bit deep mask
   m_pc_GlobalStorage->m_pc_MainImage->CreateUsedPixelMask(1,false);

} // ActivateLightness()

//-----------------------------------------------------------------------------
//
//-----------------------------------------------------------------------------


void PaintBox::ActivateSaturation()
{
   SetWidgetToMode(PaintSaturation);
   
   m_pc_GlobalStorage->m_pc_Selection->ActivateSelection(Selection::SelectionStream,
                                                  tr("Saturation"),
                                                  Callback(SelectionEventArrivedCB,this));

   // used one bit deep mask
   m_pc_GlobalStorage->m_pc_MainImage->CreateUsedPixelMask(1,false);

} // ActivateSaturation()

//-----------------------------------------------------------------------------
//
//-----------------------------------------------------------------------------

void PaintBox::ActivateFilter()
{
   // enable the valid subwidgets
   SetWidgetToMode(PaintFilter);

   m_pc_GlobalStorage->m_pc_Selection->ActivateSelection(Selection::SelectionStream,
                                                  tr("Filter"),
                                                  Callback(SelectionEventArrivedCB,this));

   // create a pixelmask, to hold the complete picture and additional a mask
   m_pc_GlobalStorage->m_pc_MainImage->CreateUsedPixelMask(32,true);
   TransparencyMaskClear();

} // ActivateFilter()

//-----------------------------------------------------------------------------
//
//-----------------------------------------------------------------------------

void PaintBox::ActivateSpeckle()
{
   // enable the valid subwidgets
   SetWidgetToMode(PaintSpeckle);

   m_pc_GlobalStorage->m_pc_Selection->ActivateSelection(Selection::SelectionStream,
                                                  tr("Speckle"),
                                                  Callback(SelectionEventArrivedCB,this));

   // create a pixelmask, to hold the complete picture and additional a mask
   m_pc_GlobalStorage->m_pc_MainImage->CreateUsedPixelMask(32,true);
   TransparencyMaskClear();

} // ActivateSpeckle()

//-----------------------------------------------------------------------------
//
//-----------------------------------------------------------------------------

void PaintBox::ActivateAirBrush()
{
   // enable the valid subwidgets
   SetWidgetToMode(PaintAirBrush);

   m_pc_GlobalStorage->m_pc_Selection->ActivateSelection(Selection::SelectionStream,
                                                  tr("Airbrush"),
                                                  Callback(SelectionEventArrivedCB,this));

   // create a pixelmask, to hold the complete picture and additional a mask
   m_pc_GlobalStorage->m_pc_MainImage->CreateUsedPixelMask(32,true);
   TransparencyMaskClear();

} // ActivateAirBrush()

//-----------------------------------------------------------------------------
//
//-----------------------------------------------------------------------------

void PaintBox::ActivateSmear()
{
   // enable the valid subwidgets
   SetWidgetToMode(PaintSmear);

   m_pc_GlobalStorage->m_pc_Selection->ActivateSelection(Selection::SelectionStream,
                                                  tr("Smear"),
                                                  Callback(SelectionEventArrivedCB,this));

} // ActivateAirSmear()

//-----------------------------------------------------------------------------
//
//-----------------------------------------------------------------------------

void PaintBox::ActivateCopy()
{
   // enable the valid subwidgets
   SetWidgetToMode(PaintCopy);

   m_b_SelectOriginalPoint = true;
   m_b_FirstPoint          = true;

    m_pc_GlobalStorage->m_pc_Selection->DeactivateSelection();
   
   m_pc_GlobalStorage->m_pc_Selection->ActivateShortTimeSelection(Selection::SelectionPoint,
                                                  tr("Select Original Point"),
                                                  Callback(SelectionEventArrivedCB,this));


} // ActivateCopy()

//-----------------------------------------------------------------------------
//
//-----------------------------------------------------------------------------

void PaintBox::ActivateRecolor()
{
   // enable the valid subwidgets
   SetWidgetToMode(PaintRecolor);

   // enable stream to recolor function
   m_pc_GlobalStorage->m_pc_Selection->ActivateSelection(Selection::SelectionStream,
                                                  tr("Change Color"),
                                                  Callback(SelectionEventArrivedCB,this));
   // used one bit deep mask
   m_pc_GlobalStorage->m_pc_MainImage->CreateUsedPixelMask(1,false);


} // ActivateRecolor()

//-----------------------------------------------------------------------------
//
//-----------------------------------------------------------------------------

void PaintBox::CopyBrushNewOrigin()
{
   m_b_SelectOriginalPoint = true;
   m_b_FirstPoint          = true;

   m_pc_GlobalStorage->m_pc_Selection->DeactivateSelection();

   DisableCopyBrushPoint();

   m_pc_GlobalStorage->m_pc_Selection->ActivateShortTimeSelection(Selection::SelectionPoint,
                                                  tr("Select Original Point"),
                                                  Callback(SelectionEventArrivedCB,this));

} // CopyBrushNewOrigin()

//-----------------------------------------------------------------------------
//
//-----------------------------------------------------------------------------

void PaintBox::OnMessageReceive(QString qs_MessageType,
                                 void *p_Data)
{
   TRACE << "Message Received member this = " << (int) this << "\n";

   if(qs_MessageType.compare("ImageResize") ==0)
   {
   }
   if(qs_MessageType.compare("ImageNewBuffer") ==0)
   {
   }
} // OnMessageReceive()


//===========================================================================
//
//===========================================================================


void *PaintBox::SelectionEventArrived(void *pc_Data)
{
   SelectionCallbackData *cb;
   cb = (SelectionCallbackData *) pc_Data;
   /*
   sprintf(stderr,"SelectionEvent %d %d %d\n",
         cb->i_CallbackReason,
         m_i_CurrentGraphikMode,
         m_pc_GlobalStorage->m_pc_Selection->GetMouseEventType());
   */
   if(cb->i_CallbackReason == SelectionCallbackData:: ReasonOperationDisabled)
   {
      // clean up unused structures
//      if(  m_i_CurrentGraphikMode == PaintCopy)
      {
         RectRegion r_PointRegion;
         m_pc_ObjectLayer->SetVisible(false);
         r_PointRegion =  m_pc_GlobalStorage->m_pc_Zoom->GetObjectPointZoomSize(m_pc_CopyBrushOriginPoint);
         m_pc_GlobalStorage->m_pc_ImageFrame->RedrawBufferRegion(r_PointRegion);
         

      }
      return NULL;
   }
   if( !cb->i_CallbackReason == SelectionCallbackData:: ReasonSelection)
   {
      return NULL;
   }
   // from now only real selections


   m_i_MouseEventType = m_pc_GlobalStorage->m_pc_Selection->GetMouseEventType();
   if(m_i_MouseEventType == MouseEvent::EventStart)
   {
      m_b_LastPoint = false;
   }

   // perform different actions depending on the graphic mode
   switch(m_i_CurrentGraphikMode)
   {
   case PaintBrush:
      DrawBrush();

      // on the last brush event force the undo setting
      if(m_i_MouseEventType == MouseEvent::EventEnd)
      {
         m_pc_GlobalStorage->m_pc_MessageServer->SendMessage(0,
                                                       this,
                                                       "ImageChanged");
      }
      break;
   case PaintLightness:
      DrawLightness();

      // on the last brush event force the undo setting
      if(m_i_MouseEventType == MouseEvent::EventEnd)
      {
         m_pc_GlobalStorage->m_pc_MessageServer->SendMessage(0,
                                                       this,
                                                       "ImageChanged");
      }
      break;
   case PaintSaturation:
      DrawSaturation();

      // on the last brush event force the undo setting
      if(m_i_MouseEventType == MouseEvent::EventEnd)
      {
         m_pc_GlobalStorage->m_pc_MessageServer->SendMessage(0,
                                                       this,
                                                       "ImageChanged");
      }
      break;
   case PaintFilter:
      DrawFilter();

      // on the last brush event force the undo setting
      if(m_i_MouseEventType == MouseEvent::EventEnd)
      {
         m_pc_GlobalStorage->m_pc_MessageServer->SendMessage(0,
                                                       this,
                                                       "ImageChanged");
      }
      break;
   case PaintFill:
      FillAtPoint();
      break;
   case PaintSpeckle:
      DrawSpeckle();

      // on the last brush event force the undo setting
      if(m_i_MouseEventType == MouseEvent::EventEnd)
      {
         m_pc_GlobalStorage->m_pc_MessageServer->SendMessage(0,
                                                       this,
                                                       "ImageChanged");
      }
      break;
   case PaintAirBrush:
      DrawAirBrush();

      // on the last brush event force the undo setting
      if(m_i_MouseEventType == MouseEvent::EventEnd)
      {
         m_pc_GlobalStorage->m_pc_MessageServer->SendMessage(0,
                                                       this,
                                                       "ImageChanged");
      }
      break;
   case PaintSmear:
      DrawSmear();

      // on the last brush event force the undo setting
      if(m_i_MouseEventType == MouseEvent::EventEnd)
      {
         m_pc_GlobalStorage->m_pc_MessageServer->SendMessage(0,
                                                       this,
                                                       "ImageChanged");
      }
      break;
   case PaintCopy:
      PaintCopyEvent(m_i_MouseEventType);

      // on the last brush event force the undo setting
      if(m_i_MouseEventType == MouseEvent::EventEnd)
      {
         m_pc_GlobalStorage->m_pc_MessageServer->SendMessage(0,
                                                       this,
                                                       "ImageChanged");
      }
      break;
   case PaintRecolor:
      DrawRecolor();

      // on the last brush event force the undo setting
      if(m_i_MouseEventType == MouseEvent::EventEnd)
      {
         m_pc_GlobalStorage->m_pc_MessageServer->SendMessage(0,
                                                       this,
                                                       "ImageChanged");
      }
      break;
   default:
      break;
   } // switch

   // at mouse end sometimes the transparency mask hast to be cleared
   if(m_i_MouseEventType == MouseEvent::EventEnd)
   {
      m_b_LastPoint = false;

      if(m_i_TransparencyClearMode == ClearModeAutomatic)
      {
         TransparencyMaskClear();
      }
   }
   else if(m_i_MouseEventType == MouseEvent::EventStart
           || m_i_MouseEventType == MouseEvent::EventDo)
   {
      m_b_LastPoint = true;
      m_pc_GlobalStorage->m_pc_Selection->GetSelectionPoint(m_i_LastX,m_i_LastY);
   }

   return NULL;
} // SelectionEventArrived()


//===========================================================================
//
//===========================================================================


void *PaintBox::SelectionEventArrivedCB(void *Thisdata,void *pc_Data)
{
   PaintBox *pc_This = (PaintBox *) Thisdata;

   return pc_This->SelectionEventArrived(pc_Data);

} // SelectionEventArrivedCB()

//===========================================================================
//
//===========================================================================


void PaintBox::DrawBrush()
{
   PixelBuffer *pc_Destination;
   RectRegion   c_ChangedRegion;
   RectRegion   c_LastChangedRegion;
   int          i_X;
   int          i_Y;

   if(m_i_MouseEventType == MouseEvent::EventOver)
   {
      return;
   }

   pc_Destination = m_pc_GlobalStorage->m_pc_MainImage;

   if(    m_pc_GlobalStorage->m_pc_MaskSettings->GetPaintMode()
       == MaskSettings::MaskPaintModeInSafety)
   {
      pc_Destination = pc_Destination->GetSafetyMask();
      if(   m_pc_GlobalStorage->m_pc_MaskSettings->GetDraw()
         == MaskSettings::MaskPaintDrawExtend)
      {
         m_pc_BufferPainter->SetMaskMode(true,true);
      }
      else
      {
         m_pc_BufferPainter->SetMaskMode(true,false);
      }
   }
   else if(    m_pc_GlobalStorage->m_pc_MaskSettings->GetPaintMode()
            == MaskSettings::MaskPaintModeInSelection)
   {
      pc_Destination = pc_Destination->GetSelectionMask();
      if(   m_pc_GlobalStorage->m_pc_MaskSettings->GetDraw()
         == MaskSettings::MaskPaintDrawExtend)
      {
         m_pc_BufferPainter->SetMaskMode(true,true);
      }
      else
      {
         m_pc_BufferPainter->SetMaskMode(true,false);
      }
   }
   else
   {
      m_pc_BufferPainter->SetMaskMode(false,false);
   }


   if(   m_b_TransparencyEnabled)
   {

      m_pc_BufferPainter->SetOneTimeDraw(true);
   }

   m_pc_GlobalStorage->m_pc_Selection->GetSelectionPoint(i_X,i_Y);

   if(    ! m_b_LastPoint
       || ! m_b_BrushContinuous)
   {
      // center brush to coordinate
      CenterCoordinateToBrush(i_X,i_Y);

      // draw brush
      m_pc_BufferPainter->DrawBrush(pc_Destination,i_X,i_Y);

      c_ChangedRegion = m_pc_BufferPainter->GetChangedRegion();
   }
   else // last point available
   {
      int i_LastX;
      int i_LastY;

      i_LastX = m_i_LastX;
      i_LastY = m_i_LastY;

      // center brush to coordinate
      CenterCoordinateToBrush(i_X,i_Y);
      CenterCoordinateToBrush(i_LastX,i_LastY);

      if(    i_LastX == i_X
          && i_LastY == i_Y)
      {
         // same point, only draw one
         // draw brush
         m_pc_BufferPainter->DrawBrush(pc_Destination,i_X,i_Y);

         c_ChangedRegion = m_pc_BufferPainter->GetChangedRegion();
      }
      else
      {
         LineIterator c_LineIterator;
         int          i_RunX;
         int          i_RunY;

         c_ChangedRegion.SetEmpty();
         c_LineIterator.Set(i_LastX,i_LastY,i_X,i_Y);

         for(c_LineIterator.Start(i_RunX,i_RunY);
             c_LineIterator.Running();
             c_LineIterator.Next(i_RunX,i_RunY)
             )
         {
            m_pc_BufferPainter->DrawBrush(pc_Destination,
                                             i_RunX,
                                             i_RunY);
            c_LastChangedRegion = m_pc_BufferPainter->GetChangedRegion();
            c_ChangedRegion.AddRegion(c_LastChangedRegion);
         } // for iterator
      } // multiple points



   } // else last point


   m_pc_GlobalStorage->m_pc_ImageFrame->RedrawBufferRegion(c_ChangedRegion);


} // DrawBrush()

//===========================================================================
//
//===========================================================================


void PaintBox::DrawAirBrush()
{
   PixelBuffer *pc_Destination;
   RectRegion c_ChangedRegion;
   RectRegion c_LastChangedRegion;
   int        i_X;
   int        i_Y;

   if(m_i_MouseEventType == MouseEvent::EventOver)
   {
      return;
   }

   pc_Destination = m_pc_GlobalStorage->m_pc_MainImage;

   if(    m_pc_GlobalStorage->m_pc_MaskSettings->GetPaintMode()
       == MaskSettings::MaskPaintModeInSafety)
   {
      pc_Destination = pc_Destination->GetSafetyMask();
      if(   m_pc_GlobalStorage->m_pc_MaskSettings->GetDraw()
         == MaskSettings::MaskPaintDrawExtend)
      {
         m_pc_BufferPainter->SetMaskMode(true,true);
      }
      else
      {
         m_pc_BufferPainter->SetMaskMode(true,false);
      }
   }
   else if(    m_pc_GlobalStorage->m_pc_MaskSettings->GetPaintMode()
            == MaskSettings::MaskPaintModeInSelection)
   {
      pc_Destination = pc_Destination->GetSelectionMask();
      if(   m_pc_GlobalStorage->m_pc_MaskSettings->GetDraw()
         == MaskSettings::MaskPaintDrawExtend)
      {
         m_pc_BufferPainter->SetMaskMode(true,true);
      }
      else
      {
         m_pc_BufferPainter->SetMaskMode(true,false);
      }
   }
   else
   {
      m_pc_BufferPainter->SetMaskMode(false,false);
   }


   if(   m_b_TransparencyEnabled)
   {

      m_pc_BufferPainter->SetOneTimeDraw(true);
   }

   m_pc_GlobalStorage->m_pc_Selection->GetSelectionPoint(i_X,i_Y);

   if( ! m_b_LastPoint
       || ! m_b_BrushContinuous)
   {
      // center brush to coordinate
      i_X -= m_i_CurrentThickness/2;
      i_Y -= m_i_CurrentThickness/2;

      // draw brush
      m_pc_BufferPainter->DrawAirBrush(pc_Destination,i_X,i_Y);

      c_ChangedRegion = m_pc_BufferPainter->GetChangedRegion();
   }
   else // last point available
   {
      int i_LastX;
      int i_LastY;

      i_LastX = m_i_LastX;
      i_LastY = m_i_LastY;

      // center brush to coordinate
      i_X -= m_i_CurrentThickness/2;
      i_Y -= m_i_CurrentThickness/2;
      i_LastX -= m_i_CurrentThickness/2;
      i_LastY -= m_i_CurrentThickness/2;

      if(    i_LastX == i_X
          && i_LastY == i_Y)
      {
         // do not double draw the point
      }
      else
      {
         // Draw a line
         m_pc_BufferPainter->DrawAirBrush(pc_Destination,
                                             i_X,
                                             i_Y);
         c_ChangedRegion = m_pc_BufferPainter->GetChangedRegion();


         m_i_LastX = i_X;
         m_i_LastY = i_Y;
      } // multiple points



   } // else last point


   m_pc_GlobalStorage->m_pc_ImageFrame->RedrawBufferRegion(c_ChangedRegion);


} // DrawAirBrush()

//===========================================================================
//
//===========================================================================


void PaintBox::DrawSpeckle()
{
   PixelBuffer *pc_Destination;
   RectRegion   c_ChangedRegion;
   RectRegion   c_LastChangedRegion;
   int          i_X;
   int          i_Y;

   if(m_i_MouseEventType == MouseEvent::EventOver)
   {
      return;
   }

   // transfer actual speckle parameter into the bufferpainter
   // invert the density high means less points in bufferpainter
   m_pc_BufferPainter->SetSpeckleParameter(m_i_SpeckleSize,
                                             m_i_SpeckleDensityMaximum
                                           - m_i_SpeckleDensity );
                                           
   pc_Destination = m_pc_GlobalStorage->m_pc_MainImage;

   if(    m_pc_GlobalStorage->m_pc_MaskSettings->GetPaintMode()
       == MaskSettings::MaskPaintModeInSafety)
   {
      pc_Destination = pc_Destination->GetSafetyMask();
      if(   m_pc_GlobalStorage->m_pc_MaskSettings->GetDraw()
         == MaskSettings::MaskPaintDrawExtend)
      {
         m_pc_BufferPainter->SetMaskMode(true,true);
      }
      else
      {
         m_pc_BufferPainter->SetMaskMode(true,false);
      }
   }
   else if(   m_pc_GlobalStorage->m_pc_MaskSettings->GetPaintMode()
           == MaskSettings::MaskPaintModeInSelection)
   {
      pc_Destination = pc_Destination->GetSelectionMask();
      if(   m_pc_GlobalStorage->m_pc_MaskSettings->GetDraw()
         == MaskSettings::MaskPaintDrawExtend)
      {
         m_pc_BufferPainter->SetMaskMode(true,true);
      }
      else
      {
         m_pc_BufferPainter->SetMaskMode(true,false);
      }
   }
   else
   {
      m_pc_BufferPainter->SetMaskMode(false,false);
   }


   if(   m_b_TransparencyEnabled
      && m_b_UseTransparencyMask )
   {

      m_pc_BufferPainter->SetOneTimeDraw(true);
   }
   else
   {
      m_pc_BufferPainter->SetOneTimeDraw(false);
   }

   m_pc_GlobalStorage->m_pc_Selection->GetSelectionPoint(i_X,i_Y);

   if( ! m_b_LastPoint
       || ! m_b_BrushContinuous)
   {
      // center brush to coordinate
      CenterCoordinateToBrush(i_X,i_Y);

      // draw brush
      m_pc_BufferPainter->DrawSpeckle(pc_Destination,i_X,i_Y);

      c_ChangedRegion = m_pc_BufferPainter->GetChangedRegion();
   }
   else // last point available
   {
      int i_LastX;
      int i_LastY;

      i_LastX = m_i_LastX;
      i_LastY = m_i_LastY;

      // center brush to coordinate
      CenterCoordinateToBrush(i_X,i_Y);
      CenterCoordinateToBrush(i_LastX,i_LastY);

      if(    i_LastX == i_X
          && i_LastY == i_Y)
      {
         // same point, only draw one
         // draw brush
         m_pc_BufferPainter->DrawSpeckle(pc_Destination,i_X,i_Y);

         c_ChangedRegion = m_pc_BufferPainter->GetChangedRegion();
      }
      else
      {
         // Draw a line
         LineIterator c_LineIterator;
         int          i_RunX;
         int          i_RunY;

         c_ChangedRegion.SetEmpty();
         c_LineIterator.Set(i_LastX,i_LastY,i_X,i_Y);

         for(c_LineIterator.Start(i_RunX,i_RunY);
             c_LineIterator.Running();
             c_LineIterator.Next(i_RunX,i_RunY)
             )
         {
            m_pc_BufferPainter->DrawSpeckle(pc_Destination,
                                             i_RunX,
                                             i_RunY);
            c_LastChangedRegion = m_pc_BufferPainter->GetChangedRegion();
            c_ChangedRegion.AddRegion(c_LastChangedRegion);
         } // for iterator             
         
      } // multiple points



   } // else last point


   m_pc_GlobalStorage->m_pc_ImageFrame->RedrawBufferRegion(c_ChangedRegion);


} // DrawSpeckle()


//===========================================================================
//
//===========================================================================


void PaintBox::DrawCopy()
{
   PixelBuffer *pc_Destination;
   RectRegion   c_ChangedRegion;
   RectRegion   c_LastChangedRegion;
   int          i_X;
   int          i_Y;

   if(m_i_MouseEventType == MouseEvent::EventOver)
   {
      return;
   }

   // transfer actual speckle parameter into the bufferpainter
   // invert the density high means less points in bufferpainter
   m_pc_BufferPainter->SetCopyBrushParameter(m_i_CopyDifferenceX,
                                             m_i_CopyDifferenceY);

   pc_Destination = m_pc_GlobalStorage->m_pc_MainImage;

   if(    m_pc_GlobalStorage->m_pc_MaskSettings->GetPaintMode()
       == MaskSettings::MaskPaintModeInSafety)
   {
      pc_Destination = pc_Destination->GetSafetyMask();
      if(   m_pc_GlobalStorage->m_pc_MaskSettings->GetDraw()
         == MaskSettings::MaskPaintDrawExtend)
      {
         m_pc_BufferPainter->SetMaskMode(true,true);
      }
      else
      {
         m_pc_BufferPainter->SetMaskMode(true,false);
      }
   }
   else if(   m_pc_GlobalStorage->m_pc_MaskSettings->GetPaintMode()
           == MaskSettings::MaskPaintModeInSelection)
   {
      pc_Destination = pc_Destination->GetSelectionMask();
      if(   m_pc_GlobalStorage->m_pc_MaskSettings->GetDraw()
         == MaskSettings::MaskPaintDrawExtend)
      {
         m_pc_BufferPainter->SetMaskMode(true,true);
      }
      else
      {
         m_pc_BufferPainter->SetMaskMode(true,false);
      }
   }
   else
   {
      m_pc_BufferPainter->SetMaskMode(false,false);
   }


   if(   m_b_TransparencyEnabled
       )
   {

      m_pc_BufferPainter->SetOneTimeDraw(true);
   }
   else
   {
      m_pc_BufferPainter->SetOneTimeDraw(false);
   }

   m_pc_GlobalStorage->m_pc_Selection->GetSelectionPoint(i_X,i_Y);

   if( ! m_b_LastPoint
       || ! m_b_BrushContinuous)
   {
      // center brush to coordinate
      CenterCoordinateToBrush(i_X,i_Y);      

      // draw brush
      m_pc_BufferPainter->DrawCopyBrush(pc_Destination,i_X,i_Y);

      c_ChangedRegion = m_pc_BufferPainter->GetChangedRegion();
   }
   else // last point available
   {
      int i_LastX;
      int i_LastY;

      i_LastX = m_i_LastX;
      i_LastY = m_i_LastY;

      // center brush to coordinate
      CenterCoordinateToBrush(i_X,i_Y);
      CenterCoordinateToBrush(i_LastX,i_LastY);

      if(    i_LastX == i_X
          && i_LastY == i_Y)
      {
         // same point, only draw one
         // draw brush
         m_pc_BufferPainter->DrawCopyBrush(pc_Destination,i_X,i_Y);

         c_ChangedRegion = m_pc_BufferPainter->GetChangedRegion();
      }
      else
      {
         // Draw a line
         LineIterator c_LineIterator;
         int          i_RunX;
         int          i_RunY;

         c_ChangedRegion.SetEmpty();
         c_LineIterator.Set(i_LastX,i_LastY,i_X,i_Y);

         for(c_LineIterator.Start(i_RunX,i_RunY);
             c_LineIterator.Running();
             c_LineIterator.Next(i_RunX,i_RunY)
             )
         {
            m_pc_BufferPainter->DrawCopyBrush(pc_Destination,
                                             i_RunX,
                                             i_RunY);
            c_LastChangedRegion = m_pc_BufferPainter->GetChangedRegion();
            c_ChangedRegion.AddRegion(c_LastChangedRegion);
         } // for iterator

      } // multiple points



   } // else last point


   m_pc_GlobalStorage->m_pc_ImageFrame->RedrawBufferRegion(c_ChangedRegion);


} // DrawCopy()

//===========================================================================
//
//===========================================================================


void PaintBox::DrawSmear()
{
   PixelBuffer *pc_Destination;
   RectRegion   c_ChangedRegion;
   RectRegion   c_LastChangedRegion;
   int          i_X;
   int          i_Y;

   if(m_i_MouseEventType == MouseEvent::EventOver)
   {
      return;
   }

   pc_Destination = m_pc_GlobalStorage->m_pc_MainImage;

   // do not smear in masks
   if(       m_pc_GlobalStorage->m_pc_MaskSettings->GetPaintMode()
          == MaskSettings::MaskPaintModeInSafety
       ||
             m_pc_GlobalStorage->m_pc_MaskSettings->GetPaintMode()
          == MaskSettings::MaskPaintModeInSelection)
   {
      return;
   }

   m_pc_BufferPainter->SetMaskMode(false,false);


   m_pc_GlobalStorage->m_pc_Selection->GetSelectionPoint(i_X,i_Y);

   // only smear if there is a last point
   if( ! m_b_LastPoint )
   {
      m_i_LastX = i_X;
      m_i_LastY = i_Y;
      return;
   }
   m_pc_BufferPainter->SetSmearParameter(m_i_SmearStrength);   
   {
      int i_LastX;
      int i_LastY;

      i_LastX = m_i_LastX;
      i_LastY = m_i_LastY;

      // center brush to coordinate
      CenterCoordinateToBrush(i_X,i_Y);
      CenterCoordinateToBrush(i_LastX,i_LastY);
      
      if(    i_LastX == i_X
          && i_LastY == i_Y)
      {
         // do not double draw the point
      }
      else
      {
         // Draw a line
         LineIterator c_LineIterator;
         int          i_RunX;
         int          i_RunY;
         int          i_VectorX;
         int          i_VectorY;

         i_VectorX = i_LastX - i_X;
         i_VectorY = i_LastY - i_Y;
         

         c_ChangedRegion.SetEmpty();
         c_LineIterator.Set(i_LastX,i_LastY,i_X,i_Y);

         for(c_LineIterator.Start(i_RunX,i_RunY);
             c_LineIterator.Running();
             c_LineIterator.Next(i_RunX,i_RunY)
             )
         {
            m_pc_BufferPainter->DrawSmear(pc_Destination,
                                             i_RunX,
                                             i_RunY,
                                             i_VectorX,
                                             i_VectorY
                                             );
            c_LastChangedRegion = m_pc_BufferPainter->GetChangedRegion();
            c_ChangedRegion.AddRegion(c_LastChangedRegion);
         } // for iterator

      } // multiple points



   } // else last point

   m_pc_GlobalStorage->m_pc_ImageFrame->RedrawBufferRegion(c_ChangedRegion);


} // DrawSmear()

//===========================================================================
//
//===========================================================================


void PaintBox::DrawLightness()
{
   DrawSpecial(PaintLightness);
   
} // DrawLightness()

//===========================================================================
//
//===========================================================================


void PaintBox::DrawSaturation()
{
   DrawSpecial(PaintSaturation);
   
} // DrawSaturation()

//===========================================================================
//
//===========================================================================

void PaintBox::DrawFilter()
{
   DrawSpecial(PaintFilter);
   
} // DrawFilter()

//===========================================================================
//
//===========================================================================

void PaintBox::DrawRecolor()
{
   DrawSpecial(PaintRecolor);
   
} // DrawRecolor()

//===========================================================================
//
//===========================================================================

void PaintBox::DrawSpecial(int i_Mode)
{
   PixelBuffer *pc_Destination;
   RectRegion c_ChangedRegion;
   RectRegion c_LastChangedRegion;
   int        i_X;
   int        i_Y;

   if(m_i_MouseEventType == MouseEvent::EventOver)
   {
      return;
   }

   pc_Destination = m_pc_GlobalStorage->m_pc_MainImage;

   if(   m_pc_GlobalStorage->m_pc_MaskSettings->GetPaintMode()
      == MaskSettings::MaskPaintModeInSafety)
   {
      pc_Destination = pc_Destination->GetSafetyMask();
      if(   m_pc_GlobalStorage->m_pc_MaskSettings->GetDraw()
         == MaskSettings::MaskPaintDrawExtend)
      {
         m_pc_BufferPainter->SetMaskMode(true,true);
      }
      else
      {
         m_pc_BufferPainter->SetMaskMode(true,false);
      }
   }
   else if(    m_pc_GlobalStorage->m_pc_MaskSettings->GetPaintMode()
            == MaskSettings::MaskPaintModeInSelection)
   {
      pc_Destination = pc_Destination->GetSelectionMask();
      if(   m_pc_GlobalStorage->m_pc_MaskSettings->GetDraw()
         == MaskSettings::MaskPaintDrawExtend)
      {
         m_pc_BufferPainter->SetMaskMode(true,true);
      }
      else
      {
         m_pc_BufferPainter->SetMaskMode(true,false);
      }
   }
   else
   {
      m_pc_BufferPainter->SetMaskMode(false,false);
   }

   // use the one time draw buffer
   m_pc_BufferPainter->SetOneTimeDraw(true);

   if(i_Mode == PaintLightness)
   {
      m_pc_BufferPainter->ResetBrushModes();
      m_pc_BufferPainter->SetLightnessMode(true);
   }
   else if(i_Mode == PaintSaturation)
   {
      m_pc_BufferPainter->ResetBrushModes();
      m_pc_BufferPainter->SetSaturationMode(true);
   }
   else if(i_Mode == PaintFilter)
   {
      m_pc_BufferPainter->ResetBrushModes();
      m_pc_BufferPainter->SetFilterMode(true);
   }
   else if(i_Mode == PaintRecolor)
   {
      m_pc_BufferPainter->ResetBrushModes();
      m_pc_BufferPainter->SetRecolorMode(true);
   }


   m_pc_GlobalStorage->m_pc_Selection->GetSelectionPoint(i_X,i_Y);

   if( ! m_b_LastPoint
       || ! m_b_BrushContinuous)
   {
      // center brush to coordinate
      CenterCoordinateToBrush(i_X,i_Y);
      
      // draw brush
      m_pc_BufferPainter->DrawBrush(pc_Destination,i_X,i_Y);

      c_ChangedRegion = m_pc_BufferPainter->GetChangedRegion();
   }
   else // last point available
   {
      int i_LastX;
      int i_LastY;

      i_LastX = m_i_LastX;
      i_LastY = m_i_LastY;

      // center brush to coordinate
      CenterCoordinateToBrush(i_X,i_Y);
      CenterCoordinateToBrush(i_LastX,i_LastY);

      if(    i_LastX == i_X
          && i_LastY == i_Y)
      {
         // same point, only draw one
         // draw brush
         m_pc_BufferPainter->DrawBrush(pc_Destination,i_X,i_Y);

         c_ChangedRegion = m_pc_BufferPainter->GetChangedRegion();
      }
      else
      {
         // Draw a line
         LineIterator c_LineIterator;
         int          i_RunX;
         int          i_RunY;

         c_ChangedRegion.SetEmpty();
         c_LineIterator.Set(i_LastX,i_LastY,i_X,i_Y);

         for(c_LineIterator.Start(i_RunX,i_RunY);
             c_LineIterator.Running();
             c_LineIterator.Next(i_RunX,i_RunY)
             )
         {
            m_pc_BufferPainter->DrawBrush(pc_Destination,
                                             i_RunX,
                                             i_RunY);
            c_LastChangedRegion = m_pc_BufferPainter->GetChangedRegion();
            c_ChangedRegion.AddRegion(c_LastChangedRegion);
         } // for iterator

      } // multiple points

   } // else last point


   // disable the lightness mode
   m_pc_BufferPainter->SetLightnessMode(false);
   m_pc_BufferPainter->SetSaturationMode(false);
   m_pc_BufferPainter->SetFilterMode(false);

   m_pc_GlobalStorage->m_pc_ImageFrame->RedrawBufferRegion(c_ChangedRegion);


} // DrawSpecial()


//===========================================================================
//
//===========================================================================

void PaintBox::ThicknessChanged(int i_Width)
{

   m_i_CurrentThickness = i_Width;
   CreateBrushShape(m_i_CurrentThickness);

} // ThicknessChanged()

//===========================================================================
//
//===========================================================================

void PaintBox::TransparencyEnableChanged(bool b_on)
{
   m_b_TransparencyEnabled = b_on;
   m_pc_BufferPainter->SetTransparencyEnable(b_on);

   if( m_b_TransparencyEnabled)
   {
      // allocate the one time draw buffer
      m_pc_GlobalStorage->m_pc_MainImage->CreateUsedPixelMask();
   }

   SetSensitivity();

} // ransparencyEnableChanged

//===========================================================================
//
//===========================================================================

void PaintBox::TransparencyFactorChanged(int i_Factor)
{
   m_i_TransparencyFactor = i_Factor;  
   m_pc_BufferPainter->SetTransparencyFactor(i_Factor);
} // TransparencyFactorChanged()

//===========================================================================
//
//===========================================================================

void PaintBox::LightnessFactorChanged(int i_Factor)
{
   m_i_LightnessFactor = i_Factor;
   m_pc_BufferPainter->SetLightnessFactor(i_Factor);
} // LightnessFactorChanged()

//===========================================================================
//
//===========================================================================

void PaintBox::BrushColorChanged(Pixel c_Color)
{
   m_pc_BufferPainter->SetBorderColor(c_Color);
   m_pc_BufferPainter->SetInnerColor(c_Color);
} // BrushColorChanged()

//===========================================================================
//
//===========================================================================

void PaintBox::BrushPatternChanged(PixelBuffer *pc_Buffer)
{
   m_pc_BufferPainter->SetBorderPattern(pc_Buffer);
   m_pc_BufferPainter->SetInnerPattern(pc_Buffer);

}  // BrushPatternChanged()

//===========================================================================
//
//===========================================================================

void PaintBox::BrushModeChanged(BrushMode i_BrushMode)
{
   if(i_BrushMode == SelectMotive)
   {
      // get the motiveLayer
      GraphicObjectLayer *pc_Motive;
      pc_Motive = m_pc_MainFrame->m_pc_MotiveBox->GetMotiveLayer();
      m_pc_BufferPainter->SetBorderMotiveLayer(pc_Motive);
      m_pc_BufferPainter->SetInnerMotiveLayer(pc_Motive);
   }
   m_pc_BufferPainter->SetBorderDrawMode(i_BrushMode);
   m_pc_BufferPainter->SetInnerDrawMode(i_BrushMode);

} // BrushModeChanged()

//===========================================================================
//
//===========================================================================

void PaintBox::CreateBrushShape(int i_Thickness)
{
   PixelBuffer *pc_Brush;
   float        f_Center;
   int          i_X;
   int          i_Y;
   int          i_Half;

   i_Half   = (i_Thickness+1) / 2;
   
   // create Buffer
   pc_Brush = new PixelBuffer(i_Thickness,i_Thickness,1,false,false);

   switch(m_i_BrushShapeStyle)
   {
   case BrushShapeCircle:
      // create Circle

      // create Buffer
      pc_Brush = new PixelBuffer(i_Thickness,i_Thickness,1,false,false);

      f_Center = (i_Thickness-1) / 2.0;
      for( i_Y=0 ; i_Y < i_Half; i_Y ++)
      {
         for( i_X=0 ; i_X < i_Half; i_X ++)
         {
            float f_DifferenceX;
            float f_DifferenceY;
            float f_TotalDifference;
            f_DifferenceX = f_Center - i_X;
            f_DifferenceY = f_Center - i_Y;
            f_TotalDifference = sqrt(   f_DifferenceX*f_DifferenceX
                                   + f_DifferenceY*f_DifferenceY );
            f_TotalDifference *= 2;
            if(f_TotalDifference <= i_Thickness)
            {
               // draw in all four quadrants
               pc_Brush->Set(i_X                ,i_Y                ,1);
               pc_Brush->Set(i_X                ,(i_Thickness-1)-i_Y,1);
               pc_Brush->Set((i_Thickness-1)-i_X,i_Y                ,1);
               pc_Brush->Set((i_Thickness-1)-i_X,(i_Thickness-1)-i_Y,1);
            }

         } // for X
      } // for Y
      break;
   case BrushShapeSquare:
      // create Buffer
      pc_Brush = new PixelBuffer(i_Thickness,i_Thickness,1,false,false);

      for( i_Y=0 ; i_Y < i_Thickness; i_Y ++)
      {
         for( i_X=0 ; i_X < i_Thickness; i_X ++)
         {
            pc_Brush->Set(i_X,i_Y,1);
         } // for X
      } // for Y
      break;
   case BrushShapeVerticalLine:
      // create Buffer
      pc_Brush = new PixelBuffer(1,i_Thickness,1,false,false);

      for( i_Y=0 ; i_Y < i_Thickness; i_Y ++)
      {
         pc_Brush->Set(0,i_Y,1);
      } // for Y
      break;
   case BrushShapeHorizontalLine:
      // create Buffer
      pc_Brush = new PixelBuffer(i_Thickness,1,1,false,false);

      for( i_X=0 ; i_X < i_Thickness; i_X ++)
      {
         pc_Brush->Set(i_X,0,1);
      } // for X
      break;
   case BrushShapeNWSELine:
      // create Buffer
      pc_Brush = new PixelBuffer(i_Thickness,i_Thickness,1,false,false);

      for( i_Y=0 ; i_Y < i_Thickness; i_Y ++)
      {
         pc_Brush->Set(i_Y,i_Y,1);
         // diagonal line something thicker, to omit holes
         // when painting opposite diagonal
         if(i_Y>0)
         {
            pc_Brush->Set(i_Y,i_Y-1,1);
         }
      } // for Y
      break;
   case BrushShapeSWNELine:
      // create Buffer
      pc_Brush = new PixelBuffer(i_Thickness,i_Thickness,1,false,false);

      for( i_Y=0 ; i_Y < i_Thickness; i_Y ++)
      {
         pc_Brush->Set(i_Y,i_Thickness-1-i_Y,1);
         // diagonal line something thicker, to omit holes
         // when painting opposite diagonal
         if(i_Y>0)
         {
            pc_Brush->Set(i_Y,i_Thickness-i_Y,1);
         }
      } // for Y
      break;
   case BrushShapeFreeFigure:
      {
         PixelBuffer *pc_Icon;
         PixelBuffer *pc_SelectionMask;
         int          i_Width;
         int          i_Height;
         
         pc_Icon = m_pc_BrushShapeFreeBuffer->CreateIconFromBuffer(
                                       i_Thickness,
                                       i_Thickness
                                       );
         pc_Brush= new PixelBuffer(pc_Icon,
                                   1,
                                   false,
                                   false,
                                   false);
         pc_Icon->GetWidthHeight(i_Width,i_Height);
         pc_SelectionMask = pc_Icon->GetSelectionMask();
         for( i_Y=0 ; i_Y < i_Height; i_Y ++)
         {
            for( i_X=0 ; i_X < i_Width; i_X ++)
            {
               Pixel p_Value;
               p_Value = pc_SelectionMask->Get(i_X,i_Y);
               pc_Brush->Set(i_X,i_Y,p_Value);
            } // for X
         } // for Y

         // icon not longer needed
         delete pc_Icon;                              
      }
      break;
   default:
      break;
   } // switch brush shape style



   // Set the brush

   m_pc_BufferPainter->SetBrushShape(pc_Brush);


} // CreateBrushShape();

//===========================================================================
//
//===========================================================================

void PaintBox::ContinuousEnableChanged(bool b_on)
{
   m_b_BrushContinuous = b_on;
   SetSensitivity();
   
} // ContinuousEnableChanged()

//===========================================================================
//
//===========================================================================


void PaintBox::FillAtPoint()
{
   PixelBuffer *pc_DestinationBuffer ;
   BufferFill   c_ActualFill;
   RectRegion   c_ChangedRegion;
   int          i_MaskMode;
   int          i_X;
   int          i_Y;

   // set the distance options
   c_ActualFill.SetColorDistance(m_i_FillDistance);
   c_ActualFill.SetColorDistanceColor(m_p_FillDistanceColor);
   if(m_b_FillFromStartPoint)
   {
      c_ActualFill.SetColorDistanceFrom(BufferFill::BufferFillDistanceOrigin);
   }
   else
   {
      c_ActualFill.SetColorDistanceFrom(BufferFill::BufferFillDistanceColor);
   }

   m_pc_BufferPainter->SetOneTimeDraw(false);
   pc_DestinationBuffer = NULL;

   i_MaskMode = m_pc_GlobalStorage->m_pc_MaskSettings->GetPaintMode();

   m_pc_GlobalStorage->m_pc_Selection->GetSelectionPoint(i_X,i_Y);

   c_ActualFill.m_pc_SourceBuffer      = m_pc_GlobalStorage->m_pc_MainImage;
   if(   i_MaskMode == MaskSettings::MaskPaintModeInSafety
      || i_MaskMode == MaskSettings::MaskPaintModeInSelection)
   {
      pc_DestinationBuffer = new PixelBuffer(m_pc_GlobalStorage->m_pc_MainImage,
                                             1,
                                             false,false);
      c_ActualFill.m_pc_DestinationBuffer = pc_DestinationBuffer;
      c_ActualFill.m_ChangeColor = m_pc_GlobalStorage->m_pc_MainImage->Get(i_X,i_Y);
      c_ActualFill.m_NewColor    = 1;
   }
   else if(m_b_TransparencyEnabled )
   {
      pc_DestinationBuffer = new PixelBuffer(m_pc_GlobalStorage->m_pc_MainImage,
                                             1,
                                             false,false);
      c_ActualFill.m_pc_DestinationBuffer = pc_DestinationBuffer;
      c_ActualFill.m_ChangeColor = m_pc_GlobalStorage->m_pc_MainImage->Get(i_X,i_Y);
      c_ActualFill.m_NewColor    = 1;
   }
   else if(m_i_FillDistance > 0 )
   {
      pc_DestinationBuffer = new PixelBuffer(m_pc_GlobalStorage->m_pc_MainImage,
                                             1,
                                             false,false);
      c_ActualFill.m_pc_DestinationBuffer = pc_DestinationBuffer;
      c_ActualFill.m_ChangeColor = m_pc_GlobalStorage->m_pc_MainImage->Get(i_X,i_Y);
      c_ActualFill.m_NewColor    = 1;
   }
   else
   {
      pc_DestinationBuffer = new PixelBuffer(m_pc_GlobalStorage->m_pc_MainImage,
                                             1,
                                             false,false);
      c_ActualFill.m_pc_DestinationBuffer = pc_DestinationBuffer;
      c_ActualFill.m_ChangeColor = m_pc_GlobalStorage->m_pc_MainImage->Get(i_X,i_Y);
      c_ActualFill.m_NewColor    = 1;
   }


   c_ActualFill.StartFill(i_X, i_Y);

   c_ChangedRegion = c_ActualFill.GetChangedRegion();
   if(i_MaskMode == MaskSettings::MaskPaintModeInSafety)
   {
      // draw transparent from the destinationBuffer into the
      // realImage
      if(    m_pc_GlobalStorage->m_pc_MaskSettings->GetDraw()
          == MaskSettings::MaskPaintDrawExtend)
      {
         m_pc_BufferPainter->SetMaskMode(true,true);
      }
      else
      {
         m_pc_BufferPainter->SetMaskMode(true,false);
      }
      m_pc_BufferPainter->DrawMarkedPixel(
                        m_pc_GlobalStorage->m_pc_MainImage->GetSafetyMask(),
                        pc_DestinationBuffer,
                        &c_ChangedRegion);
   }
   else if(i_MaskMode == MaskSettings::MaskPaintModeInSelection)
   {
      // draw transparent from the destinationBuffer into the
      // realImage
      if(   m_pc_GlobalStorage->m_pc_MaskSettings->GetDraw()
         == MaskSettings::MaskPaintDrawExtend)
      {
         m_pc_BufferPainter->SetMaskMode(true,true);
      }
      else
      {
         m_pc_BufferPainter->SetMaskMode(true,false);
      }
      m_pc_BufferPainter->DrawMarkedPixel(
                        m_pc_GlobalStorage->m_pc_MainImage->GetSelectionMask(),
                        pc_DestinationBuffer,
                        &c_ChangedRegion);
   }
   else // no mask Mode
   {
      if(   m_b_TransparencyEnabled
         || m_i_FillDistance > 0
         || pc_DestinationBuffer != NULL  
        )
      {
         // draw transparent from the destinationBuffer into the
         // realImage
         m_pc_BufferPainter->DrawMarkedPixel(m_pc_GlobalStorage->m_pc_MainImage,
                        pc_DestinationBuffer,
                        &c_ChangedRegion);
      }
   }

   c_ChangedRegion = c_ActualFill.GetChangedRegion();

   m_pc_GlobalStorage->m_pc_ImageFrame->RedrawBufferRegion(c_ChangedRegion);

   // delete the destinationbuffer
   if(pc_DestinationBuffer != NULL)
   {
      delete pc_DestinationBuffer;
   }

   m_pc_GlobalStorage->m_pc_MessageServer->SendMessage(0,
                                                 this,
                                                 "ImageChanged");
} // FillAtPoint

//===========================================================================
//
//===========================================================================

void PaintBox::TransparencyAutomaticChanged(int i_NewValue)
{
   m_i_TransparencyClearMode = i_NewValue;
   SetSensitivity();

} // TransparencyAutomaticChanged()

//===========================================================================
//
//===========================================================================

void PaintBox:: TransparencyMaskClear()
{
   bool b_Copied;

   b_Copied = false;

   // if in filter mode, then also copy the original image
   if(m_i_CurrentGraphikMode == PaintFilter)
   {
      PixelBuffer *pc_UsedPixelMask ;

      pc_UsedPixelMask = m_pc_GlobalStorage->m_pc_MainImage->GetUsedPixelMask();
      if(pc_UsedPixelMask != NULL)
      {
         if(pc_UsedPixelMask->GetDepth() >1)
         {
            // copy the original mage
            pc_UsedPixelMask->CopyFrom(m_pc_GlobalStorage->m_pc_MainImage,
                                       false,
                                       false,
                                       false);
            // clean up the additional mask
            m_pc_GlobalStorage->m_pc_MainImage->UsedPixelMaskClean();
            // remember that its done
            b_Copied = true;
         }
      }
   }

   // if not done
   if( !b_Copied)
   {
      // the do the regular way
      m_pc_GlobalStorage->m_pc_MainImage->UsedPixelMaskClean();
   }

} // TransparencyMaskClear()


//===========================================================================
//
//===========================================================================

void PaintBox::SetSensitivity()
{
   if(m_b_TransparencyEnabled)
   {
      m_TransparencySlider->setEnabled(true);

   }
   else
   {
      // transparency is disabled
      m_TransparencySlider->setEnabled(false);

   }

   // set the availiability of the filter distance slider
   if(m_b_FilterNearDistanceActive )
   {
      m_FilterDistanceSlider ->setEnabled(true);
   }
   else
   {
      m_FilterDistanceSlider ->setEnabled(false);
   }
   
   // only enable the use mask checkbox in the speckle mode
   if(m_i_CurrentGraphikMode == PaintSpeckle)
   {
      m_TransparencyMaskUseCheck->setEnabled(true);
   }
   else
   {
      m_TransparencyMaskUseCheck->setEnabled(false);
   }

   SetBrushShapeSensitivity();
      
} // SetSensitivity()

//===========================================================================
//
//===========================================================================

void PaintBox::FillColorChanged(Pixel p_Color)
{
   m_p_FillDistanceColor = p_Color;
} // FillColorChanged()

//===========================================================================
//
//===========================================================================

void PaintBox::FillDistanceChanged(int i_Factor)
{
   m_i_FillDistance = i_Factor;
} // FillDistanceChanged()

//===========================================================================
//
//===========================================================================

void PaintBox::FillDistanceFromStartPoint(bool b_On)
{
   m_b_FillFromStartPoint = b_On;
   m_FillModeFromColor  ->setChecked(!b_On);
} // FillDistanceFromStartPoint()

//===========================================================================
//
//===========================================================================

void PaintBox::FillDistanceFromColor(bool b_On)
{
   m_b_FillFromColor = b_On;
   m_FillModeFromOrigin ->setChecked(!b_On);

} // FillDistanceFromColor()

//===========================================================================
//
//===========================================================================

void PaintBox::FilterWidthChanged(int i_Factor)
{
   m_i_FilterWidth = i_Factor;
   m_pc_BufferPainter->SetFilterWidth(m_i_FilterWidth);
} // FilterWidthChanged()

//===========================================================================
//
//===========================================================================

void PaintBox::FilterDistanceEnableChanged(bool b_On)
{
   m_b_FilterNearDistanceActive = b_On;
   m_pc_BufferPainter->SetFilterNearDistanceActive(m_b_FilterNearDistanceActive);
   SetSensitivity();
} // FilterDistanceEnableChanged()

//===========================================================================
//
//===========================================================================

void PaintBox::FilterDistanceChanged(int i_Distance)
{
   m_i_FilterNearDistance = i_Distance;
   m_pc_BufferPainter->SetFilterNearDistance(m_i_FilterNearDistance);
} // FilterDistanceChanged()

//===========================================================================
//
//===========================================================================

void PaintBox::FilterAlgorithmChanged(int i_NewMode)
{
   m_i_FilterAlgorithm = i_NewMode;
   m_pc_BufferPainter->SetFilterAlgorithm(m_i_FilterAlgorithm);
} // FilterAlgorithmChanged()

//===========================================================================
//
//===========================================================================

void PaintBox::SpeckleSizeChanged(int i_Size)
{
   m_i_SpeckleSize = i_Size;
} // SpeckleSizeChanged()

//===========================================================================
//
//===========================================================================

void PaintBox::SpeckleDensityChanged(int i_Density)
{
   m_i_SpeckleDensity = i_Density;
} // SpeckleDensityChanged()

//===========================================================================
//
//===========================================================================

void PaintBox::SmearStrengthChanged(int i_Strength)
{
   m_i_SmearStrength = i_Strength;
} // SmearStrengthChanged()

//===========================================================================
//
//===========================================================================

void PaintBox::SetWidgetToMode(int i_Mode)
{
   m_i_CurrentGraphikMode = i_Mode;

   // show the continuus checkbox
   m_BrushContinuousEnable->show();
   // first disable all
   m_BrushSizeModeBox ->hide();
   m_BrushColorModeBox->hide();
   m_TransparencyBox  ->hide();
   m_MaskClearModeBox ->hide();
   m_FillModeBox      ->hide();
   m_LightnessModeBox ->hide();
   m_FilterModeBox    ->hide();
   m_SpeckleModeBox   ->hide();
   m_SmearModeBox     ->hide();
   m_CopyBrushModeBox ->hide();
   m_RecolorModeBox   ->hide();
   m_BrushShapeModeBox->hide();
   m_BrushButton      ->SetSelected(false);
   m_FillButton       ->SetSelected(false);
   m_LightnessButton  ->SetSelected(false);
   m_SaturationButton ->SetSelected(false);
   m_FilterButton     ->SetSelected(false);
   m_SpeckleButton    ->SetSelected(false);
   m_AirBrushButton   ->SetSelected(false);
   m_SmearButton      ->SetSelected(false);
   m_CopyButton       ->SetSelected(false);
   m_RecolorButton    ->SetSelected(false);

   // now enable, what is needed
   switch(i_Mode)
   {
   case PaintBrush:
      m_BrushSizeModeBox ->show();
      m_BrushShapeModeBox->show();
      m_BrushColorModeBox->show();
      m_TransparencyBox  ->show();
      m_MaskClearModeBox ->show();
      m_BrushButton      ->SetSelected(true);
      break;
   case PaintFill:
      m_BrushColorModeBox->show();
      m_TransparencyBox  ->show();
      m_FillModeBox      ->show();
      m_FillButton       ->SetSelected(true);
      break;
   case PaintLightness:
      m_BrushSizeModeBox ->show();
      m_BrushShapeModeBox->show();
      m_MaskClearModeBox ->show();
      m_LightnessModeBox ->show();
      m_LightnessButton  ->SetSelected(true);
      break;
   case PaintSaturation:
      m_BrushSizeModeBox ->show();
      m_BrushShapeModeBox->show();
      m_MaskClearModeBox ->show();
      m_LightnessModeBox ->show();
      m_SaturationButton ->SetSelected(true);
      break;
   case PaintFilter:
      m_BrushSizeModeBox ->show();
      m_BrushShapeModeBox->show();
      m_MaskClearModeBox ->show();
      m_FilterModeBox    ->show();
      m_FilterButton     ->SetSelected(true);
      break;
   case PaintSpeckle:
      m_BrushSizeModeBox ->show();
      m_BrushShapeModeBox->show();
      m_BrushColorModeBox->show();
      m_TransparencyBox  ->show();
      m_MaskClearModeBox ->show();
      m_SpeckleModeBox   ->show();
      m_SpeckleButton    ->SetSelected(true);
      break;
   case PaintAirBrush:
      m_BrushSizeModeBox ->show();
      m_BrushShapeModeBox->show();
      m_BrushColorModeBox->show();
      m_AirBrushButton   ->SetSelected(true);
      break;
   case PaintSmear:
      m_BrushSizeModeBox ->show();
      m_BrushShapeModeBox->show();
      m_BrushContinuousEnable->hide();
      m_SmearModeBox     ->show();
      m_SmearButton      ->SetSelected(true);
      break;
   case PaintCopy:
      m_BrushSizeModeBox ->show();
      m_BrushShapeModeBox->show();
      m_TransparencyBox  ->show();
      m_MaskClearModeBox ->show();
      m_CopyBrushModeBox ->show();
      m_CopyButton       ->SetSelected(true);
      break;
   case PaintRecolor:
      m_BrushSizeModeBox ->show();
      m_BrushShapeModeBox->show();
      m_MaskClearModeBox ->show();
      m_RecolorModeBox   ->show();
      m_RecolorButton    ->SetSelected(true);
      break;
   } // switch mode

   ResizeDisplay();
   SetSensitivity();

   DisableCopyBrushPoint();

} // SetWidgetToMode()

//===========================================================================
//
//===========================================================================

void PaintBox::TransparencyMaskUseCheckChanged(bool b_On)
{
   m_b_UseTransparencyMask = b_On;
} // TransparencyMaskUseCheckChanged()

//===========================================================================
//
//===========================================================================

int PaintBox::WriteFile(PericoFileWriter *pc_FileWriter,
                         QString qs_Name)
{
   pc_FileWriter->StartStructure(qs_Name,"PaintBox");

   pc_FileWriter->AddValue("BrushThickness", m_i_CurrentThickness);
   pc_FileWriter->AddValue("BrushContinous", m_b_BrushContinuous);

   m_BrushColorWidget->WriteFile(pc_FileWriter,"BrushColorWidget");
   
   pc_FileWriter->AddValue("TransparencyEnable", m_b_TransparencyEnabled);
   pc_FileWriter->AddValue("TransparencyFactor", m_i_TransparencyFactor);
   pc_FileWriter->AddValue("TransparencyUseMask", m_b_UseTransparencyMask);

   pc_FileWriter->AddValue("TransparencyClearMode",m_i_TransparencyClearMode);

   pc_FileWriter->AddValue("FillDistance",m_i_FillDistance);
   pc_FileWriter->AddValue("FillFromStartPoint",m_b_FillFromStartPoint);
   pc_FileWriter->AddValue("FillFromColor",m_b_FillFromColor);
   pc_FileWriter->AddValue("FillDistanceColor",(int)m_p_FillDistanceColor);

   pc_FileWriter->AddValue("LightnessFactor",m_i_LightnessFactor);

   pc_FileWriter->AddValue("FilterWidth",m_i_FilterWidth);
   pc_FileWriter->AddValue("FilterNearDistance",m_i_FilterNearDistance);
   pc_FileWriter->AddValue("FilterNearDistanceActive",m_b_FilterNearDistanceActive);
   pc_FileWriter->AddValue("FilterAlgorithm",m_i_FilterAlgorithm);
      
   pc_FileWriter->AddValue("SpeckleSize",m_i_SpeckleSize);
   pc_FileWriter->AddValue("SpeckleDensity",m_i_SpeckleDensity);

   pc_FileWriter->AddValue("SmearStrength",m_i_SmearStrength);

   pc_FileWriter->AddValue("RecolorFromColor",(int) m_p_RecolorFromColor) ;
   pc_FileWriter->AddValue("RecolorToColor"  ,(int) m_p_RecolorToColor) ;
   pc_FileWriter->AddValue("RecolorDistance" ,m_i_RecolorDistance) ;

   pc_FileWriter->AddValue("BrushShapeStyle" ,m_i_BrushShapeStyle);
   m_pc_BrushShapeFreeBuffer->WriteFile(pc_FileWriter,"BrushShapeFreeBuffer");
   
   pc_FileWriter->FinishStructure();

   // everything is OK
   return 0;
} // WriteFile

//===========================================================================
//
//===========================================================================

int PaintBox::LoadFile(PericoFileWriter *pc_FileWriter)
{

   PERICO_READ_STRUCTURE_START

   if(   qs_Name.compare("BrushColorWidget")==0
      && qs_Value.compare("QUColorOrPattern")==0
      && b_IsStartStructure)
   {
      m_BrushColorWidget->LoadFile(pc_FileWriter);
   }

   else if(qs_Name.compare("BrushThickness") == 0)
   {
      m_i_CurrentThickness = atoi(qs_Value);
   }
   else if(qs_Name.compare("BrushContinous") == 0)
   {
      m_b_BrushContinuous = PericoFileWriter::atob(qs_Value);
   }

   else if(qs_Name.compare("TransparencyEnable") == 0)
   {
      m_b_TransparencyEnabled = PericoFileWriter::atob(qs_Value);
   }
   else if(qs_Name.compare("TransparencyFactor") == 0)
   {
      m_i_TransparencyFactor = atoi (qs_Value);
   }
   else if(qs_Name.compare("TransparencyUseMask") == 0)
   {
      m_b_UseTransparencyMask = PericoFileWriter::atob(qs_Value);
   }

   else if(qs_Name.compare("TransparencyClearMode") == 0)
   {
      m_i_TransparencyClearMode = atoi(qs_Value);
   }

   else if(qs_Name.compare("FillDistance") == 0)
   {
      m_i_FillDistance = atoi(qs_Value);
   }
   else if(qs_Name.compare("FillFromStartPoint") == 0)
   {
      m_b_FillFromStartPoint = PericoFileWriter::atob(qs_Value);
   }
   else if(qs_Name.compare("FillFromColor") == 0)
   {
      m_b_FillFromColor = PericoFileWriter::atob(qs_Value);
   }
   else if(qs_Name.compare("FillDistanceColor") == 0)
   {
      m_p_FillDistanceColor = (Pixel) atoi(qs_Value);
   }

   else if(qs_Name.compare("LightnessFactor") == 0)
   {
      m_i_LightnessFactor =  atoi(qs_Value);
   }

   else if(qs_Name.compare("FilterWidth") == 0)
   {
      m_i_FilterWidth =  atoi(qs_Value);
   }
   else if(qs_Name.compare("FilterNearDistance") == 0)
   {
      m_i_FilterNearDistance =  atoi(qs_Value);
   }
   else if(qs_Name.compare("FilterNearDistanceActive") == 0)
   {
      m_b_FilterNearDistanceActive =  PericoFileWriter::atob(qs_Value);
   }
   else if(qs_Name.compare("FilterAlgorithm") == 0)
   {
      m_i_FilterAlgorithm =  atoi(qs_Value);
   }

   else if(qs_Name.compare("SpeckleSize") == 0)
   {
      m_i_SpeckleSize =  atoi(qs_Value);
   }
   else if(qs_Name.compare("SpeckleDensity") == 0)
   {
      m_i_SpeckleDensity =  atoi(qs_Value);
   }

   else if(qs_Name.compare("SmearStrength") == 0)
   {
      m_i_SmearStrength =  atoi(qs_Value);
   }

   else if(qs_Name.compare("RecolorFromColor") == 0)
   {
      m_p_RecolorFromColor = (Pixel) atoi(qs_Value);
   }
   else if(qs_Name.compare("RecolorToColor") == 0)
   {
      m_p_RecolorToColor = (Pixel) atoi(qs_Value);
   }
   else if(qs_Name.compare("RecolorDistance") == 0)
   {
      m_i_RecolorDistance =  atoi(qs_Value);
   }

   else if(qs_Name.compare("BrushShapeStyle") == 0)
   {
      m_i_BrushShapeStyle =  atoi(qs_Value);
   }
   else if(    qs_Name.compare("BrushShapeFreeBuffer") == 0
            && qs_Value.compare("PixelBuffer") == 0
            && b_IsStartStructure)
   {
      DELETE(m_pc_BrushShapeFreeBuffer);

      m_pc_BrushShapeFreeBuffer = new PixelBuffer(pc_FileWriter);
   }
   PERICO_READ_STRUCTURE_END

   // set to Display

   // brush part
   m_BrushContinuousEnable->setChecked(m_b_BrushContinuous);
   m_BrushThicknessSlider->SetValue(m_i_CurrentThickness);
   ThicknessChanged(m_i_CurrentThickness);

   // transparency part
   m_TransparencyEnable->setChecked(m_b_TransparencyEnabled);
   m_TransparencySlider->SetValue(m_i_TransparencyFactor);
   m_TransparencyMaskUseCheck->setChecked(m_b_UseTransparencyMask);
   TransparencyEnableChanged(m_b_TransparencyEnabled);
   TransparencyFactorChanged(m_i_TransparencyFactor);

   // Transparency clear part   
   m_TransparencyClearCombo->setCurrentItem(m_i_TransparencyClearMode);
   TransparencyAutomaticChanged(m_i_TransparencyClearMode);

   // Fill Part
   m_FillDistanceSlider->SetValue(m_i_FillDistance);
   m_FillModeFromOrigin->setChecked(m_b_FillFromStartPoint);
   m_FillModeFromColor->setChecked(m_b_FillFromColor);
   m_FillDistanceColor->SetColor(m_p_FillDistanceColor);
   FillDistanceChanged(m_i_FillDistance);
   FillDistanceFromColor(m_b_FillFromColor);
   FillDistanceFromStartPoint(m_b_FillFromStartPoint);
   FillColorChanged(m_p_FillDistanceColor);

   // Lightness Part
   m_LightnessSlider->SetValue(m_i_LightnessFactor);      
   LightnessFactorChanged(m_i_LightnessFactor);

   // Filter part
   m_FilterWidthSlider->SetValue(m_i_FilterWidth);
   m_FilterDistanceActiveCheck->setChecked(m_b_FilterNearDistanceActive);
   m_FilterDistanceSlider->SetValue(m_i_FilterNearDistance);
   if(m_i_FilterAlgorithm==BufferPainter::FilterAlgorithmMiddle)
   {
      m_FilterMiddleButton->setChecked(1);
      m_FilterMedianButton->setChecked(0);
      m_FilterSharpButton ->setChecked(0);
   }
   else if(m_i_FilterAlgorithm==BufferPainter::FilterAlgorithmMedian)
   {
      m_FilterMiddleButton->setChecked(0);
      m_FilterMedianButton->setChecked(1);
      m_FilterSharpButton ->setChecked(0);
   }
   else if(m_i_FilterAlgorithm==BufferPainter::FilterAlgorithmSharp)
   {
      m_FilterMiddleButton->setChecked(0);
      m_FilterMedianButton->setChecked(0);
      m_FilterSharpButton ->setChecked(1);
   }
   FilterWidthChanged(m_i_FilterWidth);
   FilterDistanceEnableChanged(m_b_FilterNearDistanceActive);
   FilterDistanceChanged(m_i_FilterNearDistance);
   FilterAlgorithmChanged(m_i_FilterAlgorithm);

   // speckle part
   m_SpeckleSizeSlider->SetValue(m_i_SpeckleSize);
   m_SpeckleDensitySlider->SetValue(m_i_SpeckleDensity);
   SpeckleSizeChanged(m_i_SpeckleSize);
   SpeckleDensityChanged(m_i_SpeckleDensity);
   
   // speckle part
   m_SmearStrengthSlider->SetValue(m_i_SmearStrength);
   SmearStrengthChanged(m_i_SmearStrength);

   // recolor part
   m_RecolorFromColor->SetColor(m_p_RecolorFromColor);
   m_RecolorToColor->  SetColor(m_p_RecolorToColor);
   m_RecolorDistanceSlider->SetValue(m_i_RecolorDistance);
   RecolorFromColorChanged(m_p_RecolorFromColor);
   RecolorToColorChanged(m_p_RecolorToColor);
   RecolorDistanceChanged(m_i_RecolorDistance);
   
   SetFreeBrushShapeIcon(m_pc_BrushShapeFreeBuffer);

   // shape might have changed??
   if(m_i_BrushShapeStyle == BrushShapeFreeFigure)
   {
      CreateBrushShape(m_i_CurrentThickness);
   }
     
   // finaly set all sensitive modes of GUI elements
   SetSensitivity();
   // everything is OK
   return 0;
} // LoadFile

//===========================================================================
//
//===========================================================================

void PaintBox::PaintCopyEvent(int i_MouseEventType)
{
   RectRegion r_PointRegion;
   RectRegion r_ClearRegion;
   int        i_X;
   int        i_Y;
   m_pc_GlobalStorage->m_pc_Selection->GetSelectionPoint(i_X,i_Y);
   
   if(m_i_MouseEventType != MouseEvent::EventOver)
   {
      if(m_b_SelectOriginalPoint)
      {
         m_b_SelectOriginalPoint = false;
         m_pc_GlobalStorage->m_pc_Selection->GetSelectionPoint(m_i_CopyDifferenceX,
                                                               m_i_CopyDifferenceY);
         m_pc_GlobalStorage->m_pc_Selection->DeactivateSelection();

         
         m_pc_GlobalStorage->m_pc_ObjectMain->SetVisible(true);
         m_pc_ObjectLayer->SetVisible(true);
         m_pc_CopyBrushOriginPoint->MoveTo(i_X,i_Y);
         r_PointRegion =  m_pc_GlobalStorage->m_pc_Zoom->GetObjectPointZoomSize(m_pc_CopyBrushOriginPoint);
         m_pc_GlobalStorage->m_pc_ImageFrame->RedrawBufferRegion(r_PointRegion);
         
         m_pc_GlobalStorage->m_pc_Selection->ActivateSelection(Selection::SelectionStream,
                                                  tr("Copy"),
                                                  Callback(SelectionEventArrivedCB,this));
      }
      else
      {
         if( m_b_FirstPoint)
         {
            m_b_FirstPoint = false;
            m_i_CopyDifferenceX -= i_X;
            m_i_CopyDifferenceY -= i_Y;

         }
         // draw the point
         {
            m_pc_ObjectLayer->SetVisible(true);
            
            r_ClearRegion =  m_pc_GlobalStorage->m_pc_Zoom->GetObjectPointZoomSize(m_pc_CopyBrushOriginPoint);

            m_pc_CopyBrushOriginPoint->MoveTo(i_X+m_i_CopyDifferenceX,
                                        i_Y+m_i_CopyDifferenceY);
            r_PointRegion =  m_pc_GlobalStorage->m_pc_Zoom->GetObjectPointZoomSize(m_pc_CopyBrushOriginPoint);
            m_pc_GlobalStorage->m_pc_ImageFrame->RedrawBufferRegion(r_ClearRegion);
            m_pc_GlobalStorage->m_pc_ImageFrame->RedrawBufferRegion(r_PointRegion);
         }
         DrawCopy();
      }
   }
   else  // event over
   {
      // Event Over
      if(m_b_SelectOriginalPoint)
      {
         m_pc_ObjectLayer->SetVisible(false);
         r_ClearRegion =  m_pc_GlobalStorage->m_pc_Zoom->GetObjectPointZoomSize(m_pc_CopyBrushOriginPoint);
         m_pc_GlobalStorage->m_pc_ImageFrame->RedrawBufferRegion(r_ClearRegion);
      }
      else
      {
         m_pc_ObjectLayer->SetVisible(true);
         if(m_b_FirstPoint)
         {
            r_ClearRegion = m_pc_GlobalStorage->m_pc_Zoom->GetObjectPointZoomSize(m_pc_CopyBrushOriginPoint);

            m_pc_CopyBrushOriginPoint->MoveTo(m_i_CopyDifferenceX,
                                        m_i_CopyDifferenceY);
            r_PointRegion =  m_pc_GlobalStorage->m_pc_Zoom->GetObjectPointZoomSize(m_pc_CopyBrushOriginPoint);
            m_pc_GlobalStorage->m_pc_ImageFrame->RedrawBufferRegion(r_ClearRegion);
            m_pc_GlobalStorage->m_pc_ImageFrame->RedrawBufferRegion(r_PointRegion);
         }
         else // not first point
         {
            r_ClearRegion =  m_pc_GlobalStorage->m_pc_Zoom->GetObjectPointZoomSize(m_pc_CopyBrushOriginPoint);
      
            m_pc_CopyBrushOriginPoint->MoveTo(i_X+m_i_CopyDifferenceX,
                                        i_Y+m_i_CopyDifferenceY);
            r_PointRegion =  m_pc_GlobalStorage->m_pc_Zoom->GetObjectPointZoomSize(m_pc_CopyBrushOriginPoint);
            m_pc_GlobalStorage->m_pc_ImageFrame->RedrawBufferRegion(r_ClearRegion);
            m_pc_GlobalStorage->m_pc_ImageFrame->RedrawBufferRegion(r_PointRegion);
         }
      } // else not select first point
   } // else event over
   


} // PaintCopyEvent()

//===========================================================================
//
//===========================================================================

void PaintBox::DisableCopyBrushPoint()
{
   RectRegion r_PointRegion;

   m_pc_ObjectLayer->SetVisible(false);
   r_PointRegion =  m_pc_GlobalStorage->m_pc_Zoom->GetObjectPointZoomSize(m_pc_CopyBrushOriginPoint);
   m_pc_GlobalStorage->m_pc_ImageFrame->RedrawBufferRegion(r_PointRegion);

} // DisableCopyBrushPoint()

//===========================================================================
//
//===========================================================================

void PaintBox::RecolorFromColorChanged(Pixel c_Color)
{
   m_p_RecolorFromColor = c_Color;
   m_pc_BufferPainter->SetRecolorParameter(m_p_RecolorFromColor,
                                           m_p_RecolorToColor,
                                           m_i_RecolorDistance);
} // RecolorFromColorChanged()

//===========================================================================
//
//===========================================================================

void PaintBox::RecolorToColorChanged(Pixel c_Color)
{
   m_p_RecolorToColor = c_Color;
   m_pc_BufferPainter->SetRecolorParameter(m_p_RecolorFromColor,
                                           m_p_RecolorToColor,
                                           m_i_RecolorDistance);
} // RecolorToColorChanged()

//===========================================================================
//
//===========================================================================

void PaintBox::RecolorDistanceChanged(int i_Factor)
{
   m_i_RecolorDistance = i_Factor;
   m_pc_BufferPainter->SetRecolorParameter(m_p_RecolorFromColor,
                                           m_p_RecolorToColor,
                                           m_i_RecolorDistance);
} // RecolorDistanceChanged()

//===========================================================================
//
//===========================================================================

void PaintBox::BrushShapeCircleSelected()
{
   m_i_BrushShapeStyle = BrushShapeCircle;

   CreateBrushShape(m_i_CurrentThickness);
   SetSensitivity();
} //

//===========================================================================
//
//===========================================================================

void PaintBox::BrushShapeSquareSelected()
{
   m_i_BrushShapeStyle = BrushShapeSquare;

   CreateBrushShape(m_i_CurrentThickness);
   SetSensitivity();
} //

//===========================================================================
//
//===========================================================================

void PaintBox::BrushShapeVerticalSelected()
{
   m_i_BrushShapeStyle = BrushShapeVerticalLine;

   CreateBrushShape(m_i_CurrentThickness);
   SetSensitivity();
} //

//===========================================================================
//
//===========================================================================

void PaintBox::BrushShapeHorizontalSelected()
{
   m_i_BrushShapeStyle = BrushShapeHorizontalLine;

   CreateBrushShape(m_i_CurrentThickness);
   SetSensitivity();
} //

//===========================================================================
//
//===========================================================================

void PaintBox::BrushShapeSWNESelected()
{
   m_i_BrushShapeStyle = BrushShapeSWNELine;

   CreateBrushShape(m_i_CurrentThickness);
   SetSensitivity();
} //

//===========================================================================
//
//===========================================================================

void PaintBox::BrushShapeNWSESelected()
{
   m_i_BrushShapeStyle = BrushShapeNWSELine;

   CreateBrushShape(m_i_CurrentThickness);
   SetSensitivity();
} //

//===========================================================================
//
//===========================================================================

void PaintBox::BrushShapeFreeSelected()
{
   m_i_BrushShapeStyle = BrushShapeFreeFigure;

   CreateBrushShape(m_i_CurrentThickness);
   SetSensitivity();
} //

//===========================================================================
//
//===========================================================================

void PaintBox::SetBrushShapeSensitivity()
{
   m_BrushShapeCircleButton->SetSelected(false);
   m_BrushShapeSquareButton->SetSelected(false);
   m_BrushShapeVerticalButton->SetSelected(false);
   m_BrushShapeHorizontalButton->SetSelected(false);
   m_BrushShapeNWSEButton->SetSelected(false);
   m_BrushShapeSWNEButton->SetSelected(false);
   m_BrushShapeFreeButton->SetSelected(false);

   switch(m_i_BrushShapeStyle)
   {
   case BrushShapeCircle:
      m_BrushShapeCircleButton->SetSelected(true);
      break;
   case BrushShapeSquare:
      m_BrushShapeSquareButton->SetSelected(true);
      break;
   case BrushShapeVerticalLine:
      m_BrushShapeVerticalButton->SetSelected(true);
      break;
   case BrushShapeHorizontalLine:
      m_BrushShapeHorizontalButton->SetSelected(true);
      break;
   case BrushShapeNWSELine:
      m_BrushShapeNWSEButton->SetSelected(true);
      break;
   case BrushShapeSWNELine:
      m_BrushShapeSWNEButton->SetSelected(true);
      break;
   case BrushShapeFreeFigure:
      m_BrushShapeFreeButton->SetSelected(true);
      break;
   default:
      break;
   } // switch brush shape style
} // SetBrushShapeSensitivity()

//===========================================================================
//
//===========================================================================

void PaintBox::CenterCoordinateToBrush(int &i_X,int &i_Y)
{
   PixelBuffer *pc_Brush;
   int          i_Width;
   int          i_Height;

   pc_Brush = m_pc_BufferPainter->GetBrushShape();

   pc_Brush->GetWidthHeight(i_Width,i_Height);

   i_X -= i_Width /2;
   i_Y -= i_Height/2;
} // CenterCoordinateToBrush()

//===========================================================================
//
//===========================================================================

void PaintBox::BrushShapeFreeImageDropped(PixelBuffer *pc_Buffer)
{
   // should be a valid image
   if(pc_Buffer ==NULL)
   {
      return;
   }

   DELETE(m_pc_BrushShapeFreeBuffer);
   
   
   m_pc_BrushShapeFreeBuffer = new PixelBuffer(pc_Buffer);
   SetFreeBrushShapeIcon(m_pc_BrushShapeFreeBuffer);
   
   // shape might have changed??
   if(m_i_BrushShapeStyle == BrushShapeFreeFigure)
   {
      CreateBrushShape(m_i_CurrentThickness);
   }

} // BrushShapeFreeImageDropped()

//===========================================================================
//
//===========================================================================


void PaintBox::BrushShapeFreeMenuSelected(int i_Index)
{
   MainFrame * pc_NewMainFrame;

   if(m_pc_BrushShapeFreeBuffer == NULL)
   {
      return;
   }

   if(i_Index == 0)
   {
      // edit
   
      pc_NewMainFrame = MainFrame::NewPicture(m_pc_BrushShapeFreeBuffer);
      if(pc_NewMainFrame!= NULL)
      {
         pc_NewMainFrame->SetParentPicture(&m_BrushShapeUpdateConnection);
      }
   }
   else if(i_Index == 1)
   {
      // save
      FileLoader *pc_FileWriter;
      QString     qs_Filename;

      pc_FileWriter = new FileLoader(NULL);

      qs_Filename = pc_FileWriter->AskForFilename(false,"");

      if(!qs_Filename.isEmpty())
      {
         pc_FileWriter->SaveFile(qs_Filename,m_pc_BrushShapeFreeBuffer);
      }

      delete pc_FileWriter;
      
   }
   else if(i_Index == 2)
   {
      // load
      FileLoaderOptions  c_FileOptions;
      PixelBuffer       *pc_Buffer;
      FileLoader        *pc_Loader;
      QString           qs_Filename;

      pc_Loader = new FileLoader(NULL);

      qs_Filename = pc_Loader->AskForFilename(true,"");

      if(!qs_Filename.isEmpty())
      {
         // load the file
         pc_Buffer = pc_Loader->LoadFile(qs_Filename,&c_FileOptions);

         // success?
         if(pc_Buffer != NULL)
         {
            PixelBuffer *pc_SelectionMask;
            pc_SelectionMask = pc_Buffer->GetSelectionMask();
            if(      pc_SelectionMask != NULL
                &&  (! c_FileOptions.b_HasSelectionMask)   )
            {
               pc_SelectionMask->Clean(1);
            }

            DELETE(m_pc_BrushShapeFreeBuffer);

            m_pc_BrushShapeFreeBuffer = pc_Buffer;

            SetFreeBrushShapeIcon(m_pc_BrushShapeFreeBuffer);

            // shape might have changed??
            if(m_i_BrushShapeStyle == BrushShapeFreeFigure)
            {
               CreateBrushShape(m_i_CurrentThickness);
            }
         } // if load success
      } // if filename available
      delete pc_Loader;
   } // if load
      
} // BrushShapeFreeMenuSelected()

//===========================================================================
//
//===========================================================================

void PaintBox::InitBrushShapeFree()
{
   PixelBuffer *pc_SelectionMask;
   int i_X;
   int i_Y;
   int i_Width;
   int i_Height;

   m_pc_BrushShapeFreeBuffer = g_Application->m_pc_IconLibrary
                                            ->GetIcon("BrushStyleFree");

   if(m_pc_BrushShapeFreeBuffer == NULL)
   {
      return;
   }

   m_pc_BrushShapeFreeBuffer->GetWidthHeight(i_Width,i_Height);

   // get selection mask - if if does not exist, create one
   pc_SelectionMask = m_pc_BrushShapeFreeBuffer->GetSelectionMask();
   if(pc_SelectionMask == NULL)
   {
      m_pc_BrushShapeFreeBuffer->AddSelectionMask();
      pc_SelectionMask = m_pc_BrushShapeFreeBuffer->GetSelectionMask();
   }

   // loop through the image
   for(i_Y=0; i_Y <i_Height; i_Y++)
   {
      for(i_X=0; i_X <i_Width; i_X++)
      {
         Pixel p_Value;
         int   i_Level;
         p_Value= m_pc_BrushShapeFreeBuffer->Get(i_X,i_Y);

         i_Level =   PIXEL_RED(p_Value)
                   + PIXEL_GREEN(p_Value)
                   + PIXEL_BLUE(p_Value);
         if(i_Level > 384)  // 3 * 128
         {
            // set white point
            m_pc_BrushShapeFreeBuffer->Set(i_X,i_Y,0xffffff);
            if( pc_SelectionMask != NULL)
            {
               pc_SelectionMask->Set(i_X,i_Y,0);
            }
         }
         else
         {
            // set mask point
            m_pc_BrushShapeFreeBuffer->Set(i_X,i_Y,0);
            if( pc_SelectionMask != NULL)
            {
               pc_SelectionMask->Set(i_X,i_Y,1);
            }
         }
      }
   }
   
} // InitBrushShapeFree()

//===========================================================================
//
//===========================================================================

void PaintBox::SetFreeBrushShapeIcon(PixelBuffer *pc_Buffer)
{
   PixelBuffer *pc_SelectionMask;
   PixelBuffer *pc_Icon;
   int i_X;
   int i_Y;
   int i_Width;
   int i_Height;

   if(pc_Buffer == NULL)
   {
      return ;
   }

   pc_Icon = pc_Buffer->CreateIconFromBuffer(20,20);
   
   pc_SelectionMask = pc_Icon->GetSelectionMask();

   pc_Icon->GetWidthHeight(i_Width,i_Height);
   for(i_Y=0; i_Y <i_Height; i_Y++)
   {
      for(i_X=0; i_X <i_Width; i_X++)
      {
         if(   pc_SelectionMask != NULL
            && (pc_SelectionMask->Get(i_X,i_Y)>0)
            
            )
         {
            // set black
            pc_Icon->Set(i_X,i_Y,0);   
         }
         else
         {
            // set white
            pc_Icon->Set(i_X,i_Y,0xffffff);
         }
      }

   }   
   m_BrushShapeFreeButton->SetIcon(pc_Icon);
   delete pc_Icon;
} // SetFreeBrushShapeIcon()

//===========================================================================
//
//===========================================================================

void *PaintBox::FreeBrushPictureConnectionEventArrived(void *pc_Data)
{

   PictureUpdateConnectionMessage *pc_UpdateMessage;

   pc_UpdateMessage = (PictureUpdateConnectionMessage *) pc_Data;
   if(pc_UpdateMessage == NULL)
   {
      return NULL;
   }

   if(pc_UpdateMessage->m_i_MessageType
           ==  PictureUpdateConnectionMessage::ImageUpdated)
   {
      // tell parents that e new image is available
      BrushShapeFreeImageDropped(pc_UpdateMessage->m_pc_Buffer);
   }
   else if(pc_UpdateMessage->m_i_MessageType
           ==  PictureUpdateConnectionMessage::ChildTermination)
   {
      m_BrushShapeUpdateConnection.CleanChild();
   }
   else
   {
      // wrong message - should not occur
   }

   return NULL;
} // FreeBrushPictureConnectionEventArrived()

void *PaintBox::FreeBrushPictureConnectionEventArrivedCB(
                                       void *Thisdata,
                                       void *pc_Data)
{
   PaintBox *pc_This;
   pc_This = (PaintBox *) Thisdata;
   return pc_This->FreeBrushPictureConnectionEventArrived(pc_Data);
} // FreeBrushPictureConnectionEventArrivedCB()

//===========================================================================
//
//===========================================================================




