//---------------------------------------------------------------------------
// This sourcefile is created by Peter Jodda / peter@jodda.de
// It is under the GPL license.
// If you expand this file, please contact me, that I can integrate the
// enhancements in the original file.
//---------------------------------------------------------------------------
// Module: QUColorOrPattern
//---------------------------------------------------------------------------
// Purpose: Help widget to select a color or a pattern
// for draw and paint-
//---------------------------------------------------------------------------

#include <stdlib.h>

#include <qhgroupbox.h>
#include <qlayout.h>

#include "BrushMode.h"
#include "QUColorOrPattern.h"

#include "QtUtilities.h"
#include "ApplicationStorage.h"


// width and height of the color and pattern area
#define SELCTION_WIDGET_DIMENSION 64

//===========================================================================
//
//===========================================================================

QUColorOrPattern:: QUColorOrPattern(QWidget *parent,
                    GlobalStorage *pc_GlobalStorage,
                    ColorSelectBox *pc_SelectBox)
                    : QWidget(parent)
{
   QHBoxLayout *l_BoxLayout;
   unsigned int  ui_Mode;
   
   m_pc_GlobalStorage  = pc_GlobalStorage;
   m_pc_SelectBox      = pc_SelectBox;
   m_pc_CurrentPattern = NULL;
   m_p_CurrentColor    = 0;

   m_b_InUpdate        = false;

   // create a horizonal layout
   l_BoxLayout = new QHBoxLayout(this,2);

   // add checkbox
   m_ColorSelected = new QCheckBox(tr("Color"),this);
   l_BoxLayout->addWidget(m_ColorSelected);


   ui_Mode =   QUColorDragWidget::ModePipette
             | QUColorDragWidget::ModeSetSize;
   if(pc_SelectBox != NULL)
   {
      ui_Mode |= QUColorDragWidget::ModeEnableColorselectBox;
   }
   
   // add color widget
   m_ColorWidget = new QUColorDragWidget(this,
                                            ui_Mode,
                                          m_pc_GlobalStorage,
                                          SELCTION_WIDGET_DIMENSION,
                                          SELCTION_WIDGET_DIMENSION,
                                          m_pc_SelectBox);
   l_BoxLayout->addWidget(m_ColorWidget);
   m_ColorWidget->SetColor(m_p_CurrentColor);

   // add seperator line
   l_BoxLayout->addWidget(new QtUtilLine(false,true,this));

   // add patern checkbox
   m_PatternSelected = new QCheckBox(tr("Pattern"),this);
   l_BoxLayout->addWidget(m_PatternSelected);

   // add pattern selection label
   m_PatternWidget = new QUImageDragLabel(this,
                                            QUImageDragLabel::ModeUseWidthHeight
                                          | QUImageDragLabel::ModeEnablePopup,
                                          m_pc_GlobalStorage,
                                          SELCTION_WIDGET_DIMENSION,
                                          SELCTION_WIDGET_DIMENSION
                                          );
   l_BoxLayout->addWidget(m_PatternWidget);

   // add seperator line
   l_BoxLayout->addWidget(new QtUtilLine(false,true,this));

   // add patern checkbox
   m_MotiveSelected = new QCheckBox(tr("Motive"),this);
   l_BoxLayout->addWidget(m_MotiveSelected);

   // connect all elements to slots
   connect( m_ColorSelected,
            SIGNAL(toggled(bool)),
            SLOT( ColorToggled(bool )));

   connect( m_ColorWidget,
            SIGNAL(ColorChanged(Pixel)),
            SLOT(InternalColorChanged(Pixel)) );
            
   connect( m_PatternWidget,
            SIGNAL(ImageDropped(PixelBuffer *)),
            SLOT(InternalImageDropped(PixelBuffer *)) );

   connect( m_PatternSelected,
            SIGNAL(toggled(bool)),
            SLOT( PatternToggled(bool )));

   connect( m_MotiveSelected,
            SIGNAL(toggled(bool)),
            SLOT( MotiveToggled(bool )));

   // initital switch to the color mode
   m_i_ActiveMode = SelectColor;
   m_ColorSelected->setChecked(true);

} // QUColorOrPattern() - constructor

//===========================================================================
//
//===========================================================================

QUColorOrPattern::~QUColorOrPattern()
{
   // delete the pattern, if it exists
   if(m_pc_CurrentPattern != NULL)
   {
      delete m_pc_CurrentPattern;
      m_pc_CurrentPattern = NULL;
   }
} // ~QUColorOrPattern()

//===========================================================================
//
//===========================================================================

void QUColorOrPattern::PatternToggled(bool b_Mode)
{
   // is it called from myself
   if(m_b_InUpdate)
   {
      // do nothing
      return;
   }

   if(b_Mode)
   {
      m_i_ActiveMode = SelectPattern;
   }

   m_b_InUpdate=true;
   if(b_Mode)
   {
      m_ColorSelected->setChecked(false);
      m_MotiveSelected->setChecked(false);
   }
   else
   {
      m_ColorSelected->setChecked(true);
      m_MotiveSelected->setChecked(false);
   }
   
   m_b_InUpdate=false;
   
   // signal the pattern mode
   ModeChange(m_i_ActiveMode) ;
} // PatternToggled()

//===========================================================================
//
//===========================================================================

void QUColorOrPattern::ColorToggled(bool b_Mode)
{
   // is it called from myself
   if(m_b_InUpdate)
   {
      // do nothing
      return;
   }

   if(b_Mode)
   {
      m_i_ActiveMode = SelectColor;
   }
   else

   m_b_InUpdate=true;
   if(b_Mode)
   {
      m_PatternSelected->setChecked(false);
      m_MotiveSelected->setChecked(false);
   }
   else
   {
      m_PatternSelected->setChecked(true);
      m_MotiveSelected->setChecked(false);
   }
   
   m_b_InUpdate=false;

   // signal the pattern mode
   ModeChange(m_i_ActiveMode) ;
}  // ColorToggled()

//===========================================================================
//
//===========================================================================

void QUColorOrPattern::MotiveToggled(bool b_Mode)
{
   // is it called from myself
   if(m_b_InUpdate)
   {
      // do nothing
      return;
   }

   if(b_Mode)
   {
      m_i_ActiveMode = SelectMotive;
   }

   m_b_InUpdate=true;
   if(b_Mode)
   {
      m_PatternSelected->setChecked(false);
      m_ColorSelected->setChecked(false);
   }
   else
   {
      m_PatternSelected->setChecked(false);
      m_ColorSelected->setChecked(true);
   }
   m_b_InUpdate=false;

   // signal the pattern mode
   ModeChange(m_i_ActiveMode) ;
}  // ColorToggled()

//===========================================================================
//
//===========================================================================

void QUColorOrPattern::InternalColorChanged(Pixel p_ThePixel)
{
   m_p_CurrentColor = p_ThePixel;
   // in the moment siple lead it through
   ColorChanged(p_ThePixel);
}  // InternalColorChanged()

//===========================================================================
//
//===========================================================================

void QUColorOrPattern::InternalImageDropped(PixelBuffer *pc_Image)
{
   if(pc_Image == NULL)
   {
      return;
   }

   // delete current pattern
   if(m_pc_CurrentPattern != NULL)
   {
      delete m_pc_CurrentPattern;
      m_pc_CurrentPattern = NULL;
   }

   // and create a real copy
   m_pc_CurrentPattern = new PixelBuffer(pc_Image);
   m_PatternWidget->SetPixelBuffer(m_pc_CurrentPattern);

   // in the moment siple lead it through
   ImageDropped(pc_Image);
} // InternalImageDropped()

//===========================================================================
//
//===========================================================================

void QUColorOrPattern::SetColor(Pixel p)
{
   m_p_CurrentColor = p;
   m_ColorWidget->SetColor(m_p_CurrentColor);
   
} // SetColor()

//===========================================================================
//
//===========================================================================

void QUColorOrPattern::SetPattern(PixelBuffer *pc_Pattern)
{
   // store a real copy the this widget
   if(m_pc_CurrentPattern != NULL)
   {
      delete m_pc_CurrentPattern;
      m_pc_CurrentPattern = NULL;
   }
   m_pc_CurrentPattern = new PixelBuffer(pc_Pattern);

   m_PatternWidget->SetPixelBuffer(m_pc_CurrentPattern);

} //  SetPattern()

//===========================================================================
//
//===========================================================================

Pixel QUColorOrPattern::GetColor()
{
   return m_p_CurrentColor;

} // GetColor()

//===========================================================================
//
//===========================================================================

PixelBuffer * QUColorOrPattern::GetPattern()
{
   return m_pc_CurrentPattern;

} // GetPattern()

//===========================================================================
//
//===========================================================================

BrushMode QUColorOrPattern::GetActiveMode()
{
   return m_i_ActiveMode;

} // IsColorActive()

//===========================================================================
//
//===========================================================================

void QUColorOrPattern::SetColorActive(bool b_Active)
{
   if( m_i_ActiveMode != SelectColor)
   {
      m_i_ActiveMode = SelectColor;
      m_ColorSelected->setChecked(b_Active);
   }    

} // SetColorActive()

//===========================================================================
//
//===========================================================================

int QUColorOrPattern::WriteFile(PericoFileWriter *pc_FileWriter,
                         QString qs_Name)
{
   pc_FileWriter->StartStructure(qs_Name,"QUColorOrPattern");

   pc_FileWriter->AddValue("BrushMode",(int) m_i_ActiveMode);
   pc_FileWriter->AddValue("CurrentColor",(int) m_p_CurrentColor);

   if(m_pc_CurrentPattern!= NULL)
   {
      m_pc_CurrentPattern->WriteFile(pc_FileWriter,"CurrentPattern");
   }

   pc_FileWriter->FinishStructure();

   // everything is OK
   return 0;
} // WriteFile

//===========================================================================
//
//===========================================================================

int QUColorOrPattern::LoadFile(PericoFileWriter *pc_FileWriter)
{
   PixelBuffer * pc_NewPattern;

   pc_NewPattern = NULL;
   
   PERICO_READ_STRUCTURE_START

   if(qs_Name.compare("BrushMode")==0)
   {
      m_i_ActiveMode = (BrushMode) atoi(qs_Value);
   }
   else if(qs_Name.compare("CurrentColor")==0)
   {
      m_p_CurrentColor = (Pixel) atoi(qs_Value);
   }
   else if(   qs_Name.compare("CurrentPattern") ==0
      && qs_Value.compare("PixelBuffer")==0
      && b_IsStartStructure)
   {
      pc_NewPattern = new PixelBuffer(pc_FileWriter);
   }


   PERICO_READ_STRUCTURE_END

   // set to Display

   m_b_InUpdate = true;
   m_ColorSelected->setChecked(false);
   m_PatternSelected->setChecked(false);
   m_MotiveSelected->setChecked(false);
   switch(m_i_ActiveMode)
   {
   case SelectColor:
      m_ColorSelected->setChecked(true);
      break;
   case SelectPattern:
      m_PatternSelected->setChecked(true);
      break;
   case SelectMotive:
      m_MotiveSelected->setChecked(true);
      break;
   default:
      break;
      
   }
   m_ColorWidget->SetColor(m_p_CurrentColor);

   if( m_pc_CurrentPattern != NULL)
   {
      delete m_pc_CurrentPattern;
   }
   m_pc_CurrentPattern = pc_NewPattern;
   m_PatternWidget->SetPixelBuffer(m_pc_CurrentPattern);

   // now call signals to tell the changed modes
   ImageDropped(m_pc_CurrentPattern);
   ColorChanged(m_p_CurrentColor);
   ModeChange(m_i_ActiveMode);
         
   m_b_InUpdate = false;

   // everything is OK
   return 0;
} // LoadFile

//===========================================================================
//
//===========================================================================


