//---------------------------------------------------------------------------
// This sourcefile is created by Peter Jodda / peter@jodda.de
// It is under the GPL license.
// If you expand this file, please contact me, that I can integrate the
// enhancements in the original file.
//---------------------------------------------------------------------------
// Module:  QUImageDragLabel
//---------------------------------------------------------------------------
// Purpose: small image, which allows drag and drop
//---------------------------------------------------------------------------

#include <qdragobject.h>
#include <qpopupmenu.h>
#include <qcursor.h>
#include <qimage.h>
#include <qpixmap.h>
#include <qiconset.h>
#include <qlayout.h>

#include "QUIconButton.h"
#include "Selection.h"
#include "ImageFrame.h"
#include "Tracer.h"
#include "QtUtilities.h"
#include "FileLoader.h"
#include "DragAndDropConstants.h"
#include "BufferPainter.h"
#include "MainFrame.h"
#include "ApplicationStorage.h"
#include "IconLibrary.h"

//===========================================================================
//
//===========================================================================


QUIconButton::QUIconButton(QWidget *p_Parent,
                     QString qs_IconName,
                     QString qs_DefaultText,
                     unsigned int ui_Mode,
                     GlobalStorage *pc_GlobalStorage)
                : QLabel(p_Parent) , Tracer("QUImageDragLabel")
{
   QColorGroup  qcg_ColorGroup;
   QPixmap     *pc_Icon;
   QColor       qc_Background;

   // save mode and global storage
   m_ui_Mode          = ui_Mode;
   m_pc_GlobalStorage = pc_GlobalStorage;
   m_b_Selected       = false;
   
   setFrameStyle(QFrame::NoFrame);

   // save the original background color
   qcg_ColorGroup     = colorGroup();
   m_qc_OriginalColor = qcg_ColorGroup.background();

   setMargin(5);

   // get icon from the library
   pc_Icon = g_Application->m_pc_IconLibrary->GetPixmap(qs_IconName);

   if(pc_Icon == NULL)
   {
      m_i_Width = 10;
      m_i_Height = 10;
   }
   else // icon available
   {

      m_i_Width  = pc_Icon->width();
      m_i_Height = pc_Icon->height();

      // attach the pixmap into the icon
      setPixmap(*pc_Icon);

   }
   
   setFixedSize(m_i_Width+10,m_i_Height+10);

   // add the tool tip
   QToolTip::add(this,qs_DefaultText);

   if(pc_Icon!= NULL)
   {
      delete pc_Icon;
   }

   // enable the dropping of objects
   if(m_ui_Mode & ModeImageReceiver)
   {
      setAcceptDrops(TRUE);
      m_DragAndDrop.SetAcceptType(DragAndDrop::AcceptPlain |
                                  DragAndDrop::AcceptPixelbuffer);
   }


         
} // QUIconButton()

//===========================================================================
//
//===========================================================================

QUIconButton::~QUIconButton()
{


}

//===========================================================================
//
//===========================================================================

void QUIconButton::mousePressEvent(QMouseEvent *theEvent)
{
   // right mouse botton
   if(theEvent->button() == g_Application->m_i_ButtonRegular)
   {
      // fprintf(stderr,"clicked\n");
      clicked();
   }
   else if(   theEvent->button() == g_Application->m_i_ButtonContext
      && m_ui_Mode & ModeContextMenu )
   {
      // right mouse button
      // create a popup menu
      QPopupMenu  *m_PopupMenu;
      int          i;
      int          i_Result;
      int          i_CurrentPosition;

      i_CurrentPosition = 0;
      m_PopupMenu = new QPopupMenu(this);

      for(i=0;
          i< (int)m_qsl_ContextMenuStrings.size();
          i++)
      {
         QString qs_Current;

         qs_Current = m_qsl_ContextMenuStrings[i];
         m_PopupMenu->insertItem(qs_Current,
                              i_CurrentPosition,
                              i_CurrentPosition );

         i_CurrentPosition++;
      }

      i_Result = m_PopupMenu->exec(QCursor::pos());

      if(i_Result >=0)
      {
         ContextMenuSelected(i_Result);
      }
      delete m_PopupMenu;

   }

} // mousePressEvent()

//===========================================================================
//
//===========================================================================

bool QUIconButton::GetSelected()
{
   return m_b_Selected;
} // GetSelected

//===========================================================================
//
//===========================================================================


void QUIconButton::SetSelected(bool b_Selected)
{
   QPalette    pal;

   // keep the same state?
   if(m_b_Selected == b_Selected)
   {
      // then do nothing
      return ;
   }

   m_b_Selected=b_Selected;

   if(m_b_Selected)
   {
      // make the widget red
      QColor      qc_Red;

      pal = palette();
      pal.setColor( QColorGroup::Background, QColor(255,209,236));

      setPalette( pal );
   }
   else
   {
      // make the widget normal

      pal = palette();
      pal.setColor( QColorGroup::Background, m_qc_OriginalColor );

      setPalette( pal );
   }

} // SetSelected


//===========================================================================
//
//===========================================================================


void QUIconButton::SetIcon(PixelBuffer *pc_Buffer)
{
   QPixmap *pc_Icon;

   if(pc_Buffer == NULL)
   {
      return;
   }

   // get icon from the library
   pc_Icon = IconLibrary::GetPixmapForPixelBuffer(pc_Buffer);

   m_i_Width  = pc_Icon->width();
   m_i_Height = pc_Icon->height();
   // attach the pixmap into the icon
   setPixmap(*pc_Icon);

   // icon is not needed any longer
   delete pc_Icon;
   setFixedSize(m_i_Width+10,m_i_Height+10);

} // SetIcon()

//===========================================================================
//
//===========================================================================

void QUIconButton::AddContextString(QString qs_String)
{
   m_qsl_ContextMenuStrings.push_back(qs_String);
} // AddContextString()

//===========================================================================
//
//===========================================================================

void QUIconButton::dragEnterEvent(QDragEnterEvent* event)
{
   bool       b_DoAccept;

   if(! (m_ui_Mode & ModeImageReceiver))
   {
      return;
   }

   b_DoAccept = m_DragAndDrop.IsEventAcceptable(event);

   event->accept( b_DoAccept );

} // dragEnterEvent()

//===========================================================================
//
//===========================================================================

void QUIconButton::dropEvent(QDropEvent* event)
{
   int i_AcceptType;

   if(! (m_ui_Mode & ModeImageReceiver))
   {
      return;
   }

   i_AcceptType = m_DragAndDrop.GetEventType(event);

   if( i_AcceptType == DragAndDrop::AcceptPixelbuffer)
   {
      PixelBuffer *pc_ReceivedImage;

      pc_ReceivedImage = m_DragAndDrop.GetPixelBuffer(event);
      if(pc_ReceivedImage == NULL)
      {
         return;
      }

      ImageDropped(pc_ReceivedImage);

      delete pc_ReceivedImage;
   }
   else if( i_AcceptType == DragAndDrop::AcceptPlain)
   {
      // try to load the file
      FileLoaderOptions c_FileOptions;
      PixelBuffer       *pc_Buffer;
      FileLoader        *pc_Loader;
      QString           qs_Filename;

      qs_Filename= m_DragAndDrop.GetFilename(event);
      if(qs_Filename.isEmpty())
      {
         return;
      }
      // create a file loader
      pc_Loader = new FileLoader(this);
      if(pc_Loader == NULL)
      {
         return;
      }

      // load the file
      pc_Buffer = pc_Loader->LoadFile(qs_Filename,&c_FileOptions);
      // success?
      if(pc_Buffer != NULL)
      {
         PixelBuffer *pc_SelectionMask;
         pc_SelectionMask = pc_Buffer->GetSelectionMask();
         if(      pc_SelectionMask != NULL
             &&  (! c_FileOptions.b_HasSelectionMask)   )
         {
            pc_SelectionMask->Clean(1);
         }
         ImageDropped(pc_Buffer);
         // the dropping function must copy the image
         // so we can delete it after the dropped signal
         delete pc_Buffer;
      }
      delete pc_Loader;

   }
} // dropEvent()

//===========================================================================
//
//===========================================================================



