//---------------------------------------------------------------------------
// This sourcefile is created by Peter Jodda / peter@jodda.de
// It is under the GPL license.
// If you expand this file, please contact me, that I can integrate the
// enhancements in the original file.
//---------------------------------------------------------------------------
// Module:  QUImageDragLabel
//---------------------------------------------------------------------------
// Purpose: small image, which allows drag and drop
//---------------------------------------------------------------------------

#include <qdragobject.h>
#include <qpopupmenu.h>
#include <qcursor.h>
#include <qimage.h>
#include <qpixmap.h>
#include <qiconset.h>
#include <qlayout.h>

#include "QUImageDragLabel.h"
#include "Selection.h"
#include "ImageFrame.h"
#include "Tracer.h"
#include "QtUtilities.h"
#include "FileLoader.h"
#include "DragAndDropConstants.h"
#include "BufferPainter.h"
#include "MainFrame.h"
#include "ApplicationStorage.h"

//===========================================================================
//
//===========================================================================


QUImageDragLabel::QUImageDragLabel(QWidget *p_Parent,
                     unsigned int ui_Mode,
                     GlobalStorage *pc_GlobalStorage,
                     int i_Width,
                     int i_Height)
                : QLabel(p_Parent) , Tracer("QUImageDragLabel")
{

   // save mode and global storage
   m_ui_Mode          = ui_Mode;
   m_pc_GlobalStorage = pc_GlobalStorage;

   m_i_SelectionMode = DragSelectionNone;

   setFrameStyle(QFrame::Box);
//   setLineWidth(2);
   setMargin(1);
   
   if(ui_Mode & ModeUseWidthHeight)
   {
      // set the given size
      m_i_Width  = i_Width;
      m_i_Height = i_Height;

      setFixedSize(m_i_Width+1,m_i_Height+1);
   }
   else
   {
      // set a fixed size for the widget
      m_i_Width  = 1;
      m_i_Height = 1;
      
      setFixedSize(m_i_Width,m_i_Height);
   }

   // enable the dropping of objects
   setAcceptDrops(TRUE);

   m_Pixmap         = new QPixmap();
   m_UnusedPixmap   = new QPixmap();
   m_p_DragData     = NULL;
   m_b_DragPossible = false;

   m_UpdateConnection.SetParentCallback(
                         Callback(PictureConnectionEventArrivedCB,this));

   m_DragAndDrop.SetAcceptType(DragAndDrop::AcceptPlain |
                               DragAndDrop::AcceptPixelbuffer);
} // QUImageDragLabel()

//===========================================================================
//
//===========================================================================

QUImageDragLabel::~QUImageDragLabel()
{

   delete m_Pixmap;
   delete m_UnusedPixmap ;
}

//===========================================================================
//
//===========================================================================

void QUImageDragLabel::SetImage(QImage *pc_Image)
{
   m_Pixmap->convertFromImage(*pc_Image,QPixmap::Color);

   setPixmap(*m_Pixmap);

} // SetImage()

//===========================================================================
//
//===========================================================================

void QUImageDragLabel::SetPixelBuffer(PixelBuffer *pc_ImageBuffer)
{

   QImage *pc_Image;
   int     i_SmallerFactorX;
   int     i_SmallerFactorY;
   int     i_SmallerFactor;
   int     i_ImageWidth;
   int     i_ImageHeight;
   int     i_NewWidth;
   int     i_NewHeight;
   int     i_RunX;
   int     i_RunY;
   int     i_OriginalX;
   int     i_OriginalY;
   int     i_CenterOffsetX;
   int     i_CenterOffsetY;


   // is it already stored?
   if(pc_ImageBuffer == m_p_DragData)
   {
      // then all is already done
      // return; -- did not work properly
   }
   
   if( pc_ImageBuffer == NULL)
   {
      // disable
      m_p_DragData     = NULL;
      m_b_DragPossible = false;

      setPixmap(*m_UnusedPixmap);
      return;
   }

   i_ImageWidth  = pc_ImageBuffer->GetWidth();
   i_ImageHeight = pc_ImageBuffer->GetHeight();

   i_SmallerFactorX = (i_ImageWidth  / m_i_Width)+1;
   i_SmallerFactorY = (i_ImageHeight / m_i_Height)+1;

   // take the larger of the both
   if(i_SmallerFactorX > i_SmallerFactorY)
   {
      i_SmallerFactor = i_SmallerFactorX;
   }
   else
   {
      i_SmallerFactor = i_SmallerFactorY;
   }

   i_NewWidth  = i_ImageWidth  / i_SmallerFactor;
   i_NewHeight = i_ImageHeight / i_SmallerFactor;

   pc_Image = new QImage(m_i_Width,
                         m_i_Height,
                         32);

   // first clear the image
   for(i_RunY=0; i_RunY < m_i_Height; i_RunY++)
   {
      for(i_RunX=0; i_RunX < m_i_Width; i_RunX++)
      {
         pc_Image->setPixel(i_RunX,i_RunY,0xffffffff);
      }
   }

   i_CenterOffsetX = (m_i_Width  - i_NewWidth ) / 2;
   i_CenterOffsetY = (m_i_Height - i_NewHeight) / 2;

   for(i_RunY=0 , i_OriginalY=0;
       i_RunY < i_NewHeight;
       i_RunY ++, i_OriginalY += i_SmallerFactor)
   {
      for(i_RunX=0 , i_OriginalX=0;
          i_RunX < i_NewWidth;
          i_RunX ++, i_OriginalX += i_SmallerFactor)
      {
         Pixel p;

         p = pc_ImageBuffer->Get(i_OriginalX,i_OriginalY);
         pc_Image->setPixel(i_RunX + i_CenterOffsetX,
                            i_RunY + i_CenterOffsetY,
                            p);
      } // for X
   } // for Y

   m_Pixmap->convertFromImage(*pc_Image,QPixmap::Color);

   delete pc_Image;

   setPixmap(*m_Pixmap);

   m_p_DragData     = (void *) pc_ImageBuffer;
   m_b_DragPossible = true;

} // SetPixelBuffer()

//===========================================================================
//
//===========================================================================
void QUImageDragLabel::dragMoveEvent(QDragMoveEvent* event)
{
   //fprintf(stderr,"Drag Move Event\n");
}

void QUImageDragLabel::dragEnterEvent(QDragEnterEvent* event)
{
   bool       b_DoAccept;

   b_DoAccept = m_DragAndDrop.IsEventAcceptable(event);
    
   event->accept( b_DoAccept );

} // dragEnterEvent()

//===========================================================================
//
//===========================================================================

void QUImageDragLabel::dropEvent(QDropEvent* event)
{
   int i_AcceptType;

   i_AcceptType = m_DragAndDrop.GetEventType(event);

   if(i_AcceptType == DragAndDrop::AcceptPlain)
   {
      // try to load the file
      FileLoaderOptions c_FileOptions;
      PixelBuffer       *pc_Buffer;
      FileLoader        *pc_Loader;
      QString           qs_Filename;

      qs_Filename= m_DragAndDrop.GetFilename(event);
      if(qs_Filename.isEmpty())
      {
         return;
      }
      // create a file loader
      pc_Loader = new FileLoader(this);
      if(pc_Loader == NULL)
      {
         return;
      }

      // load the file
      pc_Buffer = pc_Loader->LoadFile(qs_Filename,&c_FileOptions);
      // success?
      if(pc_Buffer != NULL)
      {
         PixelBuffer *pc_SelectionMask;
         pc_SelectionMask = pc_Buffer->GetSelectionMask();
         if(      pc_SelectionMask != NULL
             &&  (! c_FileOptions.b_HasSelectionMask)   )
         {
            pc_SelectionMask->Clean(1);
         }
         ImageDropped(pc_Buffer);
         // the dropping function must copy the image
         // so we can delete it after the dropped signal
         delete pc_Buffer;
      }
      delete pc_Loader;
      
   }
   else if(i_AcceptType == DragAndDrop::AcceptPixelbuffer)
   {
      PixelBuffer *pc_ReceivedImage;

      pc_ReceivedImage = m_DragAndDrop.GetPixelBuffer(event);
      if(pc_ReceivedImage == NULL)
      {
         return;
      }
      // image gets duplicates in imageDropped signal
      ImageDropped(pc_ReceivedImage);

      // so we may delete it here
      delete pc_ReceivedImage;      
   }

} // dropEvent()

//===========================================================================
//
//===========================================================================

void QUImageDragLabel::StartDrag()
{
   QStoredDrag * pc_Drag;
   QByteArray  * data;
   QWidget     * target;
   char          string[40];

   TRACE << "Start Drag EnterEvent\n";
   if(  ! m_b_DragPossible
      ||  m_p_DragData == NULL)
   {
      return ;
   }

   // allocate array to transfer the data
   data = new QByteArray(100);

   // write in the color
   sprintf(string,"%s=%lx",
           DRAGANDDROP_TYPE_PIXELBUFFER,
           (unsigned long) m_p_DragData);
   data->setRawData((const char *) string,strlen(string)+1);

   pc_Drag = new QStoredDrag( DRAGANDDROP_APPLICATION, this );
   pc_Drag->setEncodedData(*data);

   // attach pixmap at the drag object
   pc_Drag->setPixmap(*m_Pixmap,QPoint(0,0));

   // start the drag
   pc_Drag->dragMove();

   // get the target
   target = pc_Drag->target();
   if(target != NULL)
   {
      // if dragged, then notify
      //DragStarted(0);
   }
   //delete pc_Drag;

} // StartDrag()

//===========================================================================
//
//===========================================================================

void QUImageDragLabel::mousePressEvent(QMouseEvent *theEvent)
{
   // right mouse botton
   if(theEvent->button() == g_Application->m_i_ButtonRegular)
   {
      StartDrag();
   }

   
   // middle mouse button
   if(   theEvent->button() == g_Application->m_i_ButtonContext 
      && (   m_ui_Mode & ModeEnablePopup 
          || m_ui_Mode & ModeEnableEdit ))
   {
      // create a popup menu
      QPopupMenu  *m_SelectionPopupMenu;
      int          i_Result;
      int          i_Position=0;

      m_SelectionPopupMenu = new QPopupMenu(this);

      if(m_ui_Mode & ModeEnablePopup )
      {
         m_SelectionPopupMenu->insertItem(tr("Select a Rectangle"),0,i_Position++ );
         m_SelectionPopupMenu->insertItem(tr("Select Free Hand"),1,i_Position++ );
         m_SelectionPopupMenu->insertItem(tr("Select from Mask"),2,i_Position++ );
      }
      m_SelectionPopupMenu->insertItem(tr("Edit"),3,i_Position++);
      if(m_p_DragData != NULL)
      {
         m_SelectionPopupMenu->insertItem(tr("Save"),4,i_Position++);
      }
      m_SelectionPopupMenu->insertItem(tr("Load"),5,i_Position++);

      i_Result = m_SelectionPopupMenu->exec(QCursor::pos());

      if(i_Result ==0)
      {
         // start the rectangle

         // set the selection mode
         m_i_SelectionMode = DragSelectionRectangle;

         if(  m_pc_GlobalStorage != NULL)
         {
            m_pc_GlobalStorage->m_pc_Selection->ActivateShortTimeSelection(
                                 Selection::SelectionRectangle,
                                 tr("Cut Rectangle Motive"),
                                 Callback(SelectionEventArrivedCB,this)
                                 );

         }
      }
      else if(i_Result  == 1)
      {
         // start the rectangle
         // set the selection mode
         m_i_SelectionMode = DragSelectionFreehand;


         if(  m_pc_GlobalStorage != NULL)
         {
            m_pc_GlobalStorage->m_pc_Selection->ActivateShortTimeSelection(
                                 Selection::SelectionFreeDraw,
                                 tr("Cut free Hand Motive"),
                                 Callback(SelectionEventArrivedCB,this)
                                 );

         }
      }
      else if(i_Result  == 2)
      {
         // start the rectangle
         PixelBuffer * pc_NewImage;
         RectRegion    c_CopyRegion;

         pc_NewImage = m_pc_GlobalStorage->m_pc_MainImage
                ->CreateNewImageFromSelected( &c_CopyRegion);

         // tell parents that e new image is available
         ImageDropped(pc_NewImage);
         // application must copy it itself
         // so delete it
         delete pc_NewImage;
      }
      else if(   i_Result ==3
              && m_p_DragData != NULL)
      {
         
         MainFrame * pc_NewMainFrame;

         pc_NewMainFrame = MainFrame::NewPicture((PixelBuffer *)m_p_DragData);
         if(pc_NewMainFrame!= NULL)
         {
            pc_NewMainFrame->SetParentPicture(&m_UpdateConnection);
         }
      }         
      else if( i_Result ==4)
      {
         SaveImage();
      }
      else if( i_Result ==5)
      {
         LoadImage();
      }
      delete m_SelectionPopupMenu;
   }

} // mousePressEvent()

//===========================================================================
//
//===========================================================================

void *QUImageDragLabel::SelectionEventArrived(void *pc_Data)
{
   SelectionCallbackData *cb;

   cb = (SelectionCallbackData *) pc_Data;
   if(cb->i_CallbackReason == SelectionCallbackData:: ReasonOperationDisabled)
   {
      // clean up unused structures
      m_i_SelectionMode = DragSelectionNone;

   }
   else if(cb->i_CallbackReason == SelectionCallbackData:: ReasonSelection)
   {
      // fprintf(stderr,"MotiveBox: SelectionEventArrived\n");

      switch(m_i_SelectionMode)
      {
      case DragSelectionRectangle:
         CutRectangleImage();
         break;
      case DragSelectionFreehand:
         CutFreeHandImage();
         break;
      default:
        break;
      }
   }

   // reset the SelectionMode
   m_i_SelectionMode = DragSelectionNone;
   return NULL;
}

void *QUImageDragLabel::SelectionEventArrivedCB(void *Thisdata,void *pc_Data)
{
   QUImageDragLabel * pc_This;
   pc_This = (QUImageDragLabel *) Thisdata;

   return pc_This->SelectionEventArrived(pc_Data);
} // SelectionEventArrivedCB()

//===========================================================================
//
//===========================================================================

void QUImageDragLabel::CutRectangleImage()
{
   PixelBuffer *pc_NewImage;
   PixelBuffer *pc_SelectionMask;
   RectRegion   c_ClipRegion;
   RectRegion   c_CopyRegion;
   int          i_X1;
   int          i_X2;
   int          i_Y1;
   int          i_Y2;

   // get the Rectangle
   m_pc_GlobalStorage->m_pc_Selection->GetSelectionRectangle(i_X1,i_Y1,
                                                             i_X2,i_Y2);

   c_CopyRegion.SetXYXY( i_X1,i_Y1,i_X2,i_Y2);

   // get region of the image
   c_ClipRegion.SetXYWH(0,0,
                        m_pc_GlobalStorage->m_pc_MainImage->GetWidth(),
                        m_pc_GlobalStorage->m_pc_MainImage->GetHeight());

   // clip copyregion at the image
   c_CopyRegion.ClipAt(c_ClipRegion);

   pc_NewImage = m_pc_GlobalStorage->m_pc_MainImage->CreateNewImageFromRegion(c_CopyRegion);

   // fill the selection mask in the entire image
   pc_SelectionMask = pc_NewImage->GetSelectionMask();
   pc_SelectionMask->Clean(1);

   // tell parents that e new image is available
   ImageDropped(pc_NewImage);
   // application must copy it itself
   // so delete it
   delete pc_NewImage;
   
} // CutRectangleImage()

//===========================================================================
//
//===========================================================================

void QUImageDragLabel::CutFreeHandImage()
{
   PMemVector<PixelCoordinate> *pc_List;
   BufferPainter                  c_BufferPainter;
   PixelBuffer                   *pc_SelectionMask;
   PixelBuffer                   *pc_NewImage;
   RectRegion                     c_CopyRegion;
   int                            i_Size;

   // get the list of coordinates from the free hand selection
   pc_List = m_pc_GlobalStorage->m_pc_Selection->GetSelectionPointList();
   if(pc_List == NULL)
   {
      return;
   }

   // list should not be empty
   i_Size = pc_List->GetCount();
   if(i_Size ==0)
   {
      return;
   }


   // create a mask, same size as the original image,
   // only 1 bit depth
   pc_SelectionMask = new PixelBuffer(m_pc_GlobalStorage->m_pc_MainImage,
                        1,
                        false,false,false);
   if(pc_SelectionMask ==NULL)
   {
      return;
   }

   // prepare the buffer painter to draw in the mask
   c_BufferPainter.SetBorderColor(1);
   c_BufferPainter.SetInnerColor(1);
   c_BufferPainter.SetLineWidth(1);
   c_BufferPainter.SetDrawInnerEnable(true);

   // finaly draw it
   c_BufferPainter.DrawFreeFigure(pc_SelectionMask,
                       pc_List);


   // create image after the mask in the selection mask
   pc_NewImage = m_pc_GlobalStorage->m_pc_MainImage->CreateNewImageFromSelected(
                                  &c_CopyRegion,
                                  pc_SelectionMask);

   // delete the selection mask
   delete pc_SelectionMask;

   // tell parents that e new image is available
   ImageDropped(pc_NewImage);
   // application must copy it itself
   // so delete it
   delete pc_NewImage;

} // CutFreeHandImage()

//===========================================================================
//
//===========================================================================

void *QUImageDragLabel::PictureConnectionEventArrived(void *pc_Data)
{
   PictureUpdateConnectionMessage *pc_UpdateMessage;

   pc_UpdateMessage = (PictureUpdateConnectionMessage *) pc_Data;
   if(pc_UpdateMessage == NULL)
   {
      return NULL;
   }

   if(pc_UpdateMessage->m_i_MessageType
           ==  PictureUpdateConnectionMessage::ImageUpdated)
   {
      // tell parents that e new image is available
      ImageDropped(pc_UpdateMessage->m_pc_Buffer);
   }
   else if(pc_UpdateMessage->m_i_MessageType
           ==  PictureUpdateConnectionMessage::ChildTermination)
   {
      m_UpdateConnection.CleanChild();
   }
   else
   {
      // wrong message - should not occur
   }

   return NULL;
} // PictureConnectionEventArrived


void *QUImageDragLabel::PictureConnectionEventArrivedCB(void *Thisdata,void *pc_Data)
{
   QUImageDragLabel * pc_This;
   pc_This = (QUImageDragLabel *) Thisdata;

   return pc_This->PictureConnectionEventArrived(pc_Data);

} // PictureConnectionEventArrivedCB

//===========================================================================
//
//===========================================================================

void QUImageDragLabel::SaveImage()
{
   FileLoader *pc_FileWriter;
   QString     qs_Filename;

   pc_FileWriter = new FileLoader(NULL);
   
   qs_Filename = pc_FileWriter->AskForFilename(false,"");

   if(!qs_Filename.isEmpty())
   {
      pc_FileWriter->SaveFile(qs_Filename,(PixelBuffer *)m_p_DragData);
   }

   delete pc_FileWriter;

} // SaveImage()

//===========================================================================
//
//===========================================================================

void QUImageDragLabel::LoadImage()
{
   FileLoaderOptions  c_FileOptions;
   PixelBuffer       *pc_Buffer;
   FileLoader        *pc_Loader;
   QString           qs_Filename;

   pc_Loader = new FileLoader(NULL);

   qs_Filename = pc_Loader->AskForFilename(true,"");

   if(!qs_Filename.isEmpty())
   {
      // load the file
      pc_Buffer = pc_Loader->LoadFile(qs_Filename,&c_FileOptions);

      // success?
      if(pc_Buffer != NULL)
      {
         PixelBuffer *pc_SelectionMask;
         pc_SelectionMask = pc_Buffer->GetSelectionMask();
         if(      pc_SelectionMask != NULL
             &&  (! c_FileOptions.b_HasSelectionMask)   )
         {
            pc_SelectionMask->Clean(1);
         }
         ImageDropped(pc_Buffer);
         // the dropping function must copy the image
         // so we can delete it after the dropped signal
         delete pc_Buffer;
      } // if load success
   } // if filename available
   delete pc_Loader;

} // LoadImage()

//===========================================================================
//
//===========================================================================



