//---------------------------------------------------------------------------
// This sourcefile is created by Peter Jodda / peter@jodda.de
// It is under the GPL license.
// If you expand this file, please contact me, that I can integrate the
// enhancements in the original file.
//---------------------------------------------------------------------------
// Module: QULabelSlider.c
//---------------------------------------------------------------------------
// Purpose: Overloaded slider with labels on both sides.
//---------------------------------------------------------------------------


#include "QULabelSlider.h"

#include <stdlib.h>
#include <qhgroupbox.h>
#include <qlayout.h>

#include "QtUtilities.h"
#include "ApplicationStorage.h"


QULabelSlider::QULabelSlider(QWidget *parent,
                 int i_LabelStyle,
                 int i_From,
                 int i_To,
                 int i_Value,
                 QString qs_Description,
                 int i_DescriptionChars)
              : QWidget(parent)
{
   // create a hbox with a slider and a text on the right
   QHBoxLayout  *SliderLayout;

   m_i_LabelStyle     = i_LabelStyle;
   m_b_AutomaticLabel = false;

   m_i_FromValue = i_From;
   m_i_ToValue   = i_To;
   

   if(m_i_LabelStyle & LabelStyleAutomatic)
   {
      m_b_AutomaticLabel = true;
   }
   if(m_i_LabelStyle & LabelStyleEdit)
   {
      m_b_EditLabel = true;
   }
   

   m_i_CurrentValue   = i_Value;

   // to prevent updates whil in set functions
   m_b_DisableUpdate = false;

   m_i_DescriptionChars = i_DescriptionChars;


   // if autolabel enabled
   if(m_b_AutomaticLabel )
   {
      // set the label
      m_qs_CurrentLabel.sprintf("%5d",m_i_CurrentValue);
   }
   else
   {
      // leave it alone
      m_qs_CurrentLabel ="";
   }

   // create horizontal box
   SliderLayout  = new  QHBoxLayout( this,2);

   if( !qs_Description.isEmpty())
   {
      m_SliderDescription = new QLabel(qs_Description,this);
      if(m_i_DescriptionChars >0)
      {
         m_SliderDescription->setMinimumWidth(  g_Application->m_pc_QtUtility->GetMaximalFontWidth()
                                              * m_i_DescriptionChars);
      }
      SliderLayout->addWidget(m_SliderDescription);
   }
   else
   {
      m_SliderDescription = NULL;
   }

   // add slider
   m_Slider = new QSlider(i_From,
                          i_To,
                          1,
                          m_i_CurrentValue,
                          Horizontal,
                          this);
   // set a minimum width to the slider
   m_Slider->setMinimumWidth(  g_Application->m_pc_QtUtility->GetMaximalFontWidth()
                                              * 8);
   if( (i_To - i_From) < 20)
   {
      m_Slider->setTickmarks(QSlider::Below);
   }

   SliderLayout->addWidget(m_Slider);

   // add label
   if(m_b_EditLabel)
   {
      m_SliderLabel = NULL;

      m_SliderEdit = new QLineEdit(this);
      m_SliderEdit->setMaximumWidth(g_Application->m_pc_QtUtility->GetMaximalFontWidth() *4 );
      m_SliderEdit->setMinimumWidth(g_Application->m_pc_QtUtility->GetMaximalFontWidth() *4 );
      m_SliderEdit->setAlignment( Qt::AlignRight);

      m_qs_CurrentLabel.sprintf("%d",m_i_CurrentValue);
      m_SliderEdit->setText(m_qs_CurrentLabel);

      SliderLayout->addWidget(m_SliderEdit);

      connect( m_SliderEdit,
               SIGNAL(textChanged(const QString & )),
               SLOT(EditFieldChanged(const QString &)) );
   }
   else
   {
      m_SliderEdit = NULL;
      
      m_SliderLabel = new QLabel(m_qs_CurrentLabel,this);
      m_SliderLabel->setMinimumWidth(g_Application->m_pc_QtUtility->GetMaximalFontWidth() *3 );
      m_SliderLabel->setAlignment( Qt::AlignRight);
      m_SliderLabel->setText(m_qs_CurrentLabel);

      SliderLayout->addWidget(m_SliderLabel);
   }
   
   

   // and connect the value changed slider
   connect( m_Slider,  
               SIGNAL(valueChanged(int)), 
               SLOT(DoSliderValueChanged(int)) );


} // QULabelSlider() - constructor

//===========================================================================
//
//===========================================================================


QULabelSlider::~QULabelSlider()
{
} // ~QULabelSlider()

//===========================================================================
//
//===========================================================================


void QULabelSlider::SetValue(int i_Value,QString qs_Label)
{
   m_b_DisableUpdate= true;

   m_i_CurrentValue = i_Value;

   // set the value
   m_Slider->setValue(m_i_CurrentValue);

   // if there is a valid string, then set also the string
   if(! qs_Label.isEmpty())
   {
      SetLabel( qs_Label);
   }
   else
   {
      // no valid string
      // but automation is enabled
      if(m_b_AutomaticLabel)
      {
         if(m_SliderLabel != NULL)
         {
            m_qs_CurrentLabel.sprintf("%5d",m_i_CurrentValue);
            m_SliderLabel->setText(m_qs_CurrentLabel);
         }
         else if(m_SliderEdit != NULL)
         {
            m_qs_CurrentLabel.sprintf("%d",m_i_CurrentValue);
            m_SliderEdit->setText(m_qs_CurrentLabel);
         }
      }
   }


   m_b_DisableUpdate= false;

} // SetValue()

//===========================================================================
//
//===========================================================================

void QULabelSlider::SetLabel(QString qs_Label)
{
   m_qs_CurrentLabel = qs_Label;
   if(m_SliderLabel != NULL)
   {
      m_SliderLabel->setText(m_qs_CurrentLabel);
   }
   else if(m_SliderEdit != NULL)
   {
      m_SliderEdit->setText(m_qs_CurrentLabel);
   }

} // SetLabel()

//===========================================================================
//
//===========================================================================

int QULabelSlider::GetValue()
{
   return m_i_CurrentValue;
} // GetValue()

//===========================================================================
//
//===========================================================================


void QULabelSlider::DoSliderValueChanged(int i_NewValue)
{
   if(m_b_DisableUpdate)
   {
      //fprintf(stderr,"-- ignored\n");
      return;
   }
 
   m_i_CurrentValue = i_NewValue;

   m_b_DisableUpdate = true;
   if(m_b_AutomaticLabel)
   {
      if(m_SliderLabel != NULL)
      {
         m_qs_CurrentLabel.sprintf("%5d",i_NewValue);
         m_SliderLabel->setText(m_qs_CurrentLabel);
      }
      else if(m_SliderEdit != NULL)
      {
         m_qs_CurrentLabel.sprintf("%d",i_NewValue);
         m_SliderEdit->setText(m_qs_CurrentLabel);
      }
   }
   m_b_DisableUpdate = false;

   valueChanged( i_NewValue);

} // DoSliderValueChanged(int i_NewValue)

//===========================================================================
//
//===========================================================================

void QULabelSlider::SetLabelStringSize(int i_Letters)
{
   m_SliderDescription->setMinimumWidth(
               g_Application->m_pc_QtUtility->GetMaximalFontWidth()
                  * m_i_DescriptionChars);
  
} // SetLabelStringSize()

//===========================================================================
//
//===========================================================================

void QULabelSlider::EditFieldChanged(const QString &qs_NewText)
{
   int i_NewValue;
   
   if(m_b_DisableUpdate)
   {
      return;
   }

   // get new value as integer
   i_NewValue = atoi((const char *) qs_NewText);

   // clip at minimum and maximum value
   if(i_NewValue < m_i_FromValue)
   {
      i_NewValue = m_i_FromValue;
   }
   else if(i_NewValue > m_i_ToValue)
   {
      i_NewValue = m_i_ToValue;
   }

   // adjust the slider at this valie
   m_b_DisableUpdate = true;
   m_Slider->setValue(i_NewValue);
      
   m_b_DisableUpdate = false;

   // set the internal value
   m_i_CurrentValue = i_NewValue;

   // signal the change
   valueChanged( i_NewValue);
   
} // EditFieldChanged()

//===========================================================================
//
//===========================================================================


   

