//---------------------------------------------------------------------------
// This sourcefile is created by Peter Jodda / peter@jodda.de
// It is under the GPL license.
// If you expand this file, please contact me, that I can integrate the
// enhancements in the original file.
//---------------------------------------------------------------------------
// Module:  QUPixelButton.c
//---------------------------------------------------------------------------
// Purpose: Overload a button to display an icon instead
//---------------------------------------------------------------------------


#include "QUToolButton.h"

#include <qhgroupbox.h>
#include <qlayout.h>
#include <qpopupmenu.h>
#include <qcursor.h>

#include "QtUtilities.h"
#include "ApplicationStorage.h"
#include "IconLibrary.h"
#include "PixelBuffer.h"
#include "qtooltip.h"
#include "DragAndDropConstants.h"
#include "FileLoader.h"


//===========================================================================
//
//===========================================================================

QUToolButton::QUToolButton(QWidget *parent,
                 QString qs_IconName,
                 QString qs_DefaultText,
                 unsigned int ui_Mode) :
                 QToolButton(parent)
{
   QPixmap *pc_Icon;
   int      i_Width;
   int      i_Height;

   m_ui_Mode = ui_Mode;

   // get icon from the library
   pc_Icon = g_Application->m_pc_IconLibrary->GetPixmap(qs_IconName);

   if(pc_Icon == NULL)
   {
      // failed to load icon
      fprintf(stderr,"Failed to load icon %s\n",(const char *) qs_IconName);

      setText(qs_DefaultText);
      return;
   }

   i_Width  = pc_Icon->width();
   i_Height = pc_Icon->height();
   // attach the pixmap into the icon
   setPixmap(*pc_Icon);

   // icon is not needed any longer
   delete pc_Icon;
   setMaximumWidth  (i_Width +6);
   setMaximumHeight (i_Height+6);

   // add the tool tip
   QToolTip::add(this,qs_DefaultText);

   // enable the dropping of objects
   if(m_ui_Mode & ModeImageReceiver)
   {
      setAcceptDrops(TRUE);
      m_DragAndDrop.SetAcceptType(DragAndDrop::AcceptPlain |
                                  DragAndDrop::AcceptPixelbuffer);
   }


} // QUToolButton() - contructor

//===========================================================================
//
//===========================================================================

void QUToolButton::dragEnterEvent(QDragEnterEvent* event)
{
   bool       b_DoAccept;

   if(! (m_ui_Mode & ModeImageReceiver))
   {
      return;
   }

   b_DoAccept = m_DragAndDrop.IsEventAcceptable(event);

   event->accept( b_DoAccept );

} // dragEnterEvent()

//===========================================================================
//
//===========================================================================

void QUToolButton::dropEvent(QDropEvent* event)
{
   int i_AcceptType;
   
   if(! (m_ui_Mode & ModeImageReceiver))
   {
      return;
   }

   i_AcceptType = m_DragAndDrop.GetEventType(event);

   if( i_AcceptType == DragAndDrop::AcceptPixelbuffer)
   {
      PixelBuffer *pc_ReceivedImage;

      pc_ReceivedImage = m_DragAndDrop.GetPixelBuffer(event);
      if(pc_ReceivedImage == NULL)
      {
         return;
      }

      ImageDropped(pc_ReceivedImage);

      delete pc_ReceivedImage;
   }
   else if( i_AcceptType == DragAndDrop::AcceptPlain)
   {
      // try to load the file
      FileLoaderOptions c_FileOptions;
      PixelBuffer       *pc_Buffer;
      FileLoader        *pc_Loader;
      QString           qs_Filename;

      qs_Filename= m_DragAndDrop.GetFilename(event);
      if(qs_Filename.isEmpty())
      {
         return;
      }
      // create a file loader
      pc_Loader = new FileLoader(this);
      if(pc_Loader == NULL)
      {
         return;
      }

      // load the file
      pc_Buffer = pc_Loader->LoadFile(qs_Filename,&c_FileOptions);
      // success?
      if(pc_Buffer != NULL)
      {
         PixelBuffer *pc_SelectionMask;
         pc_SelectionMask = pc_Buffer->GetSelectionMask();
         if(      pc_SelectionMask != NULL
             &&  (! c_FileOptions.b_HasSelectionMask)   )
         {
            pc_SelectionMask->Clean(1);
         }
         ImageDropped(pc_Buffer);
         // the dropping function must copy the image
         // so we can delete it after the dropped signal
         delete pc_Buffer;
      }
      delete pc_Loader;
       
   }
} // dropEvent()

//===========================================================================
//
//===========================================================================

void QUToolButton::mousePressEvent ( QMouseEvent * theEvent )
{

   if(   theEvent->button() == g_Application->m_i_ButtonContext
      && m_ui_Mode & ModeContextMenu )
   {
      // right mouse button
      // create a popup menu
      QPopupMenu  *m_PopupMenu;
      int          i;
      int          i_Result;
      int          i_CurrentPosition;

      i_CurrentPosition = 0;
      m_PopupMenu = new QPopupMenu(this);

      for(i=0;
          i< (int)m_qsl_ContextMenuStrings.size();
          i++)
      {
         QString qs_Current;

         qs_Current = m_qsl_ContextMenuStrings[i];
         m_PopupMenu->insertItem(qs_Current,
                              i_CurrentPosition,
                              i_CurrentPosition );
      
         i_CurrentPosition++;
      }

      i_Result = m_PopupMenu->exec(QCursor::pos());

      if(i_Result >=0)
      {
         ContextMenuSelected(i_Result);
      }
      delete m_PopupMenu;
      
   }
   else // regular button
   {   
      QToolButton::mousePressEvent(theEvent);
   }
} // mousePressEvent()

//===========================================================================
//
//===========================================================================

void QUToolButton::AddContextString(QString qs_String)
{
   m_qsl_ContextMenuStrings.push_back(qs_String);
} // AddContextString()

//===========================================================================
//
//===========================================================================

void QUToolButton::SetIcon(PixelBuffer *pc_Buffer)
{
   QPixmap *pc_Icon;
   int      i_Width;
   int      i_Height;

   if(pc_Buffer == NULL)
   {
      return;
   }

   // get icon from the library
   pc_Icon = IconLibrary::GetPixmapForPixelBuffer(pc_Buffer);

   i_Width  = pc_Icon->width();
   i_Height = pc_Icon->height();
   // attach the pixmap into the icon
   setPixmap(*pc_Icon);

   // icon is not needed any longer
   delete pc_Icon;
   setMaximumWidth  (i_Width +6);
   setMaximumHeight (i_Height+6);

} // SetIcon()

//===========================================================================
//
//===========================================================================

