//---------------------------------------------------------------------------
// This sourcefile is created by Peter Jodda / peter@jodda.de
// It is under the GPL license.
// If you expand this file, please contact me, that I can integrate the
// enhancements in the original file.
//---------------------------------------------------------------------------
// Module: Undo.c
//---------------------------------------------------------------------------
// Purpose: store of undo buffers.
//---------------------------------------------------------------------------

#include <stdio.h>
#include <qwidget.h>

#include "Undo.h"
#include "IconBox.h"

//============================================================================
//
//============================================================================

Undo::Undo(QWidget *pc_Parent,
      GlobalStorage *pc_Global)  : StandardDialog(pc_Parent,pc_Global)
{
   int i;
   QBoxLayout   *l_Layout;

   LoadPosition("Undo");

   setCaption(tr("Undo Buffers"));

   l_Layout = new  QVBoxLayout( m_w_TopWidget,1);

   m_pc_Parent = pc_Parent;
   m_pc_IconBox = new ScrollIconBox(m_w_TopWidget,ScrollIconBox::UseScrollbar);
   l_Layout->addWidget(m_pc_IconBox);
   
   // default is 10
   m_i_NumberOfUndoBuffers = 10;
   // alloc array with pointers for the buffers
   m_pc_UndoBufferList     = new PixelBuffer *[m_i_NumberOfUndoBuffers];
   m_i_ActualImageIndex    = -1;
   m_i_ImagesFilled        = 0;

   // init the pointers to NULL
   for(i=0;
       i<m_i_NumberOfUndoBuffers;
       i++)
   {
      m_pc_UndoBufferList[i] = NULL;
   }

   connect(m_pc_IconBox,
           SIGNAL(IconSelected(int)),
           this,
           SLOT(BoxIconSelected(int)) );


} // Undo() - constructor

//============================================================================
//
//============================================================================

Undo::~Undo()
{
   int i;

   // delete all buffers
   for(i=0;
       i<m_i_NumberOfUndoBuffers;
       i++)
   {
      if(m_pc_UndoBufferList[i] != NULL)
      {
         delete m_pc_UndoBufferList[i];
         m_pc_UndoBufferList[i] = NULL;
      }
   }

   delete m_pc_UndoBufferList; 

} // Undo() - destructor

//============================================================================
//
//============================================================================

void Undo::Reset()
{
   int i;

   // delete all stored images
   for(i=0;
       i<m_i_NumberOfUndoBuffers;
       i++)
   {
      if(m_pc_UndoBufferList[i] != NULL)
      {
         delete m_pc_UndoBufferList[i];
         m_pc_UndoBufferList[i] = NULL;
      }
   }
   m_i_ActualImageIndex    = 0;
   m_i_ImagesFilled        = 0;

   TransferIcons();

} // Reset();

//============================================================================
//
//============================================================================

void Undo::SaveBufferToUndo(PixelBuffer *pc_Original)
{
   int i;

   if(pc_Original == NULL)
   {
      return;
   }
   
   if(m_i_ImagesFilled > m_i_ActualImageIndex)
   {
      // delete existing redo buffers
      int i;

      for(i = m_i_ImagesFilled;
          i > m_i_ActualImageIndex;
          i --
          )
      {
         if(m_pc_UndoBufferList[i] != NULL)
         {
            delete m_pc_UndoBufferList[i];
            m_pc_UndoBufferList[i] = NULL;
         }
      }
      m_i_ImagesFilled = m_i_ActualImageIndex;
   } // if redo buffers available

  
   if(m_i_ActualImageIndex +1 >= m_i_NumberOfUndoBuffers)
   {
      // all is filled, delete the oldest one
      delete m_pc_UndoBufferList[0];
      // copy all one to the front
      for(i=0;i<m_i_NumberOfUndoBuffers-1;i++)
      {
         m_pc_UndoBufferList[i] = m_pc_UndoBufferList[i+1] ;
      }
      m_pc_UndoBufferList[m_i_NumberOfUndoBuffers-1] =0;

      m_i_ActualImageIndex = m_i_NumberOfUndoBuffers-2;
      m_i_ImagesFilled = m_i_ActualImageIndex;
      
   }

   m_i_ActualImageIndex++;
   m_pc_UndoBufferList[m_i_ActualImageIndex] = new PixelBuffer(pc_Original);
   m_i_ImagesFilled = m_i_ActualImageIndex;

   // fprintf(stderr,"SaveBufferToUndo %x\n",pc_Original);
   // print();
   TransferIcons();
} // SaveBufferToUndo

//============================================================================
//
//============================================================================

void Undo::DoUndo(PixelBuffer * &pc_BufferToGetRestored,
                  bool &b_SizeChanged)
{
   // fprintf(stderr,"doUndo\n");
   // print();

   // if there are buffers before the actual buffer,
   // then restore the older one
   if(m_i_ActualImageIndex>0)
   {
      m_i_ActualImageIndex --;
      RestoreFrom(pc_BufferToGetRestored,
                  m_pc_UndoBufferList[m_i_ActualImageIndex],
                  b_SizeChanged);
      
   }
   SetActiveIcon();
} // DoUndo

//============================================================================
//
//============================================================================

void Undo::DoUndo(PixelBuffer * &pc_BufferToGetRestored,
                  bool &b_SizeChanged,
                  int i_Version)
{
   // fprintf(stderr,"doUndo\n");
   // print();

   // if there are buffers before the actual buffer,
   // then restore the older one
   b_SizeChanged = false;
   
   if(    i_Version >=0
       && i_Version <= m_i_ImagesFilled)
   {
      m_i_ActualImageIndex = i_Version;

      RestoreFrom(pc_BufferToGetRestored,
                  m_pc_UndoBufferList[m_i_ActualImageIndex],
                  b_SizeChanged);

      SetActiveIcon();
   }

} // DoUndo

//============================================================================
//
//============================================================================

void Undo::DoRedo(PixelBuffer * &pc_BufferToGetRestored,
                  bool &b_SizeChanged)
{
   // fprintf(stderr,"doRedo\n");
   // print();
   
   // if there are buffers stored behind the actual
   // position, then restore the nest buffer
   if(m_i_ActualImageIndex<m_i_ImagesFilled)
   {
      m_i_ActualImageIndex ++;
      RestoreFrom(pc_BufferToGetRestored,
                  m_pc_UndoBufferList[m_i_ActualImageIndex],
                  b_SizeChanged);

   }
   SetActiveIcon();

} // DoRedo()

//============================================================================
//
//============================================================================

bool Undo::IsRedoPossible()
{
   if(m_i_ActualImageIndex<m_i_ImagesFilled)
   {
      return true;
   }
   return false;
} // IsRedoPossible()

//============================================================================
//
//============================================================================

bool Undo::IsUndoPossible()
{
   if(m_i_ActualImageIndex>0)
   {
      return true;
   }
   return false;
} // IsUndoPossible()

//============================================================================
//
//============================================================================

PixelBuffer *Undo::GetActualPicture()
{
   return m_pc_UndoBufferList[m_i_ActualImageIndex];
} // GetActualPicture()

//============================================================================
//
//============================================================================

void Undo::RestoreFrom(PixelBuffer *pc_Destination,
                           PixelBuffer *pc_Source,
                           bool &b_SizeChanged)
{
   if(pc_Source->HasEqualSize(pc_Destination) )
   {
      // just copy the contents
      pc_Destination->CopyFrom(pc_Source);
      b_SizeChanged = false;
   }
   else // different size
   {
      // resize the original picture before
      pc_Destination->ResizeInternalTo(pc_Source->GetWidth(),
                       pc_Source->GetHeight());

      // now copy into the resized buffer
      pc_Destination->CopyFrom(pc_Source);

      b_SizeChanged = true;

   } // different size

} // RestoreFrom()

//============================================================================
//
//============================================================================

void Undo::print()
{
   int i;
   fprintf(stderr,"Undo m_i_ActualImageIndex=%d, m_i_ImagesFilled=%d\n",
         m_i_ActualImageIndex,
         m_i_ImagesFilled);
   for(i=0;i<=m_i_ImagesFilled;i++)
   {
      fprintf(stderr,"%d = %x\n",i,(int)(m_pc_UndoBufferList[i]));
   }
            
          
} // print()

//============================================================================
//
//============================================================================

void Undo::TransferIcons()
{
   QString qs_Text;
   int     i;
   m_pc_IconBox->DeleteAllImages();

   
   for(i=0;i<=m_i_ImagesFilled;i++)
   {
      qs_Text.sprintf("%d",i);
      m_pc_IconBox->AddImage(m_pc_UndoBufferList[i],qs_Text);
      
   }
   SetActiveIcon();
   
} // TransferIcons()

//============================================================================
//
//============================================================================

void Undo::SetActiveIcon()
{
   if(m_i_ImagesFilled>=0)
   {
      m_pc_IconBox->SetSelected(m_i_ActualImageIndex);
   }

} // SetActiveIcon()

//============================================================================
//
//============================================================================

void Undo::BoxIconSelected(int i_Value)
{
   IconSelected(i_Value);
} // BoxIconSelected()

//===========================================================================
//
//===========================================================================

void Undo::ShowSelectionBox()
{
   show();
} // ShowSelectionBox()

//===========================================================================
//
//===========================================================================



