#ifndef DSP2D_H
#define DSP2D_H

using namespace std;

#include <stdlib.h>
#include <iostream>
#include <fstream>
#include <math.h>
#include <stdio.h>

#include <string>
#include <vector>
#include <set>
#include <map>
#include <algorithm>

#include <qstring.h>
#include "PList.h"

class XenoKERNEL_MANAGER;              // Forward declarations
class XenoKERNEL2D;
class XenoCACHE2D;
class XenoDSP2D;

//###########################################################################
//###########################################################################

class XenoDSP2D
{
   public:
      XenoDSP2D(unsigned int width=0, unsigned int height=0);
      ~XenoDSP2D();

      void getMinMax (int&,int&);
 
      // neue groesse einstellen
      void            resize    (unsigned int width, unsigned int height);
      unsigned int    width     ();
      unsigned int    height    ();
 
      // set und get
      double          get       (int, int);
      void            set       (int, int, double);
 
      double          & operator[](unsigned int);

      void            operator *=(double); 
      void            operator +=(double); 
      void            resetPixelCounter();
      double          getNextPixel();
      void            setNextPixel(double);
private:
 //     string        x;
      unsigned int  m_i_Width;
      unsigned int  m_i_Height;
      unsigned long m_ul_PixelCounter;

      // top ist wight * height
      unsigned long m_i_NumberOfPixel;

      // daten normiert von 0.0 .. 1.0
      double        *pd_Data;

      friend class XenoCACHE2D;
      friend class XenoKERNEL2D;
};

//###########################################################################
//###########################################################################

class XenoKERNEL_MANAGER
{
   // Delivers kernel information
public:
   // The manager is needed to resolve the recursive definitions
   // inside of the kernel files.
   // It stores kernels according to their names

   // Deliver the kernel with this name or NULL if not found
   XenoKERNEL2D* resolve    (const QString);

   // Call resolve() on all known kernels
   void      resolveAll ();
   // Store a kernel
   void      checkin    (QString &, XenoKERNEL2D*);
   // Remove kernel from storage
   void      checkout   (QString &);
   // Delete all pointers - checkoutAll is called first
   void      deleteAll  ();
   // Clear all filter caches
   void      clearCaches();                    

   private:

   PPtrList<XenoKERNEL2D> m_l_KernelList;
   map<QString,XenoKERNEL2D*> kernels;               // Kernel store

   // True if checkin/out since last resolveAll()
   bool changed;

};

//###########################################################################
//###########################################################################

class XenoKFactor
{
   // Kernel factor class
public:
   // krn==NULL means 'take the input data' all other settings will
   // use recursion to other kernels
   XenoKFactor();

   // Constructor - name=="" will set krn to NULL
   XenoKFactor(QString);

   // Insert new volterra parameter
   void   ins  (int,int);
   
   double get  (XenoDSP2D&,int,int);

   // Resolve kernel name using this manager   
   void   res  (XenoKERNEL_MANAGER*); 
   void   save (ofstream&);
   int    size ();
   void   clear();

private:
   // Pointer to recursion
   // - if NULL,this kernel will take the input,
   // if not it will take the output of another kernel
   XenoKERNEL2D* krn;

   // Volterra factors
   multiset<pair<int,int> > volterra;

   // Filter Name
   QString name;

};

//###########################################################################
//###########################################################################

class XenoCACHE2D {
   // The cache works automagically.
   // it stores up to three results (whole images)
public:
   // Every kernel gets one
   XenoCACHE2D();
   ~XenoCACHE2D();
   // It avoids pixel recalculations in case of filter-of
   // filters definitions (P 1.0 0 0 KERNEL 0 0 KERNEL ....)

   // Set value for DSP2D% source image at posiiton long
   void set   (XenoDSP2D&,long,double );
   // Returns true and value if value was cached
   bool get   (XenoDSP2D&,long,double&);

   // Reset cache
   void clear();
private:
   // Cached data
   map<XenoDSP2D*,pair<bool*,double*> > cdata;
};


//###########################################################################
//###########################################################################

class XenoKERNEL2D
{
public:
   XenoKERNEL2D(XenoKERNEL_MANAGER*);
   XenoKERNEL2D(QString qs_Name,XenoKERNEL_MANAGER*);
   ~XenoKERNEL2D();
   
   // The volterra product is generated by calling insert(x,y) many times
   // after all coefficients were defined, the prduct is closed with weight()
   double getFilter (XenoDSP2D&, int, int);
   
   // Filter at position x,y
   void   filter    (XenoDSP2D&, XenoDSP2D&, int, int);

   // Filter the whole image
   void   filter    (XenoDSP2D&, XenoDSP2D&);

   // Save filter   
   void   save      (const char*);

   // Load filter
   void   load      (QString qs_Name);

   void   clear     ();
   // Resolves the names of the kernels to pointers
   void   resolve   ();

   void   insert    (int,int);
   void   weight    (double);

   // True if the filter should not be visible in the user menus
   bool   isHidden	 ();

   void   clearCache();

   QString getName   ();
   QString getFileName   ();

private:
   // Convolution
   double getConv (XenoDSP2D&, int, int);

   void   convolve(XenoDSP2D&, XenoDSP2D&, int, int);
   void   convolve(XenoDSP2D&, XenoDSP2D&);
   // Math. morphology
   double getMorph(XenoDSP2D&, int, int);

   void   morph   (XenoDSP2D&, XenoDSP2D&, int, int);
   void   morph   (XenoDSP2D&, XenoDSP2D&);
   // Ranking filter
   double getRank (XenoDSP2D&, int, int);

   void   rank    (XenoDSP2D&, XenoDSP2D&, int, int);
   void   rank    (XenoDSP2D&, XenoDSP2D&);

   // Only if !morph_on: remove zero-weighted summands
   void   clean   ();

   // Calculates some parameters used in morphology mode
   void   calcSel ();

   // Marks a morphologic filter
   bool   morph_on;

   // Marks a rank filter
   bool   rank_on;

   // Selection [0;1] - used in morphologic filters (morph_on==true)
   double morph_select;

   // Scaling of the rank filter's output
   double rank_scale;

   
   // Weighted integer selection - generated by calcSel
   int  morph_select1;
   int  morph_select2;

   double morph_weight1;
   double morph_weight2;

   XenoKFactor buffer;
   // The filter description
   map< set < XenoKFactor* >,double> FDescriptor;

   // Name of this filter
   QString m_qs_Name;
   QString m_qs_Filename;

   // The kernel manager is needed to resolve kernel names to pointers
   XenoKERNEL_MANAGER *manager;

   // The result cache
   XenoCACHE2D *ch;

};


#endif
